#ifndef __DICHHDEFS
#define __DICHHDEFS
#include <assert.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifndef WIN32
#include <unistd.h>
#endif
#ifdef __VMS
#include <starlet.h>
#endif
#include "dim_core.hxx"
#include "dim.hxx"
#include "tokenstring.hxx"

enum DimServiceType {DimSERVICE=1, DimCOMMAND, DimRPC};

class DimClient;
class DimInfo;
class DimCurrentInfo;
class DimRpcInfo;

class DllExp DimInfoHandler{
public:
	DimInfo *itsService;
    DimInfo *getInfo() { return itsService; }; 
	virtual void infoHandler() = 0;
};

class DllExp DimInfo : public DimInfoHandler, public DimTimer{

public :
	DimInfoHandler *itsHandler;

	DimInfo(){};
	DimInfo(const char *name, int nolink) 
		{ subscribe((char *)name, 0, &nolink, sizeof(int), 0); };
	DimInfo(const char *name, int time, int nolink) 
		{ subscribe((char *)name, time, &nolink, sizeof(int), 0); };
	DimInfo(const char *name, float nolink) 
		{ subscribe((char *)name, 0, &nolink, sizeof(float), 0); };
	DimInfo(const char *name, int time, float nolink) 
		{ subscribe((char *)name, time, &nolink, sizeof(float), 0); };
	DimInfo(const char *name, double nolink) 
		{ subscribe((char *)name, 0, &nolink, sizeof(double), 0); };
	DimInfo(const char *name, int time, double nolink) 
		{ subscribe((char *)name, time, &nolink, sizeof(double), 0); };
	DimInfo(const char *name, char *nolink) 
		{ subscribe((char *)name, 0, nolink, strlen(nolink)+1, 0); };
	DimInfo(const char *name, int time, char *nolink) 
		{ subscribe((char *)name, time, nolink, strlen(nolink)+1, 0); };
	DimInfo(const char *name, void *nolink, int nolinksize) 
		{ subscribe((char *)name, 0, nolink, nolinksize, 0); };
	DimInfo(const char *name, int time, void *nolink, int nolinksize) 
		{ subscribe((char *)name, time, nolink, nolinksize, 0); };

	DimInfo(const char *name, int nolink, DimInfoHandler *handler) 
		{ subscribe((char *)name, 0, &nolink, sizeof(int), handler); };
	DimInfo(const char *name, int time, int nolink, DimInfoHandler *handler) 
		{ subscribe((char *)name, time, &nolink, sizeof(int), handler); };
	DimInfo(const char *name, float nolink, DimInfoHandler *handler) 
		{ subscribe((char *)name, 0, &nolink, sizeof(float), handler); };
	DimInfo(const char *name, int time, float nolink, DimInfoHandler *handler) 
		{ subscribe((char *)name, time, &nolink, sizeof(float), handler); };
	DimInfo(const char *name, double nolink, DimInfoHandler *handler) 
		{ subscribe((char *)name, 0, &nolink, sizeof(double), handler); };
	DimInfo(const char *name, int time, double nolink, DimInfoHandler *handler) 
		{ subscribe((char *)name, time, &nolink, sizeof(double), handler); };
	DimInfo(const char *name, char *nolink, DimInfoHandler *handler) 
		{ subscribe((char *)name, 0, nolink, strlen(nolink)+1, handler); };
	DimInfo(const char *name, int time, char *nolink, DimInfoHandler *handler) 
		{ subscribe((char *)name, time, nolink, strlen(nolink)+1, handler); };
	DimInfo(const char *name, void *nolink, int nolinksize, DimInfoHandler *handler) 
		{ subscribe((char *)name, 0, nolink, nolinksize, handler); };
	DimInfo(const char *name, int time, void *nolink, int nolinksize, DimInfoHandler *handler) 
		{ subscribe((char *)name, time, nolink, nolinksize, handler); };

	virtual ~DimInfo();
	void *itsData;
	int itsDataSize;
	int itsSize;
	int getSize() {return itsSize; };
	char *getName()  { return itsName; } ;
	void *getData();
	int getInt() { return *(int *)getData(); } ;
	float getFloat() { return *(float *)getData(); } ;
	double getDouble() { return *(double *)getData(); } ;
	char *getString()  { return (char *)getData(); } ;

	virtual void infoHandler() {};
	void timerHandler();
	virtual void subscribe(char *name, int time, void *nolink, int nolinksize,
		DimInfoHandler *handler);
	virtual void doIt();
	int getQuality();
	int getTimestamp();
	int getTimestampMillisecs();
	char *getFormat();

protected :
	char *itsName;
	int itsId;
	int itsTime;
	int itsType;
	char *itsFormat;
	void *itsNolinkBuf;
	int itsNolinkSize;
	int secs, milisecs;
};

class DllExp DimStampedInfo : public DimInfo{

public :
	DimStampedInfo(){};
	DimStampedInfo(const char *name, int nolink) 
	{ subscribe((char *)name, 0, &nolink, sizeof(int), 0); };
	DimStampedInfo(const char *name, int time, int nolink) 
	{ subscribe((char *)name, time, &nolink, sizeof(int), 0); };
	DimStampedInfo(const char *name, float nolink) 
	{ subscribe((char *)name, 0, &nolink, sizeof(float), 0); };
	DimStampedInfo(const char *name, int time, float nolink) 
	{ subscribe((char *)name, time, &nolink, sizeof(float), 0); };
	DimStampedInfo(const char *name, double nolink) 
	{ subscribe((char *)name, 0, &nolink, sizeof(double), 0); };
	DimStampedInfo(const char *name, int time, double nolink) 
	{ subscribe((char *)name, time, &nolink, sizeof(double), 0); };
	DimStampedInfo(const char *name, char *nolink) 
	{ subscribe((char *)name, 0, nolink, strlen(nolink)+1, 0); };
	DimStampedInfo(const char *name, int time, char *nolink) 
	{ subscribe((char *)name, time, nolink, strlen(nolink)+1, 0); };
	DimStampedInfo(const char *name, void *nolink, int nolinksize) 
	{ subscribe((char *)name, 0, nolink, nolinksize, 0); };
	DimStampedInfo(const char *name, int time, void *nolink, int nolinksize) 
	{ subscribe((char *)name, time, nolink, nolinksize, 0); };

	DimStampedInfo(const char *name, int nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, 0, &nolink, sizeof(int), handler); };
	DimStampedInfo(const char *name, int time, int nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, time, &nolink, sizeof(int), handler); };
	DimStampedInfo(const char *name, float nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, 0, &nolink, sizeof(float), handler); };
	DimStampedInfo(const char *name, int time, float nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, time, &nolink, sizeof(float), handler); };
	DimStampedInfo(const char *name, double nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, 0, &nolink, sizeof(double), handler); };
	DimStampedInfo(const char *name, int time, double nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, time, &nolink, sizeof(double), handler); };
	DimStampedInfo(const char *name, char *nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, 0, nolink, strlen(nolink)+1, handler); };
	DimStampedInfo(const char *name, int time, char *nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, time, nolink, strlen(nolink)+1, handler); };
	DimStampedInfo(const char *name, void *nolink, int nolinksize, DimInfoHandler *handler) 
	{ subscribe((char *)name, 0, nolink, nolinksize, handler); };
	DimStampedInfo(const char *name, int time, void *nolink, int nolinksize, DimInfoHandler *handler) 
	{ subscribe((char *)name, time, nolink, nolinksize, handler); };

	virtual ~DimStampedInfo();
private :
	void doIt();
	void subscribe(char *name, int time, void *nolink, int nolinksize,
		DimInfoHandler *handler);
};

class DllExp DimUpdatedInfo : public DimInfo{

public :
	DimUpdatedInfo(){};
	DimUpdatedInfo(const char *name, int nolink) 
	{ subscribe((char *)name, 0, &nolink, sizeof(int), 0); };
	DimUpdatedInfo(const char *name, int time, int nolink) 
	{ subscribe((char *)name, time, &nolink, sizeof(int), 0); };
	DimUpdatedInfo(const char *name, float nolink) 
	{ subscribe((char *)name, 0, &nolink, sizeof(float), 0); };
	DimUpdatedInfo(const char *name, int time, float nolink) 
	{ subscribe((char *)name, time, &nolink, sizeof(float), 0); };
	DimUpdatedInfo(const char *name, double nolink) 
	{ subscribe((char *)name, 0, &nolink, sizeof(double), 0); };
	DimUpdatedInfo(const char *name, int time, double nolink) 
	{ subscribe((char *)name, time, &nolink, sizeof(double), 0); };
	DimUpdatedInfo(const char *name, char *nolink) 
	{ subscribe((char *)name, 0, nolink, strlen(nolink)+1, 0); };
	DimUpdatedInfo(const char *name, int time, char *nolink) 
	{ subscribe((char *)name, time, nolink, strlen(nolink)+1, 0); };
	DimUpdatedInfo(const char *name, void *nolink, int nolinksize) 
	{ subscribe((char *)name, 0, nolink, nolinksize, 0); };
	DimUpdatedInfo(const char *name, int time, void *nolink, int nolinksize) 
	{ subscribe((char *)name, time, nolink, nolinksize, 0); };

	DimUpdatedInfo(const char *name, int nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, 0, &nolink, sizeof(int), handler); };
	DimUpdatedInfo(const char *name, int time, int nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, time, &nolink, sizeof(int), handler); };
	DimUpdatedInfo(const char *name, float nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, 0, &nolink, sizeof(float), handler); };
	DimUpdatedInfo(const char *name, int time, float nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, time, &nolink, sizeof(float), handler); };
	DimUpdatedInfo(const char *name, double nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, 0, &nolink, sizeof(double), handler); };
	DimUpdatedInfo(const char *name, int time, double nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, time, &nolink, sizeof(double), handler); };
	DimUpdatedInfo(const char *name, char *nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, 0, nolink, strlen(nolink)+1, handler); };
	DimUpdatedInfo(const char *name, int time, char *nolink, DimInfoHandler *handler) 
	{ subscribe((char *)name, time, nolink, strlen(nolink)+1, handler); };
	DimUpdatedInfo(const char *name, void *nolink, int nolinksize, DimInfoHandler *handler) 
	{ subscribe((char *)name, 0, nolink, nolinksize, handler); };
	DimUpdatedInfo(const char *name, int time, void *nolink, int nolinksize, DimInfoHandler *handler) 
	{ subscribe((char *)name, time, nolink, nolinksize, handler); };

	virtual ~DimUpdatedInfo();
private :
	void doIt();
	void subscribe(char *name, int time, void *nolink, int nolinksize,
		DimInfoHandler *handler);
};

class DllExp DimCmnd {
public :

	int wakeUp;
	int result;
	int send(char *name, void *data, int datasize);
	void sendNB(char *name, void *data, int datasize);
  DimCmnd(){};
};

class DllExp DimCurrentInfo {

public :
	void *itsData;
	int itsDataSize;
	int itsSize;
	int wakeUp;

	DimCurrentInfo(const char *name, int nolink) { 
		subscribe((char *)name, 0, &nolink, sizeof(int)); };
	DimCurrentInfo(const char *name, float nolink) { 
		subscribe((char *)name, 0, &nolink, sizeof(float)); };
	DimCurrentInfo(const char *name, double nolink) { 
		subscribe((char *)name, 0, &nolink, sizeof(double)); };
	DimCurrentInfo(const char *name, char *nolink) { 
		subscribe((char *)name, 0, nolink, strlen(nolink)+1); };
	DimCurrentInfo(const char *name, void *nolink, int nolinksize) { 
		subscribe((char *)name, 0, nolink, nolinksize); };
	DimCurrentInfo(const char *name, int time, int nolink) { 
		subscribe((char *)name, time, &nolink, sizeof(int)); };
	DimCurrentInfo(const char *name, int time, float nolink) { 
		subscribe((char *)name, time, &nolink, sizeof(float)); };
	DimCurrentInfo(const char *name, int time, double nolink) { 
		subscribe((char *)name, time, &nolink, sizeof(double)); };
	DimCurrentInfo(const char *name, int time, char *nolink) { 
		subscribe((char *)name, time, nolink, strlen(nolink)+1); };
	DimCurrentInfo(const char *name, int time, void *nolink, int nolinksize) { 
		subscribe((char *)name, time, nolink, nolinksize); };


	virtual ~DimCurrentInfo();
	char *getName()  { return itsName; } ;
	void *getData();
	int getInt() { return *(int *)getData(); } ;
	float getFloat() { return *(float *)getData(); } ;
	double getDouble() { return *(double *)getData(); } ;
	char *getString()  { return (char *)getData(); } ;
	int getSize()  { getData(); return itsSize; } ;

private :
	char *itsName;
	void *itsNolinkBuf;
	int itsNolinkSize;
	void subscribe(char *name, int time, void *nolink, int nolinksize);
};

class DllExp DimRpcInfo {

public :
	int itsId;
	int itsInit;
	void *itsData;
	int itsDataSize;
	void *itsDataOut;
	int itsDataOutSize;
	int itsSize;
	int wakeUp;
	int itsWaiting;
	void *itsNolinkBuf;
	int itsNolinkSize;
	DimRpcInfo *itsHandler;

	DimRpcInfo(const char *name, int nolink) { 
		subscribe((char *)name, 0, 0, &nolink, sizeof(int), 0); };
	DimRpcInfo(const char *name, float nolink) { 
		subscribe((char *)name, 0, 0, &nolink, sizeof(float), 0); };
	DimRpcInfo(const char *name, double nolink) { 
		subscribe((char *)name, 0, 0, &nolink, sizeof(double), 0); };
	DimRpcInfo(const char *name, char *nolink) { 
		subscribe((char *)name, 0, 0, nolink, strlen(nolink)+1, 0); };
	DimRpcInfo(const char *name, void *nolink, int nolinksize) { 
		subscribe((char *)name, 0, 0, nolink, nolinksize, 0); };

	DimRpcInfo(const char *name, int time, int nolink) { 
		subscribe((char *)name, 0, 0, &nolink, sizeof(int), time); };
	DimRpcInfo(const char *name, int time, float nolink) { 
		subscribe((char *)name, 0, 0, &nolink, sizeof(float), time); };
	DimRpcInfo(const char *name, int time, double nolink) { 
		subscribe((char *)name, 0, 0, &nolink, sizeof(double), time); };
	DimRpcInfo(const char *name, int time, char *nolink) { 
		subscribe((char *)name, 0, 0, nolink, strlen(nolink)+1, time); };
	DimRpcInfo(const char *name, int time, void *nolink, int nolinksize) { 
		subscribe((char *)name, 0, 0, nolink, nolinksize, time); };
	
	virtual void rpcInfoHandler() {};

	virtual ~DimRpcInfo();
	char *getName()  { return itsName; } ;
	void *getData();
	int getInt() { return *(int *)getData(); } ;
	float getFloat() { return *(float *)getData(); } ;
	double getDouble() { return *(double *)getData(); } ;
	char *getString()  { return (char *)getData(); } ;
	int getSize()  { getData(); return itsSize; } ;

	void setData(void *data, int size) { doIt(data, size); };
	void setData(int &data) { doIt(&data, sizeof(int)); } ;
	void setData(float &data) { doIt(&data, sizeof(float)); } ;
	void setData(double &data) { doIt(&data, sizeof(double)); } ;
	void setData(char *data)  { doIt(data, strlen(data)+1); } ;

private :
	char *itsName;
	char *itsNameIn;
	char *itsNameOut;
	int itsTimeout;
	void subscribe(char *name, void *data, int size, 
		void *nolink, int nolinksize, int timeout);
	void doIt(void *data, int size);
};

class DllExp DimClient : public DimInfoHandler
{
public:

	static char *dimDnsNode;

	DimClient() {};
	virtual ~DimClient() 
	{
		if(dimDnsNode)
			delete dimDnsNode;
	};
	static int sendCommand(const char *name, int data) { 
		DimCmnd a;
		return a.send((char *)name, &data, sizeof(int)); };
	static int sendCommand(const char *name, float data) { 
		DimCmnd a;
		return a.send((char *)name, &data, sizeof(float)); };
	static int sendCommand(const char *name, double data) { 
		DimCmnd a;
		return a.send((char *)name, &data, sizeof(double)); };
	static int sendCommand(const char *name, const char *data) { 
		DimCmnd a;
		return a.send((char *)name, (char *)data, strlen(data)+1); };
	static int sendCommand(const char *name, void *data, int datasize) {
		DimCmnd a;
		return a.send((char *)name, data, datasize); };
	static void sendCommandNB(const char *name, int data) { 
		DimCmnd a;
		a.sendNB((char *)name, &data, sizeof(int)); };
	static void sendCommandNB(const char *name, float data) { 
		DimCmnd a;
		a.sendNB((char *)name, &data, sizeof(float)); };
	static void sendCommandNB(const char *name, double data) { 
		DimCmnd a;
		a.sendNB((char *)name, &data, sizeof(double)); };
	static void sendCommandNB(const char *name, char *data) { 
		DimCmnd a;
		a.sendNB((char *)name, data, strlen(data)+1); };
	static void sendCommandNB(const char *name, void *data, int datasize) {
		DimCmnd a;
		a.sendNB((char *)name, data, datasize); };
	static int setExitHandler(const char *serverName) {
		DimCmnd a;
		int ret, tag = 1;
		char *name = new char[strlen(serverName) + 20];
		strcpy(name,(char *)serverName);
		strcat(name,"/SET_EXIT_HANDLER");
		ret =  a.send(name, &tag, sizeof(int));
		delete name;
		return ret; };
	static int killServer(const char *serverName) {
		DimCmnd a;
		int ret, tag = 1;
		char *name = new char[strlen(serverName) + 20];
		strcpy(name,(char *)serverName);
		strcat(name,"/EXIT");
		ret = a.send(name, &tag, sizeof(int));
		delete name;
		return ret; };
	static int setDnsNode(const char *node) {
		dim_set_dns_node((char *)node);
		dic_close_dns();
		return 1;};
	static char *getDnsNode() {
		if(!dimDnsNode)
			dimDnsNode = new char[256];
		if(dim_get_dns_node(dimDnsNode))
			return dimDnsNode;
		else
			return 0; };

	virtual void infoHandler() {};

};

class DllExp DimBrowser
{
public :

	DimBrowser();

	~DimBrowser();

	int getServices(const char *serviceName);
	int getServers();
	int getServerServices(const char *serverName);
	int getServerClients(const char *serverName);
	int getNextService(char *&service, char *&format);
	int getNextServer(char *&server, char *&node);
	int getNextServerService(char *&service, char *&format);
	int getNextServerClient(char *&client, char *&node);

private:

	TokenString *itsData[4];
	int currIndex; 
	char *currToken;
	char none;
};

#endif
