#ifndef __DISHHDEFS
#define __DISHHDEFS
#include <assert.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifndef WIN32
#include <unistd.h>
#endif
#ifdef __VMS
#include <starlet.h>
#endif
#include "dim_core.hxx"
#include "dim.hxx"

#define DimINT		"I"
#define DimLONG		"L"
#define DimFLOAT	"F"
#define DimDOUBLE	"D"
#define DimSTRING	"C"

enum DisTYPE {
	DisPOINTER, DisINT, DisFLOAT, DisDOUBLE, DisSTRING, DisCOMMAND
};

class DimService;
class DimCommand;

class DllExp DimServiceHandler{
public:
	DimService *itsService;
	DimService *getService() { return itsService; } ;
	virtual void serviceHandler() = 0;
};

class DllExp DimCommandHandler{
public:
	DimCommand *itsCommand;
	DimCommand *getCommand() { return itsCommand; } ;
	virtual void commandHandler() = 0;
};

class DllExp DimClientExitHandler{
public:
	virtual void clientExitHandler() = 0;
};

class DllExp DimExitHandler{
public:
	virtual void exitHandler() = 0;
};

extern "C" {
static void client_exit_user_routine(int*);
static void exit_user_routine(int*);
}

class DllExp DimServer : public DimServiceHandler, public DimCommandHandler,
	public DimClientExitHandler, public DimExitHandler
{
public:
	static char *clientName;
	static char *itsName;
	static char *dimDnsNode;
	static int autoStart;
	static DimClientExitHandler *itsClientExit;
	static DimExitHandler *itsExit;
	DimServer() {itsClientExit = this; itsExit = this;};
	virtual ~DimServer() 
	{
		dis_stop_serving();
		if(itsName)
			delete itsName;
		if(clientName)
			delete clientName;
		if(dimDnsNode)
			delete dimDnsNode;
	};
	static void start(const char *name)
	{
		itsName = new char[strlen(name)+1];
		strcpy(itsName,name);
		dis_start_serving(itsName);
	};
	static void start()
	{
		if((itsName) && (autoStart))
			dis_start_serving(itsName);
	};
	static void stop()
	{
		dis_stop_serving();
	};
	static void autoStartOn()
	{
		autoStart = 1;
	};
	static void autoStartOff()
	{
		autoStart = 0;
	};
	// Get Current Client Identifier	
	static int getClientId()
	{
		if(!clientName)
			clientName = new char[128];
		return(dis_get_client(clientName));
    }
	// Get Current Client Name	
	static char *getClientName()
	{
		if(!clientName)
			clientName = new char[128];
		if(dis_get_client(clientName))
			return(clientName);
		return(0);
    }
	static void setClientExitHandler(int clientId)
	{
		dis_set_client_exit_handler(clientId, 1);
	};
	static void clearClientExitHandler(int clientId)
	{
		dis_set_client_exit_handler(clientId, 0);
	};
	static void addClientExitHandler(DimClientExitHandler *handler)
	{
		DimServer::itsClientExit = handler;
		dis_add_client_exit_handler(client_exit_user_routine);
	};
	void addClientExitHandler()
	{
		DimServer::itsClientExit = this;
		dis_add_client_exit_handler(client_exit_user_routine);
	};
	static void addExitHandler(DimExitHandler *handler)
	{
		DimServer::itsExit = handler;
		dis_add_exit_handler(exit_user_routine);
	};
	static int setDnsNode(const char *node) {
		return dim_set_dns_node((char *)node); 
	};
	static char *getDnsNode() {
		if(!dimDnsNode)
			dimDnsNode = new char[256];
		if(dim_get_dns_node(dimDnsNode))
			return dimDnsNode;
		else
			return 0; 
	};
	static void setWriteTimeout(int secs) {
		dim_set_write_timeout(secs); 
	};
	static int getWriteTimeout() {
		return dim_get_write_timeout();
	};
	void addExitHandler()
	{
		DimServer::itsExit = this;
		dis_add_exit_handler(exit_user_routine);
	};
	virtual void clientExitHandler() {};
	virtual void exitHandler() {};
	virtual void serviceHandler() {};
	virtual void commandHandler() {};
};

extern "C" {
static void client_exit_user_routine(int *idp)
{
	int id = *idp;

	id++;
	DimCore::inCallback = 2;
	DimServer::itsClientExit->clientExitHandler();
	DimCore::inCallback = 0;
}

static void exit_user_routine(int *idp)
{
	int id = *idp;

	id++;
	DimCore::inCallback = 2;
	DimServer::itsExit->exitHandler();
	DimCore::inCallback = 0;
}
}

class DllExp DimService : public DimServiceHandler {

public :
	DimServiceHandler *itsServiceHandler;

	// The object contains the value to be published. Service to be updated with an argument of same type;
	DimService(){};

	DimService(const char *name, int &value) {
		itsData = &value;
		itsSize = sizeof(int);
		itsType = DisINT;
		declareIt((char *)name, (char *)"L", 0);
	};
	DimService(const char *name, float &value) {
		itsData = &value;
		itsSize = sizeof(float);
		itsType = DisFLOAT;
		declareIt((char *)name, (char *)"F", 0);
	};
	DimService(const char *name, double &value) {
		itsData = &value;
		itsSize = sizeof(double);
		itsType = DisDOUBLE;
		declareIt((char *)name, (char *)"D", 0);
	};
	DimService(const char *name, char *string) {
		itsData = string;
		itsSize = strlen(string)+1;
		itsType = DisSTRING;
		declareIt((char *)name, (char *)"C", 0);
	};

	DimService(const char *name, char *format, void *structure, int size) {
		itsData = structure;
		itsSize = size;
		itsType = DisPOINTER;
		declareIt((char *)name, format, 0);
	};

	DimService(const char *name, char *format, DimServiceHandler *handler) {
		itsData = 0;
		itsSize = 0;
		itsType = DisPOINTER;
		declareIt((char *)name, format, handler);
	};

	virtual ~DimService() {
		delete itsName;
		dis_remove_service( itsId );
	};

	// Update methods
	int updateService() { return dis_update_service( itsId ) ; };
	// Update the value as well...
	int updateService( int &value ) {
		if( itsType == DisINT) {
			itsData = &value;
			return dis_update_service( itsId );
		}
		return -1;
	};
	int updateService( float &value ) {
		if( itsType == DisFLOAT) {
			itsData = &value;
			return dis_update_service( itsId );
		}
		return -1;
	};
	int updateService( double &value ) {
		if( itsType == DisDOUBLE) {
			itsData = &value;
			return dis_update_service( itsId );
		}
		return -1;
	};
	int updateService( char *string ) {
		if( itsType == DisSTRING) {
			itsData = string;
			itsSize = strlen(string)+1;
			return dis_update_service( itsId );
		}
		return -1;
	};
	
	int updateService( void *structure, int size ) {
		if( itsType == DisPOINTER) {
			itsData = structure;
			itsSize = size;
			return dis_update_service( itsId );
		}
		return -1;
	};
	
	// Selective Update methods
	int selectiveUpdateService(int *cids) {
		if( cids == 0)
		{
			int ids[2];
			ids[0] = DimServer::getClientId();
			ids[1] = 0;
			return dis_selective_update_service( itsId, ids );
		} 
		return dis_selective_update_service( itsId, cids ); };
	// Update the value as well...
	int selectiveUpdateService( int &value, int *cids) {
		if( itsType == DisINT) {
			itsData = &value;
			if( cids == 0)
			{
				int ids[2];
				ids[0] = DimServer::getClientId();
				ids[1] = 0;
				return dis_selective_update_service( itsId, ids );
			} 
			return dis_selective_update_service( itsId, cids );
		}
		return -1;
	};
	int selectiveUpdateService( float &value, int *cids ) {
		if( itsType == DisFLOAT) {
			itsData = &value;
			if( cids == 0)
			{
				int ids[2];
				ids[0] = DimServer::getClientId();
				ids[1] = 0;
				return dis_selective_update_service( itsId, ids );
			} 
			return dis_selective_update_service( itsId, cids );
		}
		return -1;
	};
	int selectiveUpdateService( double &value, int *cids ) {
		if( itsType == DisDOUBLE) {
			itsData = &value;
			if( cids == 0)
			{
				int ids[2];
				ids[0] = DimServer::getClientId();
				ids[1] = 0;
				return dis_selective_update_service( itsId, ids );
			} 
			return dis_selective_update_service( itsId, cids );
		}
		return -1;
	};
	int selectiveUpdateService( char *string, int *cids ) {
		if( itsType == DisSTRING) {
			itsData = string;
			itsSize = strlen(string)+1;
			if( cids == 0)
			{
				int ids[2];
				ids[0] = DimServer::getClientId();
				ids[1] = 0;
				return dis_selective_update_service( itsId, ids );
			} 
			return dis_selective_update_service( itsId, cids );
		}
		return -1;
	};
	
	int selectiveUpdateService( void *structure, int size, int *cids ) {
		if( itsType == DisPOINTER) {
			itsData = structure;
			itsSize = size;
			if( cids == 0)
			{
				int ids[2];
				ids[0] = DimServer::getClientId();
				ids[1] = 0;
				return dis_selective_update_service( itsId, ids );
			} 
			return dis_selective_update_service( itsId, cids );
		}
		return -1;
	};
	
	void setQuality(int quality) { dis_set_quality( itsId, quality ) ; };
	void setTimestamp(int secs, int millisecs) { 
		dis_set_timestamp( itsId, secs, millisecs ) ; };

	void *itsData;
	int itsDataSize;
	int itsSize;
	DisTYPE itsType;
	void setData(void *data, int size) { storeIt(data, size); };
	void setData(int &data) { storeIt(&data, sizeof(int)); } ;
	void setData(float &data) { storeIt(&data, sizeof(float)); } ;
	void setData(double &data) { storeIt(&data, sizeof(double)); } ;
	void setData(char *data)  { storeIt(data, strlen(data)+1); } ;

	virtual void serviceHandler() {};
	// Accessors
	char *getName() { return itsName;
	};
	int getTimeout(int clientId)
	{
		return dis_get_timeout(itsId, clientId);
	}
private :
	char *itsName;
	int itsId;
	void declareIt(char *name, char *format, DimServiceHandler *handler);
	void storeIt(void *data, int size);
};

class DllExp CmndInfo : public SLLItem {
	friend class DimCommand;
	void *itsData;
	int itsDataSize;
public:
	CmndInfo(void *data, int datasize)
	{
		itsData = new char[datasize];
		itsDataSize = datasize;
		memcpy(itsData, data, datasize);
	}
	~CmndInfo()
	{
		delete (char *)itsData;
	}
};

class DllExp DimCommand : public DimCommandHandler {

public :
	DimCommandHandler *itsCommandHandler;

	DimCommand(const char *name, char *format) {
		declareIt( (char *)name, format, 0);
	};

	DimCommand(const char *name, char *format, DimCommandHandler *handler) {
		declareIt( (char *)name, format, handler);
	};

	int getNext() {
		CmndInfo *cmndptr;
		if(currCmnd)
		{
			delete currCmnd;
			currCmnd = 0;
			itsData = 0;
			itsSize = 0;
		}
		if ((cmndptr = (CmndInfo *)itsCmndList.removeHead()))
		{
			currCmnd = cmndptr;
			itsData = currCmnd->itsData;
			itsSize = currCmnd->itsDataSize;
			return(1);
		}
		return(0);
	}
	void *itsData;
	int itsSize;
	void *getData() {return itsData; };
	int getInt() { return *(int *)itsData; } ;
	float getFloat() { return *(float *)itsData; } ;
	double getDouble() { return *(double *)itsData; } ;
	char *getString()  { return (char *)itsData; } ;
	int getSize() {return itsSize; };

	virtual void commandHandler() 
	{
		CmndInfo *cmndptr;
		cmndptr = new CmndInfo(getData(), getSize());
		itsCmndList.add(cmndptr);
	};

	// Accessors
	char *getName() { return itsName;
	};
	virtual ~DimCommand() {
		delete itsName;
		delete itsFormat;
		dis_remove_service( itsId );
	} ;

private :
	char *itsName;
	int itsId;
	char *itsFormat;
	void declareIt(char *name, char *format, DimCommandHandler *handler);
	CmndInfo *currCmnd;
	SLList itsCmndList;
};

class DllExp DimRpc
{
public :

	// The object contains the value to be published. Service to be updated with an argument of same type;
	DimRpc(){};

	DimRpc(const char *name, const char *formatin, const char *formatout) {
		declareIt( (char *)name, (char *)formatin, (char *)formatout);
	};

	// Desctructor
	virtual ~DimRpc() {
		delete itsName;
		delete itsNameIn;
		delete itsNameOut;
		dis_remove_service( itsIdIn );
		dis_remove_service( itsIdOut );
	};

	void *itsDataIn;
	int itsSizeIn;
	void *getData() {return itsDataIn; };
	int getInt() { return *(int *)itsDataIn; } ;
	float getFloat() { return *(float *)itsDataIn; } ;
	double getDouble() { return *(double *)itsDataIn; } ;
	char *getString()  { return (char *)itsDataIn; } ;
	int getSize() {return itsSizeIn; };

	void *itsDataOut;
	int itsDataOutSize;
	int itsSizeOut;

	void setData(void *data, int size) { storeIt(data,size); };
	void setData(int &data) { storeIt(&data,sizeof(int)); } ;
	void setData(float &data) { storeIt(&data,sizeof(float)); } ;
	void setData(double &data) { storeIt(&data,sizeof(double)); } ;
	void setData(char *data)  { storeIt(data,strlen(data)+1); } ;

	virtual void rpcHandler() = 0;
	// Accessors
	char *getName() { return itsName;
	};
	int itsIdIn;
	int itsIdOut;
private :
	char *itsName;
	char *itsNameIn;
	char *itsNameOut;
	void declareIt(char *name, char *formatin, char *formatout);
	void storeIt(void *data, int size);
	void timerHandler();
};


#endif

