package dim;
import java.util.Date;

public class DimService extends MutableMemory implements DataEncoder, DimServiceUpdateHandler
{
    int service_id;
	String service_name;
	int published, curr_size;
	String format;
	Format itsFormat;
	int itsLast;
	
	public DimService(String theServiceName)
	{
		published = 0;
		service_name = theServiceName;
		curr_size = 0;
		format = "";
		itsLast = 0;
	}
	public DimService(String theServiceName, int theData)
	{
		service_name = theServiceName;
		setSize(4);
		this.copyInt(theData);
        service_id = Server.addService(theServiceName, "I", this);
		published = 1;
		itsFormat = null;
	}
	public DimService(String theServiceName, float theData)
	{
		service_name = theServiceName;
		setSize(4);
		this.copyFloat(theData);
        service_id = Server.addService(theServiceName, "F", this);
		published = 1;
		itsFormat = null;
	}
	public DimService(String theServiceName, double theData)
	{
		service_name = theServiceName;
		setSize(8);
		this.copyDouble(theData);
        service_id = Server.addService(theServiceName, "D", this);
		published = 1;
		itsFormat = null;
	}
    public DimService(String theServiceName, String theData)
    {
 		service_name = theServiceName;
		setSize(theData.length()+1);
		this.copyString(theData);
        service_id = Server.addService(theServiceName, "C", this);
		published = 1;
		itsFormat = null;
    }
	public DimService(String theServiceName, int[] theData)
	{
		service_name = theServiceName;
		int size = Sizeof.sizeof(theData);
		setSize(size);
		this.copyFromIntArray(theData, 0, size/4);
        service_id = Server.addService(theServiceName, "I", this);
		published = 1;
		itsFormat = null;
	}
	public DimService(String theServiceName, float[] theData)
	{
		service_name = theServiceName;
		int size = Sizeof.sizeof(theData);
		setSize(size);
		this.copyFromFloatArray(theData, 0, size/4);
        service_id = Server.addService(theServiceName, "F", this);
		published = 1;
		itsFormat = null;
	}
	public DimService(String theServiceName, double[] theData)
	{
		service_name = theServiceName;
		int size = Sizeof.sizeof(theData);
		setSize(size);
		this.copyFromDoubleArray(theData, 0, size/8);
        service_id = Server.addService(theServiceName, "D", this);
		published = 1;
		itsFormat = null;
	}
    public void finalize()
    {
        removeService();
    }
	
    public void removeService()
    {
        Server.removeService(service_id);
    }
    public Memory encodeData()
    {
		serviceUpdateHandler();
        return this;
    }
    public void updateService(String theData)
    {
		this.setSize(theData.length()+1);
        this.copyString(theData);
        Server.updateService(service_id);
    }
    public void updateService(int theData)
    {
		this.setSize(4);
        this.copyInt(theData);
        Server.updateService(service_id);
    }
	public void updateService(float theData)
    {
		this.setSize(4);
        this.copyFloat(theData);
        Server.updateService(service_id);
    }
    public void updateService(double theData)
    {
		this.setSize(8);
        this.copyDouble(theData);
        Server.updateService(service_id);
    }
    public void updateService(int[] theData)
    {
		int size = Sizeof.sizeof(theData);
		setSize(size);
		this.copyFromIntArray(theData, 0, size/4);
        Server.updateService(service_id);
    }
    public void updateService(float[] theData)
    {
		int size = Sizeof.sizeof(theData);
		setSize(size);
		this.copyFromFloatArray(theData, 0, size/4);
        Server.updateService(service_id);
    }
    public void updateService(double[] theData)
    {
		int size = Sizeof.sizeof(theData);
		setSize(size);
		this.copyFromDoubleArray(theData, 0, size/8);
        Server.updateService(service_id);
    }
	
	int do_setup(char type, int num)
	{
		if(published == 0)
		{
			if(itsLast == 1)
			{
				System.out.println(
					"JDIM: Dynamic String Item must be at the end");
				return 0;
			}
			if(format != "")
				format += ";";
			format += type+":"+num;
			return 1;
		}
		else
		{
			char ntype = itsFormat.getType();
			int nnum = itsFormat.getNum();
			if(ntype != type)
			{
				System.out.println("JDIM: Expected "+ntype+" found "+type);
				return 0;
			}
			if((nnum != num) && (nnum != 0) && (num != 0))
			{
//				if(nnum)
					System.out.println("JDIM: Expected "+nnum+" items, found "+num);
//				else
//					System.out.println("JDIM: Expected "+nnum+" items, found "+num);
//				return 0;
			}
			if(nnum == 0)
				return 1;
			return nnum;
		}
	}
		
	public int setInt(int theData)
	{
		if(do_setup('I',1) == 0)
			return 0;
		int offset = curr_size;
		curr_size += 4;
		setSize(curr_size);
		setDataStoreOffset(offset);
		copyInt(theData);
		return 1;
	}
	public int setFloat(float theData)
	{
		if(do_setup('F',1) == 0)
			return 0;
		int offset = curr_size;
		curr_size += 4;
		setSize(curr_size);
		setDataStoreOffset(offset);
		copyFloat(theData);
		return 1;
	}
	public int setDouble(double theData)
	{
		if(do_setup('D',1) == 0)
			return 0;
		int offset = curr_size;
		curr_size += 8;
		setSize(curr_size);
		setDataStoreOffset(offset);
		copyDouble(theData);
		return 1;
	}
	public int setString(String theData, int max_size)
	{		
		if(do_setup('C',max_size) == 0)
			return 0;
		int offset = curr_size;
		curr_size += max_size;
		setSize(curr_size);
		setDataStoreOffset(offset);
		this.copyString(theData);
		return 1;
	}
	public int setString(String theData)
	{				
		int max_size;
		if(published == 0)
			max_size = do_setup('C',theData.length());
		else
			max_size = do_setup('C',0);
		if(max_size == 0)
			return 0;
		itsLast = 1;
/*
		if(published == 0)
		{
			return 0;
		}
		else
		{
			char type = itsFormat.getType();
			int num = itsFormat.getNum();
			if(type != 'C')
				return 0;
			max_size = num;
		}
*/
		int offset = curr_size;
		curr_size += max_size;
		setSize(curr_size);
		setDataStoreOffset(offset);
		this.copyString(theData);
		return 1;
	}
	public int setIntArray(int theData[])
	{		
		int size = Sizeof.sizeof(theData);
		int len = size/4;
		if(do_setup('I',len) == 0)
			return 0;
		int offset = curr_size;
		curr_size += size;
		setSize(curr_size);
		setDataStoreOffset(offset);
		this.copyFromIntArray(theData, 0, len);
		return 1;
	}
	public int setFloatArray(float theData[])
	{		
		int size = Sizeof.sizeof(theData);
		int len = size/4;
		if(do_setup('F',len) == 0)
			return 0;
		int offset = curr_size;
		curr_size += size;
		setSize(curr_size);
		setDataStoreOffset(offset);
		this.copyFromFloatArray(theData, 0, len);
		return 1;
	}
	public int setDoubleArray(double theData[])
	{		
		int size = Sizeof.sizeof(theData);
		int len = size/8;
		if(do_setup('D',len) == 0)
			return 0;
		int offset = curr_size;
		curr_size += size;
		setSize(curr_size);
		setDataStoreOffset(offset);
		this.copyFromDoubleArray(theData, 0, len);
		return 1;
	}
	public void updateService()
	{
		if(published == 0)
		{
			itsFormat = new Format(format, 1);
			service_id = Server.addService(service_name, itsFormat.getFormat(), this);
			published = 1;
		}
		else
		{
			if(itsFormat != null)
			{
				char type = itsFormat.getType();
			}
			Server.updateService(service_id);
		}
		curr_size = 0;
	}
	
    public static native void setQuality(int serviceId, int quality);
    public static native void setTimestamp(int serviceId, int secs, int millisecs);
	public void setQuality(int quality)
	{
		setQuality(service_id, quality);
	}
	public void setTimestamp(Date tstamp)
	{
		int secs, millisecs;
		long total, aux;
		
		total = tstamp.getTime();
		aux = total % 1000;
		millisecs = (int)aux;
		aux = total / 1000;
		secs = (int)aux;
		setTimestamp(service_id, secs, millisecs);
	}
	public void serviceUpdateHandler() {}
}

interface DimServiceUpdateHandler
{
	void serviceUpdateHandler();
}
