#include <signal.h>
#define DIMLIB
#include "dim.h"

#ifndef WIN32

#ifndef NOTHREADS
#include <pthread.h>
#include <semaphore.h>
#ifdef solaris
#include <synch.h>
#endif

pthread_t IO_thread = 0;
pthread_t ALRM_thread = 0;
pthread_t INIT_thread = 0;
sem_t DIM_INIT_Sema;
sem_t DIM_WAIT_Sema;
int INIT_count = 0;
int WAIT_count = 0;

void *dim_tcpip_thread(void *tag)
{
	sigset_t set1, set2;
	int signo;
	/*	
	int prio;
		
	thr_getprio(thr_self(),&prio);
	thr_setprio(thr_self(),prio+10);
	*/
	IO_thread = pthread_self();

	dim_tcpip_init(1);
	while(1)
    {
		if(INIT_thread)
			sem_post(&DIM_INIT_Sema);
		tcpip_task();
		sem_post(&DIM_WAIT_Sema);
    }
}

void *dim_dtq_thread(void *tag)
{
	sigset_t set1,set2;
	int signo, ret;
	/*
	int prio;

	thr_getprio(thr_self(),&prio);
	thr_setprio(thr_self(),prio+5);
	*/
	ALRM_thread = pthread_self();

	dim_dtq_init(1);
	while(1)
	  {
		if(INIT_thread)
			sem_post(&DIM_INIT_Sema);
		dtq_task();
		sem_post(&DIM_WAIT_Sema);
    }
}

void dim_init()
{
	static int done = 0;
	sigset_t set1;
	pthread_t t_id;
	void ignore_sigpipe();
#ifdef LYNXOS
    pthread_attr_t attr;
#endif

	if(!done)
	{
	  /*
		int prio;
	  */
		done = 1;
		dna_init();
		/*
		thr_getprio(thr_self(),&prio);
		thr_setprio(thr_self(),prio+3);
		*/
		INIT_thread = pthread_self();
		sem_init(&DIM_INIT_Sema, 0, INIT_count);
		sem_init(&DIM_WAIT_Sema, 0, WAIT_count);

		ignore_sigpipe();

#ifdef LYNXOS
		pthread_attr_create(&attr);
		pthread_create(&t_id, attr, dim_dtq_thread, 0);
#else
		pthread_create(&t_id, NULL, dim_dtq_thread, 0);
#endif
		sem_wait(&DIM_INIT_Sema);
#ifdef LYNXOS		
		pthread_create(&t_id, attr, dim_tcpip_thread, 0);
#else
		pthread_create(&t_id, NULL, dim_tcpip_thread, 0);
#endif
		sem_wait(&DIM_INIT_Sema);
		INIT_thread = 0;
	}
}

void ignore_sigpipe()
{

  struct sigaction sig_info;
  sigset_t set;
  void pipe_sig_handler();
	    
  if( sigaction(SIGPIPE, 0, &sig_info) < 0 ) 
  {
    perror( "sigaction(SIGPIPE)" );
    exit(1);
  }
  if(sig_info.sa_handler)
    return;
  sigemptyset(&set);
  sig_info.sa_handler = pipe_sig_handler;
  sig_info.sa_mask = set;
#ifndef LYNXOS 
  sig_info.sa_flags = SA_RESTART;
#else
  sig_info.sa_flags = 0;
#endif

  if( sigaction(SIGPIPE, &sig_info, 0) < 0 ) 
  {
    perror( "sigaction(SIGPIPE)" );
    exit(1);
  }
}

void pipe_sig_handler( num )
int num;
{
  /*  

	printf( "*** pipe_sig_handler called ***\n" );
  */
}

void dim_init_threads()
{
    dim_init();
}

int dim_wait(void)
{
	pthread_t id;
	
	id = pthread_self();

	if((id == ALRM_thread) || (id == IO_thread))
    {
		return(-1);
    }
	sem_wait(&DIM_WAIT_Sema);
	return(-1);
}

void show_ast()
{
sigset_t oset;
/*
	sigprocmask(SIG_SETMASK,0,&oset);
	printf("---THREAD id = %d, mask = %x %x\n",
       pthread_self(), oset.__sigbits[1], oset.__sigbits[0]);
*/
}

pthread_t Dim_thr_locker = 0;
int Dim_thr_counter = 0;
#ifdef LYNXOS
pthread_mutex_t Global_DIM_mutex;
#else
pthread_mutex_t Global_DIM_mutex = PTHREAD_MUTEX_INITIALIZER;
#endif

void dim_lock()
{
	/*printf("Locking %d ", pthread_self());*/
    if(Dim_thr_locker != pthread_self())
    {
		pthread_mutex_lock(&Global_DIM_mutex);
		Dim_thr_locker=pthread_self();
		/*printf(": Locked ");*/
	}
    /*printf("Counter = %d\n",Dim_thr_counter);*/
    Dim_thr_counter++;
}
void dim_unlock()	
{
	/*printf("Un-Locking %d ", pthread_self());*/
    Dim_thr_counter--;
    /*printf("Counter = %d ",Dim_thr_counter);*/
    if(!Dim_thr_counter)
    {
		Dim_thr_locker=0;
		pthread_mutex_unlock(&Global_DIM_mutex);
		/*printf(": Un-Locked ");*/
	}
	/*     printf("\n");*/
}


#else

void dim_init()
{
}

void dim_init_threads()
{
}

int dim_wait()
{
  pause();
  return(-1);
}

#endif

#else
#include <windows.h>

void dim_init_threads()
{
}

void dim_init()
{
}

void dim_no_threads()
{
}

int dim_wait()
{
	pause();
}

DllExp HANDLE Global_DIM_event = 0;
DllExp HANDLE Global_DIM_mutex = 0;

void dim_lock()
{
	if(!Global_DIM_mutex)
	{ 
		Global_DIM_mutex = CreateMutex(NULL,FALSE,NULL);
	}
	WaitForSingleObject(Global_DIM_mutex, INFINITE);
}

void dim_unlock()
{
	ReleaseMutex(Global_DIM_mutex);
}

void dim_pause()
{
	if(!Global_DIM_event)
	{ 
		Global_DIM_event = CreateEvent(NULL,FALSE,FALSE,NULL);
	}
	else 
	{
		WaitForSingleObject(Global_DIM_event, INFINITE);
	}
}

void dim_wake_up()
{
	SetEvent(Global_DIM_event);
}

void dim_sleep(unsigned int t)
{
	Sleep(t*1000);
}

void dim_win_usleep(unsigned int t)
{
	Sleep(t/1000);
}

#endif
