#define MAX_NAME 		132
#define MAX_OBJECTS		512
#define MAX_BUFFER		1024

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifdef VMS
#include <ssdef.h>   
#include <iodef.h>   
#include <lnmdef.h>  
#include <cfortran.h>
#endif

#include "parameters.h"
#define SMILIB
#include "../smixx_version.h"
#include "smirtl.h"

typedef struct{
	char name[MAXNAMESIZE];
	char state[MAXNAMESIZE];
}SUB;

typedef struct{
	char name[MAXNAMESIZE];
	char value[MAX_NAME];
	int type;
	char type_str[4];
}PAR;

typedef struct {
	char token_buff[MAX_BUFFER];
	char *token_ptr;
	char *curr_token_ptr;
}TOKEN_STR;

static TOKEN_STR SMI_command;

static char Domain_name[80];
static char EP_domain_name[80];
static char SMI_state[MAX_BUFFER];
static int SMI_state_id;
static void (*SMI_ast)();
static int SMI_ef;
static char Object[MAXNAMESIZE];
static char EP_object[MAXNAMESIZE];
static int Cmnd_wake_up = -1;
static int Cmnd_arrived = 0;
static int Cmnd_valid = 0;

static SUB Subs[MAXSUBOBJECTS];

static PAR Pars[MAXATRIBUTES];

static int N_sub_objects = 0;
static int N_parameters = 0;
static int Initialized = 0;

static int Info_print = 1;
static int Version_number = SMIXX_VERSION;

typedef struct{
	int (*exit_handler)();
	void *context;
} EXIT_PARS;

static EXIT_PARS Exit_pars = {0,0};

_PROTO( static int init_token, (TOKEN_STR *ptr) );
_PROTO( static int get_token,  (TOKEN_STR *ptr, char **token) );
_PROTO( static int push_token, (TOKEN_STR *ptr) );
_PROTO( static int pop_token,  (TOKEN_STR *ptr) );
_PROTO( static int cmp_token,  (char *token, char *str) );
_PROTO( static int first_token,(TOKEN_STR *ptr) );

void smi_set_print_on()
{
	Info_print = 1;
}

void smi_set_print_off()
{
	Info_print = 0;
}

void smi_print_date_time()
{
	time_t t;
	char str[128];

	t = time((time_t *)0);
	printf("SMIRTL - ");
	strcpy(str, ctime(&t));
	str[strlen(str)-1] = '\0';
	printf("%s -",str);
}

int smi_init_for(user_ef, user_ast)
int user_ef;
int user_ast;
{

	return(smi_init_(user_ef, (void (*)())user_ast));
}

int smi_init_(user_ef, user_ast)
int user_ef;
void (*user_ast)();
{

	if(user_ef)
	{
		SMI_ef = user_ef;
		SMI_ast = 0;
#ifdef VMS
		sys$clref(SMI_ef);
#endif
	}
	else
	{
		SMI_ef = 0;
		SMI_ast = user_ast;
	}
	return(1);
}


get_obj_name(obj_name, serv_name, ep)
char *obj_name, *serv_name;
int ep;
{

	char obj[MAXNAMESIZE];
	char *ptr_domain, *ptr_obj, obj_type[10];

	strcpy(obj,obj_name);
	if(ptr_obj = (char *)strstr(obj,"::"))
	{
		*ptr_obj = '\0';
		ptr_obj +=2;
		ptr_domain = obj;
	}
	else
	{
#ifdef VMS
		get_domain_name(Domain_name);
#endif
		ptr_domain = Domain_name;
		ptr_obj = obj;
	}
	strcpy(Object,ptr_obj);
	if(ep)
	{
		sprintf(serv_name,"EP/%s/%s", ptr_domain, ptr_obj);
		strcpy(EP_domain_name,ptr_domain);
		strcpy(EP_object,ptr_obj);
	}
	else
		sprintf(serv_name,"SMI/%s/%s", ptr_domain, ptr_obj);
	strcpy(Domain_name,ptr_domain);
}
	
static void capitalize(obj)
char *obj;
{
	while(*obj)
	{
		*obj = toupper(*obj);
		obj++;
	}
}

int smi_associate(obj_name)
char *obj_name;
{
	char serv_name[MAX_NAME];
	void execute_cmnd();
	int id;
	
	capitalize(obj_name);
	strcpy(SMI_state,"$DEAD");		
	get_obj_name(obj_name,serv_name,1);
	id = dis_add_service(serv_name, "C",
		SMI_state, MAX_BUFFER, 0, 0);
    if(id != 0)
		SMI_state_id = id;
	strcat(serv_name,"/CMD");
	dis_add_cmnd(serv_name, "C", execute_cmnd, 0);
	get_obj_name(obj_name,serv_name,1);
	strcat(serv_name,"/SMI_VERSION_NUMBER");
	dis_add_service(serv_name, "L",
		&Version_number, 4, 0, 0);
	if(!strchr(obj_name,':'))
		sprintf(serv_name,"%s_%s",EP_domain_name,obj_name);
	else
		strcpy(serv_name,obj_name);
	return(dis_start_serving(serv_name));
}	

void handle_exit(client_id)
int *client_id;
{
	int exit_code;

	smi_print_date_time();
	printf(" Smi Domain %s Exited - Proxy %s Exiting...\n",
	       EP_domain_name, EP_object);
	fflush(stdout);
	if(Exit_pars.exit_handler)
	{
		if(Exit_pars.exit_handler(Exit_pars.context, &exit_code))
			exit(exit_code);
	}
	else
		exit(0);
}

void smi_register_termination(exit_handler, context)
int (*exit_handler)();
void *context;
{
	Exit_pars.exit_handler = exit_handler;
	Exit_pars.context = context;
	dis_add_client_exit_handler(handle_exit);
}

int suicide_rout(context, exit_code)
int *exit_code;
void *context;
{
	*exit_code = 0;
	return(1);
}

void smi_volatile()
{
	smi_register_termination(suicide_rout, 0);
}	

/* smi_attach attaches a proxy to its associated object in SMI */
int smi_attach(obj_name, user_ast)
char *obj_name;
void (*user_ast)();
{

	char serv_name[MAX_NAME];
	void execute_cmnd();
	
	SMI_ef = 0;
	SMI_ast = user_ast;

	capitalize(obj_name);
	strcpy(SMI_state,"$DEAD");		
	get_obj_name(obj_name,serv_name,1);
	SMI_state_id = dis_add_service(serv_name, "C",
		SMI_state, MAX_BUFFER, 0, 0);

	strcat(serv_name,"/CMD");
	dis_add_cmnd(serv_name, "C", execute_cmnd, 0);
	get_obj_name(obj_name,serv_name,1);
	strcat(serv_name,"/SMI_VERSION_NUMBER");
	dis_add_service(serv_name, "L",
		&Version_number, 4, 0, 0);
	if(!strchr(obj_name,':'))
		sprintf(serv_name,"%s_%s",EP_domain_name,obj_name);
	else
		strcpy(serv_name,obj_name);
	return(dis_start_serving(serv_name));
}

get_sub_states(full_state)
char *full_state;
{
	int i, found = 0;
	int end = 0;
	char *namep, *statep, *endp;

/* for some reason some users terminate the state by a '\' */
	if(full_state[strlen(full_state)-1] == '\\')
		full_state[strlen(full_state)-1] = '\0';
/* SMI is not Case sensitive so uppercase state */
	for(i = 0; i < strlen(full_state); i++)
		full_state[i] = toupper(full_state[i]);
	strcpy(SMI_state,full_state);
	namep = full_state;
	do
	{
		if(!(statep = strchr(namep,'.')))
			break;
		*statep++ = 0;
		if(endp = strchr(statep, ','))
			*endp++ = 0;
		else
			end = 1;
		for(i = 0; i < N_sub_objects; i++)
		{
			if (!strcmp(Subs[i].name,namep))
			{
				found = 1;
				break;
			}
		}
		if(found)
			break;
		strcpy(Subs[N_sub_objects].name,namep);
		strcpy(Subs[N_sub_objects].state,statep);
		N_sub_objects++;
		namep = endp;
	}while(!end);
}

form_state(state)
char *state;
{
	int i, end = 0;
	char *namep, *statep, *endp;

/* for some reason some users terminate the state by a '\' */
	if(state[strlen(state)-1] == '\\')
		state[strlen(state)-1] = '\0';
/* SMI is not Case sensitive so uppercase state */
	for(i = 0; i < strlen(state); i++)
		state[i] = toupper(state[i]);
	if(!N_sub_objects)
	{
		strcpy(SMI_state,state);
	}
	else
	{
		namep = state;
		do
		{
			if(!(statep = strchr(namep,'.')))
				break;
			*statep++ = 0;
			if(endp = strchr(statep, ','))
				*endp++ = 0;
			else
				end = 1;
			for(i = 0; i < N_sub_objects; i++)
			{
				if(!strcmp(namep,Subs[i].name))
					strcpy(Subs[i].state,statep);
			}
			namep = endp;
		}while(!end);
		SMI_state[0] = '\0';
		for(i = 0; i < N_sub_objects; i++)
		{
			strcat(SMI_state,Subs[i].name);
			strcat(SMI_state,".");
			strcat(SMI_state,Subs[i].state);
			strcat(SMI_state,",");
		}
		SMI_state[strlen(SMI_state) - 1] = '\0';
	}
	for(i = 0; i<N_parameters; i++)
	{
		strcat(SMI_state,"/");
		strcat(SMI_state,Pars[i].name);
		strcat(SMI_state,Pars[i].type_str);
		strcat(SMI_state,"=");
		strcat(SMI_state,Pars[i].value);
	}
}
	
store_par_value(index, value, type)
int index, type;
void *value;
{
		switch(type)
		{
			case SMI_STRING:
				strcpy(Pars[index].value,(char *)value);
				strcpy(Pars[index].type_str,"(S)");
				break;
			case SMI_INTEGER:
				sprintf(Pars[index].value,"%d",*(int*)value);
				strcpy(Pars[index].type_str,"(I)");
				break;
			case SMI_FLOAT:
				sprintf(Pars[index].value,"%e",*(double*)value);
				strcpy(Pars[index].type_str,"(F)");
				break;
		}
}

int smi_set_par(par, value, type)
char *par;
void *value;
int type;
{
	int i, found = 0;

	if(!Initialized)
	{
		strcpy(Pars[N_parameters].name,par);
		Pars[N_parameters].type = type;
		store_par_value(N_parameters, value, type);
        N_parameters++;
	}
	else
	{
		for(i = 0; i < N_parameters; i++)
		{
			if(!strcmp(Pars[i].name,par))
			{
				if(type != Pars[i].type)
					return(-1);
				store_par_value(i, value, type);
				found = 1;
				break;
			}
		}
		if(!found)
			return(0);
	}
	return(1);
}

int smi_set_state(state)
char *state;
{
	static char buffer[MAX_BUFFER];

	strcpy(buffer,state);
	get_sub_states(buffer);
	if(!Initialized)
	{
		Initialized = 1;
	}
	strcpy(buffer,state);
	Cmnd_valid = 0;
	form_state(buffer);
	if(Info_print)
	{
		smi_print_date_time();
		printf(" %s::%s in state %s\n",EP_domain_name, EP_object, SMI_state);
		fflush(stdout);
	}
	dis_update_service(SMI_state_id);
	return(1);
}

int smi_terminate_command(state)
char *state;
{
	static char buffer[MAX_BUFFER];

	Cmnd_valid = 0;
	strcpy(buffer,state);

	form_state(buffer);
	if(Info_print)
	{
		smi_print_date_time();
		printf(" %s::%s in state %s\n",EP_domain_name, EP_object, SMI_state);
		fflush(stdout);
	}
	dis_update_service(SMI_state_id);
	return(1);
}

int smi_terminate_action(state)
char *state;
{
	static char buffer[MAX_BUFFER];

	Cmnd_valid = 0;
	strcpy(buffer,state);

	form_state(buffer);
	if(Info_print)
	{
		smi_print_date_time();
		printf(" %s::%s in state %s\n",EP_domain_name, EP_object, SMI_state);
		fflush(stdout);
	}
	dis_update_service(SMI_state_id);
	return(1);
}

void execute_cmnd(tag, buffer, size)
int *tag, *size;
char *buffer;
{

	strcpy(SMI_command.token_buff, buffer);
	init_token(&SMI_command);
	if(Info_print)
	{
		smi_print_date_time();
		printf(" %s::%s executing %s\n",EP_domain_name, EP_object, 
			SMI_command.token_buff);
		fflush(stdout);
	}
	Cmnd_arrived = 1;
	Cmnd_valid = 1;
	if(SMI_ef)
	{
#ifdef VMS
		sys$setef(SMI_ef);
#endif
	}
	if(SMI_ast)
	{
		SMI_ast();
	}
	if(!Cmnd_wake_up)
	{
		Cmnd_wake_up = 1;
#ifdef VMS
		sys$wake(0,0);
#endif
#ifdef WIN32
		wake_up();
#endif
	}
}

void timeout_ast(tag)
int tag;
{
	if(!Cmnd_wake_up)
	{
		Cmnd_wake_up = 1;
#ifdef VMS
		sys$wake(0,0);
#endif
#ifdef WIN32
		wake_up();
#endif
	}
}

int smi_get_command(cmnd, size)
char *cmnd;
int *size;
{
char *ptr;

#ifdef vms
	if(SMI_ef)
		sys$clref(SMI_ef);
#endif
	Cmnd_arrived = 0;
	if(!Cmnd_valid)
		return(0);
	strcpy(cmnd, SMI_command.token_buff);
	*size = strlen(cmnd);
	return(1);
}

int smi_get_commandw_for(cmnd, size, timeout)
char *cmnd;
int *size;
int timeout;
{
	return(smi_get_commandw(cmnd, size, timeout));
}

int smi_get_commandw(cmnd, size, timeout)
char *cmnd;
int *size;
int timeout;
{
	if(Cmnd_arrived)
	{
		Cmnd_arrived = 0;
		strcpy(cmnd, SMI_command.token_buff);
		*size = strlen(SMI_command.token_buff);
		return(1);
	}
	Cmnd_wake_up = 0;
	if(timeout)
		dtq_start_timer(timeout, timeout_ast, 0);
	while(!Cmnd_wake_up)
	{
#ifdef VMS
		sys$hiber();
#else
		dim_wait();
#endif
	}
	if(Cmnd_arrived)
	{
		if(timeout)
			dtq_stop_timer(0);
		Cmnd_arrived = 0;
		strcpy(cmnd, SMI_command.token_buff);
		*size = strlen(SMI_command.token_buff);
		return(1);
	}
	return(0);
}

int smi_get_action(cmnd, n_pars)
char *cmnd;
int *n_pars;
{
char *ptr;


#ifdef vms
	if(SMI_ef)
		sys$clref(SMI_ef);
#endif

	Cmnd_arrived = 0;
	if(!Cmnd_valid)
		return(0);

	init_token(&SMI_command);
	*n_pars = 0;
	get_token(&SMI_command,&ptr);
	strcpy(cmnd,ptr);
	while(get_token(&SMI_command,&ptr))
	{
		if(cmp_token(ptr,"/"))
			(*n_pars)++;
	}	
	return(1);
}

int smi_test_action(cmnd)
char *cmnd;
{
char cmnd_only[MAX_NAME];
char *ptr;

	Cmnd_arrived = 0;
	if(!Cmnd_valid)
		return(0);
	init_token(&SMI_command);
	get_token(&SMI_command,&ptr);
	strcpy(cmnd_only, ptr);
	return(!(strcmp(cmnd,cmnd_only)));
}

static int check_type(str)
char *str;
{
	int type;

	switch(*str)
	{
		case 'S':
			type = SMI_STRING;
			break;
		case 'I':
			type = SMI_INTEGER;
			break;
		case 'F':
			type = SMI_FLOAT;
			break;
	}
	return(type);
}

static int get_next_param(ptr, par, type, size)
TOKEN_STR *ptr;
char *par;
int *type, *size;
{
	char *aux_ptr;

	*type = SMI_STRING;
	push_token(ptr);
	while(get_token(ptr,&aux_ptr))
	{
		if(cmp_token(aux_ptr,"/"))
		{
			get_token(ptr,&aux_ptr);
			strcpy(par, aux_ptr);
			push_token(ptr);
			get_token(ptr,&aux_ptr);
			if(cmp_token(aux_ptr,"("))
			{
				get_token(ptr,&aux_ptr);
				*type = check_type(aux_ptr);
				get_token(ptr,&aux_ptr);
			}
			else
				pop_token(ptr);

			get_token(ptr,&aux_ptr);
			if(cmp_token(aux_ptr,"="))
			{
				get_token(ptr,&aux_ptr);
				if(*type == SMI_STRING)
				{
					*size = strlen(aux_ptr);
					if(aux_ptr[0] == '"')
						*size -=2;
				}
				else if(*type == SMI_INTEGER)
				{
					*size = sizeof(int);
				}
				else
				{
					*size = sizeof(double);
				}
			}
			else
				*size = 0;
			pop_token(ptr);
			return(1);
		}
		if(cmp_token(aux_ptr,"|"))
		{
			break;
		}
	}
	pop_token(ptr);
	return(0);
}

int smi_get_next_par(param, type, size)
char *param;
int *type, *size;
{

	if(!Cmnd_valid)
		return(0);
	if(Cmnd_valid == 1)
	{
		init_token(&SMI_command);
		Cmnd_valid = 2;
	}
	return(get_next_param(&SMI_command, param, type, size));
}

static int get_param_value(ptr, val)
TOKEN_STR *ptr;
void *val;
{
	char *aux_ptr;
	int type, ret = 0;

	type = SMI_STRING;
	push_token(ptr);
	get_token(ptr,&aux_ptr);
	if(cmp_token(aux_ptr,"("))
	{
		get_token(ptr,&aux_ptr);
		type = check_type(aux_ptr);
		get_token(ptr,&aux_ptr);
	}
	else
		pop_token(ptr);

	get_token(ptr,&aux_ptr);
	if(cmp_token(aux_ptr,"="))
	{
		get_token(ptr,&aux_ptr);
		switch(type)
		{
			case SMI_STRING:
				if(aux_ptr[0] == '"')
				{
					aux_ptr[strlen(aux_ptr) - 1] = '\0'; 
					strcpy(val, aux_ptr + 1);
				}
				else
				{
					push_token(ptr);
					strcpy(val, aux_ptr);
					while(get_token(ptr,&aux_ptr))
					{
						if(cmp_token(aux_ptr,"/"))
						{
							pop_token(ptr);
							break;
						}
						else
							strcat(val,aux_ptr);
					}
				}
				break;
			case SMI_INTEGER:
				if(strchr(aux_ptr,'x'))
					sscanf(aux_ptr,"%x",val);
				else
					sscanf(aux_ptr,"%d",val);
				break;
			case SMI_FLOAT:
				sscanf(aux_ptr,"%lf",val);
				break;
		}   	
		ret = 1;
	}
	pop_token(ptr);
	return(ret);
}

int smi_get_par_value(param, value)
char *param;
void *value;
{
	int type, size;
	char aux_param[32];

	if(!Cmnd_valid)
		return(0);
	init_token(&SMI_command);
	while(get_next_param(&SMI_command, aux_param, &type, &size))
	{
		if(!strcmp(param, aux_param))
		{
			get_param_value(&SMI_command, value);
			return(1);
		}
	}
	return(0);
}

int smi_get_state(state, max_len)
char *state;
int max_len;
{
	char *ptr;
	int len;

	strncpy(state, SMI_state, max_len);
	if(ptr = strchr(state,'/'))
		*ptr = '\0';
	else
	{
		if(strlen(SMI_state) >= max_len)
			state[max_len - 1] = '\0';
		else
			state[strlen(SMI_state)] = '\0';
	}
	return(1);
}

int smi_get_substate(subobj, state)
char *subobj, *state;
{
	int i;

	for(i = 0; i < N_sub_objects; i++)
	{
		if(!strcmp(Subs[i].name,subobj))
		{
			strcpy(state,Subs[i].state);
			return(1);
		}
	}
	return(0);
}

static int init_token(ptr)
TOKEN_STR *ptr;
{
	ptr->token_ptr = ptr->token_buff;
	ptr->curr_token_ptr = 0;
}

static int get_token(ptr,token)
TOKEN_STR *ptr;
char **token;
{
static char *Actions;
static int Actions_size = 0;
char *Actions_ptr = 0;
char *aux_ptr;
int in_tok;

	if(!ptr->curr_token_ptr)
	{
		ptr->curr_token_ptr = ptr->token_buff;
	}
	if(Actions_size < strlen(ptr->token_buff)+1)
	{
		if(Actions_size)
			free(Actions);
		Actions = malloc(strlen(ptr->token_buff)+1);
		Actions_size = strlen(ptr->token_buff)+1;
	}
	aux_ptr = ptr->curr_token_ptr;
	Actions_ptr = Actions;	

	in_tok = 0;
	while(*aux_ptr)
	{
		if(*aux_ptr == '"')
		{
			if(in_tok)
				break;
			in_tok = 1;
			aux_ptr++;
			aux_ptr = strchr(aux_ptr,'"');
			aux_ptr++;
			break;
		}
		else if(*aux_ptr == '/')
		{
			if(in_tok)
				break;
			in_tok = 1;
			aux_ptr++;
			break;
		}
		else if(*aux_ptr == '|')
		{
			if(in_tok)
				break;
			in_tok = 1;
			aux_ptr++;
			break;
		}
		else if(*aux_ptr == '=')
		{
			if(in_tok)
				break;
			in_tok = 1;
			aux_ptr++;
			break;
		}
		else if(*aux_ptr == '(')
		{
			if(in_tok)
				break;
			in_tok = 1;
			aux_ptr++;
			break;
		}
		else if(*aux_ptr == ')')
		{
			if(in_tok)
				break;
			in_tok = 1;
			aux_ptr++;
			break;
		}
		else
		{
			*Actions_ptr++ = *aux_ptr++;
			in_tok = 2;
		}
	}
	if(in_tok == 2)
	{
		*Actions_ptr = '\0';
		*token = Actions;
		ptr->curr_token_ptr = aux_ptr;
		return(1);
	}
	if(in_tok == 1)
	{
		strncpy(Actions, ptr->curr_token_ptr, aux_ptr - ptr->curr_token_ptr);
		Actions[aux_ptr - ptr->curr_token_ptr] = '\0';
		*token = Actions;
		ptr->curr_token_ptr = aux_ptr;
		return(1);
	}
	ptr->curr_token_ptr = 0;
	return(0);
}		

static int push_token(ptr)
TOKEN_STR *ptr;
{
	ptr->token_ptr = ptr->curr_token_ptr;
}

static int pop_token(ptr)
TOKEN_STR *ptr;
{
	ptr->curr_token_ptr = ptr->token_ptr;
}

static int cmp_token(token, str)
char *token, *str;
{
	if(!strcmp(token, str))
		return(1);
	return(0);
}

static int first_token(ptr)
TOKEN_STR *ptr;
{
	if(ptr->token_ptr == ptr->token_buff)
		return(1);
	return(0);
}

#ifdef VMS
int get_log_name(log_name, table, name)
char *log_name, *table, *name;
{
short length;
int ret;
struct dsc$descriptor_s tabnam_dsc;
struct dsc$descriptor_s lognam_dsc;

struct {
            short  length;
            short  code;
            char   *address;
            short  *len_add;
        } trnlnm_itmlst[] =
        {
            132,LNM$_STRING, 0, &length,
            0, 0, 0, 0
        };

	tabnam_dsc.dsc$a_pointer = table;
	tabnam_dsc.dsc$w_length = strlen(table);
	lognam_dsc.dsc$a_pointer = log_name;
	lognam_dsc.dsc$w_length = strlen(log_name);
	trnlnm_itmlst[0].address = name;
    ret = sys$trnlnm (0, &tabnam_dsc, &lognam_dsc, 0, trnlnm_itmlst);
    if (ret != SS$_NORMAL)
		return(0);
	name[length] = '\0';
	return(1);
}

get_domain_name(name)
char *name;
{
	get_log_name("SMI_DOMAIN","LNM$FILE_DEV",name);
}

int smi_send_command(obj, cmnd)
char *obj, *cmnd;
{
	char serv_name[MAX_NAME];
			
	get_obj_name(obj,serv_name,0);
	strcat(serv_name,"/CMD");
	dic_cmnd_service(serv_name, cmnd, strlen(cmnd)+1);
	return(1);
}

#endif

/* CFORTRAN WRAPPERS */

#ifdef VMS
FCALLSCFUN2(INT, smi_attach, SMI_ATTACH, smi_attach,
                 STRING, PVOID)
FCALLSCFUN2(INT, smi_init_for, SMI_INIT_FOR, smi_init_for,
                 INT, INT)
FCALLSCFUN1(INT, smi_associate, SMI_ASSOCIATE, smi_associate,
                 STRING)
FCALLSCFUN1(INT, smi_set_state, SMI_SET_STATE, smi_set_state,
                 STRING)
FCALLSCFUN2(INT, smi_get_state, SMI_GET_STATE, smi_get_state,
                 STRING, INT)
FCALLSCFUN2(INT, smi_get_substate, SMI_GET_SUBSTATE, smi_get_substate,
                 STRING, STRING)
FCALLSCFUN1(INT, smi_terminate_command, SMI_TERMINATE_COMMAND, 
				 smi_terminate_command,
                 STRING)
FCALLSCFUN1(INT, smi_terminate_action, SMI_TERMINATE_ACTION, 
				 smi_terminate_action,
                 STRING)
FCALLSCFUN2(INT, smi_get_command, SMI_GET_COMMAND,
				 smi_get_command,
                 PSTRING, PINT)
FCALLSCFUN3(INT, smi_get_commandw_for, SMI_GET_COMMANDW_FOR,
				 smi_get_commandw_for,
                 PSTRING, PINT, INT)
FCALLSCFUN2(INT, smi_get_action, SMI_GET_ACTION,
				 smi_get_action,
                 PSTRING, PINT)
FCALLSCFUN1(INT, smi_test_action, SMI_TEST_ACTION,
				 smi_test_action,
                 STRING)
FCALLSCFUN3(INT, smi_get_next_par, SMI_GET_NEXT_PAR,
				 smi_get_next_par,
                 PSTRING, PINT, PINT)
FCALLSCFUN2(INT, smi_get_par_value, SMI_GET_PAR_VALUE,
				 smi_get_par_value,
                 STRING, PVOID)
FCALLSCFUN3(INT, smi_set_par, SMI_SET_PAR,
				 smi_set_par,
                 STRING, PVOID, INT)
FCALLSCFUN2(INT, smi_send_command, SMI_SEND_COMMAND, smi_send_command,
                 STRING, STRING)
FCALLSCSUB0(smi_volatile, SMI_VOLATILE, smi_volatile)
#endif

