#ifndef __SMIRTLHHDEFS
#define __SMIRTLHHDEFS

#include <dis.hh>
#include <sllist.hh>
#include "smirtl_core.hh"
#include <tokenstring.hh>

#ifdef WIN32
#ifdef SMILIB
#	define DllExp __declspec(dllexport)
#else
#	define DllExp __declspec(dllimport)
#endif
#else
#	define DllExp
#endif

#ifndef SMI_PAR_TYPES
#ifndef WIN32
typedef enum { STRING, INTEGER, FLOAT } SHORT_PAR_TYPES;
#endif
typedef enum { SMI_STRING, SMI_INTEGER, SMI_FLOAT } PAR_TYPES;
#define SMI_PAR_TYPES
#endif

class ParInfo : public SLLItem {
	char *itsName;
	int itsType;
	char *itsData;
	char itsTypeStr[8];
public:
	ParInfo(char *name, int type)
	{
		itsName = new char[strlen(name)+1];
		strcpy(itsName, name);
		itsType = type;
		switch(type)
		{
			case SMI_INTEGER:
				strcpy(itsTypeStr,"(I)");
				itsData = new char[32];
				break;
			case SMI_FLOAT:
				strcpy(itsTypeStr,"(F)");
				itsData = new char[32];
				break;
			case SMI_STRING:
				itsData = new char[32];
				strcpy(itsTypeStr,"(S)");
				break;
		}
	}
	~ParInfo()
	{
		delete itsName;
		delete itsData;
	}
	int setInfo(void *data, int type)
	{
		if(itsType != type)
			return -1;
		switch(type)
		{
			case SMI_INTEGER:
				sprintf(itsData,"%d",*(int *)data);
				break;
			case SMI_FLOAT:
				sprintf(itsData,"%e",*(double *)data);
				break;
			case SMI_STRING:
				delete itsData;
				itsData = new char[ strlen((char *)data) +1];
				strcpy(itsData,(char *)data);
				break;
		}
		return 1;
	}
	int setStrInfo(char *data, int type)
	{
		if(itsType != type)
			return -1;
		delete itsData;
		itsData = new char[ strlen(data) +1];
		strcpy(itsData,(char *)data);
		return 1;
	}
	char *getName() { return itsName; }
	int getType() { return itsType; }
	int getDataSize() { return strlen(itsData)+1; }
	char *getTypeStr() { return itsTypeStr; }
	char *getData() { return itsData; }
	
};

class DllExp SmiProxy: public DimCommandHandler, public DimClientExitHandler
{
private:
	char *itsDomain;
	char *itsName;
	char *itsFullName;
	char *itsState;
	char *itsShortState;
	char *itsAction;
	char *itsCommand;
	char *itsParValue;
	DimService *itsDimState;
	DimCommand *itsDimCmnd;
	DimService *itsDimVersion;
	void capitalize(char *str);
	SLList itsParList;
	SLList itsCmndParList;
	ParInfo *currCmndPar;
//	int findPar(char *name);
	int addPar(char *name, void *value, int type);
	TokenString *cmndTokens;
	int getType(char *str);
	int getStateLen(char *state);
	void formState(char *state);
	int storeCmndPar();
	char *getCmndPar(char *name, int type);
	void publish_it();
	void clientExitHandler() {smiDomainExitHandler();}
	void commandHandler();
public:
	void attach(char *domain);
	void attach(char *domain, char *name);
	void setVolatile();
	int setState(char *state);
	char *getState();
	int setParameter(char *parName, int parValue);
	int setParameter(char *parName, double parValue);
	int setParameter(char *parName, char *parValue);
	int getParameterInt(char *parName);
	double getParameterFloat(char *parName);
	char *getParameterString(char *parName);
	int getParameterType(char *parName);
	char *getNextParameter();
	int testAction(char *action);
	char *getAction();
	
	SmiProxy(char *name){
		itsName = new char[strlen(name)+1];
		strcpy(itsName,name);
		itsState = 0;
		itsAction = 0;
		itsShortState = 0;
		itsCommand = 0;
		cmndTokens = 0;
	}

	SmiProxy(){
		itsName = 0;
		itsState = 0;
		itsAction = 0;
		itsShortState = 0;
		itsCommand = 0;
		cmndTokens = 0;
	}

	~SmiProxy() 
	{
		ParInfo *parptr;
		delete itsName;
		delete itsFullName;
		if(itsState)
			delete itsState;
		if(itsShortState)
			delete itsShortState;
		if(itsAction)
			delete itsAction;
		if(itsCommand)
			delete itsCommand;
		if(itsParValue)
			delete itsParValue;
		if(cmndTokens)
			delete cmndTokens;
		while(parptr = (ParInfo *)itsParList.removeHead())
			delete parptr;
		while(parptr = (ParInfo *)itsCmndParList.removeHead())
			delete parptr;
		delete itsDimState;
		delete itsDimCmnd;
		delete itsDimVersion;
	};
	virtual void smiCommandHandler() {};
	virtual void smiDomainExitHandler() {exit(0);};
};

#endif
