#define MAX_OBJECTS		512
#define MAX_BUFFER		1024

#define SMIUILIB
#include "smiuirtl.h"

#include <dim.h>

#ifdef VMS
#include <cfortran.h>
#include <ssdef.h>   
#include <iodef.h>   
#include <lnmdef.h>    
#endif

#include <stdlib.h>
#include <string.h>
#include "parameters.h"

typedef struct {
	int state;
	char action[MAX_BUFFER];
}BUSY;

typedef struct {
	char action_buff[MAX_NAME * MAXACTIONS];
	int index;
	void (*user_rout)();
	int par;
	int id;
}ACTIONS;

typedef struct {
	char token_buff[MAX_NAME * MAXACTIONS];
	char *token_ptr;
	char *curr_token_ptr;
}TOKEN_STR;

typedef struct dom_info{
	struct dom_info *next;
	char domain[80];
	int dom_id;
	int objlist_id;
	char *info;
	TOKEN_STR *objs;
	int size;
	int obj_index;
	void (*user_rout)();
	int par;
	int nobjs;
	int nanswers;
}DOMAIN_INFO;

typedef struct {
	char obj_name[MAX_NAME];
	int state_id;
	int busy_id;
	int action_id;
	void (*user_rout)();
	int par;
	TOKEN_STR *state;
	int busy_state;
	char busy_action[MAX_BUFFER];
	TOKEN_STR *actions;
	DOMAIN_INFO *domain_ptr;
}STATECHANGE;

typedef char STR_ARR[MAXNAMESIZE];

static char My_address[MAX_NAME];
static int Get_obj_busy = -1;
static int Get_obj_state = 0;
static char No_link_str[8] = "DEAD";
static int No_link_busy = 0;
static int No_link = -1;
static char No_link_char = -1;
static int Wake_up = 0;
static int Domain_serv_id = 0;
static DOMAIN_INFO *Domain_head = (DOMAIN_INFO *) 0;

_PROTO( static int init_token, (TOKEN_STR *ptr) );
_PROTO( static int get_token,  (TOKEN_STR *ptr, char **token) );
_PROTO( static int push_token, (TOKEN_STR *ptr) );
_PROTO( static int pop_token,  (TOKEN_STR *ptr) );
_PROTO( static int cmp_token,  (char *token, char *str) );
_PROTO( static int first_token,(TOKEN_STR *ptr) );

static void capitalize(name)
char *name;
{
	while(*name)
	{
		*name = toupper(*name);
		name++;
	}
}

static void capitalize_sel(name)
char *name;
{
	int par_value = 0;

	while(*name)
	{
		if(*name == '=')
			par_value = 1;
		if(*name == '/')
			par_value = 0;
		if(!par_value)
			*name = toupper(*name);
		name++;
	}
}

get_service_name(obj_name, serv_name, ep)
char *obj_name, *serv_name;
int ep;
{

	char obj[MAX_NAME];
	char *ptr_domain, *ptr_obj, obj_type[10];

	strcpy(obj,obj_name);
	if(ptr_obj = (char *)strstr(obj,"::"))
	{
		*ptr_obj = '\0';
		ptr_obj +=2;
		ptr_domain = obj;
	}
	else
	{
		strcpy(serv_name,"NONE");
		return(0);
	}
	if(ep)
		sprintf(serv_name,"EP/%s/%s", ptr_domain, ptr_obj);
	else
		sprintf(serv_name,"SMI/%s/%s", ptr_domain, ptr_obj);
	return(1);
}
	
void get_domain_info_rout(domain_ptr_ptr, buff, size)
DOMAIN_INFO **domain_ptr_ptr;
char *buff;
int *size;
{
DOMAIN_INFO *domain_ptr;
int n_objs;

	domain_ptr = *domain_ptr_ptr;
	domain_ptr->nanswers++;
	if(!strcmp(buff,No_link_str))
		domain_ptr->nobjs = 0;
	else
	{
		domain_ptr->info = malloc((*size)+1);
		memcpy(domain_ptr->info,buff,*size);
		domain_ptr->info[*size] = '\0';
		domain_ptr->size = *size;
		n_objs = domain_ptr->size/MAXNAMESIZE;
		if(!strcmp(domain_ptr->info,"&ALLOC"))
			n_objs--;
		domain_ptr->nobjs = n_objs;
	}
	if(domain_ptr->nanswers == 2)
	{
		if(domain_ptr->user_rout)
		{
			domain_ptr->user_rout(&domain_ptr->par, &domain_ptr->nobjs);
		}
		domain_ptr->nanswers = 0;
	}
	Wake_up++;
#ifdef VMS
	sys$wake(0,0);
#endif
#ifdef WIN32
	wake_up();
#endif
}

static int get_next_attr(ptr, attr)
TOKEN_STR *ptr;
char *attr;
{
	char *aux_ptr;

	push_token(ptr);
	while(get_token(ptr,&aux_ptr))
	{
		if(cmp_token(aux_ptr,"/"))
		{
			get_token(ptr,&aux_ptr);
			strcpy(attr, aux_ptr);
			return(1);
		}
		if(cmp_token(aux_ptr,"|"))
		{
			break;
		}
	}
	pop_token(ptr);
	return(0);
}
/*
static int get_next_obj(ptr, obj)
TOKEN_STR *ptr;
char *obj;
{
	char *aux_ptr;
	int ret = 0, found = 0;

	if(ptr == 0)
		return(0);

	if(get_token(ptr,&aux_ptr))
	{
		strcpy(obj,aux_ptr);
		if(cmp_token(aux_ptr,"&ALLOC"))
			ret = 2;
		else
			ret = 1;
		push_token(ptr);
		while(get_token(ptr,&aux_ptr))
		{
			if(cmp_token(aux_ptr,"|"))
			{
				found = 1;
				break;
			}
			push_token(ptr);
		}
	}
	if(!found)
		pop_token(ptr);
	return(ret);
}
*/

static int get_next_obj(ptr, obj)
TOKEN_STR *ptr;
char *obj;
{
	char *aux_ptr;
	int ret = 0, found = 0;

	if(ptr == 0)
		return(0);

	push_token(ptr);
	if(!get_token(ptr, &aux_ptr))
	{
		pop_token(ptr);
		return(0);
	}
	if(cmp_token(aux_ptr,"/"))
	{
		push_token(ptr);
		while(get_token(ptr,&aux_ptr))
		{
			if(cmp_token(aux_ptr,"|"))
			{
				found = 1;
				break;
			}
			push_token(ptr);
		}
		if(!found)
			pop_token(ptr);
	}
	else if(cmp_token(aux_ptr,"|"))
	{
	}
	else
		pop_token(ptr);
	if(get_token(ptr,&aux_ptr))
	{
		strcpy(obj,aux_ptr);
		if(cmp_token(aux_ptr,"&ALLOC"))
			ret = 2;
		else
			ret = 1;
	}
	return(ret);
}

void get_objlist_info_rout(domain_ptr_ptr, buff, size)
DOMAIN_INFO **domain_ptr_ptr;
char *buff;
int *size;
{
DOMAIN_INFO *domain_ptr;
int n_objs = 0, ret;
char obj[MAX_NAME];

	
	domain_ptr = *domain_ptr_ptr;
	domain_ptr->nanswers++;
	if(!strcmp(buff,No_link_str))
		domain_ptr->nobjs = 0;
	else
	{
		domain_ptr->objs = malloc(sizeof(TOKEN_STR));
		strcpy(domain_ptr->objs->token_buff,buff);
		domain_ptr->size = *size;
		init_token(domain_ptr->objs);
		while(ret = get_next_obj(domain_ptr->objs, obj))
		{
			if(ret == 1)
				n_objs++;
		}
		domain_ptr->nobjs = n_objs;
		init_token(domain_ptr->objs);
	}
	if(domain_ptr->nanswers == 2)
	{
		if(domain_ptr->user_rout)
		{
			domain_ptr->user_rout(&domain_ptr->par, &domain_ptr->nobjs);
		}
		domain_ptr->nanswers = 0;
	}
	Wake_up++;
#ifdef VMS
	sys$wake(0,0);
#endif
#ifdef WIN32
	wake_up();
#endif

}

static DOMAIN_INFO *find_domain(domain)
char *domain;
{
	DOMAIN_INFO *domain_ptr;

    if( !Domain_head ) {
		return((DOMAIN_INFO *)0);
	}
	domain_ptr = (DOMAIN_INFO *)sll_search( (SLL *)Domain_head, 
						domain, strlen(domain));
	return( domain_ptr);
}

static DOMAIN_INFO *connect_domain(domain, user_rout, par)
char *domain;
void (*user_rout)();
int par;
{
	char serv_name[MAX_NAME];
	DOMAIN_INFO *domain_ptr;

    if( !Domain_head ) {
		Domain_head = (DOMAIN_INFO *)malloc(sizeof(DOMAIN_INFO));
		sll_init( (SLL *) Domain_head );
	}
	if(!(domain_ptr = find_domain(domain)))
	{
		domain_ptr = malloc(sizeof(DOMAIN_INFO));
		strcpy(domain_ptr->domain,domain);
		domain_ptr->user_rout = user_rout;
		domain_ptr->par = par;
		domain_ptr->info = 0;
		domain_ptr->objs = 0;       
		domain_ptr->nobjs = 0;
		domain_ptr->nanswers = 0;
		Wake_up = 0;
		sll_insert_queue( (SLL *) Domain_head, (SLL *) domain_ptr );
		sprintf(serv_name,"SMI/%s/OBJECT_LIST",domain);
		domain_ptr->objlist_id = dic_info_service(serv_name, MONITORED, 0, 
			0, 0, 
			get_objlist_info_rout, (int)domain_ptr, No_link_str, 5);
		sprintf(serv_name,"SMI/%s/DOMAIN",domain);
		domain_ptr->dom_id = dic_info_service(serv_name, MONITORED, 0, 0, 0, 
			get_domain_info_rout, (int)domain_ptr, No_link_str, 5);
	}
	return( domain_ptr);
}

static disconnect_domain(domain, remove)
char *domain;
int remove;
{
DOMAIN_INFO *domain_ptr;

    if( !Domain_head ) {
		Domain_head = (DOMAIN_INFO *)malloc(sizeof(DOMAIN_INFO));
		sll_init( (SLL *) Domain_head );
	}
	if(domain_ptr = find_domain(domain))
	{
		dic_release_service(domain_ptr->dom_id);
		dic_release_service(domain_ptr->objlist_id);
		if(remove)
		{
			if(domain_ptr->info)
				free(domain_ptr->info);
			if(domain_ptr->objs)
				free(domain_ptr->objs);
			sll_remove( (SLL *) Domain_head, (SLL *) domain_ptr );
			free(domain_ptr);
		}
		return(1);
	}
	return(0);
}

int smiui_connect_domain(domain)
char *domain;
{
	DOMAIN_INFO *domain_ptr;

	capitalize(domain);
	domain_ptr = find_domain(domain);
	if(!domain_ptr)
	{
		domain_ptr = connect_domain(domain, 0, 0);
		while(Wake_up < 2)
		{
#ifdef VMS
			sys$hiber();
#else
			dim_wait();
#endif
		}
		if(!domain_ptr->nobjs)
		{
			disconnect_domain(domain, 1);
			return(0);
		}
		disconnect_domain(domain, 0);
	}
	domain_ptr->obj_index = 0;
	return(domain_ptr->nobjs);
}
                     
int smiui_book_connect_domain(domain, user_rout, par)
char *domain;
void (*user_rout)();
int par;
{
	DOMAIN_INFO *domain_ptr;

	capitalize(domain);
	if(!(domain_ptr = find_domain(domain)))
	{
		domain_ptr = connect_domain(domain, user_rout, par);
		domain_ptr->obj_index = 0;
		return(0);
	}
	else
		init_token(domain_ptr->objs);
	domain_ptr->obj_index = 0;
	return(domain_ptr->nobjs);
}
    
int smiui_cancel_connect_domain(domain)
char *domain;
{
	char serv_name[MAX_NAME];
	DOMAIN_INFO *domain_ptr;

	capitalize(domain);
	return(disconnect_domain(domain, 1));
}

int smiui_shutdown_domain(domain)
char *domain;
{
	char serv_name[MAX_NAME];
	int exit_code = 1;

	capitalize(domain);
	/*
	sprintf(serv_name,"SMI/%s/&ALLOC/CMD",domain);
	dic_cmnd_service(serv_name, "EXIT", 5);
	*/
	sprintf(serv_name,"%s_SMI/EXIT",domain);
	dic_cmnd_service(serv_name, &exit_code, 4);
	return(1);
}

int smiui_number_of_objects(domain)
char *domain;
{

	return(smiui_connect_domain(domain));
}

void get_proxy(tag, buff, size)
int *tag;
int *buff;
int *size;
{
	Wake_up++;
#ifdef VMS
	sys$wake(0,0);
#endif
#ifdef WIN32
	wake_up();
#endif

}

int smiui_check_proxy(obj)
char *obj;
{
	char serv_name[MAX_NAME];
	static char state[MAX_NAME];

	capitalize(obj);
	strcpy(serv_name,obj);
	strcat(serv_name,"/VERSION_NUMBER");
	Wake_up = 0;
	dic_info_service(serv_name, ONCE_ONLY, 20, state, MAX_NAME, 
		get_proxy, 0, No_link_str, 5);
	while(!Wake_up)
	{
#ifdef VMS
		sys$hiber();
#else
		dim_wait();
#endif
	}
	if(!strcmp(state, No_link_str))
		return(0);
	return(1);
}                 

void get_exit_res(tag, res)
int *tag;
int *res;
{
	Wake_up++;
#ifdef VMS
	sys$wake(0,0);
#endif
#ifdef WIN32
	wake_up();
#endif

}

int smiui_kill(obj)
char *obj;
{
	char serv_name[MAX_NAME];
	int exit_code = 1;

	capitalize(obj);
	strcpy(serv_name,obj);
	if(strstr(obj,"::"))
	  strcat(serv_name,"/EXIT");
	else
	  strcat(serv_name,"_SMI/EXIT");
	Wake_up = 0;
	dic_cmnd_callback(serv_name, &exit_code, sizeof(int),
		get_exit_res, 0);
	while(!Wake_up)
	{
#ifdef VMS
		sys$hiber();
#else
		dim_wait();
#endif
	}
	return(1);
}                 

int smiui_get_next_object(domain,object)
char *domain;
char *object;
{
	DOMAIN_INFO *domain_ptr;
	STR_ARR *info_ptr;
	char obj[MAX_NAME];
	int ret;

	capitalize(domain);
	capitalize(object);
	domain_ptr = find_domain(domain);
	if(!domain_ptr)
	{
		smiui_connect_domain(domain);
		domain_ptr = find_domain(domain);
	}
	if((!domain_ptr) || (!domain_ptr->nobjs))
		return(0);
	
	
	if(domain_ptr->objs)
	{
		if(ret = get_next_obj(domain_ptr->objs, obj))
		{
			if(ret == 2)
				get_next_obj(domain_ptr->objs, obj);
			strcpy(object,domain);
			strcat(object,"::");
			strcat(object,obj);
			return(1);
		}
		else
			return(0);
	}
	info_ptr = (STR_ARR *)domain_ptr->info;
	if(!info_ptr[domain_ptr->obj_index][0])
	{
		domain_ptr->obj_index = 0;
		return(0);
	}
	if(!strcmp(info_ptr[domain_ptr->obj_index],"&ALLOC"))
	{
		domain_ptr->obj_index++;
		if(!info_ptr[domain_ptr->obj_index][0])
		{
			domain_ptr->obj_index = 0;
			return(0);
		}
	}
	strcpy(object,domain);
	strcat(object,"::");
	strcat(object,info_ptr[domain_ptr->obj_index++]);
	return(1);
}

int smiui_get_next_attribute(object, attr)
char *object;
char *attr;
{
	DOMAIN_INFO *domain_ptr;
	char domain[MAX_NAME], *ptr;
	int ret;

	capitalize(object);
	strcpy(domain, object);
	ptr = strstr(domain,"::");
	*ptr = 0;
	ptr += 2;
	domain_ptr = find_domain(domain);
	if(!domain_ptr)
	{
		smiui_connect_domain(domain);
		domain_ptr = find_domain(domain);
	}
	if((!domain_ptr) || (!domain_ptr->nobjs))
		return(0);
	
	
	if(domain_ptr->objs)
	{
		return(get_next_attr(domain_ptr->objs, attr));
	}
	else
		return(0);
}

void get_state_rout(tag)
int *tag;
{
	if(*tag)
	{
		Get_obj_state = 1;
	}
	if((!Get_obj_busy) && (Get_obj_state))
	{
		Wake_up = 1;
#ifdef VMS
		sys$wake(0,0);
#endif
#ifdef WIN32
	wake_up();
#endif
	}
}
                     
int smiui_wait_stable_state(obj, state)
char *obj;
char *state;
{
	char serv_name[MAX_NAME];
	int index, state_id, busy_id;
	void get_state_rout();

	Wake_up = 0;
	get_service_name(obj,serv_name,0);
	Get_obj_state = 0;
	Get_obj_busy = -1;
	state_id = dic_info_service(serv_name, MONITORED, 0, state, MAXNAMESIZE, 
		get_state_rout, 1, No_link_str, 5);
	strcat(serv_name,"/BUSY");
	busy_id = dic_info_service(serv_name, MONITORED, 0, &Get_obj_busy, 4, 
		get_state_rout, 0, &No_link_busy, 4);
	while(!Wake_up)
	{
#ifdef VMS
		sys$hiber();
#else
		dim_wait();
#endif
	}
	dic_release_service(state_id);
	dic_release_service(busy_id);
	if(!strcmp(state,No_link_str))
		return(0);
	return(1);
}
                     
static BUSY Get_obj_state_busy = {-1};

void get_state_busy_rout(tag)
int *tag;
{
	if(*tag)
	{
		Get_obj_state = 1;
	}
	if((Get_obj_state_busy.state != -1) && (Get_obj_state))
	{
		Wake_up = 1;
#ifdef VMS
		sys$wake(0,0);
#endif
#ifdef WIN32
		wake_up();
#endif
	}
}
                     
int smiui_current_state(obj, state, action)
char *obj;
char *state;
char *action;
{
	char serv_name[MAX_NAME];
	int index, state_id, busy_id;

	capitalize(obj);
	get_service_name(obj,serv_name,0);

	Wake_up = 0;
	Get_obj_state = 0;
	Get_obj_state_busy.state = -1;
	state_id = dic_info_service(serv_name, MONITORED, 0, state, MAXNAMESIZE, 
		get_state_busy_rout, 1, No_link_str, 5);
	strcat(serv_name,"/BUSY");
	busy_id = dic_info_service(serv_name, MONITORED, 0, &Get_obj_state_busy, 
		4+MAX_BUFFER, get_state_busy_rout, 0, &No_link_busy, 4);
	while(!Wake_up)
	{
#ifdef VMS
		sys$hiber();
#else
		dim_wait();
#endif
	}
	dic_release_service(state_id);
	dic_release_service(busy_id);
	if(Get_obj_state_busy.state == 1)
	{
		strcpy(action,Get_obj_state_busy.action);
	}
	else
		action[0] = '\0';
	if(!strcmp(state,No_link_str))
		return(0);
	return(1);
}

void state_change_rout(ptr_ptr)
STATECHANGE **ptr_ptr;
{
	STATECHANGE *ptr;

	ptr = *ptr_ptr;
	init_token(ptr->state);
	init_token(ptr->actions);
	ptr->user_rout((int *)&ptr, &ptr->par);
}

int smiui_book_statechange(obj, user_rout, par)
char *obj;
void (*user_rout)();
int par;
{
	char serv_name[MAX_NAME];
	int index, state_id, busy_id;
	STATECHANGE *ptr;

	capitalize(obj);
	ptr = malloc(sizeof(STATECHANGE));
	strcpy(ptr->obj_name,obj);
	ptr->user_rout = user_rout;
	ptr->par = par; 
	ptr->state = malloc(sizeof(TOKEN_STR));
	init_token(ptr->state);
	ptr->actions = malloc(sizeof(TOKEN_STR));
	init_token(ptr->actions);
	get_service_name(obj,serv_name,0);
	ptr->state_id = dic_info_service(serv_name, MONITORED, 0, 
		ptr->state->token_buff, MAX_NAME * MAXACTIONS, 
		0, 0, "No link", 8);
	get_service_name(obj,serv_name,0);
	strcat(serv_name,"/ACTIONS&PARS");
	ptr->action_id = dic_info_service(serv_name, MONITORED, 0,
		ptr->actions->token_buff, MAX_NAME * MAXACTIONS, 
		0, 0, "No link", 8);
	get_service_name(obj,serv_name,0);
	strcat(serv_name,"/BUSY");
	ptr->busy_id = dic_info_service(serv_name, MONITORED, 0, 
		&(ptr->busy_state), sizeof(BUSY), 
		state_change_rout, (int)ptr, &No_link, 4);
	return((int)ptr);
}

int smiui_book_alloc_statechange(domain, user_rout, par)
char *domain;
void (*user_rout)();
int par;
{
	char serv_name[MAX_NAME];

	capitalize(domain);
	strcpy(serv_name,domain);
	strcat(serv_name,"::");
	strcat(serv_name,"&ALLOC");
	return((int)smiui_book_statechange(serv_name,user_rout,par));
		
}
                     
int smiui_cancel_statechange(id)
int id;
{
STATECHANGE *ptr;

	ptr = (STATECHANGE *)id; 
	dic_release_service(ptr->state_id);
	dic_release_service(ptr->action_id);
	dic_release_service(ptr->busy_id);
	free(ptr->state);
	free(ptr->actions);
	free(ptr);
}

int smiui_cancel_alloc_statechange(id)
int id;
{

	smiui_cancel_statechange(id);
}
                     

int smiui_get_alloc_state(id, busyflg, state)
int id;
int *busyflg;
char *state;
{
	int n_actions;

	return( smiui_get_state(id, busyflg, state, &n_actions));
}


int smiui_get_action_in_progress(id, action)
int id;
char *action;
{
	int index = 0;
	STATECHANGE *ptr;

        ptr = (STATECHANGE *)id; 
	if(ptr->busy_state == 1)
		strcpy(action,ptr->busy_action);
	else
		action[0] = '\0';
	return(1);
}

int smiui_get_name(id, name)
int id;
char *name;
{
STATECHANGE *ptr;

	ptr = (STATECHANGE *)id; 
	strcpy(name,ptr->obj_name);
	return(1);
}

int smiui_get_state(id, busyflg, state, n_actions)
int id;
int *busyflg;
char *state;
int *n_actions;
{
	int index = 0;
	char *aux_ptr;
	STATECHANGE *ptr;

	ptr = (STATECHANGE *)id; 
	init_token(ptr->state);
	init_token(ptr->actions);
	*busyflg = ptr->busy_state;
	if(*busyflg == No_link)
	{
		state[0] = '\0';
		*n_actions = 0;
		*busyflg = 0;
		return(0);
	}
	get_token(ptr->state, &aux_ptr);
	strcpy(state,aux_ptr);
	*n_actions = 0;
	
	if(get_token(ptr->actions, &aux_ptr))
	{
		(*n_actions)++;
    	while(get_token(ptr->actions, &aux_ptr))
		{
			if(cmp_token(aux_ptr,"|"))
				(*n_actions)++;
		}
	}
	init_token(ptr->actions);
	return(1);
}

int smiui_get_first_action(id, action, n_pars)
int id;
char *action;
int *n_pars;
{
STATECHANGE *ptr;

	ptr = (STATECHANGE *)id; 
	init_token(ptr->actions);
	return(smiui_get_next_action(id, action, n_pars));
}

int smiui_get_next_action(id, action, n_pars)
int id;
char *action;
int *n_pars;
{
	int i;
	char *aux_ptr;
	STATECHANGE *ptr;

	ptr = (STATECHANGE *)id; 
	if(first_token(ptr->actions))
	{
		if(get_token(ptr->actions,&aux_ptr))
		{
			strcpy(action, aux_ptr);
			*n_pars = count_pars(ptr);
			return(1);
		}
	}
	else
	{
		while(get_token(ptr->actions,&aux_ptr))
		{
			if(cmp_token(aux_ptr,"|"))
			{
				get_token(ptr->actions,&aux_ptr);
				strcpy(action, aux_ptr);
				*n_pars = count_pars(ptr);
				return(1);
			}
		}
	}
	return(0);
}

int count_pars(ptr)
STATECHANGE *ptr;
{
	char *aux_ptr;
	int n_pars = 0;

	push_token(ptr->actions);
	while(get_token(ptr->actions,&aux_ptr))
	{
		if(cmp_token(aux_ptr,"/"))
		{
			n_pars++;
		}
		if(cmp_token(aux_ptr,"|"))
			break;
	}
	pop_token(ptr->actions);
	return(n_pars);
}	

static int check_type(str)
char *str;
{
	int type;

	switch(*str)
	{
		case 'S':
			type = SMI_STRING;
			break;
		case 'I':
			type = SMI_INTEGER;
			break;
		case 'F':
			type = SMI_FLOAT;
			break;
	}
	return(type);
}

static int get_next_param(ptr, par, type, size)
TOKEN_STR *ptr;
char *par;
int *type, *size;
{
	char *aux_ptr;

	*type = SMI_STRING;
	push_token(ptr);
	while(get_token(ptr,&aux_ptr))
	{
		if(cmp_token(aux_ptr,"/"))
		{
			get_token(ptr,&aux_ptr);
			strcpy(par, aux_ptr);
			push_token(ptr);
			get_token(ptr,&aux_ptr);
			if(cmp_token(aux_ptr,"("))
			{
				get_token(ptr,&aux_ptr);
				*type = check_type(aux_ptr);
				get_token(ptr,&aux_ptr);
			}
			else
				pop_token(ptr);

			get_token(ptr,&aux_ptr);
			if(cmp_token(aux_ptr,"="))
			{
				get_token(ptr,&aux_ptr);
				if(*type == SMI_STRING)
				{
					*size = strlen(aux_ptr);
					if(aux_ptr[0] == '"')
						*size -=2;
				}
				else if(*type == SMI_INTEGER)
				{
					*size = sizeof(int);
				}
				else
				{
					*size = sizeof(double);
				}
			}
			else
				*size = 0;
			pop_token(ptr);
			return(1);
		}
		if(cmp_token(aux_ptr,"|"))
		{
			break;
		}
	}
	pop_token(ptr);
	return(0);
}

int smiui_get_next_param(id, par, type, def_value_size)
int id;
char *par;
int *type, *def_value_size;
{
	STATECHANGE *ptr;

	ptr = (STATECHANGE *)id; 
	return(get_next_param(ptr->actions,par, type, def_value_size));
}

int smiui_get_next_obj_param(id, par, type, value_size)
int id;
char *par;
int *type, *value_size;
{
	STATECHANGE *ptr;

	ptr = (STATECHANGE *)id; 
	return(get_next_param(ptr->state, par, type, value_size));
}

static int get_param_value(ptr, val)
TOKEN_STR *ptr;
void *val;
{
	char *aux_ptr;
	int type, ret = 0;

	type = SMI_STRING;
	push_token(ptr);
	get_token(ptr,&aux_ptr);
	if(cmp_token(aux_ptr,"("))
	{
		get_token(ptr,&aux_ptr);
		type = check_type(aux_ptr);
		get_token(ptr,&aux_ptr);
	}
	else
		pop_token(ptr);

	get_token(ptr,&aux_ptr);
	if(cmp_token(aux_ptr,"="))
	{
		get_token(ptr,&aux_ptr);
		switch(type)
		{
			case SMI_STRING:
				if(aux_ptr[0] == '"')
				{
					aux_ptr[strlen(aux_ptr) - 1] = '\0'; 
					strcpy(val, aux_ptr + 1);
				}
				else
					strcpy(val, aux_ptr);
				break;
			case SMI_INTEGER:
				if(strchr(aux_ptr,'x'))
					sscanf(aux_ptr,"%x",val);
				else
					sscanf(aux_ptr,"%d",val);
				break;
			case SMI_FLOAT:
				sscanf(aux_ptr,"%lf",val);
				break;
		}   	
		ret = 1;
	}
	pop_token(ptr);
	return(ret);
}

int smiui_get_param_default_value(id, val)
int id;
void *val;
{
	STATECHANGE *ptr;

	ptr = (STATECHANGE *)id; 
	return(get_param_value(ptr->actions, val));
}

int smiui_get_obj_param_value(id, val)
int id;
void *val;
{
	STATECHANGE *ptr;

	ptr = (STATECHANGE *)id;
	return(get_param_value(ptr->state, val));
}


int smiui_send_command(obj, cmnd)
char *obj, *cmnd;
{
	char serv_name[MAX_NAME];
	char obj_aux[MAX_NAME];
	char *ptr_domain, *ptr_obj;
	int ret;
			
	capitalize(obj);
	capitalize_sel(cmnd);
	get_service_name(obj,serv_name,0);

	strcat(serv_name,"/CMD");
/*
	printf("Sending %s to %s\n",cmnd,serv_name);
*/
	dic_cmnd_service(serv_name, cmnd, strlen(cmnd)+1);
	ret = 1;
	return(ret);
}

int smiui_ep_send_command(obj, cmnd)
char *obj, *cmnd;
{
	char serv_name[MAX_NAME];

	capitalize(obj);
	capitalize_sel(cmnd);
	get_service_name(obj,serv_name,1);
	strcat(serv_name,"/CMD");
/*
	printf("Sending %s to %s\n",cmnd,serv_name);
*/
	dic_cmnd_service(serv_name, cmnd, strlen(cmnd)+1);
}

int smiui_allocate(domain)
char *domain;
{
	char serv_name[MAX_NAME], cmnd[MAX_NAME];
	int ret, cmnd_id;

	capitalize(domain);
	sprintf(serv_name,"SMI/%s/&ALLOC/CMD",domain);
	dic_cmnd_service(serv_name, "ALLOCATE", 9);
	return(1);
}

int smiui_allocate_to(domainA, domainB)
char *domainA, *domainB;
{
	char serv_name[MAX_NAME], cmnd[MAX_NAME];

	capitalize(domainA);
	capitalize(domainB);
	sprintf(serv_name,"SMI/%s::%s/&ALLOC/CMD",domainA,domainB);
	sprintf(cmnd,"ALLOCATE/ID=%s",domainB);
	dic_cmnd_service(serv_name, cmnd, strlen(cmnd)+1);
	return(1);
}

int smiui_release(domain)
char *domain;
{
	char serv_name[MAX_NAME], cmnd[MAX_NAME];
	int ret, cmnd_id;

	capitalize(domain);
	sprintf(serv_name,"SMI/%s/&ALLOC/CMD",domain);
	dic_cmnd_service(serv_name, "RELEASE", 8);
	return(1);
}

int smiui_release_from(domainA,domainB)
char *domainA, *domainB;
{
	char serv_name[MAX_NAME], cmnd[MAX_NAME];
	int ret, cmnd_id;

	capitalize(domainA);
	capitalize(domainB);
	sprintf(serv_name,"SMI/%s::%s/&ALLOC/CMD",domainA,domainB);
	dic_cmnd_service(serv_name, "RELEASE", 8);
	return(1);
}

static int init_token(ptr)
TOKEN_STR *ptr;
{
	ptr->token_ptr = ptr->token_buff;
	ptr->curr_token_ptr = 0;
}

static int get_token(ptr,token)
TOKEN_STR *ptr;
char **token;
{
static char *Actions;
static int Actions_size = 0;
char *Actions_ptr = 0;
char *aux_ptr;
int in_tok;

	if(!ptr->curr_token_ptr)
	{
		ptr->curr_token_ptr = ptr->token_buff;
	}
	if(Actions_size < strlen(ptr->token_buff)+1)
	{
		if(Actions_size)
			free(Actions);
		Actions = malloc(strlen(ptr->token_buff)+1);
		Actions_size = strlen(ptr->token_buff)+1;
	}
	aux_ptr = ptr->curr_token_ptr;
	Actions_ptr = Actions;	

	in_tok = 0;
	while(*aux_ptr)
	{
		if(*aux_ptr == '"')
		{
			if(in_tok)
				break;
			in_tok = 1;
			aux_ptr++;
			aux_ptr = strchr(aux_ptr,'"');
			aux_ptr++;
			break;
		}
		else if(*aux_ptr == '/')
		{
			if(in_tok)
				break;
			in_tok = 1;
			aux_ptr++;
			break;
		}
		else if(*aux_ptr == '|')
		{
			if(in_tok)
				break;
			in_tok = 1;
			aux_ptr++;
			break;
		}
		else if(*aux_ptr == '=')
		{
			if(in_tok)
				break;
			in_tok = 1;
			aux_ptr++;
			break;
		}
		else if(*aux_ptr == '(')
		{
			if(in_tok)
				break;
			in_tok = 1;
			aux_ptr++;
			break;
		}
		else if(*aux_ptr == ')')
		{
			if(in_tok)
				break;
			in_tok = 1;
			aux_ptr++;
			break;
		}
		else
		{
			*Actions_ptr++ = *aux_ptr++;
			in_tok = 2;
		}
	}
	if(in_tok == 2)
	{
		*Actions_ptr = '\0';
		*token = Actions;
		ptr->curr_token_ptr = aux_ptr;
		return(1);
	}
	if(in_tok == 1)
	{
		strncpy(Actions, ptr->curr_token_ptr, aux_ptr - ptr->curr_token_ptr);
		Actions[aux_ptr - ptr->curr_token_ptr] = '\0';
		*token = Actions;
		ptr->curr_token_ptr = aux_ptr;
		return(1);
	}
	ptr->curr_token_ptr = 0;
	return(0);
}		

static int push_token(ptr)
TOKEN_STR *ptr;
{
	ptr->token_ptr = ptr->curr_token_ptr;
}

static int pop_token(ptr)
TOKEN_STR *ptr;
{
	ptr->curr_token_ptr = ptr->token_ptr;
}

static int cmp_token(token, str)
char *token, *str;
{
	if(!strcmp(token, str))
		return(1);
	return(0);
}

static int first_token(ptr)
TOKEN_STR *ptr;
{
	if(ptr->token_ptr == ptr->token_buff)
		return(1);
	return(0);
}

#ifdef VMS
FCALLSCFUN3(INT, smiui_current_state, SMIUI_CURRENT_STATE,
				 smiui_current_state,
                 STRING,PSTRING,PSTRING)
FCALLSCFUN3(INT, smiui_book_statechange, SMIUI_BOOK_STATECHANGE,
				 smiui_book_statechange,
                 STRING,PVOID,INT)
FCALLSCFUN1(INT, smiui_cancel_statechange, SMIUI_CANCEL_STATECHANGE,
				 smiui_cancel_statechange,
                 INT)
FCALLSCFUN2(INT, smiui_get_name, SMIUI_GET_NAME,
				 smiui_get_name,
                 INT,PSTRING)
FCALLSCFUN4(INT, smiui_get_state, SMIUI_GET_STATE,
				 smiui_get_state,
                 INT,PINT,PSTRING,PINT)
FCALLSCFUN1(INT, smiui_connect_domain, SMIUI_CONNECT_DOMAIN,
				 smiui_connect_domain,
                 STRING)
FCALLSCFUN3(INT, smiui_book_connect_domain, SMIUI_BOOK_CONNECT_DOMAIN,
				 smiui_book_connect_domain,
                 STRING,PVOID,INT)
FCALLSCFUN1(INT, smiui_cancel_connect_domain, SMIUI_CANCEL_CONNECT_DOMAIN,
				 smiui_cancel_connect_domain,
                 STRING)
FCALLSCFUN2(INT, smiui_get_next_object, SMIUI_GET_NEXT_OBJECT,
				 smiui_get_next_object,
                 STRING,PSTRING)
FCALLSCFUN3(INT, smiui_get_next_action, SMIUI_GET_NEXT_ACTION,
				 smiui_get_next_action,
                 INT,PSTRING,PINT)
FCALLSCFUN2(INT, smiui_send_command, SMIUI_SEND_COMMAND,
				 smiui_send_command,
                 STRING,STRING)
FCALLSCFUN2(INT, smiui_get_next_attribute, SMIUI_GET_NEXT_ATTRIBUTE,
				 smiui_get_next_attribute,
                 STRING,PSTRING)
#endif

