//-------------------------- Action Class ----------------------------------
#include <iostream.h>
#include <stdio.h>
#include <assert.h>

#include "parameters.hh"
#include "action.hh"
#include "smiobject.hh"
#include "inslist.hh"
//
//                                                        Author: B. Franek
//                                                     Date : 23 September 1996
//-----------------------------------------------------------------------------

Action::Action
      ( char lines[][MAXRECL],char* action_nm,int *no_lines, 
       char* objName, SMIObject *pobj, State* pstat){
//
// First, the Action class is instantiated and as a part of its initialisation,
// the Instruction block number 0 is also instantiated. Even though the action
// may contain many blocks, it points directly only to this block (0).
// The other blocks of the action are accessed hirachically from block(0)

   enum { maxInsBlocks = 500 };     // maximal number of Instruction blocks
                                    // of the current action

   int NoOfBlocks;
   InsList *pblocks[maxInsBlocks];  // pointers to all the blocks 
                                    // of the current action

   _objName = objName;
   _pParentObject = pobj;
   _pParentState = pstat;

   strcpy(action_nm,lines[1]);

   _actionName = action_nm; 


#ifdef DEBUG
   cout << "    action : " << _actionName << "\n";
   cout.flush();
#endif

   int numOfPar;
   sscanf(lines[2],"%d",&numOfPar);

   int lin = 3;

   if ( numOfPar > 0 ) {
      char par_type[MAXRECL];
      char par_name[MAXRECL];
      char par_default[MAXRECL];
      sscanf(lines[lin],"%s",par_type);
      if ( !strcmp(par_type,"STRING") ||
           !strcmp(par_type,"INT") ||
           !strcmp(par_type,"FLOAT")      ) { // New format incl the types
         for ( int ip = 0; ip < numOfPar; ip++) {
             sscanf(lines[lin],"%s",par_type); 
             lin++;
             sscanf(lines[lin],"%s",par_name); 
             lin++;
             sscanf(lines[lin],"%s",par_default);
             lin++;
             if ( !strcmp(par_default,"&nodefault") ) {
                _declaredParms.add(par_name,par_type);
             }
             else {
                _declaredParms.add(par_name,par_default,par_type);
             }
         }
      }
      else {  //old type without the type info....all parameters are strings
         strcpy(par_type,"STRING");
         for ( int ip = 0; ip < numOfPar; ip++) {
             sscanf(lines[lin],"%s",par_name); 
             lin++;
             sscanf(lines[lin],"%s",par_default);
             lin++;
             if ( !strcmp(par_default,"&nodefault") ) {
                _declaredParms.add(par_name,par_type);
             }
             else {
                _declaredParms.add(par_name,par_default,par_type);
             }
         }
      }
#ifdef DEBUG
      cout << " Action " << _actionName << "  declared parameters :" << endl;
      _declaredParms.out();
      cout.flush();
#endif
   }
   

  NoOfBlocks = 0;

//-----------  First loop over the action lines and instantiate all blocks ----
  for (int ii=lin;;ii++){
    if (!strcmp(lines[ii],"*END_ACTION")) {
       break;
    }
    else if (!strcmp(lines[ii],"*BLOCK")) {
// Now read two integers, making sure they are interpreted as decimal
      ii++;
      int curr_block,curr_level;
      sscanf(lines[ii],"%d %d",&curr_block,&curr_level);
//      cout << " Block :  " << curr_block << " level " << curr_level
//           << endl;
      assert ( curr_block == NoOfBlocks );
      char tmpobjname[MAXOBJECTNAME];      //temporary fix untill InsList
//                                           takes Names
      _objName.whatAreYou(tmpobjname,MAXOBJECTNAME);
      pblocks[curr_block] = new InsList
          (curr_block,curr_level,tmpobjname,_pParentObject,_pParentState,this);
      assert( pblocks[curr_block] != 0 );
      assert ( NoOfBlocks < maxInsBlocks );
      NoOfBlocks++;
    }
  }
   


   if ( NoOfBlocks == 0 ) {
      _pblock = 0;
   }
   else {
      _pblock = pblocks[0];
   }


//---------------------- Loop over the action lines ---------------------------

  int blkno;
  blkno = 0;

  for (int il=lin;;il++){
    if (!strcmp(lines[il],"*END_ACTION")) {
       *no_lines = il+1;
       break;
    }
    else if (!strcmp(lines[il],"*BLOCK")) {
      int no_blklines;
      pblocks[blkno]->initialise(&lines[il],pblocks,&no_blklines);
      blkno++;
      il = il + no_blklines -1;
    }
    else {
      cout << " Action::Action  funny line encountered \n"
           << "|" << lines[il] << "|\n";
      cout.flush();
      exit(1);
    }      
  }
//------------------------------------------------------------------------


  return;
}  


void Action::whatAreYou() {
  cout << "\n";
  cout << "    action : " << _actionName << "\n \n";
  cout.flush();

  if ( _pblock == 0 ) {return;}

  _pblock->listInstructions();
  return;
}



//--------------------------------  execute  -----------------------------------
int Action::execute(char* pstate, const Name& actionstr) {
//

#ifdef DEBUG
  cout << "\n" << " Executing action : " << _actionName << "\n \n";

  Parms temppar;
  temppar.initFromParmString(actionstr);


  if ( temppar.numOfEntries() > 0 ) {
     cout << " Incomming action Parameters :\n";
     temppar.out();
  }
  cout << "\n";
  cout.flush();
#endif
//


  if ( _declaredParms.numOfEntries() > 0 ) {
#ifdef DEBUG
     cout << " Declared Parameters:" << endl;
     _declaredParms.out();
     cout << "\n";
     cout.flush();
#endif
     _currentParameters = _declaredParms;
     _currentParameters.setFromParmString(actionstr);
     if ( !_currentParameters.allHaveValues() ) {
        cout << " There are parameters without a default value and not supplied"
             << endl;
        cout << "\n";
        cout.flush();
     }
#ifdef DEBUG
     cout << " Current Parameters : " << endl;
     _currentParameters.out();
     cout << "\n";
     cout.flush();
#endif
  }

// temp bodge

   if ( _objName == "&ALLOC" ) 
   { _currentParameters.initFromParmString(actionstr);}

//-----------------------------------------

  int temp;
  char state[MAXNAMESIZE];
  if (_pblock == 0) { return 0;}   // some crazy guys have no instructions
  temp = _pblock->execute(state);
  strcpy(pstate,state);

  return temp;
}


//------------------------------- pCurrentParameters -------------------------
//

Parms* Action::pCurrentParameters()  {
   return &_currentParameters;
}
//------------------------------- pDeclaredParameters -------------------------
//

Parms* Action::pDeclaredParms() {
   return &_declaredParms;
}

//---------------------------- name ----------------------------------------
void Action::name ( char* name, int* numParam) {
   _actionName.whatAreYou(name,MAXNAMESIZE);
//
   *numParam = _declaredParms.numOfEntries();
//
   return;
}

//---------------------------- actionString -------------------------------
//  action/par1/par2=/par3
//
void  Action::actionString (Name& actionstr) {
//
   actionstr = _actionName;

   if ( _declaredParms.numOfEntries() > 0 ) {
      Name parmString;
      _declaredParms.buildParmString(parmString);
      actionstr += parmString;
   }
//
   return;
}
