//-------------------------- CommHandler Class ----------------------------------
#define SMI
#include <iostream.h>
#include <assert.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>

#ifndef WIN32
#ifdef __VMS /* VMS doesn't take includes relative to the source directory! */
#include "../../smixx_version.h"
#else
#include "../../smixx_version.h"
#endif
#else
#include "../smixx_version.h"
#endif
#include "commhandler.hh"

//#define COMMS_DEBUG 1

char *CommHandler::domainName;
Queue_TwoNames* CommHandler::pStateQ;
Queue_TwoNames* CommHandler::pExternalActionQ;
PtrVector CommHandler::itsObjs;

void CommHandler::printDateTime()
{
    time_t t;
	char *s;

    t = time((time_t *)0);
    s = ctime(&t);
	*(char *)strchr(s,'\n') = '\0';
    cout << "COMMS - " << s << " - ";
}

void CommObj::serviceHandler() 
{
	DimService *curr = getService();

	if(curr == dimState)
	{
		curr->setData(currState.getString());
	}
	else if(curr == dimActions)
	{
		curr->setData(currActions.getString());
	}
	else if(curr == dimBusy)
	{
		curr->setData(currBusy,strlen(currBusy->action)+1+sizeof(int));
	}
}

void CommObj::create()
{
	char servname[MAX_NAME];
	char tmpstr[MAX_NAME];

	if(created)
		return;
	created = 1;
	strcpy(tmpstr,"SMI/");
	strcat(tmpstr,CommHandler::domainName);
	strcat(tmpstr,"/");
	if(!(topName == itsName))
	{
		strcat(tmpstr,topName.getString());
		strcat(tmpstr,".");
	}	        
	strcat(tmpstr,itsName.getString());
	strcpy(servname,tmpstr);

	dimState = new DimService(servname,"C",this);

	strcat(tmpstr,"/ACTIONS&PARS");
	dimActions = new DimService(tmpstr,"C",this);

	strcpy(tmpstr,servname);
	strcat(tmpstr,"/BUSY");
	dimBusy = new DimService(tmpstr,"L:1;C",this);

	strcpy(tmpstr,servname);
	strcat(tmpstr,"/CMD");
	dimCmd = new DimCommand(tmpstr,"C",this);

}

Name &CommObj::findState(State *pState)
{
	currState = pState->stateName();
	int n_actions = pState->numActions();
	currActions = "";
	for(int i = 0; i< n_actions; i++)
	{
		currActions += pState->actionString(i);
		if(i != n_actions-1 )
			currActions += "|";
	}
	return currState;
}

void CommObj::setBusy(char *cmnd)
{
	if(	currBusy->busy == 1)
	{
//		if(!strcmp(currBusy->action,cmnd))
		return;
	}
	delete currBusy;
	currBusy = (BUSY *)new char[strlen(cmnd)+1+sizeof(int)];
	currBusy->busy = 1;
	strcpy(currBusy->action,cmnd);
	updateBusy();
}

void CommObj::clearBusy()
{
	if(currBusy->busy)
	{
		delete currBusy;
		currBusy = (BUSY *)new char[1+sizeof(int)];
		currBusy->busy = 0;
		currBusy->action[0] = '\0';
	}
	updateBusy();
}

void CommObj::releaseDomain()
{
	int conn_id;
	static char *cmndptr = 0;

#ifdef COMMS_DEBUG
	CommHandler::printDateTime();
	cout << itsName << " executing : "<< "RELEASE" << "\n";
	cout.flush();
#endif
	if(!cmndptr)
		cmndptr = new char[MAX_NAME+8];
	if(conn_id = DimServer::getClientId());
	{
		sprintf(cmndptr,"RELEASE/&ID(S)=%s/&CONN_ID(I)=%d",
			DimServer::getClientName(),conn_id); 
	}
	TwoNames  actionInfo;
	if(!(topName == itsName))
		actionInfo.set(topName.getString(),cmndptr);
	else
		actionInfo.set(itsName.getString(),cmndptr);
	(CommHandler::pExternalActionQ)->add(actionInfo);
#ifdef __VMS
	sys$wake(0,0);
#endif
#ifdef WIN32
	wake_up();
#endif

}

void CommObj::commandHandler()
{
	char name[MAX_NAME];
	int conn_id;
	static char *cmndptr = 0;
	static int cmndsize = 0;
	
	DimCommand *curr = getCommand();
	setBusy(curr->getString());
#ifdef COMMS_DEBUG
	CommHandler::printDateTime();
	cout << itsName << " executing : "<< curr->getString() << "\n";
	cout.flush();
#endif
	int size = curr->getSize();
	if(cmndsize < ( size + MAX_NAME))
	{
		cmndsize = size + MAX_NAME;
		if(cmndptr)
			delete cmndptr;
		cmndptr = new char[cmndsize];
	}
	strcpy(cmndptr,curr->getString());
//	if(conn_id = dis_get_client(name))
	if(conn_id = DimServer::getClientId());
	{
		sprintf(&cmndptr[size-1],"/&ID(S)=%s/&CONN_ID(I)=%d",
			DimServer::getClientName(),conn_id); 
	}
	TwoNames  actionInfo;
	if(!(topName == itsName))
		actionInfo.set(topName.getString(),cmndptr);
	else
		actionInfo.set(itsName.getString(),cmndptr);
	(CommHandler::pExternalActionQ)->add(actionInfo);
#ifdef __VMS
	sys$wake(0,0);
#endif
#ifdef WIN32
	wake_up();
#endif

}

Obj::Obj(Name &name, SMIObject *pobj, int nsubobjs) : CommObj(name, name)
{
	char tmpname[MAX_NAME]; 
	CommObj *subptr;
	int i;

	pObj = pobj;
	itsNSubObjs = nsubobjs;
	if(itsNSubObjs)
	{
		getDimName(tmpname);
		itsProxy = new DimInfo(tmpname,"$DEAD",this); 
	}
	if(itsNSubObjs > 1)
	{
		for(i = 0; i < itsNSubObjs; i++)
		{
			Name subname = pObj->subobjectName(i);
			subptr = new CommObj(subname, name);
			itsSubObjs += subptr;
		}
	}
}

int Obj::findState()
{
	int i;
	State *pState;
	
	if(itsNSubObjs > 1)
	{
		currState = "";
		for(i = 0; i < itsNSubObjs; i++)
		{
			pState = pObj->pCurrentState(i);
			currState += ((CommObj *)itsSubObjs[i])->getName();
			currState += ".";
			currState += ((CommObj *)itsSubObjs[i])->findState(pState);
			if(i != itsNSubObjs - 1)
				currState += ",";
		}
	}
	else
	{
		pState = pObj->pCurrentState(0);
		CommObj::findState(pState);
	}
	currState += pObj->parmString();
	return 1;
}

void Obj::updateState()
{
	int i;
	
	if(itsNSubObjs > 1)
	{
		for(i = 0; i < itsNSubObjs; i++)
		{
			((CommObj *)itsSubObjs[i])->updateState();
			((CommObj *)itsSubObjs[i])->updateActions();
			((CommObj *)itsSubObjs[i])->clearBusy();
		}
	}
	CommObj::updateState();
	CommObj::updateActions();
	CommObj::clearBusy();
}

void Obj::clearBusy()
{
	int i;
	
	if(itsNSubObjs > 1)
	{
		for(i = 0; i < itsNSubObjs; i++)
		{
			((CommObj *)itsSubObjs[i])->clearBusy();
		}
	}
	CommObj::clearBusy();
}

void Obj::setBusy(char *cmnd)
{
	int i;
	
	if(itsNSubObjs > 1)
	{
		for(i = 0; i < itsNSubObjs; i++)
		{
			((CommObj *)itsSubObjs[i])->setBusy(cmnd);
		}
	}
	CommObj::setBusy(cmnd);
}

void Obj::create()
{
	int i;
	
	if(itsNSubObjs > 1)
	{
		for(i = 0; i < itsNSubObjs; i++)
		{
			((CommObj *)itsSubObjs[i])->create();
		}
	}
	CommObj::create();
}

void Obj::sendCommand(char *cmnd)
{
	int i;
	char servname[MAX_NAME];
	
	setBusy(cmnd);
#ifdef COMMS_DEBUG
	CommHandler::printDateTime();
	cout << itsName << " executing : "<< cmnd << "\n";
	cout.flush();
#endif
	getDimName(servname);
	strcat(servname,"/CMD");
	DimClient::sendCommand(servname,cmnd);
}

void Obj::getDimName(char *servname)
{
	char tmpstr[MAX_NAME];
	int index;

	if ((index = itsName.existsAt("::")) != -1)
	{
		strcpy(servname,"SMI/");
		strcpy(tmpstr,itsName.getString());
		tmpstr[index] = '\0';
		strcat(servname,tmpstr);
		strcat(servname,"/");
		strcpy(tmpstr,&itsName[index+2]);
		strcat(servname,tmpstr);
	}
	else
	{
		strcpy(servname,"EP/");
		strcat(servname,CommHandler::domainName);
		strcat(servname,"/");
		strcat(servname,itsName.getString());
	}
}

void Obj::infoHandler()
{
	TwoNames  stateInfo;
	DimInfo *proxy;
	char *data;

	proxy = getInfo();
	data = (char *)proxy->getData();
	stateInfo.set(this->itsName.getString(), data);
	(CommHandler::pStateQ)->add(stateInfo);
/*
	if((this->currState == "$DEAD") || 
	   (!strncmp(this->currState.getString(),"DEAD",4)))
*/
	if((this->currDimState == "$DEAD") && 
	   (strcmp(data,"$DEAD")))
	{
		char servname[MAX_NAME];
		strcpy(servname,CommHandler::domainName);
		strcat(servname,"::");
		strcat(servname,this->itsName.getString());
		DimClient::setExitHandler(servname);
	} 
	this->currDimState = data;
#ifdef __VMS
	sys$wake(0,0);
#endif
#ifdef WIN32
	wake_up();
#endif

}

CommHandler::CommHandler(char* domainname, Queue_TwoNames* ipStateQ,
	Queue_TwoNames* ipExternalActionQ) 
{ 
				 
	domainName = domainname;
	pStateQ = ipStateQ;
	pExternalActionQ = ipExternalActionQ;
	nObjs = 0;
	started = 0;
}

void CommHandler::declareObj(SMIObject *pObj)
{
	Obj *objptr;
	char name[MAX_NAME];
	int nsubobjs;

	Name tmpname = pObj->name();
	if(findObjectByName(tmpname.getString()))
	{
		cout << "CommHandler: Object Already Declared - " << tmpname << "\n";
		cout.flush();
		return;
	}
	nsubobjs = pObj->numSubobj();
	objptr = new Obj(tmpname, pObj, nsubobjs);
	if(nsubobjs > 1)
		nObjs += nsubobjs;
	else
		nObjs++;
	itsObjs += objptr;
#ifdef COMMS_DEBUG
	CommHandler::printDateTime();
	cout << objptr->getName() << " in state : "<< objptr->getState() << "\n";
	cout.flush();
#endif
}

Obj *CommHandler::findObject(SMIObject *pobj)
{
	int i;
	Obj *objptr;
	
	for(i = 0; i < itsObjs.length(); i++)
	{
		objptr = (Obj *)itsObjs[i];
		if(	objptr->getpObj() == pobj)
		return (Obj *)itsObjs[i];
	}
	return (Obj *)0;
}

Obj *CommHandler::findObjectByName(const char *objname) const
{
	int i;
	Obj *objptr;
	
	for(i = 0; i < itsObjs.length(); i++)
	{
		objptr = (Obj *)itsObjs[i];
		if(	objptr->getName() == objname)
		return (Obj *)itsObjs[i];
	}
	return (Obj *)0;
}

void CommHandler::clientExitHandler()
{
	Obj *objptr;

		CommHandler::printDateTime();
		cout << DimServer::getClientName() << " died, releasing Domain\n";
		cout.flush();
		objptr = (Obj *)CommHandler::itsObjs[0];
		objptr->releaseDomain();
}

void CommHandler::declareObjList()
{
	char tmpstr[MAXNAMESIZE];
	int nsub;
	Obj *objptr;
	int size;
	int index = 0;

	size = nObjs*MAXNAMESIZE;

	itsObjList = (char *)new char[size];
	memset(itsObjList,0,size);

	for(int i = 0; i < itsObjs.length(); i++)
	{
		objptr = (Obj *)itsObjs[i];
		if((nsub = objptr->getNSubObjs()) > 1)
		{
			for(int j=0; j<nsub; j++)
			{
				strcpy(tmpstr,objptr->getName().getString());
				strcat(tmpstr,".");
				strcat(tmpstr,
					objptr->getSubObj(j)->getName().getString());
				strcpy(&itsObjList[(index++)*MAXNAMESIZE], tmpstr); 
			}
		}
		else
		{
			strcpy(tmpstr,objptr->getName().getString());
			strcpy(&itsObjList[(index++)*MAXNAMESIZE], tmpstr); 
		}
	}
	itsVersion = SMIXX_VERSION;
	strcpy(tmpstr,"SMI/");
	strcat(tmpstr,CommHandler::domainName);
	strcat(tmpstr,"/SMI_VERSION_NUMBER");
	version = new DimService(tmpstr,itsVersion);
	strcpy(tmpstr,"SMI/");
	strcat(tmpstr,CommHandler::domainName);
	strcat(tmpstr,"/DOMAIN");
	objList = new DimService(tmpstr,"C",itsObjList, size);
	declareDynObjList();

//	dis_add_client_exit_handler((void (*)(...))client_exit_handler);
}

void CommHandler::declareDynObjList()
{
	int nobjs, nsub;
	Obj *objptr;
	char tmpstr[MAX_NAME], *ptr;;

	itsDynObjList = "";

	nobjs = itsObjs.length();
	for(int i = 0; i < nobjs; i++)
	{
		objptr = (Obj *)itsObjs[i];
		if((nsub = objptr->getNSubObjs()) > 1)
		{
			for(int j=0; j<nsub; j++)
			{
				itsDynObjList += objptr->getName();
				itsDynObjList += ".";
				itsDynObjList += objptr->getSubObj(j)->getName();
				itsDynObjList += "/ASSOCIATED";
				itsDynObjList += "|";
			}
		}
		else
		{
			itsDynObjList += objptr->getName();
			if(nsub == 1)
				itsDynObjList += "/ASSOCIATED";
			itsDynObjList += "|";
		}
	}
	ptr = itsDynObjList.getString();
	ptr[strlen(ptr)-1] = '\0';
	strcpy(tmpstr,"SMI/");
	strcat(tmpstr,CommHandler::domainName);
	strcat(tmpstr,"/OBJECT_LIST");
	dynObjList = new DimService(tmpstr,"C",ptr, strlen(ptr)+1);
}

void CommHandler::publishState(SMIObject *pObj)
{
	Obj *objptr;

	if(objptr = findObject(pObj))
	{
		objptr->findState();
		objptr->updateState();
		if(!started)
		{
			objptr->create();
		}
#ifdef COMMS_DEBUG
		CommHandler::printDateTime();
		cout << objptr->getName() << " in state : "<< objptr->getState() << "\n";
		cout.flush();
#endif
	}
	else
	{
		cout << "Publish State, object not created yet\n";
		cout.flush();
	}
	if(!started)
		startIt();
}

void CommHandler::publishBusy(SMIObject *pObj, char *cmnd)
{
	Obj *objptr;

	if(objptr = findObject(pObj))
	{
		objptr->setBusy(cmnd);
#ifdef COMMS_DEBUG
		CommHandler::printDateTime();
		cout << objptr->getName() << " executing : "<< cmnd << "\n";
		cout.flush();
#endif
	}
}

void CommHandler::startIt()
{
	int i;
	Obj *objptr;
	int missing = 0;
	char tmpstr[MAXNAMESIZE];
	
	for(i = 0; i < itsObjs.length(); i++)
	{
		objptr = (Obj *)itsObjs[i];
		if(!objptr->created)
		{
			missing = 1;
			break;
		}
	}
	if(!missing)
	{
		strcpy(tmpstr,CommHandler::domainName);
		strcat(tmpstr,"_SMI");
		DimServer::start(tmpstr);
		DimServer::addClientExitHandler(this);
		started = 1;
	}
}

void CommHandler::sendCommand( const char *extobj, char *cmnd) const
{
	Obj *objptr;

	if(objptr = findObjectByName(extobj))
	{
		objptr->sendCommand(cmnd);
	}
}	

void CommHandler::invalidAction(SMIObject *pObj)
{
	Obj *objptr;

	if(objptr = findObject(pObj))
	{
		objptr->clearBusy();
#ifdef COMMS_DEBUG
		CommHandler::printDateTime();
		cout << objptr->getName() << " Invalid action\n";
		cout.flush();
#endif
	}
}


void CommHandler::domainAllocated(int conn_id)
{
	DimServer::setClientExitHandler(conn_id);
//	dis_set_client_exit_handler(conn_id,2);
}

void CommHandler::domainDeallocated(int conn_id)
{
	DimServer::clearClientExitHandler(conn_id);
//	dis_set_client_exit_handler(conn_id,0);
}


