#ifndef commhandler_HH
#define commhandler_HH

#ifndef solaris
#ifdef sun
#define solaris
#endif
#endif
#include "name.hh"
#include "twonames.hh"
#include "queue_twonames.hh"
#include "ptrvector.hh"
class CommHandler;
#include "smiobject.hh"
#include "state.hh"
#include <dic.hh>
#include <dis.hh>

#define MAX_NAME 132
#define MAX_BUFFER 1024

typedef struct{
	int busy;
	char action[1];
}BUSY; 

class CommObj : public DimServer
{
protected:
	Name topName;
	Name itsName;
	Name currState;
	Name currDimState;
	Name currActions;
	BUSY   *currBusy;
	DimService *dimState;
	DimService *dimActions;
	DimService *dimBusy;
	DimCommand *dimCmd;
	void serviceHandler();
public :
	CommObj(Name &name, Name &topname) 
	{	
		itsName = name; 
		topName = topname; 
		currState = "$DEAD";
		currDimState = "$DEAD";
		currActions = "";
		dimState = 0; dimActions = 0; dimBusy = 0;
		currBusy = new BUSY;
		currBusy->busy = 0;
		currBusy->action[0] = '\0';
		created = 0;
//		create();
	};
    int created;
	void create();
	Name &getName() {return itsName;};
	Name &getState() {return currState;};
	Name &findState(State *pstate);
	void setBusy(char *cmnd);
	void clearBusy();
	void updateState() 
		{if(dimState) /*create(); else*/ dimState->updateService();};
	void updateActions() 
		{if(dimActions) /*create(); else*/ dimActions->updateService();};
	void updateBusy() 
		{if(dimBusy) /*create(); else*/ dimBusy->updateService();};
	void commandHandler();
	void releaseDomain();
};

class Obj : public CommObj, public DimClient
{
	SMIObject *pObj;
	int itsNSubObjs;
	PtrVector itsSubObjs;
	DimInfo *itsProxy;
	void getDimName(char *servname);
	void infoHandler();
public :
	Obj(Name &name, SMIObject *pobj, int nsubobjs);
	void create();
	int findState();
	int getNSubObjs() {return itsNSubObjs;};
	CommObj *getSubObj(int index) {return (CommObj *)itsSubObjs[index];};
	SMIObject *getpObj() {return pObj;};
	void updateState();
	void setBusy(char *cmnd);
	void clearBusy();
	void sendCommand(char *cmnd);
};

class CommHandler : public DimServer
{
public :
	CommHandler(char* domainname, Queue_TwoNames* ipStateQ, 
		Queue_TwoNames* ipExternalActionQ);
	void startIt();
	void publishState(SMIObject *pObj);
	void publishBusy(SMIObject *pObj, char *command);
	void invalidAction(SMIObject *pObj);
	void sendCommand( const char* object_nm, char* command) const;
	void declareObj(SMIObject *pObj);
	void declareObjList();
	void domainAllocated(int conn_id);
	void domainDeallocated(int conn_id);
	void clientExitHandler();

	static char *domainName;
	static Queue_TwoNames* pStateQ;
	static Queue_TwoNames* pExternalActionQ;
	static PtrVector itsObjs;
	static void printDateTime();
private :
/* class vars */
	char *itsObjList;
	Name itsDynObjList;
	int itsVersion;
	DimService *objList, *dynObjList, *version;
	int nObjs;
	Obj *findObject(SMIObject *pobj);
	Obj *findObjectByName(const char *objname) const;
	void declareDynObjList();
	int started;
};

#endif

