//--------------------------  Scheduler Class  -------------------------------
#include <iostream.h>
#include <assert.h>
#include "utilities.hh"
//----------- Externals ------------------
#include "queue_name.hh"
   extern Queue_Name executableObjectQ;
#include "objectregistrar.hh"
   extern ObjectRegistrar allSMIObjects;
#include "ifhandler.hh"
   extern IfHandler ifHandler;
   extern Name allocator;
//---------------------------------------
#include "scheduler.hh"
#include "smiobject.hh"
#include "queue_twonames.hh"
//                                                         8-November-1996
//                                                          B. Franek
//---------------------------------------------------------------------------
//
Scheduler::Scheduler( 
                      CommHandler *pCommHandler,
                      Queue_TwoNames *pStateQ,
                      Queue_TwoNames *pExternalActionQ ) {
    _pCommHandler = pCommHandler;
    _pStateQ = pStateQ;
    _pExternalActionQ = pExternalActionQ;
}


void Scheduler::initialise(){
//------  First startup all the objects -----------------------

#ifdef DEBUG
  cout << " ------------------------------  Start up  "
       << "------------------------------\n";
  cout.flush();
#endif

  int numOfObjects = allSMIObjects.numOfObjects();


  for (int io=0; io < numOfObjects; io++) {
     SMIObject *pSMIObj;
     pSMIObj = allSMIObjects.gimePointer(io);
//     pSMIObj->whatAreYou();
	 _pCommHandler->declareObj(pSMIObj);
     pSMIObj->startUp(_pStateQ,_pCommHandler);
  }
  _pCommHandler->declareObjList();

#ifdef DEBUG
  cout << " ---------------------------  End of Start up  "
       << "---------------------------\n\n";
  cout.flush();
#endif
  return;
//-----------------------------------------------------------
}

void Scheduler::execute(){

while   ( !_pStateQ->isEmpty()          || 
          !_pExternalActionQ->isEmpty() ||
          !executableObjectQ.isEmpty()
        ) {

     for (; ;) {
         if( _pStateQ->isEmpty() ) {break;}
         TwoNames stateInfo;
         stateInfo = _pStateQ->remove();
         char object[MAXOBJECTNAME];
         Name stateNm,objectNm;
         stateInfo.gimeNames(objectNm,stateNm);
         objectNm.whatAreYou(object,MAXOBJECTNAME);
#ifdef DEBUG
         print_obj(object);
         cout << " StateQ ->state : " << stateNm;
         cout << "\n";
         cout.flush();
#endif
         SMIObject *pSMIObj;
         pSMIObj = allSMIObjects.gimePointer(object);
         pSMIObj->setState(stateNm);
	     _pCommHandler->publishState(pSMIObj);
//             char tempstr[120];
//             pSMIObj->parmString(tempstr,120);
//             cout << tempstr << endl;
//             cout.flush();
         handleWhens(object);
     }

     for (; ;) {
         if( _pExternalActionQ->isEmpty() ) {break;}
         TwoNames actionInfo;
         actionInfo = _pExternalActionQ->remove();
         Name actionstring, objectstring;
         actionInfo.gimeNames(objectstring,actionstring);
         cout << endl;
         print_obj(objectstring);
         cout << "  External action :" << actionstring << "\n"; 
         cout.flush();
         SMIObject *pSMIObj;
         pSMIObj = allSMIObjects.gimePointer(objectstring);
         if ( guardianOK(objectstring,actionstring) ) {
            pSMIObj->queueAction(actionstring);
         }
         else {
             cout << " Action rejected " << endl; cout.flush();
             _pCommHandler->invalidAction(pSMIObj); 
        }
     }

     for (; ;) {
         if( executableObjectQ.isEmpty() ) {break;}
//         cout << " Executab object queue \n";
//         executableObjectQ.out();
         char object[MAXOBJECTNAME];
         executableObjectQ.remove(object,MAXOBJECTNAME);
#ifdef DEBUG
         print_obj(object);
         cout << " is executable \n"; 
         cout.flush();
#endif
         SMIObject *pSMIObj;
         pSMIObj = allSMIObjects.gimePointer(object);
         pSMIObj->execute();
         ifHandler.execute();
     }

}
     return;
}

//------------------------------- handleWhens ----------------------------------

void Scheduler::handleWhens( char *objName ) {


  int numOfObjects = allSMIObjects.numOfObjects();


  for (int io=0; io < numOfObjects; io++) {
     SMIObject *pSMIObj;
     pSMIObj = allSMIObjects.gimePointer(io);
     pSMIObj->executeWhens(objName); // objName is the object that changed st.
  }
//
  return;

}
//----------------------------------------------------------------------------
  int Scheduler::guardianOK(const Name& objectName, Name& actionstring) {

     Name actionnm;
     actionstring.element(0,'/',actionnm);
     Parms pars;
     pars.initFromParmString(actionstring);

     cout << " guardian:  Object " << objectName 
          << " Action string: " << actionstring << endl; cout.flush();

     Name nameAmpersId("&ID"), valAmpersId("\0");
     Name nameAmpersConnId("&CONN_ID"),valAmpersConnId("\0");
     char type[7];
     Name doubleq("\""),nullstr("\0");

     int numPars = pars.numOfEntries();
     int ampersIdPresent = pars.get(nameAmpersId,valAmpersId,type);
     int ampersConnIdPresent = pars.get(nameAmpersConnId,valAmpersConnId,type);

     if (ampersIdPresent) {valAmpersId.replace(doubleq,nullstr);}

     cout << " Value of &ID : " << valAmpersId << endl; cout.flush();
     cout << " ampersIdPresent : " << ampersIdPresent << endl; cout.flush();

     if ( allocator == "\0" ) { } // domain not allocated
     else {  // it is allocated
         cout << " Domain allocated to : " << allocator << endl; cout.flush();
         if ( ampersIdPresent ) { // &ID present
            if ( allocator == valAmpersId ) {}  //this is OK
            else { return 0;} //down the drainpipe
         }
         else { return 0;} //down the drainpipe
     }

// Now just tydiing up

     if ( objectName == "&ALLOC" ) {
        if ( !ampersIdPresent || !ampersConnIdPresent ) {
           cout << " Actions to &ALLOC must have &ID and &CONN_ID" << endl;
           cout.flush();
           return 0;  // down the drainpipe
        }
     }
     else { // for all other objects remove &ID and &CONN_ID
         Parms temp;

         if ( ampersIdPresent ) {
            pars.remove(nameAmpersId,temp);
            pars = temp;
         }

         if ( ampersConnIdPresent ) {
            pars.remove(nameAmpersConnId,temp);
            pars = temp;
         }

         Name parString;
         pars.buildParmString(parString);
         actionstring = actionnm;
         actionstring += parString;
     }
     return 1;
  }
