
//---------------------- SMIObject   Class ---------------------------
#ifndef SMIOBJECT_HH
#define SMIOBJECT_HH
#include "parameters.hh"
#include "objectregistrar.hh"
#include "smiclassregistrar.hh"
#include "smiclass.hh"

class State;
#include "state.hh"

#include "commhandler.hh"
#include "twonames.hh"
#include "queue_twonames.hh"

#include "queue_name.hh"
#include "parms.hh"
//                                                          19-August-1996
//                                                            B. Franek
//--------------------------------------------------------------------------


class SMIObject {
public :
  SMIObject
  ( const SMIClassRegistrar* const pnt2,char lines[][MAXRECL]);

  void whatAreYou();

  void startUp( Queue_TwoNames* pStateQ, CommHandler *ptnr);

  void setState( const Name& stateString);

  void execute();

  void resume();

  void executeWhens(char* objName);

  void queueAction(Name& actionstr);

  void queueAction(char* actionstr);

  void queueHpAction(Name& action_nm);

  void queueHpAction(char* action_nm);

  int lock(char* objname); // objname ... the name of the object requesting lock   
//              ret 1- locked; 0- not locked but lock put on the action queue

  void unlock();

  Name name() const;

  int numSubobj() const;

  bool associated() const;

  Name externDomain() const;

  int name(char* obj_nm, int *numSubobj);	 //returns associated flag
                                                 //can probably be made obsolete
  Name subobjectName(const int index) const;

  void subobjectName(int index, char* name);     //can be made obsolete?

  Name currentState() const;

  void currentState(int subobjinx,  State** pState);

  State* pCurrentState( const int subobjinx ) const ;

  int  currentState(char* subobjnm, char* statenm); // returns -1 if object busy
//                                                    otherwise 1

  Name parmString() const;

  void parmString(Name& str) const;

  void parmString(char* str, int mxsize) const;

  Parms* pObjectParameters();

  State* pTaggedState( const Name& tagname) const ;  

private :
//--------------------------  private methods ----------------------------------
  void bodyLog( char lines[][MAXRECL]);

  void executeState();

  void split( const Name& statestr, Name& stateStr, Name& paramStr) const;

  int stateInx( const int isub, const Name& state ) const ;

  int stateInx( const Name& stname) const ;  // returns -1 if not found

  int stateInx( const char stname[]) const ;  // returns -1 if not found

  int stateInxDeadState() const;

  int stateInxDeadState(const int isub) const ;

  int stateInxInitialState(const int isub) const ;

  int stateInxTaggedState() const;

  void setStateAlloc( const Name& statestr);

  void setCurrStateInxes( const Name& stateNm ) ;
//------------------------------------
  Name _objectName;


  int _classFlag;                 // =1 indicates the object belongs to a class 
  char _className[MAXNAMESIZE];


  int _numOfAtributes;
  Name _atribute[MAXATRIBUTES];


  int _numOfStates;

  State *_pState[MAXSTATES];         // Pointers to states


  int _associated;            // =1 indicates that the object is associated

// When object is associated, it has at least 1 subobject (&dummy id 1)

  int _numOfSubobjects;

  Name _subobject[MAXSUBOBJECTS]; // List of subobjects
  int  _stateIndex[MAXSUBOBJECTS]; // list of indexes to _state array
  int  _numOfSubobjStates[MAXSUBOBJECTS];
//
// E.g. _state[_stateIndex[i]] is the 0th state of (i+1)st subobject

//------------------------ Dynamical part -------------------------------------

   Queue_TwoNames* _pStateQ;
   CommHandler * _pCommHandler;

   int _currStateInx;  // Index to the current state

   int _currStateInxes[MAXSUBOBJECTS]; // Same thing for subobjects

   char _internalState[32];

   Queue_Name _actionQ,_hpActionQ;        // Action & High priority action queue

   int _lock;

   Name _currentActionString;

   Parms _objectParameters;
};
#endif
