//----------------------------------------------------------------------------
//
//                                                    B. Franek  January 1998
//----------------------------------------------------------------------------
#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include "name.hh"

   void strSqueeze(char str[]){
      int iout(0);

      for (int i=0;;i++) {
         if (str[i] == '\0') { break;}
         if ( (str[i] != ' ') && (str[i] != '\t') ) {
             str[iout] = str[i];
             iout++;
         }         
      }
      str[iout] = '\0';
   }
//-----------------------------------------------------------------------

   void strTrim(char str[]){

      int ln = strlen(str);

      if ( ln <= 0 ) { return; }

      for ( int i = (ln-1); i>=0 ; i--) {
         if ( (str[i] != ' ') && (str[i] != '\t') ) { str[i+1] = '\0'; return; } 
      }
      str[0] = '\0';
      return;
   }

//--------------------------------------------------------------------------

   void strUpcase(char str[]){

      int ln = strlen(str);

      if ( ln <= 0 ) { return; }

      for ( int i=0; i<ln; i++) {
          str[i] = toupper(str[i]);
      }

      return;
   }

//---------------------------------------------------------------------------

   void process_line(char str[]){

      char TAB = '\t';
      int ln = strlen(str);

      if ( ln <= 0 ) { return; }

      int iout = 0;
      int mode = 0;  //0..beginning; 1..normal; 2..after blank; 3...after "

      for ( int i=0; i<ln; i++) {
          if ( str[i]==TAB ) {str[i] = ' ';}
          if (mode==0) {
             if ( str[i] == ' ' ) { continue;} // ignore blanks at the begining
             mode = 1;
             i--; // consider the character again
          }
          else if (mode==1) {
             if ( str[i] == '!' ) { str[iout] = '\0'; return; }
             if ( str[i] == ' ' ) {    // First blank found
                if ( str[iout-1] == ':' || 
                     str[iout-1] == '(' ||
                     str[iout-1] == ')' ||
                     str[iout-1] == '/' ||
                     str[iout-1] == '='   ) { continue; }
                mode = 2;
             }    
             if ( str[i] == '"' ) {  // First " found
                mode = 3;
             }
             str[iout] = toupper(str[i]);
             iout++;
          }
          else if (mode==2) {
             if ( str[i] == '!' ) { str[iout-1] = '\0'; return; }
             if ( str[i] == ' ' ) { continue; }
             if ( str[i] == '"' ) { mode = 3;}
             else { mode = 1;}
             if ( str[i] == ':' ||
                  str[i] == '(' ||
                  str[i] == ')' ||
                  str[i] == '/' ||
                  str[i] == '='   ) { iout--; }
             str[iout] = toupper(str[i]);
             iout++;
          }
          else if (mode==3) {
             str[iout] = toupper(str[i]);
             iout++;
             if ( str[i] == '"' ) { mode = 1; }
          }
          else {
             cout << " Internal error, illegal mode = " << mode << endl;
             abort();
          }
      }   

      if ( mode == 0 ) { str[0] = '\0'; }
      if ( mode == 1 ) { str[iout] = '\0'; return; }
      if ( mode == 2 ) { str[iout-1] = '\0'; }
      if ( mode == 3 ) {
         cout << "  Illegal number of double quotes on the line" << endl;
         cout.flush();
         abort();
      }
   }
//-------------------------------------------------------------------------------
   int readLine(ifstream& file, Name& line) {
       enum { bufLen = 80 };
       char buf[bufLen];
 
       line ="\0";

       for (;;) {
           file.getline(buf,bufLen);
          
           if (file.eof() ) {
              file.close();
              return 2;
           }
           else if (file.bad()) {
              return 0;
           }
           line += buf;
           if (strlen(buf) < (bufLen-1)) {break;}
       }
       return 1;
   }

char* getToken(char str[], char delim, Name& token ) {

    if (strlen(str) == 0 ) { token = "\0"; return 0;}

    char *pend,*ptokbeg,*ptokend,*pnext;

    pend = str+strlen(str);

    if ( delim == ' ' ) { 
       ptokbeg = strtok(str," ");
       assert (ptokbeg != 0);
       token = ptokbeg;
       ptokend = ptokbeg+strlen(ptokbeg);
       pnext = ptokend+1;
       if ( ptokend < pend ) { *ptokend = ' ';}
       if ( pnext < pend ) { return pnext; }
       return 0;
    }
    else {

       char* pdel = strstr(str,&delim);

       if (pdel) { *pdel = '\0'; }

       char* pendd = str+strlen(str);

        ptokbeg = strtok(str," ");
        assert (ptokbeg != 0);
        ptokend = ptokbeg+strlen(ptokbeg);
        if ( ptokend < pendd ) {
           *ptokend = ' ';
        }
        token = ptokbeg;
        if (pdel) {*pdel = delim;}
        token.trim();
        if (pdel) { return pdel+1;}
        return 0;
    }
}



bool check_name(const Name& name) {
    char charset[] = "_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
    char ch;

    ch = name[0];

    if ( strchr(charset,ch) ) {
       return 1;
    }
    else {
       return 0;
    }
}

bool check_int(const Name& name) {
     int tempi;
     char ch;

     if ( sscanf(&name[0],"%d %c",&tempi,&ch) == 1) { return 1; }     
     return 0;
}

bool check_float(const Name& name) {
     int tempf;
     char ch;

     if ( sscanf(&name[0],"%e %c",&tempf,&ch) == 1) { return 1; }     
     return 0;
}

bool check_string(const Name& name) {

     if ( name[0] == '\"' ) { return 1; }//anything beginning with " is string
     return 0;          
}
