/* 
 * Written by    : tome.anticic@cern.ch
 */
#include <stdio.h>
#include <dic.h>
#include <dis.h>
#include <signal.h>

#include <stdlib.h>
#include <unistd.h>
#include <smirtl.h>

#include "CollectorDATE.h"
#include "hostInfo.h"
#include "collectorlib.h"

#include "event.h"
#include "rcShm.h"
#include "dateDb.h"
#include "banksManager.h"
#include "dbConfig.h"


#include <ctype.h>
#include <mysql.h>


int startFlag = 0;


void SMI_handle_command() {
   char action[64], param[64];
   int n_params;
   smi_get_action(action,&n_params);
   if (n_params >=1) {
      smi_get_par_value("PARAM",param);
   }
   else {
      strcpy(param,"");
   }
   if (strcmp(action,"STOP") == 0) {
       hostInfo.statusFlag = 0;
       // dis_update_service(kStartCollecting);
       printf("Affair: SMI exit on machine %s monitoring %s \n",hname,hostInfo.monset);
       exit (0);
   }
   smi_set_state("RUNNING");
}


////////////////////////////////////////////////////////////////////////////////////////////////
void startCollecting(tag, buf, size)
    int *tag, *size;
    char *buf;
{
    int ncols;
    char buffer[256];
    char dummy[256];
    int timeDelayTemp;

    char    **argv;
    int     argc;

    int ddlSerial;
    int ddlChannel;
    char ddlName[128];

    int timeIndex        = 3;
    int par1Index        = 5;
    int par2Index        = 6;

    int Mask             = 0xFFFF;
    int idddl,i;
    
    argv = string_to_argv(buf, &argc);
    
    timeDelay = atoi(argv[timeIndex]);    
    if (timeDelay <= 0) {
      printf("Warning.timeDelay <= 0 =%i. setting it to 10 sec \n",timeDelay);
      timeDelay = 10; //consistency
    }    
    //par1 and par2 not really used at all. Placed here for compatibility obnly. Might put to use at a later date
    //also,if LDC, for DDL names, the incoming labels (dummy auto whatever) completely ignored.Fill with DDL names
    if ( strcmp(argv[par1Index],"AUTO") !=0 )     sprintf(sSYSMONPAR.parameter1,"%s",argv[par1Index]);
    if ( strcmp(argv[par2Index],"AUTO") !=0 )     sprintf(sSYSMONPAR.parameter2,"%s",argv[par2Index]);
    sprintf(hostInfo.parameterLine,"%s %s %i  DUMMY %s %s", hname,hostInfo.monset,timeDelay,sSYSMONPAR.parameter1,sSYSMONPAR.parameter2);

    // do the following only for LDC
    if (strcmp(hostInfo.monset,"LDC") == 0) {    
      for (i=0; i < MAXIMUM_DDL_PER_LDC; i++) {
	//convert here to ascii form if idddl with shifting bits
	idddl       =   DAQCONTROL->monitoringDdl[i].idDdl;
	ddlSerial   =   (idddl >> 16) &  Mask;
        ddlChannel  =   idddl &  Mask;
	// printf(" idddl = %i ddlSerial = %i ddlChannel = %i \n",idddl,ddlSerial,ddlChannel);
	sprintf(ddlName,"%i.%i",ddlSerial,ddlChannel); 
	if (ddlSerial == 0 && ddlChannel) sprintf(ddlName,"DUMMY"); 
	sprintf(hostInfo.parameterLine,"%s %s",hostInfo.parameterLine,ddlName);
      }
      if (gDebugLevel) printf(" par line is %s",hostInfo.parameterLine);
    }
    
    sprintf(gTextString,"Received command from AFFAIR_MONITOR=%s\n",buf);
    writeOutput(gTextString);

    // send info to monitor about the parameter line
    dis_update_service(kParameterLine);
    startFlag = 1;    
}

void setup(char * monset)
{
    int errCode;
    // DATE stuff
    /* access to the control region */
    

    if (strcmp(monset,"GDC") == 0) {
	errCode = MAPCONTROLREGION(dbRoleGdc);
    }
    else if (strcmp(monset,"LDC") == 0) {
      errCode = MAPCONTROLREGION(dbRoleLdc);
    }
    if ( errCode <= 0) {
      printf("Error: affair on machine %s could not monitor LDC or GDC. exiting  errCode=%i \n ",hname,errCode);
      exit(0);
    }
    else {
      printf("affair on machine %s monitoring %s \n ",hname,monset);
    }

    sprintf(sSYSMONPAR.parameter1,"%s","DUMMY");
    sprintf(sSYSMONPAR.parameter2,"%s","DUMMY");

}


main(argc,argv)
    int argc;
    char **argv;
{
    char aux[256];
    int i;
    char obj[128];

    int loopCount;
    double kBytesRecordedOld;
    double kBytesInjectedOld; 
    char*  dateRolePtr;
    char*  dateRole[256];
    double kByteDdlOld[MAXIMUM_DDL_PER_LDC];
    double fragmentDdlOld[MAXIMUM_DDL_PER_LDC];

    printf("Affair.Starting initialization for DATE LDC or GDC \n");

    sprintf(gUserHname,"%s","DUMMY");  

    dateRolePtr=getenv("DATE_ROLE");
    if(dateRolePtr) {
      if (strcmp(dateRolePtr,"LDC") == 0 ) {
	strcpy(monitorSet,dateRolePtr);
      } else if ( strcmp(dateRolePtr,"GDC") == 0 ) {
	strcpy(monitorSet,dateRolePtr);
      } else {
	printf("DATE_ROLE is neither LDC or GDC. exit \n");	
	exit;
      }
    } else { 
      printf("no DATE_ROLE Environment variable! This is bad! It needs to be LDC or GDC.exit \n");	
      exit;
    }

    sprintf(monitorSet,"%s",monitorSet);    
    sprintf(hostInfo.monset,"%s",monitorSet);

    // setup services, initial parameters...
    //setupCommon(logOption,&monitorData, sizeof(monitorData));


    // setup services, initial parameters...
    if (strcmp(hostInfo.monset,"LDC") == 0) {
        setupCommon(logOption,&monitorData, sizeof(monitorData));
    }
    if (strcmp(hostInfo.monset,"GDC") == 0) {
        setupCommon(logOption,&monitorDataGDC, sizeof(monitorDataGDC));
    }

    //setup up monitor specific stuff
    setup(monitorSet);
    
    // smi stuff
    strcpy(obj,argv[1]);      
    smi_attach(obj, SMI_handle_command);
    smi_volatile();
    smi_set_state("RUNNING");
    /* end smi stuff */
    
    // start the data collector
    sprintf(aux,"%s_%s_Collector",hname,monitorSet);
    dis_start_serving(aux);

    // send the hostInfo information to the monitoring station, so it can
    // set things up over there
    // need the statusFlag = -1 condition for time being...
    //monitorData.statusFlag = -1;

    if (strcmp(hostInfo.monset,"LDC") == 0) {
        monitorData.statusFlag = -1;
    }
    if (strcmp(hostInfo.monset,"GDC") == 0) {
        monitorDataGDC.statusFlag = -1;
    }


    dis_update_service(kStartCollecting);
    sprintf(gTextString,"DATE. Data collector setup  \n");
    writeOutput(gTextString);

    sprintf(aux,"%s_%s",hname,monitorSet);
    dic_cmnd_service("MonAffair/newHost",&aux,sizeof(aux));
    sprintf(gTextString,"Initialization done. DIM MonAffair/newHost command sent to AFFAIR_MONITOR. Waiting for response \n");
    writeOutput(gTextString);

    // wait for monitor to acknowledge that it sees this node/service
    while(!startFlag) dim_wait();
    sprintf(gTextString,"Received response from AFFAIR_MONITOR! Start collecting performance  data\n");
    writeOutput(gTextString);

//    monitorData.statusFlag = 1;

    if (strcmp(hostInfo.monset,"LDC") == 0) {
        monitorData.statusFlag = 1;
    }
    if (strcmp(hostInfo.monset,"GDC") == 0) {
        monitorDataGDC.statusFlag = 1;
    }


    ///////////////////////////////////////////////////////////////////////////////////////////////////
    // do an endless loop with provided delay, get current data and send data to monitor in each cycle
    /////////////////////////////////////////////////////////////////////////////////////////////////////
    loopCount = 0;
    while(1){	


        if (strcmp(hostInfo.monset,"LDC") == 0) {
	    
	    monitorData.statusFlag  = (double) time(0);    		
	    
	    monitorData.runNumber      = DAQCONTROL->runNumber;
	    monitorData.kBytesRecorded = DAQCONTROL-> bytesRecorded*KBYTES_PER_BYTE;
	    monitorData.kBytesInjected = DAQCONTROL-> bytesInjected*KBYTES_PER_BYTE; 
	    monitorData.runStatus      = DAQCONTROL->runStatus;
	    monitorData.eventsRecorded = DAQCONTROL->eventsRecorded;
	    monitorData.eventCount     = DAQCONTROL->eventCount;
	    
	    
	    if ( loopCount > 0 ) {
		monitorData.kBytesRecRate = (monitorData.kBytesRecorded - kBytesRecordedOld)/((double) timeDelay);
		monitorData.kBytesInjRate = (monitorData.kBytesInjected - kBytesInjectedOld)/((double) timeDelay); 
	    } else {
		monitorData.kBytesRecRate = 0;
		monitorData.kBytesInjRate = 0; 
	    }
	    
	    kBytesRecordedOld = monitorData.kBytesRecorded ;
	    kBytesInjectedOld = monitorData.kBytesInjected ; 
	    
	    // DDL stuff
	    if ( loopCount > 0 ) {
		for (i=0; i < MAXIMUM_DDL_PER_LDC; i++) {
		    monitorData.kByteDdlRate[i]      =  (DAQCONTROL->monitoringDdl[i].byteDdl*MBYTES_PER_BYTE - kByteDdlOld[i])/((double) timeDelay)     ;
		    monitorData.fragmentDdlRate[i]   =  (DAQCONTROL->monitoringDdl[i].fragmentDdl             - fragmentDdlOld[i])/((double) timeDelay) ;
		}
	    } else {
		for (i=0; i < MAXIMUM_DDL_PER_LDC; i++) {
		    monitorData.kByteDdlRate[i]      =  0. ;
		    monitorData.fragmentDdlRate[i]   =  0. ;	    
		}
	    }
	    
	    for (i=0; i < MAXIMUM_DDL_PER_LDC; i++) {
		monitorData.idDdl[i] = DAQCONTROL->monitoringDdl[i].idDdl;
		kByteDdlOld[i]       = DAQCONTROL->monitoringDdl[i].byteDdl*MBYTES_PER_BYTE;
		fragmentDdlOld[i]    = DAQCONTROL->monitoringDdl[i].fragmentDdl;
		//		printf("i=%i idddl =%f byteddl=%f fragddl=%f \n",i,(double) DAQCONTROL->monitoringDdl[i].idDdl,
		//      (double) DAQCONTROL->monitoringDdl[i].byteDdl, (double) DAQCONTROL->monitoringDdl[i].fragmentDdl);
	    }
	}


        if (strcmp(hostInfo.monset,"GDC") == 0) {
            monitorDataGDC.statusFlag  = (double) time(0);
            monitorDataGDC.runNumber      = DAQCONTROL->runNumber;
            monitorDataGDC.kBytesRecorded = DAQCONTROL-> bytesRecorded*KBYTES_PER_BYTE;
            monitorDataGDC.kBytesInjected = DAQCONTROL-> bytesInjected*KBYTES_PER_BYTE;
            monitorDataGDC.runStatus      = DAQCONTROL->runStatus;
            monitorDataGDC.eventsRecorded = DAQCONTROL->eventsRecorded;
            monitorDataGDC.eventCount     = DAQCONTROL->eventCount;


            if ( loopCount > 0 ) {
                monitorDataGDC.kBytesRecRate = (monitorDataGDC.kBytesRecorded - kBytesRecordedOld)/((double) timeDelay);
                monitorDataGDC.kBytesInjRate = (monitorDataGDC.kBytesInjected - kBytesInjectedOld)/((double) timeDelay);
            } else {
                monitorDataGDC.kBytesRecRate = 0;
                monitorDataGDC.kBytesInjRate = 0;
            }

            kBytesRecordedOld = monitorDataGDC.kBytesRecorded ;
            kBytesInjectedOld = monitorDataGDC.kBytesInjected ;
        }


	
	// data sent to monitor
	dis_update_service(kUpdateData);	

	if (gDebugLevel == 1) {
	  sprintf(gTextString,"new data sent. iteration=%i. run=%f kb rec=%f kb inj=%f status=%f evRec=%f evCount=%f recrate=%f injRate=%f \n",
		  loopCount, monitorData.runNumber,monitorData.kBytesRecorded,monitorData.kBytesInjected,monitorData.runStatus,
		  monitorData.eventsRecorded,monitorData.eventCount,monitorData.kBytesRecRate,monitorData.kBytesInjRate );
	  writeOutput(gTextString);
	}

	loopCount++;

	// wait for time delay
	dtq_sleep(timeDelay);		
    }
}









