#include <stdlib.h>
#include <TROOT.h>
#include <TApplication.h>
#include <TVirtualX.h>
#include <TGResourcePool.h>
#include <TGListBox.h>
#include <TGListTree.h>
#include <TGFSContainer.h>
#include <TGClient.h>
#include <TGFrame.h>
#include <TGIcon.h>
#include <TGLabel.h>
#include <TGButton.h>
#include <TGTextEntry.h>
#include <TGNumberEntry.h>
#include <TGMsgBox.h>
#include <TGMenu.h>
#include <TGCanvas.h>
#include <TGComboBox.h>
#include <TGTab.h>
#include <TGSlider.h>
#include <TGDoubleSlider.h>
#include <TGFileDialog.h>
#include <TGTextEdit.h>
#include <TGShutter.h>
#include <TGProgressBar.h>
#include <TGColorSelect.h>
#include <TDataProcess.h>

//#include <TQObject.h>
//#include <RQ_OBJECT.h>

#include <TRootEmbeddedCanvas.h>
#include <TCanvas.h>
#include <TH1.h>
#include <TH2.h>
#include <TRandom.h>
#include <TSystem.h>
#include <TSystemDirectory.h>
#include <TEnv.h>
#include <TFile.h>
#include <TKey.h>
#include <string>

#include "TAffairSql.h"
#include "affairGlobals.h"
#include "MonitorConstants.h"


#define MAX_PLOTS 50
/* constants */


enum EFrameIdentifier { F_1, F_2, F_3, F_4, F_5, F_6, F_7};
const int kSetupOffset = 2000;
const int kEditConfigTablesOffset  = 2100;
const int kSetupComputerParsOffset = 2200;
const int kSetupWebPasswordsOffset = 2300;

const int kSetupMonitorSet         = 1;
const int kSetupComputerPars       = 2;
const int kSetupMonitorSetDefaults = 3;
const int kSetupRootfiles          = 4;
const int kSetupHostinfo           = 5;
const int kSetupRrdinfo            = 6;

const int kSetupWebPars            = 7;
const int kSetupCreateNewMonitoringSet = 8;
const int kSetupMonitorSetNewMonitoringSet         = 9;

const int kMaxEntriesPerColumn = 30;
string   *gMonitorSetList;
Int_t     gMonsetCount;        
Char_t    gMonitorSet[256];
Char_t     gDatabase[128];

char** string_to_argv(char *argstring, int *argc)
{
  char  inquotes = 0, *workstring, **argv;
  int i, nchars;
  
  if((nchars = strlen(argstring)) > 0) {
    workstring = (char*)calloc((nchars + 2), sizeof(char*));
    //	fill the array with space characters
    for(i=0; i < (nchars + 2); i++) workstring[i] = ' '; 	  
    //	copy the argstring into the workstring (padded with spaces on both ends)
    for(i=0; i < nchars; i++) workstring[i+1] = argstring[i];
    for((*argc) = 1, i=0; i < nchars + 2; i++) {
      //	count the number of arguements
      if(isgraph(workstring[i]) > 0) {
	//	if current character is not whitespace and previous 
	//      character is whitespace, increment (*argc)
	if((isgraph(workstring[i-1]) == 0) && (inquotes == 0)) (*argc)++;
	//	check for the quote character
	if(workstring[i] == '"') {
	  //	toggle inquotes flag
	  inquotes = (char)(!inquotes);
	}		
      } else {
	//	convert whitespace to null characters
	if(!inquotes) workstring[i] = '\0';
      }
    }
    
    //	no arguements in the string
    if((*argc) == 0){
      free(workstring);
      return NULL;
    } else {
      inquotes = 0;		  
      argv = (char**)calloc((*argc), sizeof(char**));	    
      argv[0] = &workstring[0];
      
      for((*argc) = 1, i=1; i < nchars + 2; i++) {
	// count the number of arguements
	if(isgraph(workstring[i]) > 0) {
	  // if current character is not whitespace and previous 
	  // character is whitespace, increment (*argc)
	  if((isgraph(workstring[i-1]) == 0) && (inquotes == 0)) {
	    argv[(*argc)] = &workstring[i];
	    (*argc)++;
	  }
	  //	check for the quote character
	  if(workstring[i] == '"') {
	    //	toggle inquotes flag
	    inquotes = (char)(!inquotes);
	  }
	}
      }	  
      return argv;
    }
  }
  else {
    (*argc) = 0;
    return NULL;
  }
}



class EditConfigTables: public TGTransientFrame {

private:
  TGCompositeFrame    **fCompositeFrames;
  TGButton            **fButton;
  TGComboBox          **fCombo;
  TGTextBuffer        **fTextBuffer;
  TGTextEntry         **fTextEntry;
  TGLabel             **fLabel;
  TGLayoutHints       *fL1, *fL2, *fL3, *fL4, *fL5, *fL5b, *fL6, *fL6b, *fL7;
  TGTextButton        *fbtExit;

  TAffairSql          *fAffairSql;
  //  Char_t              fMonitorSet[256];
  TGGroupFrame        *fGroupFrame;
  Int_t               fRowId;
  Int_t               fFieldNumber;
  Int_t               fRowNumber;
  string              *fMonitorSetLabels;
  string              *fMonsetParameterDescription;
  Pixel_t             fWhite;
  Pixel_t             fRed;
  Pixel_t             fGreen;
  Pixel_t             fBlue;
  Pixel_t             fYellow;
  Pixel_t             fGray;
  Int_t               fTableOption; 
  Int_t               UpdateRow();
  Int_t               UpdateLabels();


public:
  EditConfigTables(const TGWindow *p, const TGWindow *main, UInt_t w, UInt_t h, TAffairSql *affairSql,const int tableOption, UInt_t options = kVerticalFrame);
  virtual ~EditConfigTables();
  virtual void   CloseWindow();
  virtual Bool_t ProcessMessage(Long_t msg, Long_t parm1, Long_t);
};


void EditConfigTables::CloseWindow()
{
  // Called when window is closed (via the window manager or not).
  DeleteWindow();
}

Int_t EditConfigTables::UpdateRow()
{
  
  Char_t sql[256];

  if (fTableOption == kSetupMonitorSet) {
    sprintf(sql," select * FROM AFFAIR_%s ORDER BY ID",gMonitorSet);
  } else if (fTableOption == kSetupComputerPars) {
    sprintf(sql,"SELECT * FROM AFFAIR_COMPUTERPARS WHERE MONSET='%s' ORDER BY COMPUTER",gMonitorSet);
  } else if (fTableOption == kSetupMonitorSetDefaults) {
    sprintf(sql,"SELECT * FROM AFFAIR_MONSET_DEFAULTPARS");
  } else if (fTableOption == kSetupRootfiles) {
    sprintf(sql,"SELECT * FROM AFFAIR_ROOTFILELIST WHERE MONSET='%s' ORDER BY COMPUTER",gMonitorSet);
  } else if (fTableOption == kSetupHostinfo) {
    sprintf(sql,"SELECT * FROM AFFAIR_HOSTINFO ORDER BY COMPUTER");
  } else if (fTableOption == kSetupRrdinfo) {
    sprintf(sql,"SELECT COMPUTER,MONSET,RRD_NAME FROM AFFAIR_RRDLIST  WHERE MONSET='%s' ORDER BY COMPUTER ",gMonitorSet);
  } else if (fTableOption == kSetupWebPars) {
    sprintf(sql,"SELECT * FROM AFFAIR_PARS");
  } else if (fTableOption == kSetupCreateNewMonitoringSet) {
        sprintf(sql,"SELECT * FROM AFFAIR_MONSET_DEFAULTPARS WHERE MONSET='%s' ",gMonitorSet);
  } else if (fTableOption == kSetupMonitorSetNewMonitoringSet) {
    sprintf(sql," select * FROM AFFAIR_%s ORDER BY ID",gMonitorSet);
  }

  if (gDebug) printf(" fRowId=%d\n",fRowId);
  Int_t dummy1,dummy2;
  string * rowData = fAffairSql->GetRow(sql,fRowId,&dummy1,&dummy2);
  
  if (gDebug) printf(" fRowId=%d\n",fRowId);
  if (rowData) {
    for (Int_t i = 0; i < fFieldNumber; i++) {
      //if (i == 0 || i == 1 || i == 3) fTextEntry[i]->SetEnabled(kTRUE);
      fTextEntry[i]->SetFocus();
      fTextBuffer[i]->Clear();	    
      fTextBuffer[i]->AddText(0,rowData[i].c_str());	    
      if ((fTableOption == kSetupMonitorSet) && (i == 0 || i == 3)) fTextEntry[i]->SetEnabled(kFALSE);
      if ((fTableOption == kSetupComputerPars) && (i == 0 || i == 1 || i == 3)) fTextEntry[i]->SetEnabled(kFALSE);
      if ((fTableOption == kSetupMonitorSetDefaults) && (i == 0 )) fTextEntry[i]->SetEnabled(kFALSE);
      if ((fTableOption == kSetupRootfiles) && (i == 0 || i == 3 ||  i == 4 ||  i == 5 ||  i == 6 ||  i == 7 )) fTextEntry[i]->SetEnabled(kFALSE);
      //       if ((fTableOption == kSetupWebPars) && (i == 0  )) fTextEntry[i]->SetEnabled(kFALSE);
      if ((fTableOption == kSetupWebPars) && (i == 0  || i == 2 ||  i == 3||  i == 4 ||  i == 6 ||  i == 7  )) fTextEntry[i]->SetEnabled(kFALSE);
      if ((fTableOption == kSetupMonitorSetNewMonitoringSet) && (i == 0 || i == 3)) fTextEntry[i]->SetEnabled(kFALSE);

    }  
  }

  return 1;
}



Int_t EditConfigTables::UpdateLabels()
{
  
  Char_t sql[256];

  if (fTableOption != kSetupComputerPars) return 0;


  for (Int_t i = 0; i < fFieldNumber; i++) {
    Char_t myLabel[200];
    fMonsetParameterDescription = fAffairSql->GetMonsetParameterDescription(gMonitorSet);
    sprintf(myLabel,"%s (%s)",fMonitorSetLabels[i].c_str(),fMonsetParameterDescription[i].c_str());
    fLabel[i]->Resize(200,fLabel[i]->GetDefaultHeight());
    fLabel[i]->SetText(myLabel);
  }
  
  fGroupFrame->Resize();
  MapSubwindows();
  Resize();


  return 1;
}



EditConfigTables::EditConfigTables(const TGWindow *p, const TGWindow *main, UInt_t w,UInt_t h, TAffairSql  *affairSql ,const int tableOption , UInt_t options)
  :TGTransientFrame(p, main, w, h, options)
{


  SetCleanup(kDeepCleanup);

  string *rowData          ;


  fMonitorSetLabels = 0;
  rowData           = 0;
  fRowId            = 0;
  gClient->GetColorByName("white", fWhite);
  gClient->GetColorByName("red", fRed);
  gClient->GetColorByName("green", fGreen);
  gClient->GetColorByName("blue", fBlue);
  gClient->GetColorByName("yellow", fYellow);
  gClient->GetColorByName("gray", fGray);
  
  Char_t sql[500];


  fAffairSql = affairSql;
  fTableOption = tableOption;

  if (gDebug)cout << "EditConfigTables.0. fMonitorSet= " <<gMonitorSet<<" tableOption=" <<fTableOption << " gMonsetCount="<< gMonsetCount<<endl;  
  if (fTableOption != kSetupMonitorSetNewMonitoringSet) {
    sprintf(gMonitorSet,"%s",gMonitorSetList[0].c_str());
  } else {
    // a new monitoring set. assume it is the last created
    //    sprintf(fMonitorSet,"%s",gMonitorSetList[gMonsetCount-1].c_str());
  }
  if (gDebug)cout << "EditConfigTables. fMonitorSet= " <<gMonitorSet<<endl;  
  Char_t tableName[256];

  //note that same sql format needed in updaterow
  if (fTableOption == kSetupMonitorSet) {
    sprintf(sql,"SELECT * FROM AFFAIR_%s ORDER BY ID",gMonitorSet);
  } else if (fTableOption == kSetupComputerPars) {
    //first, extract desciption of labels foirm monset
    fMonsetParameterDescription = fAffairSql->GetMonsetParameterDescription(gMonitorSet);
    sprintf(sql,"SELECT * FROM AFFAIR_COMPUTERPARS WHERE MONSET='%s'  ORDER BY COMPUTER",gMonitorSet);
  } else if (fTableOption == kSetupMonitorSetDefaults) {
    sprintf(sql,"SELECT * FROM AFFAIR_MONSET_DEFAULTPARS ORDER BY MONSET");
  } else if (fTableOption == kSetupRootfiles) {
    sprintf(sql,"SELECT * FROM AFFAIR_ROOTFILELIST  WHERE MONSET='%s' ORDER BY COMPUTER",gMonitorSet);
  } else if (fTableOption == kSetupHostinfo) {
    sprintf(sql,"SELECT * FROM AFFAIR_HOSTINFO ORDER BY COMPUTER");
  } else if (fTableOption == kSetupRrdinfo) {
    sprintf(sql,"SELECT COMPUTER,MONSET,RRD_NAME FROM AFFAIR_RRDLIST  WHERE MONSET='%s' ORDER BY COMPUTER,MONSET ",gMonitorSet);
  } else if (fTableOption == kSetupWebPars) {
    sprintf(sql,"SELECT * FROM AFFAIR_PARS");
  } else if (fTableOption == kSetupCreateNewMonitoringSet) {
    sprintf(gMonitorSet,"%s",fAffairSql->CreateMonitoringSetDefaultparsRow());	    	 
    sprintf(sql,"SELECT * FROM AFFAIR_MONSET_DEFAULTPARS  WHERE MONSET='%s'",gMonitorSet);
  }else if (fTableOption == kSetupMonitorSetNewMonitoringSet) {
    sprintf(sql,"SELECT * FROM AFFAIR_%s  ORDER BY ID",gMonitorSet);
  }

  fMonitorSetLabels  = fAffairSql->GetLabels(sql,&fFieldNumber,&fRowNumber);    
 
  if (gDebug) cout << " sql statement is  = " <<sql << endl;
  if (gDebug) cout << " fRowNumber = " << fRowNumber << endl;

  fL1  = new TGLayoutHints(kLHintsTop    | kLHintsRight,2, 2, 2, 2);
  fL3  = new TGLayoutHints(kLHintsTop    | kLHintsLeft, 5,  5,  5, 5);
  fL4  = new TGLayoutHints(kLHintsTop    | kLHintsLeft    | kLHintsExpandX, 5,  5,  5, 5);
  fL5  = new TGLayoutHints(kLHintsBottom | kLHintsExpandX | kLHintsExpandY, 2, 2, 5, 1);
  fL5b = new TGLayoutHints(kLHintsBottom | kLHintsExpandX | kLHintsExpandY, 2, 2, 25, 1);
  fL6  = new TGLayoutHints(kLHintsTop    | kLHintsExpandX, 5, 5, 80, 5);
  fL6b = new TGLayoutHints(kLHintsTop    | kLHintsExpandX, 5, 5, 145, 5);
  fL7  = new TGLayoutHints(kLHintsTop    | kLHintsLeft    |  kLHintsExpandY, 5,  5,  5, 5);
  

  AddFrame(fbtExit = new TGTextButton(this, "Exit monitor set editing", kEditConfigTablesOffset+9),fL1);
  fbtExit->SetBackgroundColor(fRed);
  fbtExit->Associate(this);
  
  fCompositeFrames      = new TGCompositeFrame*[3];
  
  fButton               = new TGButton*[8];
  fCombo                = new TGComboBox*[1];
  fTextEntry            = new TGTextEntry*[fFieldNumber];
  fTextBuffer           = new TGTextBuffer*[fFieldNumber];
  fLabel                = new TGLabel*[fFieldNumber];
  
  
   fCompositeFrames[F_1] = new TGCompositeFrame(this, 60, 20, kHorizontalFrame);
   fCompositeFrames[F_2] = new TGCompositeFrame(fCompositeFrames[F_1], 60, 20, kVerticalFrame);
   fCompositeFrames[F_3] = new TGCompositeFrame(fCompositeFrames[F_1], 60, 20, kVerticalFrame);
  
   fButton[0]  = new TGTextButton(fCompositeFrames[F_3],"Choose monitor set",kEditConfigTablesOffset+0 );
   fButton[1]  = new TGTextButton(fCompositeFrames[F_3],"go to previous row", kEditConfigTablesOffset+4);
   fButton[2]  = new TGTextButton(fCompositeFrames[F_3],"go to next row", kEditConfigTablesOffset+5);
   fButton[3]  = new TGTextButton(fCompositeFrames[F_3],"save changes to this row", kEditConfigTablesOffset+6);
   fButton[4]  = new TGTextButton(fCompositeFrames[F_3],"delete this row", kEditConfigTablesOffset+7);
   fButton[5]  = new TGTextButton(fCompositeFrames[F_3],"create new row", kEditConfigTablesOffset+8);
   fButton[6]  = new TGTextButton(fCompositeFrames[F_3],"setup tables for new monitoring set", kEditConfigTablesOffset+10);
   fButton[7]  = new TGTextButton(fCompositeFrames[F_3],"delete this monitoring set", kEditConfigTablesOffset+11);

   fButton[0]->Resize(80, fButton[0]->GetDefaultHeight());
   fButton[1]->Resize(80, fButton[1]->GetDefaultHeight());
   fButton[2]->Resize(80, fButton[2]->GetDefaultHeight());
   fButton[3]->Resize(80, fButton[3]->GetDefaultHeight());
   fButton[4]->Resize(80, fButton[4]->GetDefaultHeight());
   fButton[5]->Resize(80, fButton[5]->GetDefaultHeight());
   fButton[6]->Resize(80, fButton[6]->GetDefaultHeight());
   fButton[7]->Resize(80, fButton[7]->GetDefaultHeight());
   
   fButton[0]->SetBackgroundColor(fGray);
   fButton[1]->SetBackgroundColor(fYellow);
   fButton[2]->SetBackgroundColor(fYellow);
   fButton[3]->SetBackgroundColor(fBlue);
   fButton[4]->SetBackgroundColor(fRed);
   fButton[5]->SetBackgroundColor(fGreen);
   fButton[6]->SetBackgroundColor(fGray);
   fButton[7]->SetBackgroundColor(fGray);
   if (gDebug)cout << "EditConfigTables. 3 gMonsetCount= " <<gMonsetCount<<endl;  

   fCombo[0] = new TGComboBox(fCompositeFrames[F_3],kEditConfigTablesOffset+1);
   for (Int_t i = 0; i < gMonsetCount; i++) fCombo[0]->AddEntry(gMonitorSetList[i].c_str(),i); fCombo[0]->Select(0); 

   if (fTableOption == kSetupMonitorSetNewMonitoringSet) {
     for (Int_t i = 0; i < gMonsetCount; i++) {
       if (strcmp(gMonitorSet,gMonitorSetList[i].c_str()) == 0) fCombo[0]->Select(i); 
     }
   }

   fCombo[0]->Resize(250,fButton[0]->GetDefaultHeight());
   fCombo[0]->Associate(this);


   fButton[0]->Associate(this);
   fButton[1]->Associate(this);
   fButton[2]->Associate(this);
   fButton[3]->Associate(this);
   fButton[4]->Associate(this);
   fButton[5]->Associate(this);
   fButton[6]->Associate(this);
   fButton[7]->Associate(this);
     
   if (fTableOption == kSetupHostinfo) {
     fButton[0]->SetEnabled(kFALSE);
     fCombo[0]->SetEnabled(kFALSE);

     fButton[5]->SetBackgroundColor(fGray);
     fButton[5]->SetEnabled(kFALSE);

   }



     
   if (fTableOption == kSetupHostinfo ) {
     fButton[0]->SetEnabled(kFALSE);
     fCombo[0]->SetEnabled(kFALSE);

     fButton[5]->SetBackgroundColor(fGray);
     fButton[5]->SetEnabled(kFALSE);

   }



     
   if ( fTableOption ==  kSetupWebPars ) {
     fButton[0]->SetEnabled(kFALSE);
     fCombo[0]->SetEnabled(kFALSE);

     fButton[5]->SetBackgroundColor(fGray);
     fButton[5]->SetEnabled(kFALSE);


     fButton[1]->SetBackgroundColor(fGray);
     fButton[1]->SetEnabled(kFALSE);

     fButton[2]->SetBackgroundColor(fGray);
     fButton[2]->SetEnabled(kFALSE);

     fButton[4]->SetBackgroundColor(fGray);
     fButton[4]->SetEnabled(kFALSE);


   }


   if ( fTableOption == kSetupMonitorSetDefaults ) {
     fButton[0]->SetEnabled(kFALSE);
     fCombo[0]->SetEnabled(kFALSE);

     fButton[5]->SetBackgroundColor(fGray);
     fButton[5]->SetEnabled(kFALSE);
   }



   if (fTableOption == kSetupCreateNewMonitoringSet ) {
     fButton[0]->SetEnabled(kFALSE);
     fCombo[0]->SetEnabled(kFALSE);

     fButton[5]->SetBackgroundColor(fGray);
     fButton[5]->SetEnabled(kFALSE);

     fButton[4]->SetBackgroundColor(fGray);
     fButton[4]->SetEnabled(kFALSE);

     fButton[2]->SetBackgroundColor(fGray);
     fButton[2]->SetEnabled(kFALSE);

     fButton[1]->SetBackgroundColor(fGray);
     fButton[1]->SetEnabled(kFALSE);
   }

   if (fTableOption == kSetupMonitorSetNewMonitoringSet) {
     fButton[0]->SetEnabled(kFALSE);
     fCombo[0]->SetEnabled(kFALSE);
   }

   fButton[6]->SetEnabled(kFALSE);
   fButton[7]->SetEnabled(kFALSE);

   fCompositeFrames[F_3]->AddFrame(fButton[0],fL4);
   fCompositeFrames[F_3]->AddFrame(fCombo[0],fL4);
   fCompositeFrames[F_3]->AddFrame(fButton[1],fL6);
   fCompositeFrames[F_3]->AddFrame(fButton[2],fL4);
   fCompositeFrames[F_3]->AddFrame(fButton[3],fL6);
   fCompositeFrames[F_3]->AddFrame(fButton[4],fL4);
   fCompositeFrames[F_3]->AddFrame(fButton[5],fL4);
   fCompositeFrames[F_3]->AddFrame(fButton[6],fL4);
   fCompositeFrames[F_3]->AddFrame(fButton[7],fL4);

   // last column
   if (gDebug)cout << " 3.2 gMonsetCount= " <<gMonsetCount<<endl;  
   
   fGroupFrame = new TGGroupFrame(fCompositeFrames[F_1], " parameters ", kVerticalFrame);
   
   Int_t columnNumber = 2;
   if (fFieldNumber > kMaxEntriesPerColumn ) columnNumber = 4;

   fGroupFrame ->SetLayoutManager(new TGMatrixLayout(fGroupFrame, 0, columnNumber, 3));

   
   for (Int_t i = 0; i < fFieldNumber; i++) {
     if (fTableOption == kSetupComputerPars) {
       Char_t myLabel[200];
       sprintf(myLabel,"%s %s",fMonitorSetLabels[i].c_str(),fMonsetParameterDescription[i].c_str());
       fLabel[i] = new TGLabel(fGroupFrame, new TGHotString(myLabel));
     } else {
       fLabel[i] = new TGLabel(fGroupFrame, new TGHotString(fMonitorSetLabels[i].c_str()));
     }
     fLabel[i]->Resize(200,fLabel[i]->GetDefaultHeight());
     fTextEntry[i] = new TGTextEntry(fGroupFrame, fTextBuffer[i] = new TGTextBuffer(110));
     fTextEntry[i]->Resize(200,fTextEntry[i]->GetDefaultHeight());
     fGroupFrame->AddFrame(fLabel[i]);
     fGroupFrame->AddFrame(fTextEntry[i],fL4);       
   }
   
   fGroupFrame->Resize();
   UpdateRow();

   fCompositeFrames[F_1]->AddFrame(fCompositeFrames[F_2], fL3);
   fCompositeFrames[F_1]->AddFrame(fCompositeFrames[F_3], fL3);
   fCompositeFrames[F_1]->AddFrame(fGroupFrame, fL3);

   AddFrame(fCompositeFrames[F_1], fL3);

   MapSubwindows();
   Resize();
   // position relative to the parent's window
   CenterOnParent();
   SetWindowName("Edit Config Tables");
   MapWindow();

}

EditConfigTables::~EditConfigTables()
{
  // DeleteWindow();  // deletes fMain
}


//
//
//
Bool_t EditConfigTables::ProcessMessage(Long_t msg, Long_t parm1, Long_t)
{

  Char_t tableName[256];
  Char_t sql[500];

  // Process messages coming from widgets associated with the dialog.  
  switch (GET_MSG(msg)) {
  case kC_COMMAND:
    
    switch (GET_SUBMSG(msg)) {
    case kCM_BUTTON:
      switch(parm1) {
      case (kEditConfigTablesOffset+0): //set
	{
	  Int_t selMSet = fCombo[0]->GetSelected();
	  sprintf(gMonitorSet,gMonitorSetList[selMSet].c_str());
	  //

	  if (fTableOption == kSetupMonitorSet) {
	    sprintf(sql," select * FROM AFFAIR_%s ORDER BY ID",gMonitorSet);
	  } else if (fTableOption == kSetupComputerPars) {
	    sprintf(sql,"SELECT * FROM AFFAIR_COMPUTERPARS WHERE MONSET='%s' ORDER BY COMPUTER",gMonitorSet);
	  } else if (fTableOption == kSetupMonitorSetDefaults) {
	    sprintf(sql,"SELECT * FROM AFFAIR_MONSET_DEFAULTPARS");
	  } else if (fTableOption == kSetupRootfiles) {
	    sprintf(sql,"SELECT * FROM AFFAIR_ROOTFILELIST WHERE MONSET='%s' ORDER BY COMPUTER",gMonitorSet);
	  } else if (fTableOption == kSetupHostinfo) {
	    sprintf(sql,"SELECT * FROM AFFAIR_HOSTINFO ORDER BY COMPUTER");
	  } else if (fTableOption == kSetupRrdinfo) {
	    sprintf(sql,"SELECT COMPUTER,MONSET,RRD_NAME FROM AFFAIR_RRDLIST WHERE MONSET='%s' ORDER BY COMPUTER ",gMonitorSet);
	  } else if (fTableOption == kSetupWebPars) {
	    sprintf(sql,"SELECT * FROM AFFAIR_PARS");
	  }

	  	  
	  fRowNumber = fAffairSql->GetNumberOfRowsSql(sql); 

	  if (gDebug) cout << " fMonitorSet = " << gMonitorSet<< " fRowNumber =" << fRowNumber << endl;
	  fRowId = 0;
	  UpdateLabels();
	  UpdateRow();
	}
	break;	
      

      case (kEditConfigTablesOffset+3): //set
	{
	  cout << "SetupMonitorSetWindow::ProcessMessage 3" << endl;
	}
	break;	


      case (kEditConfigTablesOffset+4): //set
	{
	  if (fRowId==0) {
	    cout << " first row. do nothing " << endl;
	  } else  if (fRowId > fRowNumber-1) {
	    cout << " shoud not happen. fRowId > fRowNumber ="<<  fRowNumber << endl;
	  } else {
	    fRowId--;
	    UpdateLabels();
	    UpdateRow();
	  }
	}
	break;	


      case (kEditConfigTablesOffset+5): //set
	{
	  if (fRowId == fRowNumber-1 ) {
	    cout << " last row. do nothing " << endl;
	  } else  if (fRowId  < 0 ) {
	    cout << " should not happen. fRowId  < 0= "<< fRowId << endl;
	  } else  if (fRowId  > fRowNumber-1 ) {
	    cout << " should not happen. fRowId  > fRowNumber-1.fRowId= "<< fRowId<<" fRowNumber="<<fRowNumber << endl;
	  } else {
	    fRowId++;
	    UpdateLabels();
	    UpdateRow();
	  }
	}
	break;	


      case (kEditConfigTablesOffset+6): //set
	{
	  string oneRowData = "";
	  Char_t tmp[128];

	  for (Int_t i = 0; i < fFieldNumber; i++) {
	    sprintf(tmp," %s ",fTextBuffer[i]->GetString());
	    oneRowData += tmp;	   
	  }  
	  printf("are in kEditConfigTablesOffset+6.fTableOption= %d \n",fTableOption);

	  if (fTableOption == kSetupMonitorSet) {
	    sprintf(tableName,"AFFAIR_%s",gMonitorSet);
	  } else if (fTableOption == kSetupComputerPars) {
	    sprintf(tableName,"AFFAIR_COMPUTERPARS");
	  } else if (fTableOption == kSetupMonitorSetDefaults) {
	    sprintf(tableName,"AFFAIR_MONSET_DEFAULTPARS");
	  } else if (fTableOption == kSetupRootfiles) {
	    sprintf(tableName,"AFFAIR_ROOTFILELIST");
	  }  else if (fTableOption == kSetupHostinfo) {
	    sprintf(tableName,"AFFAIR_HOSTINFO");
	  }  else if (fTableOption == kSetupRrdinfo) {
	    sprintf(tableName,"AFFAIR_RRDLIST");
	  } else if (fTableOption == kSetupWebPars) {
	    sprintf(tableName,"AFFAIR_PARS");
	  } else if (fTableOption == kSetupCreateNewMonitoringSet) {
	    sprintf(tableName,"AFFAIR_MONSET_DEFAULTPARS");
	  } else if (fTableOption == kSetupMonitorSetNewMonitoringSet) {
	      sprintf(tableName,"AFFAIR_%s",gMonitorSet);
	  }


	  fAffairSql->ReplaceRowWithString(tableName,(Char_t *) oneRowData.c_str(),fFieldNumber);
	  printf(" sss fMonitorSet=%s fTextBuffer[0]->GetString()=%s \n",gMonitorSet,fTextBuffer[0]->GetString());
	  if (fTableOption == kSetupCreateNewMonitoringSet) {

	    sprintf(gMonitorSet,fTextBuffer[0]->GetString());
	    fAffairSql->CreateMonitoringSetTable(gMonitorSet,1);

	    int retval;	    
	    printf(" 2 sss fMonitorSet=%s fTextBuffer[0]->GetString()=%s gMonsetCount=%d \n",gMonitorSet,fTextBuffer[0]->GetString(),gMonsetCount);
	    
	    gMonitorSetList = fAffairSql->GetColumn("AFFAIR_MONSET_DEFAULTPARS",&gMonsetCount,0," ");
	    if (gDebug) cout << "2ssss  gMonsetCount = " <<gMonsetCount<< endl;
	    

	    new TGMsgBox(gClient->GetRoot(), fMain, "Warning",  "You will next  need to specify the variables monitored.\n  The appropriate window will open automatically",
			 kMBIconExclamation, 4, &retval);
	    new EditConfigTables(gClient->GetRoot(), this, 1400, 1220, fAffairSql,kSetupMonitorSetNewMonitoringSet);
	    //
	  }

	}
	break;	



      case (kEditConfigTablesOffset+7): //set
	{
	  //deleting rows
	  int retval;	    
	  Char_t tmpMonset[100];
	  if (fTableOption == kSetupMonitorSet) {
	    sprintf(sql,"DELETE FROM %s.AFFAIR_%s WHERE ID=%i",gDatabase,gMonitorSet,atoi(fTextBuffer[0]->GetString()));
	  } else if (fTableOption == kSetupComputerPars) {
	    sprintf(sql,"DELETE FROM %s.AFFAIR_COMPUTERPARS WHERE COMPUTER='%s' AND MONSET='%s'",gDatabase,fTextBuffer[0]->GetString(),fTextBuffer[1]->GetString());
	  } else if (fTableOption == kSetupMonitorSetDefaults) {
	    sprintf(sql,"DELETE FROM %s.AFFAIR_MONSET_DEFAULTPARS WHERE  MONSET='%s'",gDatabase,fTextBuffer[0]->GetString());
	    sprintf(tmpMonset,"%s",fTextBuffer[0]->GetString());
	  } else if (fTableOption == kSetupRootfiles) {
	    sprintf(sql,"DELETE FROM %s.AFFAIR_ROOTFILELIST WHERE FILE='%s'",gDatabase,fTextBuffer[0]->GetString());
	  } else if (fTableOption == kSetupHostinfo) {
	    sprintf(sql,"DELETE FROM %s.AFFAIR_HOSTINFO WHERE COMPUTER='%s'",gDatabase,fTextBuffer[0]->GetString());	    
	  }  else if (fTableOption == kSetupRrdinfo) {
	    sprintf(tableName,"DELETE FROM %s.AFFAIR_RRDLIST WHERE RRD_NAME='%s'",gDatabase,fTextBuffer[0]->GetString());
	  } else if (fTableOption == kSetupMonitorSetNewMonitoringSet) {
	    //	    sprintf(sql,"DELETE FROM affair.AFFAIR_MONSET_DEFAULTPARS WHERE  MONSET='%s'",atoi(fTextBuffer[0]->GetString()));
	    //  sprintf(tmpMonset,"%s",fTextBuffer[0]->GetString());
	    sprintf(sql,"DELETE FROM %s.AFFAIR_%s WHERE ID=%i",gDatabase,gMonitorSet,atoi(fTextBuffer[0]->GetString()));
	  }
	  
	  if (gDebug) printf("sql delete statement is %s  fTableOption is %d \n",sql,fTableOption);
	  // insert here a question if one wants to delete

	  new TGMsgBox(gClient->GetRoot(), fMain, "Warning",  "Are you sure you want to delete this row?",
		       kMBIconQuestion, 3, &retval);
	  printf(" retval=%d\n",retval);

	  if ( retval == 2) break;

	  fAffairSql->Exec(sql);
	  if (fTableOption == kSetupMonitorSet || fTableOption == kSetupMonitorSetNewMonitoringSet  ) {
	    new TGMsgBox(gClient->GetRoot(), fMain, "Warning",  "You deleted a monitored variable. \n Make sure that the ID parameters go sequentially from 0 for all the parameters",
			 kMBIconExclamation, 4, &retval); 
	  }

	  if (fTableOption == kSetupMonitorSetDefaults || fTableOption == kSetupCreateNewMonitoringSet) {
	    Char_t questionMsg[1000];
	    
	    //	    sprintf(questionMsg,"You deleted the default parameters for a monitoring set. \n Do you also want to delete the table where the monitored \n variables are defined for this monitoring set (i.e. delete table AFFAIR_%s)? \n",tmpMonset);
	    //	    new TGMsgBox(gClient->GetRoot(), fMain, "Warning",questionMsg, kMBIconQuestion, 3, &retval); 
	    retval = 1;
	    if ( retval == 1) {
	      sprintf(sql,"drop table  if exists AFFAIR_%s ",tmpMonset);
	      fAffairSql->Exec(sql);
	      //check this. update list of monitoring sets 
	      gMonitorSetList = fAffairSql->GetColumn("AFFAIR_MONSET_DEFAULTPARS",&gMonsetCount,0," ");

	    } else {
	    }	  
	  }
	  fRowId  = 0;
	  UpdateLabels();
	  UpdateRow();
	}
	break;	



      case (kEditConfigTablesOffset+8): //set
	{
	  string oneRowData = "";
	  Char_t tmp[128];

	  for (Int_t i = 0; i < fFieldNumber; i++) {
	    sprintf(tmp," %s ",fTextBuffer[i]->GetString());
	    oneRowData += tmp;	   
	  }  

	  if (fTableOption == kSetupMonitorSet || fTableOption ==  kSetupMonitorSetNewMonitoringSet) {
	    sprintf(tableName,"AFFAIR_%s",gMonitorSet);
	    // add here if want to add new row. It will be a copy of the last one, but with a new name
	    fAffairSql->CreateMonitoringSetRow(gMonitorSet);
	    sprintf(sql,"SELECT * FROM %s ",tableName);
	    fRowNumber = fAffairSql->GetNumberOfRowsSql(sql); 
	    fRowId = fRowNumber-1;
	    UpdateRow();
	    	 
	  } else if (fTableOption == kSetupComputerPars) {
	    sprintf(sql,"INSERT INTO %s.AFFAIR_COMPUTERPARS WHERE COMPUTER='%s' AND MONSET='%s'",gDatabase,fTextBuffer[0]->GetString(),fTextBuffer[1]->GetString());
	  } else if (fTableOption == kSetupMonitorSetDefaults) {
	    sprintf(tableName,"AFFAIR_MONSET_DEFAULTPARS");
	    sprintf(gMonitorSet,"AFFAIR_MONSET_DEFAULTPARS");
	    fAffairSql->CreateMonitoringSetDefaultparsRow();	    	 
	    fButton[6]->SetEnabled(kTRUE);
	    fButton[6]->SetBackgroundColor(fBlue);
	    fButton[6]->Resize(fButton[5]->GetWidth(), fButton[5]->GetHeight());

	    //should now get fRowId 
	    sprintf(sql,"SELECT * FROM AFFAIR_MONSET_DEFAULTPARS ");
	    fRowNumber = fAffairSql->GetNumberOfRowsSql(sql); 
	    fRowId = fRowNumber-1;
	    UpdateRow();

	    Int_t retval;
	    new TGMsgBox(gClient->GetRoot(), fMain, "Warning",  "After editing and saving changes to this new monitoring set, you need to press the \n \
SETUP TABLES FOR NEW MONITORING SET  button \n to create the AFFAIR_YOURNEWMONITOTING SET table,\n  where you will add all the new variables and their plotting options ",
			 kMBIconExclamation, 4, &retval);


	  } else if (fTableOption == kSetupRootfiles) {
	    sprintf(tableName,"AFFAIR_ROOTFILELIST");
	    cout << " new entry not implemented yet for this table " << endl;
	  } else if (fTableOption == kSetupHostinfo) {
	    sprintf(tableName,"AFFAIR_HOSTINFO");
	    cout << " new entry not implemented yet for this table " << endl;
	  }  else if (fTableOption == kSetupRrdinfo) {
	    sprintf(tableName,"AFFAIR_RRDLIST");
	    cout << " new entry not implemented yet for this table " << endl;
	  }
	  
	  //	  fAffairSql->Exec(sql);
	}
	break;	



      case (kEditConfigTablesOffset+9): //set
	{
	  int retval;	  
	  new TGMsgBox(gClient->GetRoot(), this, "Warning",  "Are you sure you want to exit?",
		       kMBIconQuestion, 3, &retval);
	  if ( retval == 1) {
	    CloseWindow();
	  } else {
	  }	  
	}
	break;	



      case (kEditConfigTablesOffset+10): //set
	{
	  //	  fButton[6]->SetEnabled(kTRUE);
	  //	  printf("buffer is  %s %s %s     \n",fTextBuffer[0]->GetString(),fTextBuffer[1]->GetString(),fTextBuffer[2]->GetString() );
	  /*
	  sprintf(fMonitorSet,fTextBuffer[0]->GetString());
	  fAffairSql->CreateMonitoringSetTable(fMonitorSet);
	  int retval;	  
	  new TGMsgBox(gClient->GetRoot(), this, "Warning",  "The new monitoring set table has been created. \n You should add the variables  that  should be monitored \n by clicking on the VIEW/EDIT MONITORED VARIABLES AND PLOTTING OPTIONS button on the initail setup tab  ",
		       kMBIconQuestion, 4, &retval);
	  */
	  cout << " kEditConfigTablesOffset+10. do nothing " << endl;
	  // first check if table already exists.
	  // if not, create it

	}
      }
    }
    
  case kCM_RADIOBUTTON:
  case kCM_CHECKBUTTON:
    break;
  default:
    break;
  }
  return kTRUE;
}



//////////////////////////
/////////////////////////
////////////////////////


class SetupWebPasswords: public TGTransientFrame {

private:
  TGCompositeFrame    **fCompositeFrames;
  TGButton            **fButton;
  TGComboBox          **fCombo;
  TGTextBuffer        **fTextBuffer;
  TGTextEntry         **fTextEntry;
  TGLabel             **fLabel;
  TGLayoutHints       *fL1, *fL2, *fL3, *fL4, *fL5, *fL5b, *fL6, *fL6b, *fL7;
  TGTextButton        *fbtExit;

  TGListBox           *fList;
  Pixel_t             fRed;
  Pixel_t             fGreen;
  Pixel_t             fBlue;
  Pixel_t             fYellow;
 

public:
  SetupWebPasswords(const TGWindow *p, const TGWindow *main, UInt_t w, UInt_t h, UInt_t options = kVerticalFrame);
  virtual ~SetupWebPasswords();
  virtual void   CloseWindow();
  virtual Bool_t ProcessMessage(Long_t msg, Long_t parm1, Long_t);
};


void SetupWebPasswords::CloseWindow()
{
  // Called when window is closed (via the window manager or not).
  DeleteWindow();
}


SetupWebPasswords::SetupWebPasswords(const TGWindow *p, const TGWindow *main, UInt_t w,UInt_t h, UInt_t options)
  :TGTransientFrame(p, main, w, h, options)
{


  SetCleanup(kDeepCleanup);


  gClient->GetColorByName("red", fRed);
  gClient->GetColorByName("green", fGreen);
  gClient->GetColorByName("blue", fBlue);
  gClient->GetColorByName("yellow", fYellow);
  

  fL1  = new TGLayoutHints(kLHintsTop    | kLHintsRight,2, 2, 2, 2);
  fL3  = new TGLayoutHints(kLHintsTop    | kLHintsLeft, 5,  5,  5, 5);
  fL4  = new TGLayoutHints(kLHintsTop    | kLHintsLeft    | kLHintsExpandX, 5,  5,  5, 5);
  fL5  = new TGLayoutHints(kLHintsBottom | kLHintsExpandX | kLHintsExpandY, 2, 2, 5, 1);
  fL5b = new TGLayoutHints(kLHintsBottom | kLHintsExpandX | kLHintsExpandY, 2, 2, 25, 1);
  fL6  = new TGLayoutHints(kLHintsTop    | kLHintsExpandX, 5, 5, 80, 5);
  fL6b = new TGLayoutHints(kLHintsTop    | kLHintsExpandX, 5, 5, 145, 5);
  fL7  = new TGLayoutHints(kLHintsTop    | kLHintsLeft    |  kLHintsExpandY, 5,  5,  5, 5);
  
  
  AddFrame(fbtExit = new TGTextButton(this, "Exit Setup Web Passwords", kSetupWebPasswordsOffset+9),fL1);
  fbtExit->SetBackgroundColor(fRed);
  fbtExit->Associate(this);
  
  fCompositeFrames      = new TGCompositeFrame*[3];  
  fButton               = new TGButton*[6];
  fList                 = new TGListBox;
    
  fCompositeFrames[F_1]->AddFrame(fCompositeFrames[F_2], fL3);
  fCompositeFrames[F_1]->AddFrame(fCompositeFrames[F_3], fL3);
  
  fList = new TGListBox(fCompositeFrames[F_1], 89);
  fCompositeFrames[F_2]->AddFrame(fList, fL7);


  fButton[0]  = new TGTextButton(fCompositeFrames[F_3],"add user:",kSetupWebPasswordsOffset+0 );
  fButton[1]  = new TGTextButton(fCompositeFrames[F_3],"set password for user()", kSetupWebPasswordsOffset+4);
  fButton[2]  = new TGTextButton(fCompositeFrames[F_3],"go to next row", kSetupWebPasswordsOffset+5);


  fButton[0]->Resize(80, fButton[0]->GetDefaultHeight());
  fButton[1]->Resize(80, fButton[1]->GetDefaultHeight());
  fButton[2]->Resize(80, fButton[2]->GetDefaultHeight());

  fButton[0]->Associate(this);
  fButton[1]->Associate(this);
  fButton[2]->Associate(this);

  fCompositeFrames[F_3]->AddFrame(fButton[0],fL4);
  fCompositeFrames[F_3]->AddFrame(fButton[1],fL4);
  fCompositeFrames[F_3]->AddFrame(fButton[2],fL4);
  
  fCompositeFrames[F_1]->AddFrame(fCompositeFrames[F_2], fL3);
  fCompositeFrames[F_1]->AddFrame(fCompositeFrames[F_3], fL3);

  AddFrame(fCompositeFrames[F_1], fL3);
  
  MapSubwindows();
  Resize();
  // position relative to the parent's window
  CenterOnParent();
  SetWindowName("Edit Config Tables");
  MapWindow();

}

SetupWebPasswords::~SetupWebPasswords()
{
  // DeleteWindow();  // deletes fMain
}


//
//
//
Bool_t SetupWebPasswords::ProcessMessage(Long_t msg, Long_t parm1, Long_t)
{

  // Process messages coming from widgets associated with the dialog.  
  switch (GET_MSG(msg)) {
  case kC_COMMAND:
    
    switch (GET_SUBMSG(msg)) {
    case kCM_BUTTON:
      switch(parm1) {
      case (kSetupWebPasswordsOffset+0): //set
	{
	}
	break;	
      




      case (kEditConfigTablesOffset+9): //set
	{
	  int retval;	  
	  new TGMsgBox(gClient->GetRoot(), this, "Warning",  "Are you sure you want to exit?",
		       kMBIconQuestion, 3, &retval);
	  if ( retval == 1) {
	    CloseWindow();
	  } else {
	  }	  
	}
	break;	


      }
    }

  case kCM_RADIOBUTTON:
  case kCM_CHECKBUTTON:
    break;
  default:
    break;
  }
  return kTRUE;
}




//////////////////////////
////////////////////////

class MainFrame : public TGMainFrame {
  ///  RQ_OBJECT("MainFrame")

private:
   TGMainFrame         *fMain;
   TGCompositeFrame    *fCFTab;
   TGCompositeFrame    *fStatusFrame;
   TGCompositeFrame    *fF1,*fF2,*fF3,*fF4,*fF5,*fF6;
   TGCompositeFrame    *fDateF1,*fDateF2,*fDateF3,*fDateF4,*fDateF5;
   TGCanvas            *fCanvasWindow;
   TGTextEntry         *fTestText;
   TGButton            *fTestButton,*bt3;
   TGColorSelect       *fColorSel;
   TGTab               *fTab;
   TGTextEntry         *fTxt1, *fTxt2,*fTxt3, *fTxt4, *fTxt6;
   TGTextEntry         *fDateTxt1, *fDateTxt2;
   TGLayoutHints       *fL1, *fL2, *fL3, *fL4, *fL5, *fL5b, *fL6, *fL6b, *fL6c, *fL6d, *fL6e, *fL7;
   TGTextBuffer        *fNewNodeBuffer;
   TGTextBuffer        *fAffairPathBuffer;
   TGTextBuffer        *fAffairCodeTarGzBuffer;
   TGTextBuffer        *fDateCodeTarGzBuffer;
   TGTextBuffer        *fDomainNameBuffer;
   TGTextBuffer        *fDateNewNodeBuffer;
   TGTextBuffer        *fDimDnsNodeBuffer;
   TGTextView          *fTxtView1;
   TGTextButton        *fbt1, *fbt2, *fbt3, *fbt4, *fbt5, *fbt6, *fbt7, *fbt8;   
   TGCompositeFrame    *fMonitorF1,*fMonitorF2,*fMonitorF3,*fMonitorF4,*fMonitorF5;
   TGTextEntry         *fMonitorTxt1;
   TGTextButton        *fbtExit;

   TGListBox           *fListBox;
   TGListBox           *fDateListBox;
  TGListBox           *fMonitorSetListBox;
   TGListBox           *fOfflineStorageListBox;

  TGListBox           *fNodeMonitoredStatusListBox;
   TGListBox           *fWebPlotsListBox;

   Char_t               fDimDnsNode[256];
   Char_t               fTempText[1024];

   TGMenuBar          *fMenuBar;
   TGPopupMenu        *fMenuFile, *fMenuTest, *fMenuView, *fMenuHelp;
   TGPopupMenu        *fCascadeMenu, *fCascade1Menu, *fCascade2Menu;
   TGPopupMenu        *fMenuNew1, *fMenuNew2;
   TGLayoutHints      *fMenuBarLayout, *fMenuBarItemLayout, *fMenuBarHelpLayout;
   Char_t              fAffairMonitorPath[256];
   Char_t              fAffairPath[256];
   Char_t              fAffairCodeTarGz[256];
   Char_t              fDateCodeTarGz[256];
   Char_t              fDomainName[256];
   Pixel_t             fRed;
   Pixel_t             fRed2;
   Pixel_t             fRed3;
   Pixel_t             fRed4;
   Pixel_t             fLightGreen;
   Pixel_t             fGreenYellow;
   Pixel_t             fGreen;
   Pixel_t             fWhite;
   Pixel_t             fPink;
   Pixel_t             fLightPink;
   Pixel_t             fBlue;
   Pixel_t             fYellow;
  TDataProcess         *cc;
  TAffairSql           *fAffairSql;
  Int_t                gSumOrSuperimposedOption;


  TRootEmbeddedCanvas *fEc1;
  Bool_t               fFillHistos;
  Bool_t               fPrintHistos;
  TCanvas             *fC1;
  Char_t               fTimePeriod[128];
  Char_t               fMonitorSet[128];
  Char_t               fOnlinePlotBaseName[128];
  Float_t              fBinDeltaFactor;
  Float_t              fBinDeltaFactorList[6];
  string               fTimePeriodList[6];
  TGCompositeFrame    **fCFCanvas;
  TGCompositeFrame    **fSetupCompositeFrames;
  TGCompositeFrame    **fMonitorControlFrames;

  TGButton            **fSetupButton;
  TGComboBox          **fComboCanvas;
  TGButton            *fStartB, *fStopB, *fPrintB;
  TGButton            **fButtonDate;
  TGButton            **fButtonCanvas;
  TGButton            **fButtonMonitor;
  string              *fPlotList;
  Bool_t              fbOneNodeOnly;
  Char_t              fNodeList[1000][50];
  Char_t              fMyHostName[256];
  Char_t              fCurrentVersion[32];

  TGRadioButton        **fRBPickMonitorMode;
  TGRadioButton        **fRBPickOfflineMode;


  Int_t      fUseMysql;
  string    *fRrdString;
  Int_t      fSelPlotId;

  TGLabel             **fSetupLabel;
  TGLabel             **fDatemonLabel;
  TGLabel             **fMonitorLabel;



  
  void Init();
  void CreateTabCanvas();
  void CreateTabMonitor();
  void CreateTabDatemon();
  void CreateTabSysmon();
  void CreateTabSetup();
  void CreateTabMessages();
  Int_t SetPlotList(Int_t plotScope);
  Int_t SetNodeList();
  void RemoveEntryFromFile(char* fileName , char* nodeName); 
  void AddEntryToFile(char* fileName , char* nodeName);
  Int_t  GetActiveRrds(Char_t * monset);
  Int_t  GetNodeNumber();

public:

  //  MainFrame();
  MainFrame(const TGWindow *p, UInt_t w, UInt_t h);
  virtual ~MainFrame();  
  void CloseWindow();
  void HandleMenu(Int_t id);
  void DoWarning(Char_t *);
  void MainFrame::OutputMessage(Char_t * mytext);
  void MainFrame::FillListBox(char * configfile,  TGListBox * myListBox);
  void MainFrame::ShowStatusOfMonitoringSet(void);

  virtual Bool_t ProcessMessage(Long_t msg, Long_t parm1, Long_t);
  Int_t MainFrame::FillHistos(void);
};


Int_t MainFrame::SetNodeList()
{
  Char_t aux[256];
  Char_t sysCommand[256];
  Int_t  ncols;
  Char_t line[256];
  Char_t baseName[256];
  Char_t rrdName[256];
  Char_t rootName[256];
  Char_t monitorSetName[256];
  Char_t nodeName[256];
  Int_t  nodeCount = 0;
  Char_t   *ptr;

  Int_t selMSet = fComboCanvas[1]->GetSelected();

  if (selMSet < 0) {
    if (gDebug) cout << "SetNodeList. selMSet="<<selMSet<< " Cannot set any plots, as no monitor set selected. " << endl;
    return 0;
  }

  sprintf(fMonitorSet,gMonitorSetList[selMSet].c_str());
  

  fComboCanvas[3]->RemoveEntries(0,1000);
  fComboCanvas[3]->AddEntry("aggregate plots",0); 
  sprintf(fNodeList[0],"aggregate plots");


  if ( !fUseMysql ) {
    // create rrdlist file
    sprintf(sysCommand,"%s/support/listRRD.pl",fAffairMonitorPath);	
    gSystem->Exec(sysCommand);
    
    sprintf(aux,"%s/log/rrdlist.txt",fAffairMonitorPath);
    FILE *table_fp = fopen(aux,"r");        
    while (fgets(line,256,table_fp)) {		
      for(ptr=line;;ptr++){
	if (*ptr==0) break;
	if (!isspace(*ptr)) break;
      }       
      if (*ptr=='#')        continue;
      if (strlen(line) < 2) continue;
      ncols = sscanf(line,"%s %s %s %s %s",&rrdName[0], &rootName[0],&baseName[0],&monitorSetName[0],&nodeName[0]);
      if (strcmp(fMonitorSet,monitorSetName) == 0) {
	nodeCount++;
	fComboCanvas[3]->AddEntry(nodeName,nodeCount); 
	sprintf(fNodeList[nodeCount],nodeName);
      }
    }
    fclose(table_fp);
  } else {
    char **argv;
    int    argc;
    fRrdString = 0;
    fRrdString = fAffairSql->GetRrdRows(fMonitorSet,&nodeCount);

    if (gDebug) cout << "setnodelist.fRrdString = " << fRrdString <<endl ; 
    if (fRrdString) {
      for (Int_t ii = 0; ii < nodeCount;  ii++) {
	if((argv = string_to_argv((Char_t *) fRrdString[ii].c_str(), &argc)) != NULL) {
	  fComboCanvas[3]->AddEntry(argv[5],ii+1); 
	  sprintf(fNodeList[ii+1],argv[5]);
	}      
      } 
    }
  }
  fComboCanvas[3]->Select(0); 
  fbOneNodeOnly = false;
  return 1;
}


Int_t MainFrame::SetPlotList(Int_t plotScope)
{
  Char_t commandStr[256];
  Char_t line[4096];

  char **argv;
  int    argc;

  Int_t selMSet = fComboCanvas[1]->GetSelected();
  if (selMSet < 0) {
    if (gDebug) cout << "SetPlotList. selMSet="<<selMSet<< " Cannot set any plots, as no monitor set selected. " << endl;
    return 0;
  }
  sprintf(fMonitorSet,gMonitorSetList[selMSet].c_str());
  
  // note that getplotlist does not get box or multibox plots 
  //kludge to avoid problem with Box and MultiBox
  // in future would want to include the multibox plots too here
  //TO BE DONE

  fComboCanvas[2]->RemoveEntries(-1,MAX_PLOTS);
  if (gDebug) cout << " SetPlot.List.fUseMysql ="<<fUseMysql <<endl;

  if (!fUseMysql) {
    
    if ( fPlotList != 0) delete [] fPlotList ;fPlotList = 0;
    fPlotList = new  string[MAX_PLOTS] ;    
    Char_t plotName[256]; 
    sprintf(commandStr,"%s/support/getPlotList.pl %s %i %i Box",fAffairMonitorPath, fMonitorSet,plotScope,MAX_PLOTS);
    FILE *fp = gSystem->OpenPipe(commandStr, "r");
    if(fp !=NULL) fgets(line,2048,fp);
    gSystem->ClosePipe(fp);
    
    argv = string_to_argv(line, &argc);
    if (argc > MAX_PLOTS) argc = MAX_PLOTS;
    for (Int_t i = 1; i <= argc; i++) {
      if (gDebug >0) cout << "i= "<< i << " argv[i] = "  << argv[i] << endl;
      sprintf(plotName,"%s",argv[i]);
      fPlotList[i-1] = plotName;
    }
    free(argv[0]);
    free(argv);

    
    for (Int_t i=0; i< MAX_PLOTS; i++) {
      if (strcmp(fPlotList[i].c_str(),"dummy")!= 0  ) {
	if (gDebug >0) cout << "plot name is   "<< fPlotList[i] << endl;
	fComboCanvas[2]->AddEntry(fPlotList[i].c_str(),i); 
      }
    }    
  } else {
    //mysql case
    Int_t plotNumber = 0;
    Char_t whereCondition[500];

    // sprintf(whereCondition," WHERE (USER = 'user' AND MONSET = '%s' AND PLOT_SCOPE = %d ) ",fMonitorSet,plotScope);
    // the UPTIME bit a kludge, as the name for the node up has wrnog format and can't be displayed online.    
    sprintf(whereCondition," WHERE (USER = 'user' AND MONSET = '%s' AND PLOT_SCOPE = %d AND PLOT_TYPE <> 'UPTIME') ",fMonitorSet,plotScope);
    fPlotList = fAffairSql->GetColumn("AFFAIR_PLOTLIST",&plotNumber,0,whereCondition);
    if (gDebug) cout << " fPlotList= " <<fPlotList <<" plotNumber = " << plotNumber<<endl;
    if (gDebug) printf( " plotNumber = %d \n",plotNumber);

    if (plotNumber && fPlotList) {
      for (Int_t i = 0; i < plotNumber; i++) {
	fComboCanvas[2]->AddEntry(fPlotList[i].c_str(),i);      
	if (gDebug) cout << " fPlotList[i]= " <<fPlotList[i]<<" plotScope= " << plotScope<<endl;
	if (gDebug) printf(" fPlotList[i]= %s plotScope= %d \n  \n ",fPlotList[i].c_str(),plotScope);
      }
    } else {      
      // if no plots, then probably not ran affair web interface
      // run it, for the appropriate scope
      if (fUseMysql) cc->SetMysql(fAffairSql);
      cc->UnsetPlotName();
      Int_t    returnInitValue;
      Int_t binNumber = 120;
      Int_t binDeltaTimeHour = (Int_t) 3600/binNumber;
      Int_t TimeNow  = (Int_t) (time(0)-20.0);    
      if (plotScope == kOneNodeScope) {
	Int_t selNode = fComboCanvas[3]->GetSelected();
	if ( selNode > 0 ) {
	  returnInitValue = cc->Init(fMonitorSet,binNumber,RRD_ONE_COMPUTER_OPTION,fNodeList[selNode],"none","each");
	  if (returnInitValue) cc->EachPlotsRrd(fTimePeriod,((Float_t)binDeltaTimeHour)*fBinDeltaFactor,12,TimeNow);  
	}
      } else {
 	returnInitValue = cc->Init(fMonitorSet,binNumber,RRD_GLOBAL_OPTION,"ONLINE_RRD_LIST","none","all");
	if (returnInitValue) cc->GlobalPlotsRrd(fTimePeriod,((Float_t)binDeltaTimeHour)*fBinDeltaFactor,12,TimeNow,gSumOrSuperimposedOption);
      }
      sprintf(whereCondition," WHERE (USER = 'user' AND MONSET = '%s' AND PLOT_SCOPE = %d AND PLOT_TYPE <> 'UPTIME') ",fMonitorSet,plotScope);
      fPlotList = fAffairSql->GetColumn("AFFAIR_PLOTLIST",&plotNumber,0,whereCondition);
      
      if (plotNumber && fPlotList) {
  	for (Int_t i = 0; i < plotNumber; i++) {
	  fComboCanvas[2]->AddEntry(fPlotList[i].c_str(),i);      
 	}
      } else {
	if (gDebug) cout << "no plots available " <<endl;
	if ( fPlotList != 0) delete [] fPlotList ; fPlotList = 0;
	fPlotList = new  string[1] ;    
	fPlotList[0] = "no plots available";
	fComboCanvas[2]->AddEntry(fPlotList[0].c_str(),-1);      
      }
    }
  }


  fComboCanvas[2]->Select(0); 
  fSelPlotId = fComboCanvas[2]->GetSelected();
  if (fSelPlotId >= 0) sprintf(fOnlinePlotBaseName,fPlotList[fSelPlotId].c_str());

  Int_t selTime = fComboCanvas[0]->GetSelected();
  sprintf(fTimePeriod,fTimePeriodList[selTime].c_str());
  fBinDeltaFactor = fBinDeltaFactorList[selTime];

  sprintf(fTempText,"new set name  %s plot base name is %s ",fMonitorSet,fOnlinePlotBaseName);

  if (gDebug) cout << "SetPlotList. end" <<endl;
  
  return 1;
}


void MainFrame::CreateTabCanvas()
{

  if (gDebug >0) cout << "CreateTabCanvas 1  " << endl;

   fCFTab = fTab->AddTab("performance plots");
   fCFCanvas = new TGCompositeFrame*[7];
   fButtonCanvas = new TGButton*[4];
   fComboCanvas = new TGComboBox*[4];

   fCFCanvas[F_1] = new TGCompositeFrame(fCFTab, 60, 20, kVerticalFrame);
   fCFCanvas[F_2] = new TGCompositeFrame(fCFCanvas[F_1], 60, 20, kHorizontalFrame);
   fCFCanvas[F_3] = new TGCompositeFrame(fCFCanvas[F_2], 60, 20, kVerticalFrame);
   fCFCanvas[F_4] = new TGCompositeFrame(fCFCanvas[F_2], 60, 20, kVerticalFrame);
   //   fCFCanvas[F_5] = new TGCompositeFrame(fCFCanvas[F_2], 60, 20, kVerticalFrame);

  if (gDebug >0) cout << "CreateTabCanvas 2  " << endl;

   fButtonCanvas[0]  = new TGTextButton(fCFCanvas[F_3], "Change time period:", 604);
   fButtonCanvas[1]  = new TGTextButton(fCFCanvas[F_3], "Change monitored set:", 605);
   fButtonCanvas[2]  = new TGTextButton(fCFCanvas[F_3], "Select plot:", 606);
   fButtonCanvas[3]  = new TGTextButton(fCFCanvas[F_3], "Select node:", 607);


   fComboCanvas[0] = new TGComboBox(fCFCanvas[F_4],9991);
   for (Int_t i =0; i < 6; i++) fComboCanvas[0]->AddEntry(fTimePeriodList[i].c_str(),i); fComboCanvas[0]->Select(1); 
   fComboCanvas[0]->Resize(150, fButtonCanvas[0]->GetDefaultHeight());

   if (gDebug >0) cout << "CreateTabCanvas 3.gMonsetCount=  " << gMonsetCount << endl;

   fComboCanvas[1] = new TGComboBox(fCFCanvas[F_4],9992);
   for (Int_t i = 0; i < gMonsetCount; i++) fComboCanvas[1]->AddEntry(gMonitorSetList[i].c_str(),i); fComboCanvas[1]->Select(0); 
   fComboCanvas[1]->Resize(250,fButtonCanvas[1]->GetDefaultHeight());

   if (gDebug >0) cout << "CreateTabCanvas 4  " << endl;

   fComboCanvas[2] = new TGComboBox(fCFCanvas[F_4],9993);

   //   SetPlotList(kGlobalScope);

   fComboCanvas[2]->Resize(250,fButtonCanvas[2]->GetDefaultHeight());

   fComboCanvas[3] = new TGComboBox(fCFCanvas[F_4],9993);
   fComboCanvas[3]->Resize(250,fButtonCanvas[3]->GetDefaultHeight());
   SetNodeList();

   fButtonCanvas[0]->Associate(this);
   fButtonCanvas[1]->Associate(this);
   fButtonCanvas[2]->Associate(this);
   fButtonCanvas[3]->Associate(this);

  if (gDebug >0) cout << "CreateTabCanvas 6  " << endl;

   fCFCanvas[F_3]->AddFrame(fButtonCanvas[0],fL4);
   fCFCanvas[F_3]->AddFrame(fButtonCanvas[1],fL4);
   fCFCanvas[F_3]->AddFrame(fButtonCanvas[2],fL4);
   fCFCanvas[F_3]->AddFrame(fButtonCanvas[3],fL4);

   fCFCanvas[F_4]->AddFrame(fComboCanvas[0],fL4);
   fCFCanvas[F_4]->AddFrame(fComboCanvas[1],fL4);
   fCFCanvas[F_4]->AddFrame(fComboCanvas[2],fL4);
   fCFCanvas[F_4]->AddFrame(fComboCanvas[3],fL4);
  
   if (gDebug >0) cout << "CreateTabCanvas 9  " << endl;

   fCFCanvas[F_2]->AddFrame(fCFCanvas[F_3], fL3);
   fCFCanvas[F_2]->AddFrame(fCFCanvas[F_4], fL3);
   //   fCFCanvas[F_2]->AddFrame(fCFCanvas[F_5], fL3);

   // TDataProcess canvas part
   fCFCanvas[F_6] = new TGCompositeFrame(fCFCanvas[F_1], 60, 20, kHorizontalFrame);
   fStartB = new TGTextButton(fCFCanvas[F_6], "Show performance", 601); fStartB->SetBackgroundColor(fGreen);
   fStopB  = new TGTextButton(fCFCanvas[F_6], "Stop", 602);             fStopB->SetBackgroundColor(fRed);
   fPrintB  = new TGTextButton(fCFCanvas[F_6], "Print", 603);           fPrintB->SetBackgroundColor(fBlue);

  if (gDebug >0) cout << "CreateTabCanvas 10  " << endl;

   fStartB->Associate(this);
   fStopB->Associate(this);
   fPrintB->Associate(this);


   fCFCanvas[F_6]->AddFrame(fStartB, fL4);
   fCFCanvas[F_6]->AddFrame(fStopB,  fL4);
   fCFCanvas[F_6]->AddFrame(fPrintB, fL4);

   fCFCanvas[F_7] = new TGCompositeFrame(fCFCanvas[F_1], 60, 60, kHorizontalFrame);

   fEc1 = new TRootEmbeddedCanvas("ec1",fCFCanvas[F_7], 300, 200);
   fCFCanvas[F_7]->AddFrame(fEc1, fL5);
   //
   fCFCanvas[F_1]->AddFrame(fCFCanvas[F_2], fL4);
   fCFCanvas[F_1]->AddFrame(fCFCanvas[F_6], fL4);
   fCFCanvas[F_1]->AddFrame(fCFCanvas[F_7], fL4);

   fCFTab->AddFrame(fCFCanvas[F_1], fL4);
   if (gDebug >0) cout << "CreateTabCanvas 11  cc="<<cc << endl;

   fC1 = fEc1->GetCanvas();

   if (cc != 0)  {
     delete cc; cc = 0;
   }
   cc = new TDataProcess;
   cc->SetCanvas(fC1);
   if (gDebug) cout << "CreateTabCanvas 15  " << endl;
   cc->SetOutputMode(k_NO_EPS);
   if (gDebug)  cout <<  " k_NO_EPS " << k_NO_EPS << endl;

   //   SetPlotList(kGlobalScope);

}


void MainFrame::CreateTabMonitor()
{
  Char_t  configfile[256];

   fCFTab = fTab->AddTab("Monitor control");
   fMonitorControlFrames = new TGCompositeFrame*[6];

   fButtonMonitor      = new TGButton*[4];
   fRBPickMonitorMode  = new TGRadioButton*[2];
   fRBPickOfflineMode  = new TGRadioButton*[3];
   fMonitorLabel       = new TGLabel*[6];

   //
   fMonitorControlFrames[F_1] = new TGCompositeFrame(fCFTab, 60, 20, kHorizontalFrame);
   fMonitorControlFrames[F_2] = new TGCompositeFrame(fMonitorControlFrames[F_1], 60, 20, kVerticalFrame);

   fButtonMonitor[0] = new TGTextButton(fMonitorControlFrames[F_2],"(re)start monitoring for selected monitoring set(s): ", 302);fButtonMonitor[0]->SetBackgroundColor(fGreen);
   fButtonMonitor[1] = new TGTextButton(fMonitorControlFrames[F_2],"(re)start default DATE monitoring (offline,with plots)", 303);fButtonMonitor[1]->SetBackgroundColor(fGreen);
   fButtonMonitor[2] = new TGTextButton(fMonitorControlFrames[F_2],"Stop monitoring  for selected monitoring set(s)", 304);fButtonMonitor[2]->SetBackgroundColor(fRed);
  
   for (Int_t i = 0; i < 3; i++){
     fButtonMonitor[i]->Resize(80, fButtonMonitor[i]->GetDefaultHeight());   
     fButtonMonitor[i]->Associate(this);
   }

   // do the radio button choice
   fRBPickMonitorMode[0]= new TGRadioButton(fMonitorControlFrames[F_2],"create plots for web interface", 305);
   fRBPickMonitorMode[1]= new TGRadioButton(fMonitorControlFrames[F_2],"no plots for web interface", 306);
   fRBPickMonitorMode[0]->SetState(kButtonDown);   
   //   fRBPickMonitorMode[0]->SetBackgroundColor(fLightGreen);
    fRBPickMonitorMode[1]->SetBackgroundColor(fLightPink);
   // do the radio button offline storage choice

   fRBPickOfflineMode[0]= new TGRadioButton(fMonitorControlFrames[F_2],"monitor, with offline storage", 307);
   fRBPickOfflineMode[1]= new TGRadioButton(fMonitorControlFrames[F_2],"monitor, with no offline storage", 308);
   fRBPickOfflineMode[2]= new TGRadioButton(fMonitorControlFrames[F_2],"no monitoring", 311);
   //   fRBPickOfflineMode[2]= new TGRadioButton(fMonitorControlFrames[F_2],"monitor, with offline storage as a separate process", 310);

   // put default to be no offline storage
   fRBPickOfflineMode[1]->SetState(kButtonDown);   

   //   fRBPickOfflineMode[0]->SetBackgroundColor(fLightGreen);
   //   fRBPickOfflineMode[1]->SetBackgroundColor(fLightGreen);
   // fRBPickOfflineMode[2]->SetBackgroundColor(fLightGreen);
    fRBPickOfflineMode[2]->SetBackgroundColor(fLightPink);


   fMonitorControlFrames[F_2]->AddFrame(fButtonMonitor[0], fL4);

   fMonitorControlFrames[F_2]->AddFrame(fRBPickMonitorMode[0], fL4);
   fMonitorControlFrames[F_2]->AddFrame(fRBPickMonitorMode[1], fL4);

   fMonitorControlFrames[F_2]->AddFrame(fRBPickOfflineMode[0], fL6d);
   fMonitorControlFrames[F_2]->AddFrame(fRBPickOfflineMode[1], fL4);
   fMonitorControlFrames[F_2]->AddFrame(fRBPickOfflineMode[2], fL4);
   //   fMonitorControlFrames[F_2]->AddFrame(fRBPickOfflineMode[3], fL4);

   fMonitorControlFrames[F_2]->AddFrame(fButtonMonitor[1], fL6d);


   fMonitorControlFrames[F_2]->AddFrame(fButtonMonitor[2], fL4);

   /// next column
   // vertical, where title is,
   // followed by a horizontal
   // where four verticalls are
   // each has title and listbox
   //followed 
  fMonitorControlFrames[F_3] = new TGCompositeFrame(fMonitorControlFrames[F_1], 60, 20, kVerticalFrame);
  fMonitorControlFrames[F_4] = new TGCompositeFrame(fMonitorControlFrames[F_3], 60, 20, kHorizontalFrame);  

  fMonitorControlFrames[F_4]->AddFrame(fMonitorSetListBox = new TGListBox(fMonitorControlFrames[F_4], 89), fL7);

   if (fUseMysql) {
     sprintf(configfile,"AFFAIR_MONSET_DEFAULTPARS");     
   } else {
     sprintf(configfile,"%s/config/monsetDefault.conf",fAffairMonitorPath);
   }
   FillListBox(configfile, fMonitorSetListBox);

   fMonitorSetListBox->Resize(120, 180);
   fMonitorSetListBox->SetMultipleSelections(1);

   fMonitorControlFrames[F_4]->AddFrame(fNodeMonitoredStatusListBox = new TGListBox(fMonitorControlFrames[F_4], 89), fL7);
   fNodeMonitoredStatusListBox->Resize(110, 180);

   fMonitorControlFrames[F_4]->AddFrame(fOfflineStorageListBox = new TGListBox(fMonitorControlFrames[F_4], 89), fL7);
   fOfflineStorageListBox->Resize(80, 180);

   fMonitorControlFrames[F_4]->AddFrame(fWebPlotsListBox = new TGListBox(fMonitorControlFrames[F_4], 89), fL7);
   fWebPlotsListBox->Resize(80, 240);


   fMonitorControlFrames[F_5] = new TGCompositeFrame(fMonitorControlFrames[F_3], 60, 20, kVerticalFrame);  

   
  fButtonMonitor[3] = new TGTextButton(fMonitorControlFrames[F_5],"                                   Refresh current status                                  ", 309);
  fButtonMonitor[3]->SetBackgroundColor(fYellow);
  fButtonMonitor[3]->Resize(500, fButtonMonitor[3]->GetDefaultHeight());   
  fButtonMonitor[3]->Associate(this);

  //  fButtonMonitor[2]->Resize(80, fButtonMonitor[2]->GetDefaultHeight());   
  // fButtonMonitor[2]->Associate(this);

  fMonitorControlFrames[F_5]->AddFrame(fButtonMonitor[3], fL4);


   fRBPickMonitorMode[0]->Associate(this);
   fRBPickMonitorMode[1]->Associate(this);


   fRBPickOfflineMode[0]->Associate(this);
   fRBPickOfflineMode[1]->Associate(this);
   fRBPickOfflineMode[2]->Associate(this);
   //   fRBPickOfflineMode[3]->Associate(this);

   fMonitorControlFrames[F_3]->AddFrame(fMonitorControlFrames[F_4],fL3);
   fMonitorControlFrames[F_3]->AddFrame(fMonitorControlFrames[F_5],fL3);


   fMonitorControlFrames[F_1]->AddFrame(fMonitorControlFrames[F_2],fL3);
   fMonitorControlFrames[F_1]->AddFrame(fMonitorControlFrames[F_3],fL3);
   fCFTab->AddFrame(fMonitorControlFrames[F_1], fL3);

}


////////////////////////////////////////////////////////////////////////
// DATEMON TAB
////////////////////////////////////////////////////////////////////////
void MainFrame::CreateTabDatemon()
{
   char configfile[256];
   Int_t buttonNumber = 5;


 


   fCFTab = fTab->AddTab("set list of computers");
   fDateF3 = new TGCompositeFrame(fCFTab, 60, 20, kHorizontalFrame);
   fButtonDate = new TGButton*[buttonNumber];
   //
   //list box with list of nodes
   //
   fDateF3->AddFrame(fDateListBox = new TGListBox(fDateF3, 89), fL7);

   if (fUseMysql) {
     sprintf(configfile,"AFFAIR_COMPUTERLIST");     
   } else {
   sprintf(configfile,"%s/config/computerlist.conf",fAffairMonitorPath);
   }
   FillListBox(configfile, fDateListBox);

   fDateListBox->Resize(150, 300);
   fDateListBox->SetMultipleSelections(1);


   //
   // buttons, right of list box
   //
   fDateF4 = new TGCompositeFrame(fDateF3, 60, 20, kVerticalFrame);


   fButtonDate[0]  = new TGTextButton(fDateF4, "Add node:", 202);
   fButtonDate[0]->SetBackgroundColor(fGreen);

   fButtonDate[1]  = new TGTextButton(fDateF4, "Delete selected node/s", 201);
   fButtonDate[1]->SetBackgroundColor(fRed);

   fButtonDate[2]  = new TGTextButton(fDateF4, "Delete all node/s", 206);
   fButtonDate[2]->SetBackgroundColor(fRed);


   if (!fUseMysql) {
     fButtonDate[3]  = new TGTextButton(fDateF4, "Edit file with list of nodes", 203);
   } else {
     fButtonDate[3]  = new TGTextButton(fDateF4, "Use temp text file to add nodes", 203);
   }
   fButtonDate[3]->SetBackgroundColor(fYellow);

   fButtonDate[4]  = new TGTextButton(fDateF4, "Import GDC/LDC nodes from DATE Roles table", 204);
   fButtonDate[4]->SetBackgroundColor(fGreen);



   for (Int_t i = 0; i < 4; i++) {
     fDateF4->AddFrame(fButtonDate[i], fL4);
     fButtonDate[i]->Resize(80,fButtonDate[i]->GetDefaultHeight());
     fButtonDate[i]->Associate(this);
   }
   fDateF4->AddFrame(fButtonDate[4], fL6e);
   fButtonDate[4]->Resize(80,fButtonDate[4]->GetDefaultHeight());
   fButtonDate[4]->Associate(this);


   //
   // text box, right of buttons
   //
   
   fDateF5 = new TGCompositeFrame(fDateF3, 60, 20, kVerticalFrame);
   fDateF5->AddFrame(fDateTxt1 = new TGTextEntry(fDateF5, fDateNewNodeBuffer = new TGTextBuffer(110)), fL4);
   fDateTxt1->Resize(250, fDateTxt1->GetDefaultHeight());


   Char_t myDomain[100];
   Int_t domainNameStatus = getdomainname(myDomain,100);
   if (domainNameStatus != 0 || strlen(myDomain) < 2 || ( strncmp (myDomain,"(none)",6) == 0) ) {
     printf("could not get domain name. Set it to cern.ch. error=%d \n",domainNameStatus);
     //     cout << " could not get domain name. Set it to cern.ch. error= " << domainNameStatus <<" domain is= " << myDomain <<  endl;      
     sprintf(myDomain,"cern.ch");
   }
   //   sprintf(myDomain,"xcern.ch");
   if (gDebug) cout << " myDomain="<<myDomain<<endl; 
   if (gDebug)  printf("mydomain is %s  domainNameStatus is %d strlen(myDomain)  is %d \n",myDomain,domainNameStatus,strlen(myDomain) );
   sprintf(fDomainName,"%s",myDomain);
   fDatemonLabel                = new TGLabel*[1];
   fDatemonLabel[0] = new TGLabel(fDateF5, new TGHotString("domain name (e.g. cern.ch):"));
   fDatemonLabel[0]->Resize(200,fDatemonLabel[0]->GetDefaultHeight());
   fDateF5->AddFrame(fDatemonLabel[0],fL6b);

   fDateF5->AddFrame(fDateTxt2 = new TGTextEntry(fDateF5, fDomainNameBuffer = new TGTextBuffer(110)), fL4);
   fDateTxt2->Resize(250, fDateTxt2->GetDefaultHeight());
   fDomainNameBuffer->AddText(0,fDomainName);

   // should this be here??
   ShowStatusOfMonitoringSet();

   fDateF3->AddFrame(fDateF4,fL3);
   fDateF3->AddFrame(fDateF5,fL3);
   fCFTab->AddFrame(fDateF3, fL3);
}



////////////////////////////////////////////////////////////////////////
// SYSMON TAB
////////////////////////////////////////////////////////////////////////
void MainFrame::CreateTabSysmon()
{
   char configfile[256];
   fCFTab = fTab->AddTab("setup nodes:no DATE installed");
   fF3 = new TGCompositeFrame(fCFTab, 60, 20, kHorizontalFrame);
   //list box with list of nodes
   fF3->AddFrame(fListBox = new TGListBox(fF3, 89), fL7);


   if (fUseMysql) {
     sprintf(configfile,"AFFAIR_COMPUTERLIST");     
   } else {
     sprintf(configfile,"%s/config/sysmonlist.conf",fAffairMonitorPath);
   }
   FillListBox(configfile, fListBox);

   fListBox->Resize(150, 200);
   fListBox->SetMultipleSelections(1);

   //
   // buttons, right of list box
   //
   fF4 = new TGCompositeFrame(fF3, 60, 20, kVerticalFrame);

   fF4->AddFrame(fbt1 = new TGTextButton(fF4, "Add entry:", 102), fL4);
   fbt1->Associate(this);
   fbt1->Resize(80, fbt1->GetDefaultHeight());


   fF4->AddFrame(fbt2 = new TGTextButton(fF4, "Delete enty/ies ", 101),fL4);
   fbt2->Associate(this);
   fbt2->Resize(80, fbt2->GetDefaultHeight());

   fF4->AddFrame(fbt3 = new TGTextButton(fF4, "Edit file  ", 103),fL4);
   fbt3->Associate(this);
   fbt3->Resize(80, fbt3->GetDefaultHeight());

   //
   // stuff on bottom. sysmon control commands
   //
   fF4->AddFrame(fbt6 = new TGTextButton(fF4, "Set AFFAIR env. variable: ", 111), fL6);
   fbt6->Associate(this);
   fbt6->Resize(100, fbt6->GetDefaultHeight());
   fbt6->SetBackgroundColor(fBlue);
   fbt6->SetToolTipText("This is the AFFAIR environment variable on the remote nodes.If it does not exist on the remote node, it will be created. In the gui for the time being it is assumed to be always the same.");


   fF4->AddFrame(fbt7 = new TGTextButton(fF4, "Copy,untar and install using scp/ssh: ", 116), fL4);
   fbt7->Associate(this);
   fbt7->Resize(100, fbt7->GetDefaultHeight());
   fbt7->SetBackgroundColor(fYellow);
   fbt7->SetToolTipText("Just the file name,no path. Assume in AFFAIR_MONITOR directory. Setting up ssh to not use passwords makes life easier.See manual.");


   fF4->AddFrame(fbt5 = new TGTextButton(fF4, "Start via ssh ", 114), fL4);
   fbt5->Associate(this);
   fbt5->Resize(100, fbt5->GetDefaultHeight());
   fbt5->SetBackgroundColor(fGreen);
   fbt5->SetToolTipText("Setting up ssh to not use passwords makes life easier.See manual.");

   fF4->AddFrame(fbt8 = new TGTextButton(fF4, "Stop via ssh ", 115), fL4);
   fbt8->Associate(this);
   fbt8->Resize(100, fbt6->GetDefaultHeight());
   fbt8->SetBackgroundColor(fRed);
   fbt8->SetToolTipText("Kills all Collector processes on the remote nodes. Setting up ssh to not use passwords makes life easier.See manual.");

   //
   // text box, right of buttons
   //
   fF5 = new TGCompositeFrame(fF3, 60, 20, kVerticalFrame);
   fF5->AddFrame(fTxt3 = new TGTextEntry(fF5, fNewNodeBuffer = new TGTextBuffer(110)), fL4);
   fTxt3->Resize(250, fTxt3->GetDefaultHeight());

   fF5->AddFrame(fTxt4 = new TGTextEntry(fF5, fAffairPathBuffer = new TGTextBuffer(110)), fL6b);
   fTxt4->SetToolTipText("This is the AFFAIR environment variable on the remote nodes. \n In the gui for the time being it is assumed to be always the same.");
   fTxt4->Resize(250, fTxt4->GetDefaultHeight());
   fAffairPathBuffer->AddText(0,fAffairPath);

   fF5->AddFrame(fTxt6 = new TGTextEntry(fF5, fAffairCodeTarGzBuffer = new TGTextBuffer(110)), fL4);
   fTxt6->SetToolTipText("Just the file name,no path. Assume in AFFAIR_MONITOR directory.");
   fTxt6->Resize(250, fTxt6->GetDefaultHeight());
   fAffairCodeTarGzBuffer->AddText(0,fAffairCodeTarGz);

   fF3->AddFrame(fF4,fL3);
   fF3->AddFrame(fF5,fL3);
   fCFTab->AddFrame(fF3, fL3);
}



////////////////////////////////////////////////////////////////////////
// SETUP TAB
////////////////////////////////////////////////////////////////////////
void MainFrame::CreateTabSetup()
{
   fCFTab = fTab->AddTab("initial setup");
   Int_t buttonNumber = 11;

   fSetupLabel                = new TGLabel*[5];

   fSetupCompositeFrames      = new TGCompositeFrame*[3];
   fSetupButton               = new TGButton*[buttonNumber];
  
   fSetupCompositeFrames[F_1] = new TGCompositeFrame(fCFTab, 60, 20, kHorizontalFrame);
   fSetupCompositeFrames[F_2] = new TGCompositeFrame(fSetupCompositeFrames[F_1], 60, 20, kVerticalFrame);
   fSetupCompositeFrames[F_3] = new TGCompositeFrame(fSetupCompositeFrames[F_1], 60, 20, kVerticalFrame);

   fSetupButton[0]  = new TGTextButton(fSetupCompositeFrames[F_3],"Screen dump Contents of all tables",kSetupOffset+0 );
   fSetupButton[1]  = new TGTextButton(fSetupCompositeFrames[F_2],"Create And fill default tables  ", kSetupOffset+2);
   fSetupButton[1]->SetBackgroundColor(fGreen);
   fSetupButton[2]  = new TGTextButton(fSetupCompositeFrames[F_2],"monitored variables and plotting options for a monitoring set", kSetupOffset+3);
   fSetupButton[3]  = new TGTextButton(fSetupCompositeFrames[F_2],"parameters for individual nodes",kSetupOffset+4 );
   fSetupButton[4]  = new TGTextButton(fSetupCompositeFrames[F_2],"default parameters for a monitoring set",kSetupOffset+5 );
   fSetupButton[5]  = new TGTextButton(fSetupCompositeFrames[F_2],"rootfiles saved offline",kSetupOffset+6 );
   fSetupButton[6]  = new TGTextButton(fSetupCompositeFrames[F_2],"hostinfo",kSetupOffset+7 );
   fSetupButton[7]  = new TGTextButton(fSetupCompositeFrames[F_2],"rrdlist",kSetupOffset+8 );

   fSetupButton[0]->SetBackgroundColor(fYellow);
   fSetupButton[2]->SetBackgroundColor(fYellow);
   fSetupButton[3]->SetBackgroundColor(fYellow);
   fSetupButton[4]->SetBackgroundColor(fYellow);
   fSetupButton[5]->SetBackgroundColor(fYellow);
   fSetupButton[6]->SetBackgroundColor(fYellow);
   fSetupButton[7]->SetBackgroundColor(fYellow);

   fSetupButton[8]  = new TGTextButton(fSetupCompositeFrames[F_3],"upgrade tables from 3.0* ",kSetupOffset+9 );
   fSetupButton[9]  = new TGTextButton(fSetupCompositeFrames[F_3],"view/edit warning colors,timeouts etc",kSetupOffset+10 );
   fSetupButton[10]  = new TGTextButton(fSetupCompositeFrames[F_2],"Create a new monitoring set",kSetupOffset+11 );

   fSetupButton[8]->SetBackgroundColor(fYellow);
   fSetupButton[9]->SetBackgroundColor(fYellow);
   fSetupButton[10]->SetBackgroundColor(fYellow);

   for (Int_t i = 0; i < buttonNumber; i++) {
     fSetupButton[i]->Resize(80, fSetupButton[i]->GetDefaultHeight());
     fSetupButton[i]->Associate(this);
   }

   fSetupLabel[0] = new TGLabel(fSetupCompositeFrames[F_2], new TGHotString("Necessary initial database configuration"));
   fSetupLabel[1] = new TGLabel(fSetupCompositeFrames[F_2], new TGHotString("View/modify initially created tables"));
   fSetupLabel[2] = new TGLabel(fSetupCompositeFrames[F_2], new TGHotString("View/modify other tables"));
   fSetupLabel[3] = new TGLabel(fSetupCompositeFrames[F_3], new TGHotString("Other"));
   fSetupLabel[4] = new TGLabel(fSetupCompositeFrames[F_2], new TGHotString("New monitoring set"));

   fSetupLabel[0]->Resize(200,fSetupLabel[0]->GetDefaultHeight());
   fSetupLabel[0]->SetBackgroundColor(fPink);

   fSetupLabel[1]->Resize(200,fSetupLabel[1]->GetDefaultHeight());
   fSetupLabel[1]->SetBackgroundColor(fPink);

   fSetupLabel[2]->Resize(200,fSetupLabel[2]->GetDefaultHeight());
   fSetupLabel[2]->SetBackgroundColor(fPink);

   fSetupLabel[3]->Resize(200,fSetupLabel[2]->GetDefaultHeight());
   fSetupLabel[3]->SetBackgroundColor(fPink);

   fSetupLabel[4]->Resize(200,fSetupLabel[2]->GetDefaultHeight());
   fSetupLabel[4]->SetBackgroundColor(fPink);


   fSetupCompositeFrames[F_2]->AddFrame(fSetupLabel[0],fL4);
   fSetupCompositeFrames[F_2]->AddFrame(fSetupButton[1],fL4);

   fSetupCompositeFrames[F_2]->AddFrame(fSetupLabel[1],fL6d);
   fSetupCompositeFrames[F_2]->AddFrame(fSetupButton[2],fL4);
   fSetupCompositeFrames[F_2]->AddFrame(fSetupButton[4],fL4);

   fSetupCompositeFrames[F_2]->AddFrame(fSetupLabel[2],fL4);
   fSetupCompositeFrames[F_2]->AddFrame(fSetupButton[3],fL4);
   fSetupCompositeFrames[F_2]->AddFrame(fSetupButton[5],fL4);
   fSetupCompositeFrames[F_2]->AddFrame(fSetupButton[6],fL4);
   fSetupCompositeFrames[F_2]->AddFrame(fSetupButton[7],fL4);

   fSetupCompositeFrames[F_2]->AddFrame(fSetupLabel[4],fL4);
   fSetupCompositeFrames[F_2]->AddFrame(fSetupButton[10],fL4);


   // second column
   fSetupCompositeFrames[F_3]->AddFrame(fSetupLabel[3],fL4);
   fSetupCompositeFrames[F_3]->AddFrame(fSetupButton[0],fL4);
   fSetupCompositeFrames[F_3]->AddFrame(fSetupButton[8],fL4);
   fSetupCompositeFrames[F_3]->AddFrame(fSetupButton[9],fL4);


   fSetupCompositeFrames[F_1]->AddFrame(fSetupCompositeFrames[F_2], fL3);
   fSetupCompositeFrames[F_1]->AddFrame(fSetupCompositeFrames[F_3], fL3);
   fCFTab->AddFrame(fSetupCompositeFrames[F_1], fL3);
}

//
//
//
void MainFrame::CreateTabMessages()
{
   
   fCFTab = fTab->AddTab("log");

   fF1 = new TGCompositeFrame(fCFTab, 60, 20, kVerticalFrame);
   fTxtView1 = new TGTextView(fF1,400,200,kSunkenFrame);

   fF1->AddFrame(fTxtView1,fL5);


   sprintf(fTempText,"AFFAIR_MONITOR  is=%s",fAffairMonitorPath);
   OutputMessage(fTempText);
   fCFTab->AddFrame(fF1, fL5);
}


//
//
//
void MainFrame::Init()
{
   Char_t  *affairPath = 0;
   Char_t  *dimDnsNode = 0;

   fFillHistos = kFALSE;
   fPrintHistos = kFALSE;



  if (gDebug) printf(" AffairControl: Init 1 \n");


   affairPath = getenv("AFFAIR_MONITOR");

   if ( !affairPath) {
     DoWarning("You do not have AFFAIR_MONITOR defined! \n");
     exit(0);
   }
   else {
     sprintf(fAffairMonitorPath,getenv("AFFAIR_MONITOR"));
   }

  if (gDebug) printf(" AffairControl: Init 2 \n");


   fAffairSql = 0;
   if (fAffairSql != 0) delete fAffairSql;
   fAffairSql = new TAffairSql;

  if (gDebug) printf(" AffairControl: Init 3 \n");
   fUseMysql = 0;
   fUseMysql = fAffairSql->IsConfiguredMysql();

   if (gDebug >0) printf("Init. fUseMysql= %d \n",fUseMysql);

   if (fUseMysql) {
    fUseMysql = fAffairSql->OpenDatabase();
    sprintf(gDatabase,"%s",fAffairSql->GetDatabaseName());
    if(gDebug) printf("AffairControl.Init. sql databasse is %s \n",gDatabase);
   } 
   
   if (gDebug) printf(" AffairControl: Init 4 \n");

   if (!fUseMysql) {
     DoWarning("mysql not used. \n");
   }

   if (gDebug) printf(" AffairControl: Init 5 \n");
   if (gDebug) printf(" Init 2. fUseMysql= %d \n",fUseMysql );

   gClient->GetColorByName("lightpink", fLightPink);
   gClient->GetColorByName("lightgreen", fLightGreen);
   gClient->GetColorByName("pink", fPink);
   gClient->GetColorByName("white", fWhite);
   gClient->GetColorByName("red", fRed);
   gClient->GetColorByName("yellow", fYellow);
   gClient->GetColorByName("green", fGreen);
   gClient->GetColorByName("blue", fBlue);

   gClient->GetColorByName("LimeGreen", fRed2);
   gClient->GetColorByName("LawnGreen", fRed3);
   gClient->GetColorByName("lightcoral", fRed4);
   gClient->GetColorByName("honeydew", fGreenYellow);


   fL1 = new TGLayoutHints(kLHintsTop | kLHintsRight, 2, 2, 2, 2);
   fL3 = new TGLayoutHints(kLHintsTop | kLHintsLeft, 5,  5,  5, 5);
   fL4 = new TGLayoutHints(kLHintsTop | kLHintsLeft | kLHintsExpandX, 5,  5,  5, 5);
   fL5 = new TGLayoutHints(kLHintsBottom | kLHintsExpandX | kLHintsExpandY, 2, 2, 5, 1);
   fL5b = new TGLayoutHints(kLHintsBottom | kLHintsExpandX | kLHintsExpandY, 2, 2, 25, 1);
   fL6 =  new TGLayoutHints(kLHintsTop | kLHintsExpandX, 5, 5, 80, 5);
   fL6b=  new TGLayoutHints(kLHintsTop | kLHintsExpandX, 5, 5, 145, 5);
   fL6c=  new TGLayoutHints(kLHintsTop | kLHintsExpandX, 5, 5, 85, 5);
   fL6d=  new TGLayoutHints(kLHintsTop | kLHintsExpandX, 5, 5, 45, 5);
   fL6e=  new TGLayoutHints(kLHintsTop | kLHintsExpandX, 5, 5, 75, 5);
   fL7 =  new TGLayoutHints(kLHintsTop | kLHintsLeft |  kLHintsExpandY, 5,  5,  5, 5);

 

  if (gDebug) printf(" AffairControl: Init 6 \n");



  if (!fUseMysql) {
    Char_t  configfile[256];
    Int_t nodeCount;
    
    nodeCount = 0;
    FILE *nodeListFile;
    static char line[256];
    static char nodeName[256];
    sprintf(configfile,"%s/config/monsetDefault.conf",fAffairMonitorPath);
    
    if (gDebug >0) cout << " Init 4 " << endl;
    
    if ((nodeListFile = fopen(configfile, "r")) != NULL) {
      while (fgets(line, 256, nodeListFile) != NULL) {
	//ignore comments (if first or second char is#. Make this more robust in future) and newlines
	if (line[0] != '#' && line[1] != '#' && strlen(line) > 1 ) {
	  sscanf(line, "%s", &nodeName);
	  nodeCount++;
	}
      }
      fclose(nodeListFile);
    }
    
    gMonsetCount = nodeCount;
    
    gMonitorSetList = new string[nodeCount];
    nodeCount = 0;
    Char_t *ptr;
  if (gDebug) printf(" AffairControl: Init 7 \n");
    
    if ((nodeListFile = fopen(configfile, "r")) != NULL) {
      while (fgets(line, 256, nodeListFile) != NULL) {
	for(ptr=line;;ptr++){
	  if (*ptr==0) break;
	  if (!isspace(*ptr)) break;
	}       
	if (*ptr=='#') continue;
	if (strlen(line) < 2) continue;
	sscanf(line, "%s", &nodeName);
	gMonitorSetList[nodeCount] = nodeName;
	nodeCount++;
      }
      fclose(nodeListFile);
    }
  } else {
    // mysql case
    gMonitorSetList = fAffairSql->GetColumn("AFFAIR_MONSET_DEFAULTPARS",&gMonsetCount,0," ");
    if (gDebug) {
      cout << "Init. gMonsetCount = " <<gMonsetCount<< endl;
      for (Int_t i = 0; i < gMonsetCount; i++) {
	cout << " testtstr = " << gMonitorSetList[i].c_str() << endl;
      }
    }
  }

  if (gDebug) printf(" AffairControl: Init 8 \n");

   fTimePeriodList[0] = "20min";
   fTimePeriodList[1] = "hour";
   fTimePeriodList[2] = "6hour";
   fTimePeriodList[3] = "day";
   fTimePeriodList[4] = "7days";
   fTimePeriodList[5] = "month";
   //   fTimePeriodList[6] = "current";

   fBinDeltaFactorList[0] = 0.333333333;
   fBinDeltaFactorList[1] = 1;
   fBinDeltaFactorList[2] = 6;
   fBinDeltaFactorList[3] = 24;
   fBinDeltaFactorList[4] = 168;
   fBinDeltaFactorList[5] = 720;
   fbOneNodeOnly = false;


   dimDnsNode = getenv("DIM_DNS_NODE");

   if ( !dimDnsNode) {
     DoWarning("You do not have DIM_DNS_NODE defined!  Assume it will be this computer \n");
     sprintf(fDimDnsNode,dimDnsNode);
   }
   else {
     sprintf(fDimDnsNode,getenv("DIM_DNS_NODE"));
   }


   //sprintf(fAffairPath,"~/affairRC");
   sprintf(fCurrentVersion,"2.0");
   sprintf(fDateCodeTarGz,"affairDate%s.tar.gz",fCurrentVersion);
   sprintf(fAffairCodeTarGz,"affairCollector%s.tar.gz",fCurrentVersion);

   cc = 0;
   fPlotList = 0;
   fSelPlotId = -1;
  if (gDebug) printf(" AffairControl: Init end \n");
}

//
//
//
MainFrame::MainFrame(const TGWindow *p, UInt_t w, UInt_t h): TGMainFrame(p, w, h)
{

  if (gDebug) printf(" AffairControl: MainFrame 1 \n");

   Init();

  if (gDebug) printf(" AffairControl: MainFrame 2 \n");
   fMain  = new TGMainFrame(p, w, h);
  if (gDebug) printf(" AffairControl: MainFrame 3 \n");
   fMain->AddFrame(fbtExit = new TGTextButton(fMain, "Exit Affair Control", 501),fL1);
   fbtExit->SetBackgroundColor(fRed);
   fbtExit->Associate(this);
  if (gDebug) printf(" AffairControl: MainFrame 4 \n");


   //*************************************************************************
   //--------- create Tab widget
   //*************************************************************************

   fTab = new TGTab(fMain, 300, 300);
  if (gDebug) printf(" AffairControl: MainFrame 5 \n");
   CreateTabMonitor();
  if (gDebug) printf(" AffairControl: MainFrame 6 \n");
   CreateTabDatemon();
  if (gDebug) printf(" AffairControl: MainFrame 7 \n");
   CreateTabCanvas();
  if (gDebug) printf(" AffairControl: MainFrame 8 \n");
   //CreateTabSysmon();
   CreateTabSetup();
   //CreateTabMessages();
  if (gDebug) printf(" AffairControl: MainFrame 9 \n");

   fMain->AddFrame(fTab, fL5b);
  if (gDebug) printf(" AffairControl: MainFrame 10 \n");
   fMain->SetWindowName("AFFAIR control");
   fMain->MapSubwindows();
  if (gDebug) printf(" AffairControl: MainFrame 11 \n");
   fMain->Resize();
  if (gDebug) printf(" AffairControl: MainFrame 12 \n");
   fMain->MapWindow();
  if (gDebug) printf(" AffairControl: MainFrame 13 \n");
}



//
//
//
MainFrame::~MainFrame()
{
   // Delete all created widgets.


   delete fTab;
   delete fL1;
   delete fL3;
   delete fL4;
   delete fL5;
   delete fL5b;
   delete fL6;
   delete fL6b;
   delete fL7;
   delete fF1;
   delete fF3;
   delete fF4;
   delete fF5;
   delete fDateF3;
   delete fDateF4;
   delete fDateF5;
   delete fListBox;
   delete fbt1;
   delete fbt2;
   delete fbt3;
   delete fbt4;
   delete fbt5;
   delete fbt6;
   delete fbt7;
   delete fbt8;
   delete fbtExit;


   delete fTxtView1;
   delete fTxt3;
   delete fTxt4;
   delete fTxt6;
   delete fDateTxt1;
   delete fDateTxt2;

   delete fMonitorF1;
   delete fMonitorF2;
   delete fMonitorF3;
   delete fMonitorTxt1;

   delete fNewNodeBuffer;
   delete fAffairPathBuffer;
   delete fAffairCodeTarGzBuffer;
   delete fDateNewNodeBuffer;
   delete fDimDnsNodeBuffer;
   delete fMain;
   delete fMenuFile;

   delete fStartB;
   delete fStopB;
   delete fPrintB;
   delete [] fButtonDate; fButtonDate = 0;
   delete [] fButtonCanvas; fButtonCanvas = 0;
   delete [] fButtonMonitor; fButtonMonitor = 0;
   delete [] fCFCanvas; fCFCanvas = 0;
   delete [] fComboCanvas; fComboCanvas = 0;
   delete  fEc1;

}



void MainFrame::CloseWindow()
{
   // Got close message for this MainFrame. Terminates the application.
   gApplication->Terminate(0);
}

//
//
//
void MainFrame::FillListBox(char * configfile,  TGListBox * myListBox)
{

  //first remove all entries from list box
  // do this if have more than zero elements
  //kludge. assume not more than 10000 elements...

    myListBox->RemoveEntries(1,10000);
  Int_t nodeCount;
  nodeCount = 0;
  Char_t *ptr;

  if (fUseMysql) {
    fAffairSql->FillListBox(configfile,myListBox);    
  } else {    
    FILE *nodeListFile;
    static char line[256];
    static char nodeName[256];
    
    if ((nodeListFile = fopen(configfile, "r")) != NULL) {
      while (fgets(line, 256, nodeListFile) != NULL) {
	for(ptr=line;;ptr++){
	  if (*ptr==0) break;
	  if (!isspace(*ptr)) break;
	}       
	if (*ptr=='#') continue;
	if (strlen(line) < 2) continue;
	sscanf(line, "%s", &nodeName);
	nodeCount++;
	//	  fLastEntry = nodeCount;
	myListBox->AddEntry(nodeName,nodeCount);
      }
      /* Close serial file */
      fclose(nodeListFile);
    }
  }
  
  
  myListBox->MapSubwindows();
  myListBox->Layout();

}



Int_t MainFrame::FillHistos()
{
   // Fill histograms till user clicks "Stop Filling" button.
   // create rrdlist file
   Char_t   sysCommand[256];
   Char_t  *gAffairPath;
   Char_t   plotName[256];
   Int_t    returnInitValue;
   Int_t    selNode;

   gAffairPath = getenv("AFFAIR_MONITOR");

   if (fSelPlotId < 0 ) {
     fFillHistos = kFALSE;
     return 0;
   }

   if (!fUseMysql) {
     sprintf(sysCommand,"%s/support/listRRD.pl",gAffairPath);
     gSystem->Exec(sysCommand); 
   }
   Int_t binNumber = 120;
   Int_t binDeltaTimeHour = (Int_t) 3600/binNumber;

   gSumOrSuperimposedOption = k_SUM_AND_SUPERIMPOSE;

   if ( fbOneNodeOnly == true) {
     selNode = fComboCanvas[3]->GetSelected();
     if (fUseMysql) cc->SetMysql(fAffairSql);
     returnInitValue = cc->Init(fMonitorSet,binNumber,RRD_ONE_COMPUTER_OPTION,fNodeList[selNode],"none","each");
   }else {
     if (fUseMysql) cc->SetMysql(fAffairSql);
     returnInitValue = cc->Init(fMonitorSet,binNumber,RRD_GLOBAL_OPTION,"ONLINE_RRD_LIST","none","all");
   }

   sprintf(plotName,"%s_%s",fOnlinePlotBaseName,fTimePeriod);
  
   if (returnInitValue) {	      
     while (fFillHistos) {
       Int_t TimeNow  = (Int_t) (time(0)-20.0);    
       //       Int_t TimeNow  = (Int_t) (time(0));    
       cc->SetPlotName(plotName);
       if (fPrintHistos) {
	 cc->SetOutputMode(k_YES_EPS_AFFAIR_DIR);
       }

       gSystem->ProcessEvents();  // handle GUI events
       if (gDebug) printf("about to plot plots.fbOneNodeOnly=%d  \n",fbOneNodeOnly);
       if ( fbOneNodeOnly == true) {
	 cc->EachPlotsRrd(fTimePeriod,((Float_t)binDeltaTimeHour)*fBinDeltaFactor,12,TimeNow);  
	 if (gDebug) printf("b about to plot plots.fbOneNodeOnly=%d  \n",fbOneNodeOnly);
       }else {
	 cc->GlobalPlotsRrd(fTimePeriod,((Float_t)binDeltaTimeHour)*fBinDeltaFactor,12,TimeNow,gSumOrSuperimposedOption);
       }

       if (fPrintHistos) {
	 fPrintHistos = kFALSE;
	 cc->SetOutputMode(k_NO_EPS);	 
	 sprintf(fTempText,"Plot has been generated in AFFAIR_MONITOR directory(%s) with name %s",fAffairMonitorPath,plotName);
	 DoWarning(fTempText);
       }

       if ( fbOneNodeOnly == true) {
       }else {
	 (Int_t) cc->InitQuick("ONLINE_RRD_LIST","global");
       }

       gSystem->ProcessEvents();  // handle GUI events
       gSystem->Sleep(500);
       gSystem->ProcessEvents();  // handle GUI events
     }
   } 
   else {
     cout << " could not load root file. exiting." << endl;
   }	    

   return 1;
}


//
//
//
void MainFrame::DoWarning(Char_t * warningText)
{

   int buttons, retval;
   EMsgBoxIcon icontype = kMBIconStop;
   buttons = 4;
   //   new TGMsgBox(gClient->GetRoot(), fMain,"Warning",warningText,icontype, buttons, &retval);
   //   OutputMessage(warningText);
}



void MainFrame::RemoveEntryFromFile(char* fileName , char* nodeName) 
{

  if (fUseMysql) {
    //    fAffairSql->RemoveEntryFromTable(fileName,nodeName);    
  } else {    
    char  strCommand[256];
    char affairPath[256];
    sprintf(affairPath,getenv("AFFAIR_MONITOR"));
    sprintf(strCommand,"%s/support/removeEntryFromFile.pl %s %s",
	    affairPath,fileName,nodeName);
    system(strCommand);
  }
  return;
}


void MainFrame::AddEntryToFile(char* fileName , char* nodeName)
{

  if (gDebug) cout <<" AddEntryToFile.begin " << endl;

  if (fUseMysql) {
    fAffairSql->AddEntryToTable(fileName,nodeName);    
  } else {        
    char  strCommand[256];
    char  affairPath[256];
    sprintf(affairPath,getenv("AFFAIR_MONITOR"));
    
    sprintf(strCommand,"%s/support/fillLogfile.pl %s %s",
	    affairPath,fileName,nodeName);
    system(strCommand);
  }
  if (gDebug) cout <<" AddEntryToFile.end " << endl;
  return;
}



//
//
//
void MainFrame::ShowStatusOfMonitoringSet(void)
{

  Int_t entryId;
  Char_t entryName[200];
  TList *monitorSetList;
 
  // don't know how to loop thruogh unmarked list, so
  // mark all (don't know how to mark only shown, so mark up tp say 10000 entries)
  // and then loop through them and see which ones up, and change color if so, then
  // unmark all

  for (Int_t i = 1; i < 10000; ++i) {
    fMonitorSetListBox->Select(i);
  }
  
  //also, show status of offline stuff.kludge by deleting all entries and filling them
  // should really all be filled before...
  fNodeMonitoredStatusListBox->RemoveEntries(1,10000);
  fOfflineStorageListBox->RemoveEntries(1,10000);
  fWebPlotsListBox->RemoveEntries(1,10000);

    
  monitorSetList = new TList;
  fMonitorSetListBox->GetSelectedEntries(monitorSetList);
  monitorSetList->Print();
  
  TIter nextDateIter = monitorSetList->MakeIterator();
  TGTextLBEntry * dateObj;

  Char_t aux[256];        
  Char_t line[256];        
  Int_t monsetStatus;	    	
  Int_t monsetStatus2;	    	

  Int_t counter = 0;
  while ( (dateObj = (TGTextLBEntry *) nextDateIter()) ) {
    counter++;
    entryId = dateObj->EntryId();
    sprintf(entryName,dateObj->GetText()->GetString());

    sprintf(aux,"ps -weo command | grep AffairMonitor | grep -v grep | grep -ce ' '%s' ' ",entryName);	
    FILE *fp = gSystem->OpenPipe(aux,"r");
    if(fp != NULL) {
      // consistency check. Should get -1 if not exists, and 1 if exists
      fgets(line,256,fp);
    }
    else {		
      cout << " fp is null. This should not happen " << endl;
    }
    gSystem->ClosePipe(fp);
    monsetStatus = atoi(line);	    	
    
    //   printf("entryname is %s id is %d  line is %s monsetStatus is %d \n",entryName,entryId,line,monsetStatus);

    if (monsetStatus) {
      // dateObj->SetBackgroundColor(fGreen);
      Int_t   rrdup = GetActiveRrds(entryName);
      Int_t   nodesmonitored = GetNodeNumber();
      Char_t  messageOut[100]; 
      sprintf(messageOut,"%i/%i nodes monitored",rrdup,nodesmonitored);
      fNodeMonitoredStatusListBox->AddEntry(messageOut,counter);
      if (rrdup == nodesmonitored && nodesmonitored > 0) {
	fNodeMonitoredStatusListBox->GetEntry(counter)->SetBackgroundColor(fGreen);
      } else {
	fNodeMonitoredStatusListBox->GetEntry(counter)->SetBackgroundColor(fYellow);
      }
    } else {
      // dateObj->SetBackgroundColor(fPink);
      fNodeMonitoredStatusListBox->AddEntry("not monitored",counter);
      fNodeMonitoredStatusListBox->GetEntry(counter)->SetBackgroundColor(fPink);
    }


    //
    // now do offline status
    // 
    sprintf(aux,"ps -weo command | grep AffairMonitor | grep -v grep | grep -ce ' '%s' '2''",entryName);	
    fp = gSystem->OpenPipe(aux,"r");
    if(fp != NULL) {
      // consistency check. Should get -1 if not exists, and 1 if exists
      fgets(line,256,fp);
    }
    else {		
      cout << " fp is null. This should not happen " << endl;
    }
    gSystem->ClosePipe(fp);
    monsetStatus = atoi(line);	    	

    sprintf(aux,"ps -weo command | grep AffairMonitor | grep -v grep | grep -ce ' '%s' '1''",entryName);	
    fp = gSystem->OpenPipe(aux,"r");
    if(fp != NULL) {
      // consistency check. Should get -1 if not exists, and 1 if exists
      fgets(line,256,fp);
    }
    else {		
      cout << " fp is null. This should not happen " << endl;
    }
    gSystem->ClosePipe(fp);
    monsetStatus2 = atoi(line);	    	


    if (monsetStatus > 0 ||  monsetStatus2 > 0 ) {
      fOfflineStorageListBox->AddEntry("offline",counter);
      fOfflineStorageListBox->GetEntry(counter)->SetBackgroundColor(fGreen);
    } else {
      fOfflineStorageListBox->AddEntry("no offline",counter);
      fOfflineStorageListBox->GetEntry(counter)->SetBackgroundColor(fPink);
    }

    // now do web plot status
    sprintf(aux,"ps -weo command | grep AffairDataProcess | grep -v grep | grep -ce '-m '%s' -r RRD_GLOBAL -s ALL_PLOT_TYPES -p all'",entryName);	
    fp = gSystem->OpenPipe(aux,"r");
    if(fp != NULL) {
      // consistency check. Should get -1 if not exists, and 1 if exists
      fgets(line,256,fp);
    }
    else {		
      cout << " fp is null. This should not happen " << endl;
    }
    gSystem->ClosePipe(fp);
    monsetStatus = atoi(line);	    	
    if (monsetStatus) {
      fWebPlotsListBox->AddEntry("web plots",counter);
      fWebPlotsListBox->GetEntry(counter)->SetBackgroundColor(fGreen);
    } else {
      fWebPlotsListBox->AddEntry("no web plots",counter);
      fWebPlotsListBox->GetEntry(counter)->SetBackgroundColor(fPink);
    }

  }
  
  fMonitorSetListBox->SetMultipleSelections(0);
  fMonitorSetListBox->SetMultipleSelections(1);
 
  fNodeMonitoredStatusListBox->MapSubwindows();
  fNodeMonitoredStatusListBox->Layout();

  fOfflineStorageListBox->MapSubwindows();
  fOfflineStorageListBox->Layout();

  fWebPlotsListBox->MapSubwindows();
  fWebPlotsListBox->Layout();

  

}


void MainFrame::OutputMessage(Char_t * mytext)
{

  /*
  TGText txt;
  txt.LoadBuffer(mytext);
  fTxtView1->AddText(&txt);
  txt.LoadBuffer("-------------------------------");
  fTxtView1->AddText(&txt);
  */
}


Int_t  MainFrame::GetActiveRrds(Char_t * monset)
{

  Char_t aux[256];
  Char_t line[256];
  Int_t count = 0;

  sprintf(aux,"find %s/rrd/*%s*  -mmin -2 | wc -l ",fAffairMonitorPath,monset);	
  FILE *fp = gSystem->OpenPipe(aux,"r");
  if(fp != NULL) {
    // consistency check. Should get -1 if not exists, and 1 if exists
    fgets(line,256,fp);
  }
  else {		
    cout << " fp is null. This should not happen " << endl;
  }
  gSystem->ClosePipe(fp);
  count = atoi(line);	    	
  
  return(count);
}


Int_t  MainFrame::GetNodeNumber()
{

  Char_t sql[256];
  // Int_t fRowNumber = 0;
  Int_t rowNumber = 0;

  sprintf(sql,"SELECT * FROM AFFAIR_COMPUTERLIST ");	

  rowNumber = fAffairSql->GetNumberOfRowsSql(sql); 	    	
  //printf("GetNodeNumber. = %d sql = %s\n",fRowNumber,sql);
  return(rowNumber);
}




//
//
//
Bool_t MainFrame::ProcessMessage(Long_t msg, Long_t parm1, Long_t)
{
  // Handle different buttons.
  Int_t entryId;
  Char_t  entryName[128];
  char configfile[256];
  Char_t systemCmd[256];
  Int_t monOption = -1;
  Int_t offlineOption = -1;
  Char_t *ptr;
  FILE *table_fp;
  Char_t aux[256];        
  Char_t line[256];        
  Char_t sql[500];        


  // Process messages coming from widgets associated with the dialog.
  
  switch (GET_MSG(msg)) {
  case kC_COMMAND:
    
    switch (GET_SUBMSG(msg)) {
    case kCM_BUTTON:
      switch(parm1) {
	//
	// SYSMON STUFF
	//
      case 101:  // 
	{
	  TList *removeList;
	  removeList = new TList;
	  fListBox->GetSelectedEntries(removeList);
	  removeList->Print();     
	  //     TIter nextIter = removeList->MakeIterator();
	  TIter nextIter(removeList);
	  TGTextLBEntry * obj;

	  if (fUseMysql) {
	    sprintf(configfile,"AFFAIR_COMPUTERLIST");     
	  } else {
	    sprintf(configfile,"%s/config/sysmonlist.conf",fAffairMonitorPath);
	  }
	  
	  while ( (obj = (TGTextLBEntry *)nextIter()) ) {
	    entryId = obj->EntryId();
	    sprintf(entryName,obj->GetText()->GetString());	 
	    sprintf(fTempText,"entry to be deleted  for sysmon is %s",entryName);
	    OutputMessage(fTempText);
	    if (fUseMysql) {
	      sprintf(sql,"delete from %s.%s where  computer = '%s'",gDatabase,configfile,entryName);
	      fAffairSql->Exec(sql);
	    }else {
	      RemoveEntryFromFile(configfile,entryName);
	    }
	  }       
	  FillListBox(configfile, fListBox);
	}
	break;
	
      case 102:  
	if (fUseMysql) {
	  sprintf(configfile,"AFFAIR_COMPUTERLIST");     
	} else {
	  sprintf(configfile,"%s/config/sysmonlist.conf",fAffairMonitorPath);
	}

	sprintf(entryName,fNewNodeBuffer->GetString());
	AddEntryToFile(configfile,entryName);
	FillListBox(configfile, fListBox);
	sprintf(fTempText,"entry to be added for sysmon is %s",entryName);
	OutputMessage(fTempText);
	break;
	
      case 103:  //    
	if (fUseMysql) {
	  sprintf(configfile,"AFFAIR_COMPUTERLIST");     
	} else {
	  sprintf(configfile,"%s/config/sysmonlist.conf",fAffairMonitorPath);
	}

	sprintf(systemCmd,"emacs %s",configfile);
	gSystem->Exec(systemCmd);
	sprintf(fTempText,"manually editing %s",configfile);
	OutputMessage(fTempText);
	
	FillListBox(configfile, fListBox);
	
	break;
		
	//Sysmon control stuff	
	
      case 111:  //    
	sprintf(fAffairPath,fAffairPathBuffer->GetString());
	sprintf(fTempText,"AFFAIR environment variable is %s",fAffairPath);
	OutputMessage(fTempText);
	break;


      case 112:  //    
	
	if (strlen(fAffairPath) <= 1 ) {
	  DoWarning("You do not have AFFAIR for the remote nodes defined!");
	} else if (strlen(fAffairCodeTarGzBuffer->GetString()) <= 1 ) {
	  DoWarning("You do not have the affair tar.gz file defined!!");
	} else {
	  sprintf(fAffairCodeTarGz,fAffairCodeTarGzBuffer->GetString());
	  sprintf(configfile,"%s/config/sysmonlist.conf",fAffairMonitorPath);
	  sprintf(systemCmd,"%s/support/connectAllNodes.sh %s copyAndUntar  %s %s >& %s/log/runControl_untar.log",fAffairMonitorPath,configfile,fAffairPath,fAffairCodeTarGz,fAffairMonitorPath);
	  gSystem->Exec(systemCmd);
	  sprintf(fTempText,"copying and untarring affair monitoring on nodes in %s. command is: %s ",configfile,systemCmd);
	  OutputMessage(fTempText);
	}
	  break;
	
	
      case 113:  //    
	
	if (strlen(fAffairPath) <= 1 ) {
	  DoWarning("You do not have AFFAIR for the remote nodes defined!");
	} else {
	  sprintf(configfile,"%s/config/sysmonlist.conf",fAffairMonitorPath);
	  sprintf(systemCmd,"%s/support/connectAllNodes.sh %s install %s   >& %s/log/runControl_install.log",fAffairMonitorPath,
		  configfile,fAffairPath,fAffairMonitorPath);
	  gSystem->Exec(systemCmd);
	  sprintf(fTempText,"installing SYSMON affair monitoring on nodes in %s. command is %s",configfile,systemCmd);
	  OutputMessage(fTempText);
	}
	break;
	
		
      case 114:  //   
	if (strlen(fDimDnsNode) <= 1 ) {
	  DoWarning("You do not have DIM_DNS_NODE defined! You can do that in the Monitor tab.");
	}  else if (strlen(fAffairPath) <= 1 ) {
	  DoWarning("You do not have AFFAIR for the remote nodes defined!");
	}  else {
	  sprintf(configfile,"%s/config/sysmonlist.conf",fAffairMonitorPath);
	  sprintf(systemCmd,"%s/support/connectAllNodes.sh %s run  %s %s >& %s/log/runControl_run.log ",fAffairMonitorPath,configfile,
		  fAffairPath,fDimDnsNode,fAffairMonitorPath);
	  gSystem->Exec(systemCmd);
	  sprintf(fTempText,"starting SYSMON affair monitoring on nodes in %s. command is %s",configfile,systemCmd);
	  OutputMessage(fTempText);
	}
	break;
	
	
      case 115:  //
	
	if ( strlen(fAffairPath)  <= 1 ) {
	  DoWarning("You do not have AFFAIR for the remote nodes defined!");
	} else {    
	  sprintf(configfile,"%s/config/sysmonlist.conf",fAffairMonitorPath);
	  sprintf(systemCmd,"%s/support/connectAllNodes.sh %s stop %s   >& %s/log/runControl_stop.log ",fAffairMonitorPath,configfile,fAffairPath,fAffairMonitorPath);
	  gSystem->Exec(systemCmd);
	  sprintf(fTempText,"stopping SYSMON affair monitoring on nodes in %s. command is %s",configfile,systemCmd);
	  OutputMessage(fTempText);
	}
	break;

      case 116:  //    	
	if (strlen(fAffairPath)  <= 1 ) {
	  DoWarning("You do not have AFFAIR for the remote nodes defined!");
	} else if (strlen(fAffairCodeTarGzBuffer->GetString())  <= 1 ) {
	  DoWarning("You do not have the affair tar.gz file defined!!");
	} else {    
	  sprintf(fAffairCodeTarGz,fAffairCodeTarGzBuffer->GetString());
	  sprintf(configfile,"%s/config/sysmonlist.conf",fAffairMonitorPath);
	  sprintf(systemCmd,"%s/support/connectAllNodes.sh %s copyAndUntarAndInstall  %s %s >& %s/log/runControl_untar_install.log",fAffairMonitorPath,configfile,fAffairPath,fAffairCodeTarGz,fAffairMonitorPath);
	  gSystem->Exec(systemCmd);
	  sprintf(fTempText,"copying and untarring affair monitoring on nodes in %s. command is: %s ",configfile,systemCmd);
	  OutputMessage(fTempText);
	}
	  break;
		       	
	//
	// DATEMON STUFF
	//
	
      case 201:  //
	{
	  TList *removeDateList;
	  removeDateList = new TList;
	  fDateListBox->GetSelectedEntries(removeDateList);
	  removeDateList->Print();
	  
	  TIter nextDateIter = removeDateList->MakeIterator();
	  TGTextLBEntry * dateObj;

	  if (fUseMysql) {
	    sprintf(configfile,"AFFAIR_COMPUTERLIST");     
	  } else {
	    sprintf(configfile,"%s/config/computerlist.conf",fAffairMonitorPath);
	  }
	  
	  while ( (dateObj = (TGTextLBEntry *) nextDateIter()) ) {
	    entryId = dateObj->EntryId();
	    sprintf(entryName,dateObj->GetText()->GetString());
	    sprintf(fTempText,"entry to be deleted for DATEmon is %s",entryName);
	    OutputMessage(fTempText);
	    if (fUseMysql) {
	      sprintf(sql,"delete from %s.%s where  computer = '%s'",gDatabase,configfile,entryName);
	      fAffairSql->Exec(sql);
	    }else {
	      RemoveEntryFromFile(configfile,entryName);
	    }
	  }       
	  FillListBox(configfile, fDateListBox);
	}
	break;
	
      case 202:  //      
	if (fUseMysql) {
	  sprintf(configfile,"AFFAIR_COMPUTERLIST");     
	} else {
	  sprintf(configfile,"%s/config/computerlist.conf",fAffairMonitorPath);
	}
	sprintf(entryName,fDateNewNodeBuffer->GetString());
	sprintf(fTempText,"entry to be added for DATEMON is %s",entryName);
	OutputMessage(fTempText);
	AddEntryToFile(configfile,entryName);
	FillListBox(configfile, fDateListBox);
	break;
	
      case 203:  //    
	//

	if (fUseMysql) {
	  sprintf(configfile,"%s/log/tmpnodes.txt",fAffairMonitorPath);
	  sprintf(systemCmd,"rm -f %s;emacs %s",configfile,configfile);
	  gSystem->Exec(systemCmd);
	  // add list to mysql table
	  // loop over file and add entry
	  table_fp = fopen(configfile,"r");        
	  while (fgets(line,256,table_fp)) {		
	    for(ptr=line;;ptr++){
	      if (*ptr==0)        break;
	      if (!isspace(*ptr)) break;
	    }       
	    if (*ptr=='#')        continue;
	    if (strlen(line) < 2) continue;
	    sscanf(line,"%s",&entryName[0]);
	    AddEntryToFile("AFFAIR_COMPUTERLIST",entryName);
	  }
	  FillListBox("AFFAIR_COMPUTERLIST", fDateListBox);
	  fclose(table_fp);
	  
	} else {
	  sprintf(configfile,"%s/config/computerlist.conf",fAffairMonitorPath);
	  sprintf(systemCmd,"emacs %s",configfile);
	  gSystem->Exec(systemCmd);
	  sprintf(fTempText,"manually editing %s",configfile);
	  OutputMessage(fTempText);
	  FillListBox(configfile, fDateListBox);       
	}

	break;
	     
      case 204:  //    

	if (fUseMysql) {
	  if (strlen(fDomainNameBuffer->GetString()) <= 1 ) {
	    DoWarning("You do not have the domain name defined");
	  } else {
	    sprintf(fDomainName,fDomainNameBuffer->GetString());
	    fAffairSql->FillComputerListTableFromRolesTable(fDomainName);	  
	    FillListBox("AFFAIR_COMPUTERLIST", fDateListBox);  
	  }
	} else {
	  cout << " only works if mysql used " << endl;
	}

	break;

      case 205:  //    

	if (strlen(fDimDnsNode) <= 1 ) {
	  DoWarning("You do not have DIM_DNS_NODE defined! You can do that in the Monitor tab.");
	} else {
	  sprintf(systemCmd,"%s/support/installDate.sh setDimNode %s  >& %s/log/date_setDimNode.log ",fAffairMonitorPath,fDimDnsNode,fAffairMonitorPath);
	  gSystem->Exec(systemCmd);
	  sprintf(fTempText,"setting the DIM_DNS_NODE for system monitoring to  %s  in /date/affairRC ",fDimDnsNode);
	  OutputMessage(fTempText);
	}

	break;

	
      case 206:  //
	{
	  
	  for (Int_t i = 1; i < 10000; ++i) {
	    fDateListBox->Select(i);
	  }
	  
	  TList *removeDateList;
	  removeDateList = new TList;
	  fDateListBox->GetSelectedEntries(removeDateList);
	  removeDateList->Print();
	  
	  TIter nextDateIter = removeDateList->MakeIterator();
	  TGTextLBEntry * dateObj;

	  if (fUseMysql) {
	    sprintf(configfile,"AFFAIR_COMPUTERLIST");     
	  } else {
	    sprintf(configfile,"%s/config/computerlist.conf",fAffairMonitorPath);
	  }
	  
	  while ( (dateObj = (TGTextLBEntry *) nextDateIter()) ) {
	    entryId = dateObj->EntryId();
	    sprintf(entryName,dateObj->GetText()->GetString());
	    sprintf(fTempText,"entry to be deleted for DATEmon is %s",entryName);
	    OutputMessage(fTempText);
	    if (fUseMysql) {
	      sprintf(sql,"delete from %s.%s where  computer = '%s'",gDatabase,configfile,entryName);
	      fAffairSql->Exec(sql);
	    }else {
	      RemoveEntryFromFile(configfile,entryName);
	    }
	  }       
	  FillListBox(configfile, fDateListBox);
	}
	break;


       	//
	// Monitor TAB
	//
      case 301:  //    	
	sprintf(fDimDnsNode,fDimDnsNodeBuffer->GetString());
	sprintf(fTempText,"DIM_DNS_NODE is %s",fDimDnsNode);
	OutputMessage(fTempText);
	
	break;
	
	
      case 302:  //    
	{
	
	  printf("strlen= %d \n",strlen(fDimDnsNode));
	if (strlen(fDimDnsNode) <= 1 ) {
	  DoWarning("You do not have DIM_DNS_NODE defined!");
	}  else {
	  printf("a \n");

	  for (Int_t i = 0; i < 2; ++i) {
	    if (fRBPickMonitorMode[i]->GetState() == kButtonDown){
	      monOption = i; 
	      break; 
	    }
	  }
	  	  
	  for (Int_t i = 0; i < 3; ++i) {	    
	    if (fRBPickOfflineMode[i]->GetState() == kButtonDown){
	      offlineOption = i; 
	      break; 
	    }
	  }
	  
	  
	  TList *monitorSetList;
	  monitorSetList = new TList;
	  fMonitorSetListBox->GetSelectedEntries(monitorSetList);
	  monitorSetList->Print();
	  
	  TIter nextDateIter = monitorSetList->MakeIterator();
	  TGTextLBEntry * dateObj;
	  
	  Int_t storeDataFlag = -1;
	  Char_t verboseOption[100];
	  
	  if (offlineOption == 0) storeDataFlag = 1 ;
	  if (offlineOption == 1) storeDataFlag = 0 ;
	  if (offlineOption == 2) storeDataFlag = 2 ;

	  Int_t  selmoncount = 0;
	  
	  printf("ba \n");

	  while ( (dateObj = (TGTextLBEntry *) nextDateIter()) ) {
	  printf("ca \n");

	    selmoncount++;
	    entryId = dateObj->EntryId();
	    sprintf(entryName,dateObj->GetText()->GetString());
	    sprintf(fTempText,"monitor set to start monitoring is %s",entryName);
	    OutputMessage(fTempText);	    

	    sprintf(verboseOption," ");
	    if (gDebug) sprintf(verboseOption,"verbose");
	    printf("about to start monitoring for %s monitoring set \n",entryName);

	    if (storeDataFlag == 0 || storeDataFlag == 1) {
	      sprintf(systemCmd,"source %s/startProcess.sh restartMonitor dummy %s %i %s &",fAffairMonitorPath,entryName,storeDataFlag,verboseOption);	    
	      gSystem->Exec(systemCmd);
	    }

	    if (storeDataFlag == 2) {
	      sprintf(systemCmd,"source %s/startProcess.sh restartMonitor dummy %s 0 %s &",fAffairMonitorPath,entryName,verboseOption);	    
	      gSystem->Exec(systemCmd);

	      sprintf(systemCmd,"source %s/startProcess.sh restartMonitor dummy %s 2 %s &",fAffairMonitorPath,entryName,verboseOption);	    
	      gSystem->Exec(systemCmd);
	    }

	    //sprintf(systemCmd,"source %s/startProcess.sh dim &",fAffairMonitorPath);	    
	    //gSystem->Exec(systemCmd);
	  printf("a mpon opt =%d  \n",monOption);
	    if ( monOption == 0 ) {
	      sprintf(verboseOption," ");
	      printf("about to start plot creation for %s monitoring set \n",entryName);
	      sprintf(systemCmd,"source %s/startProcess.sh restartDataProcessor %s %s &",fAffairMonitorPath,entryName,verboseOption);	    
	      gSystem->Exec(systemCmd);
	    }
	  }

	  if (selmoncount == 0) printf("You need to select a monitoring set! \n");
       
	}

	//
	ShowStatusOfMonitoringSet();
	}
	break;

      case 303:  //    

	if (strlen(fDimDnsNode) <= 1 ) {
	  DoWarning("You do not have DIM_DNS_NODE defined!");
	}  else {
	  

	  printf("about to kill processes for LDC monitoring set \n");
	  sprintf(systemCmd,"%s/support/killProcess.pl AffairMonitor LDC",fAffairMonitorPath);
	  gSystem->Exec(systemCmd);
	  
	  printf("about to kill processes for DATEMON monitoring set \n");
	  sprintf(systemCmd,"%s/support/killProcess.pl AffairMonitor DATEMON ",fAffairMonitorPath);
	  gSystem->Exec(systemCmd);
	  
	  printf("about to kill processes for GDC monitoring set \n");
	  sprintf(systemCmd,"%s/support/killProcess.pl AffairMonitor GDC ",fAffairMonitorPath);
	  gSystem->Exec(systemCmd);
	  
	  
	  sprintf(systemCmd,"%s/support/killProcess.pl AffairDataProcess LDC ",fAffairMonitorPath);
	  gSystem->Exec(systemCmd);	    
	  
	  sprintf(systemCmd,"%s/support/killProcess.pl AffairDataProcess DATEMON ",fAffairMonitorPath);
	  gSystem->Exec(systemCmd);	    
	  
	  sprintf(systemCmd,"%s/support/killProcess.pl AffairDataProcess GDC ",fAffairMonitorPath);
	  gSystem->Exec(systemCmd);	    

	  
	  printf("about to start monitoring for LDC, GDC, and DATEMON monitoring sets, with offline storage, and plots  \n ");
	  sprintf(systemCmd,"source %s/startProcess.sh monitor",fAffairMonitorPath);
	
	  //	  if ( monOption == 0  &&  offlineOption == 1) sprintf(systemCmd,"source %s/startProcess.sh monitor_nopermstore",fAffairMonitorPath);
	  //if ( monOption == 1  &&  offlineOption == 1) sprintf(systemCmd,"source %s/startProcess.sh monitor_noweb_nopermstore",fAffairMonitorPath);
	  //if ( monOption == 1  &&  offlineOption == 0) sprintf(systemCmd,"source %s/startProcess.sh monitor_noweb",fAffairMonitorPath);
	  
	  
	  gSystem->Exec(systemCmd);
	  sprintf(fTempText,"starting monitoring for DATE nodes. command is %s",systemCmd);
	  OutputMessage(fTempText);
	}	
	ShowStatusOfMonitoringSet();
	break;
	
      case 304:  //    
	///
	///
	{
	  TList *monitorSetList;
	  monitorSetList = new TList;
	  fMonitorSetListBox->GetSelectedEntries(monitorSetList);
	  monitorSetList->Print();
	  
	  TIter nextDateIter = monitorSetList->MakeIterator();
	  TGTextLBEntry * dateObj;
	  
	  
	  Int_t  selmoncount = 0;
	  while ( (dateObj = (TGTextLBEntry *) nextDateIter()) ) {
	    selmoncount++;

	    entryId = dateObj->EntryId();
	    sprintf(entryName,dateObj->GetText()->GetString());
	    
	    printf("about to kill processes for %s monitoring set \n",entryName);
	    sprintf(systemCmd,"%s/support/killProcess.pl AffairMonitor %s ",fAffairMonitorPath,entryName);
	    gSystem->Exec(systemCmd);
	    
	    
	    sprintf(systemCmd,"%s/support/killProcess.pl AffairDataProcess %s ",fAffairMonitorPath,entryName);
	    gSystem->Exec(systemCmd);	    
	  }       
	  if (selmoncount == 0) printf("You need to select a monitoring set! \n");
	}
	////
	
	//	sprintf(fTempText,"killing AFFAIR monitoring processes. command is %s",systemCmd);
	//OutputMessage(fTempText);
	
	
	//sprintf(fTempText,"killing AFFAIR monitoring processes. command is %s",systemCmd);
	// OutputMessage(fTempText);
	
	ShowStatusOfMonitoringSet();
	break;
	
      case 309:  //    
	ShowStatusOfMonitoringSet();
	break;

      case 501:  //    
	
	int retval;
	
	new TGMsgBox(gClient->GetRoot(), fMain, "Warning",  "Are you sure you want to exit Affair control?",
		     kMBIconQuestion, 3, &retval);
	if ( retval == 1) {
	  CloseWindow();
	} else {
	}
	break;

      case 601:  // start histogram filling
	fFillHistos = kTRUE;      
	FillHistos();
	break;
      case 602:  // stop histogram filling
	fFillHistos = kFALSE;
	break;
      case 603:  // print histogram
	fPrintHistos = kTRUE;
	break;
      case 604:  // change time period
	{
	
	fFillHistos = kFALSE;
	//	TGLBEntry * selTime = fComboB->GetSelectedEntry();
        Int_t selTime = fComboCanvas[0]->GetSelected();
	sprintf(fTimePeriod,fTimePeriodList[selTime].c_str());
	fBinDeltaFactor = fBinDeltaFactorList[selTime];


	fFillHistos = kTRUE;
	FillHistos();
	}
	break;	

      case 605:  // change monitor set
	{

	fFillHistos = kFALSE;
        Int_t selMSet = fComboCanvas[1]->GetSelected();
	sprintf(fMonitorSet,gMonitorSetList[selMSet].c_str());
	SetPlotList(kGlobalScope);
	SetNodeList();

        fSelPlotId = fComboCanvas[2]->GetSelected();
	
	if (fSelPlotId >=0) {
	  sprintf(fOnlinePlotBaseName,fPlotList[fSelPlotId].c_str());
	  sprintf(fTempText,"new set name  %s plot base name is %s ",fMonitorSet,fOnlinePlotBaseName);
	  OutputMessage(fTempText);	  	  
	  fFillHistos = kTRUE;
	  FillHistos();
	}
	}
	break;

      case 606:  // change plot
	{
	  fFillHistos = kFALSE;
	  
	  fSelPlotId = fComboCanvas[2]->GetSelected();
	  if (fSelPlotId >=0) {
	    sprintf(fOnlinePlotBaseName,fPlotList[fSelPlotId].c_str());
	    sprintf(fTempText,"new set name  %s plot base name is %s ",fMonitorSet,fOnlinePlotBaseName);
	    OutputMessage(fTempText);
	    
	    fFillHistos = kTRUE;
	    FillHistos();
	  }
	}
	break;


      case 607:  // individual node
	{
	fFillHistos = kFALSE;

        Int_t selNode = fComboCanvas[3]->GetSelected();
	if (selNode == 0) {
	  fbOneNodeOnly = false;
	  SetPlotList(kGlobalScope);
	} else {
	  fbOneNodeOnly = true;
	  SetPlotList(kOneNodeScope);
	}

	if (fSelPlotId >=0) {
	  fFillHistos = kTRUE;
	  FillHistos();
	}
	}
	break;	
	//
	// SETUP TAB
	//
      case (kSetupOffset+0): //set
	{
	  if (fUseMysql) {
	    fAffairSql->ShowContentsAllTables();
	  }
	}
	break;	
      
      case (kSetupOffset+2): //set
	{
	  if (fUseMysql) {
	    int retval;	    
	    new TGMsgBox(gClient->GetRoot(), fMain, "Warning",  "Are you sure you want to delete all previous Affair mysql tables, and create and fill them with default values?",
	    		 kMBIconQuestion, 3, &retval);
	    if ( retval == 1) {
	      fAffairSql->CreateDefaultTables();
	      new TGMsgBox(gClient->GetRoot(), fMain, "Warning",  "Please stop Affair Control and restart it, for the changes to take effect.",
	      		   kMBIconExclamation, 4, &retval);
	    }
	  }
	}
	break;	

      case (kSetupOffset+3): //set
	{
	  if (fUseMysql) {
	    new EditConfigTables(gClient->GetRoot(), this, 1400, 1220, fAffairSql,kSetupMonitorSet);
	  } else {
	    cout << " this option only works if mysql is used " << endl;
	  }
	}
	break;	

      case (kSetupOffset+4): //set
	{
	  if (fUseMysql) {
	    new EditConfigTables(gClient->GetRoot(), this, 1400, 1220, fAffairSql,kSetupComputerPars);
	  } else {
	    cout << " this option only works if mysql is used " << endl;
	  }
	}
	break;	

      case (kSetupOffset+5): //set
	{
	  if (fUseMysql) {
	    new EditConfigTables(gClient->GetRoot(), this, 1400, 1220, fAffairSql,kSetupMonitorSetDefaults);
	  } else {
	    cout << " this option only works if mysql is used " << endl;
	  }
	}
	break;	


      case (kSetupOffset+6): //set
	{
	  if (fUseMysql) {
	    new EditConfigTables(gClient->GetRoot(), this, 1400, 1220, fAffairSql,kSetupRootfiles);
	  } else {
	    cout << " this option only works if mysql is used " << endl;
	  }
	}
	break;	


      case (kSetupOffset+7): //set
	{
	  if (fUseMysql) {
	    new EditConfigTables(gClient->GetRoot(), this, 1400, 1220, fAffairSql,kSetupHostinfo);
	  } else {
	    cout << " this option only works if mysql is used " << endl;
	  }
	}
	break;	


      case (kSetupOffset+8): //set
	{
	  if (fUseMysql) {
	    new EditConfigTables(gClient->GetRoot(), this, 1400, 1220, fAffairSql,kSetupRrdinfo);
	  } else {
	    cout << " this option only works if mysql is used " << endl;
	  }
	  
	}
	break;	



      case (kSetupOffset+9): //set
	{
	  if (fUseMysql) {
	    fAffairSql->UpgradeTablesForAffairVersion315();

	  } 
	}
	break;	

      case (kSetupOffset+10): //set
	{
	  // new SetupWebPasswords(gClient->GetRoot(), this, 1400, 1220);	  
	  if (fUseMysql) {
	    new EditConfigTables(gClient->GetRoot(), this, 1400, 1220, fAffairSql,kSetupWebPars);
	  } else {
	    cout << " this option only works if mysql is used " << endl;
	  }
	}
	break;	


      case (kSetupOffset+11): //set
	{
	  // new SetupWebPasswords(gClient->GetRoot(), this, 1400, 1220);	  
	  if (fUseMysql) {
	    new EditConfigTables(gClient->GetRoot(), this, 1400, 1220, fAffairSql,kSetupCreateNewMonitoringSet);
	  } else {
	    cout << " this option only works if mysql is used " << endl;
	  }
	}
	break;	
	
      }
      
      




    }
    
  case kCM_RADIOBUTTON:
  case kCM_CHECKBUTTON:
      if ( parm1 == 305 || parm1 == 306) {
	if (fRBPickMonitorMode[0]->WidgetId() != parm1) {
	  fRBPickMonitorMode[0]->SetState(kButtonUp); 
	}
	if (fRBPickMonitorMode[1]->WidgetId() != parm1) {
	  fRBPickMonitorMode[1]->SetState(kButtonUp); 
	}
      } 

      if ( parm1 == 307 || parm1 == 308 || parm1 == 310 || parm1 == 311) {
	if (fRBPickOfflineMode[0]->WidgetId() != parm1) {
	  fRBPickOfflineMode[0]->SetState(kButtonUp); 
	}
	if (fRBPickOfflineMode[1]->WidgetId() != parm1) {
	  fRBPickOfflineMode[1]->SetState(kButtonUp); 
	}
	if (fRBPickOfflineMode[2]->WidgetId() != parm1) {
	  fRBPickOfflineMode[2]->SetState(kButtonUp); 
	}
	//	if (fRBPickOfflineMode[3]->WidgetId() != parm1) {
	//  fRBPickOfflineMode[3]->SetState(kButtonUp); 
	//	}
      } 
    break;
  default:
    break;
  }
  return kTRUE;
}





int main(int argc, char **argv)
{
   gDebug = 0;

   TApplication theApp("App", &argc, argv);

   if ( argc > 1) {
     for(Int_t i=1; i<argc; i++) {
       if ( strncmp(argv[i],"-V", 2 ) == 0 ) {
	 gDebug = 1;
       }
     }   
   }

   printf(" AffairControl: gDebug  is %d argc is %d \n",gDebug,argc );
   if (gDebug) printf(" AffairControl: main 1 \n");



   if (gDebug) printf(" AffairControl: main 2 \n");
   if (gROOT->IsBatch()) {
      fprintf(stderr, "%s: cannot run in batch mode\n", argv[0]);
      return 1;
   }



   if (gDebug) printf(" AffairControl: main 3 \n");



   MainFrame mainWindow(0, 1400, 1220);
   if (gDebug) printf(" AffairControl: main 4 \n");
   theApp.Run();
   if (gDebug) printf(" AffairControl: main 5 \n");

   return 0;
}
