#include "AffairMonitor.h"
#include "TAffairSql.h"

Char_t     gDatabase[128];

class TOperateFile:public TObject {
private:
    const static    Int_t kMaxVariables = 100;
    TFile          *fFile;
    Int_t           fLoopCount;
          
    TTree          *fTree;   //pointer to the analyzed TTree or TChain
    TTree          *fTreeTimeRange;   //pointer to the analyzed TTree or TChain
    Char_t          fFileName[256];
    Bool_t          fbTakeData;
    Int_t           fMaxIterations;
    
   Char_t           NameAndMonitorSet[256];
   UInt_t           MonitorTime;
   UInt_t           OrigMonitorTime;
   UInt_t           nVariables; 
   Float_t          MonitorValue[kMaxVariables];

   UInt_t           BeginTime;
   UInt_t           EndTime;
   Char_t           NameAndMonitorSet2[256];
   Char_t           Name[128];
   Char_t           MonitorSet[128];

   Char_t          fNodeName[256];
   Char_t          fMonitorSet[256];
   TAffairSql     *fAffairSql;
   Bool_t          fUseMysql;
   

   //List of branches
   TBranch        *b_NameAndMonitorSet;
   TBranch        *b_MonitorTime;  
   TBranch        *b_OrigMonitorTime;  
   TBranch        *b_nVariables;  
   TBranch        *b_MonitorValue;

   //
   TBranch        *b_BeginTime;  
   TBranch        *b_EndTime;  
   TBranch        *b_NameAndMonitorSet2;
   TBranch        *b_Name;
   TBranch        *b_MonitorSet;

public:

   TOperateFile();
   ~TOperateFile();
   void	  SaveNewData(Char_t * hnameAndMonset,Int_t arraySize,Double_t * buf, Int_t timeNow);
   Int_t   OpenFile(Char_t * monset);
   void   CloseFile();
   void    SetTakeData(Bool_t takeDataFlag){ fbTakeData = takeDataFlag;}
   Int_t    UseMysql(TAffairSql * affairSql);
   //ClassDef (TOperateFile,0) // affair class

};



void  TOperateFile::CloseFile()
{
  
  fbTakeData = false;
  // gSystem->Sleep(100);

  if (fTree != 0) fTree->AutoSave();		   
  EndTime = time(0); 

  fTreeTimeRange->Fill();
  fTreeTimeRange->AutoSave();

  if (fFile != 0) {		   
    fFile->Save();
    fFile->Close();    delete fFile; fFile = 0;
  }
  
  //now rename the file
  Char_t beginTimeText[64];
  Char_t endTimeText[64];
  Char_t newFileName[256];

  char  strCommand[256];     

  Char_t beginTimeText2[128];
  Char_t endTimeText2[128];

  struct tm *tm_ptr;

  tm_ptr = localtime((const time_t *)&BeginTime);    
  strftime(beginTimeText,64,"%H_%M_%d_%b_%y",tm_ptr);

  strftime(beginTimeText2,64,"%Y-%m-%d %H:%M:%S",tm_ptr);

  tm_ptr = localtime((const time_t *)&EndTime);    
  strftime(endTimeText,64,"%H_%M_%d_%b_%y",tm_ptr);
  strftime(endTimeText2,64,"%Y-%m-%d %H:%M:%S",tm_ptr);
  
  sprintf(newFileName,"%s_%s___%s.root",NameAndMonitorSet,beginTimeText,endTimeText);
  if (gDebug == 1) {cout << "closing file " << " file name is " << newFileName << endl;}

  if (fUseMysql) {
    Char_t sql[1024];
    Char_t dataDir[512];
    sprintf(dataDir,"%s/backup/",gAffairPath);
    sprintf(sql,"INSERT INTO %s.AFFAIR_ROOTFILELIST (DIR,FILE,COMPUTER,MONSET,BEGIN,END) VALUES ('%s','%s','%s','%s','%s','%s')",
	    gDatabase,dataDir,newFileName,fNodeName,fMonitorSet,beginTimeText2,endTimeText2 );
    printf("closing file. sql command is %s \n",sql);
    fAffairSql->Exec(sql);

  }
  sprintf(strCommand,"mv %s/rootfiles/%s.root %s/backup/%s ",gAffairPath,NameAndMonitorSet,gAffairPath,newFileName);	
  gSystem->Exec(strCommand);
}

Int_t  TOperateFile::OpenFile(Char_t * fileName)
{

    Char_t rootFile[256];
    Char_t aux[256];    
    Char_t line[256];
    Int_t  errFlag = -1;
    fbTakeData = false;


    sprintf(NameAndMonitorSet,"%s",fileName);

    sscanf(fileName,"%[^_]_%s",fNodeName,fMonitorSet);
    sprintf(fFileName,"%s",fileName);  // fFileName probably reduntant. eliminate in future
    sprintf(rootFile,"%s/rootfiles/%s.root",gAffairPath,fFileName);
    int CheckIfExists = -1;
    sprintf(aux,"source %s/support/isFile.sh %s",gAffairPath,rootFile);	
    FILE *fp = gSystem->OpenPipe(aux,"r");
    if(fp != NULL) {
      // consistency check. Should get -1 if not exists, and 1 if exists
      fgets(line,256,fp);
    }
    else {		
      cout  << getTime()  << " fp is null. This should not happen " << endl;
    }
    gSystem->ClosePipe(fp);
    CheckIfExists = atoi(line);	    	



    if (CheckIfExists > 0) {
      if (gDebug) cout  << getTime()  << " OLD NTUPLE, not new. rootFile="<< rootFile << endl;
      //      cout  << " Something is wrong. old files sholud not exist in this directory.=" << endl;
      // cout  << " Marked it as wrong, with current time, and move it to backup and open new file." << endl;

      
      Char_t endTimeText[64];
      Char_t newFileName[256];      
      char  strCommand[256];     


      struct tm *tm_ptr;
      tm_ptr = localtime((const time_t *)&EndTime);    


      strftime(endTimeText,64,"%H_%M_%d_%b_%y",tm_ptr);
      sprintf(newFileName,"%s_CHECKME___%s.root",fileName,endTimeText);
      // cout << " file name is " << newFileName <<endl;      
      //      sprintf(strCommand,"mv %s/rootfiles/%s.root %s/backup/%s ",
      //	      gAffairPath,fileName,gAffairPath,newFileName);	
      sprintf(strCommand,"rm -f %s/rootfiles/%s.root  ", gAffairPath,fileName);	
      gSystem->Exec(strCommand);
      //      gSystem->Sleep(2000);
    }

    gSystem->Sleep(100);
    // file does not exist. Create it
    fFile = new TFile(rootFile,"RECREATE","ntuple for online monitoring");
    fFile->SetCompressionLevel(2);

    fTree = new TTree("ntuple","DAQ monitoring ntuple");
    fTree->SetAutoSave(5000);


    fTree->Branch("NameAndMonitorSet",&NameAndMonitorSet,"NameAndMonitorSet[256]/C");
    fTree->Branch("MonitorTime",&MonitorTime,"MonitorTime/i");
    fTree->Branch("OrigMonitorTime",&OrigMonitorTime,"OrigMonitorTime/i");
    fTree->Branch("nVariables",&nVariables,"nVariables/i");
    fTree->Branch("MonitorValue",&MonitorValue,"MonitorValue[nVariables]/F");
    
    fTreeTimeRange = new TTree("TimeRange","Time range of data");
    fTreeTimeRange->Branch("BeginTime",&BeginTime,"BeginTime/i");
    fTreeTimeRange->Branch("EndTime",&EndTime,"EndTime/i");
    fTreeTimeRange->Branch("NameAndMonitorSet2",&NameAndMonitorSet2,"NameAndMonitorSet2[256]/C");
    fTreeTimeRange->Branch("Name",&Name,"Name[128]/C");
    fTreeTimeRange->Branch("MonitorSet",&MonitorSet,"MonitorSet[128]/C");

    fTree->SetBranchAddress("NameAndMonitorSet",&NameAndMonitorSet);
    fTree->SetBranchAddress("MonitorTime",&MonitorTime);
    fTree->SetBranchAddress("OrigMonitorTime",&OrigMonitorTime);
    fTree->SetBranchAddress("nVariables",&nVariables);  
    fTree->SetBranchAddress("MonitorValue",&MonitorValue);

    fTreeTimeRange->SetBranchAddress("BeginTime",&BeginTime);  
    fTreeTimeRange->SetBranchAddress("EndTime",&EndTime);
    fTreeTimeRange->SetBranchAddress("NameAndMonitorSet2",&NameAndMonitorSet2);  
    fTreeTimeRange->SetBranchAddress("Name",&Name);  
    fTreeTimeRange->SetBranchAddress("MonitorSet",&MonitorSet);  

    fLoopCount = 0;
    BeginTime = time(0); 
    fbTakeData = true;
    
    if (fUseMysql) {
      //now rename the file
      //Char_t beginTimeText[64];
      Char_t beginTimeText2[128];
      char  strCommand[256];
      struct tm *tm_ptr;
      tm_ptr = localtime((const time_t *)&BeginTime);
      strftime(beginTimeText2,64,"%Y-%m-%d %H:%M:%S",tm_ptr);
 
      Char_t sql[1024];
      Char_t dataDir[512];
      Char_t  fileName[512];
      sprintf(dataDir,"%s/rootfiles/",gAffairPath);
      sprintf(fileName,"%s.root",NameAndMonitorSet);

      sprintf(sql,"REPLACE INTO %s.AFFAIR_ROOTFILELIST (DIR,FILE,COMPUTER,MONSET,BEGIN,END) VALUES ('%s','%s','%s','%s','%s','%s')",
	      gDatabase,dataDir,fileName,fNodeName,fMonitorSet,beginTimeText2,beginTimeText2 );
      // for some reason this causes crashes and disconnection from the mysql server
      // remove this for time being
      // it basically adds the current files 
      //      fAffairSql->Exec(sql);
    }
    return(errFlag);
}


TOperateFile::TOperateFile()
{
 /* initialize random generator */
  srand(static_cast<unsigned>(time(0)));
  fFile = 0;

  if (fFile != 0){
   delete fFile; 
    fFile = 0;
  }

  fbTakeData = false;
  Float_t randFrac =  (Float_t)  ((Double_t) rand())/ ((Double_t)RAND_MAX);
  fMaxIterations = kMaxIterations +  (Int_t) (0.1*kMaxIterations*randFrac);
  fUseMysql=kFALSE;

 
}


TOperateFile::~TOperateFile()
{
  //  Shutdown();
}


void TOperateFile::SaveNewData(Char_t* hnameAndMonset,Int_t monitorValueNumber, Double_t *monData, Int_t timeNow)
{ 
  Int_t errFlag = -1;
  if ( fbTakeData == false ) {
    if (gDebug) cout <<" TOperateFile::SaveNewData. new data,but to prevent collisons, don't take data while opening,closing,moving files " <<endl; 
    return;
  }

  ifstream fin;
  ifstream finTest;

  OrigMonitorTime = (UInt_t) monData[0];
  MonitorTime = timeNow;
  nVariables =  monitorValueNumber-1; 
  
  sprintf(NameAndMonitorSet,"%s",hnameAndMonset);

  for (Int_t i = 0; i < (Int_t)  nVariables; i++) MonitorValue[i] = monData[i+1];
    if (gDebug == 1) {
      cout << hnameAndMonset << " ";cout << (Int_t) monData[0];  for (Int_t i = 1; i < (Int_t)  nVariables+1; i++) cout << " " << monData[i]; 
      cout << " iteration="<< fLoopCount  <<endl;
    }
  fTree->Fill();		
  fLoopCount++;


  if (fLoopCount % fMaxIterations == 0) {
    CloseFile();
    errFlag = OpenFile(fFileName);
  }

}

Int_t TOperateFile::UseMysql(TAffairSql * affairSql)
{
  fAffairSql = affairSql;
  fUseMysql    = kTRUE;
  // use from databse the max iterations
  if (fUseMysql) {
    Int_t maxIterations;
    maxIterations = atoi(fAffairSql->GetAffairParameter(kMaxIterationsForRootStorageIndex)); 
    if (gDebug == 1) {
      printf("TOperateFile::UseMysql. orig fMaxIterations = %d new maxIterations = %d \n",fMaxIterations,maxIterations);
    }
    Float_t randFrac =  (Float_t)  ((Double_t) rand())/ ((Double_t)RAND_MAX);
    fMaxIterations   = maxIterations +  (Int_t) (0.1*maxIterations*randFrac);
    if (gDebug == 1) {
      printf("TOperateFile::UseMysql. final fMaxIterations = %d \n",fMaxIterations);
    }

  }

  return 1;
}


//////////////////////////////////////////
// Class DIMDATA
//////////////////////////////////////////

class DimData: public DimInfo
{

private:
  char    hnameAndMonset[256];
  char    serviceType[256];
  char    dummy[256];
  char   *_serviceName;
  char    storeDataCmd[2024];
  Bool_t  fUseMysql;

  TOperateFile  * cc;
  Int_t          fOfflineStorageFlag;
  Bool_t         fbUseRemoteNodeTime;

  TAffairSql    *fAffairSql;

   void    infoHandler();
  void     FillLogfile(char* hname, int timeDelay); 
  Int_t    GetStatusFlag(void *buf,int size);
  Char_t** string_to_argv(char *argstring, int *argc);

 public:
  DimData(char *service_name,int nolink);
  ~DimData(){};
  Int_t   OpenFile(char * fileName, Int_t offlineStorageFlag);
  void    Shutdown();
  void    SetUseRemoteNodeTime(Int_t useRemoteNodeTime);
  Int_t   UseMysql(TAffairSql * affairSql);
};


DimData::DimData(char *service_name,int nolink):DimInfo(service_name,&nolink,4){
  fUseMysql=kFALSE;
  //  fbUseLocalNodeTime = true;
}


void DimData::infoHandler() {    
  char hname[80];
  char monset[80];
  char line[512];
  char line2[512];
  if (gDebug == 1) cout <<getTime() <<" DimData. service_name=" << getName() << endl;
  
  _serviceName = getName();
  //int lentmp = strlen(_serviceName);
  sscanf(_serviceName,"Affair/%[^/]/%s",hnameAndMonset,serviceType);
  sscanf(hnameAndMonset,"%[^_]_%s",hname,monset);
  
  if (gDebug == 1) cout <<getTime() <<" type = " << serviceType << " hnameAndMonset = "<< hnameAndMonset <<endl;
  //
  // check which particular service is subscribed
  //
  if ( strcmp(serviceType,"StartFlag")== 0 ) {
    int statusFlag = GetStatusFlag(getData(), getSize());
    if (statusFlag > 0) {
      char createDBCmd[512];
      char strCmd[256];
      char defaultParLine[1000];
      char dummy[256];
      char aux[256];
      FILE* read_parameter;
      FILE* read_parameter2;
      int chars_read;
      // see if there is a line in computers.conf matching this
      // machine and monitor set
      int compConfFlag = 0;
      string * sqlRow;
      

      sprintf(defaultParLine," ");
      if (!fUseMysql) {
	sprintf(strCmd,"$AFFAIR_MONITOR/support/getLogfileMatchTwo.pl $AFFAIR_MONITOR/config/computerpars.conf %s %s",hname,monset);
	read_parameter = popen(strCmd,"r");
	if(read_parameter !=NULL){
	  chars_read = fread(line2,sizeof(char),512,read_parameter);
	  if (chars_read > 0 && strcmp(line2,"NONE\n") !=0 ) {
	    sprintf(defaultParLine,"%s",line2);
	    compConfFlag = 1;  
	  }
	}
	pclose(read_parameter);      
	int timeDelay;      
	if (compConfFlag == 0) {
	  // no entry in computer.conf. Add from default par line
	  // get the parline. First, get it form the default file
	  sprintf(strCmd,"$AFFAIR_MONITOR/support/getLogfile.pl $AFFAIR_MONITOR/config/monsetDefault.conf %s",monset);
	  read_parameter2 = popen(strCmd,"r");
	  if(read_parameter2 !=NULL ){
	    chars_read = fread(line,sizeof(char),512,read_parameter);
	    if (chars_read > 0) 
	      sprintf(defaultParLine,"%s",line);
	  }
	  pclose(read_parameter2);
	  // now get the time delay from the par line
	  if (gDebug == 1) cout <<getTime() <<" defaultParLine Stage 2= " <<defaultParLine<<endl;
	  sscanf(defaultParLine,"%s %s %s %i %[^\n]",&dummy,&dummy,&dummy,&timeDelay,&dummy);
	} else {
	  if (gDebug == 1) cout <<getTime() <<" defaultParLine Stage 3= " <<defaultParLine<<endl;
	  sscanf(defaultParLine,"%s %s %i %[^\n]",&dummy,&dummy,&timeDelay,&dummy);	
	}	
      } else {
	// mysql case
	Char_t sql[500];
	Int_t rowNumber,colNumber;
	Char_t *ptr;
	sprintf(sql,"SELECT * FROM %s.AFFAIR_COMPUTERPARS WHERE MONSET='%s' AND COMPUTER='%s'",gDatabase,monset,hname);
 	sqlRow = fAffairSql->GetRow(sql,0,&rowNumber,&colNumber);

	if(sqlRow){
	  for (Int_t i = 0; i < colNumber; i++) 
	    if (sqlRow[i].c_str() ) sprintf(defaultParLine,"%s %s ",defaultParLine,sqlRow[i].c_str());	  
	} else {
	  Char_t fieldAfterUnderscore[100];
	  sprintf(sql,"SELECT * FROM %s.AFFAIR_MONSET_DEFAULTPARS WHERE MONSET='%s' ",gDatabase,monset);
	  sqlRow = fAffairSql->GetRow(sql,0,&rowNumber,&colNumber);
	  for (Int_t i = 0; i < colNumber; i++) {
	    // a nonelegant solution now. delete forst par
	    Int_t offsetUnderline = 0;
	    for (ptr=(Char_t *) sqlRow[i].c_str();;ptr++){
	      if (*ptr==0) break;
	      if (*ptr=='_')  offsetUnderline = (ptr - sqlRow[i].c_str() )/sizeof(Char_t);
	    }       	
	    
	    if (offsetUnderline) {
	      sscanf( sqlRow[i].c_str() + offsetUnderline + 1, "%s",fieldAfterUnderscore);
	      if (fieldAfterUnderscore) sprintf(defaultParLine,"%s %s ",defaultParLine,fieldAfterUnderscore);	      
	    } else {
	      if (sqlRow[i].c_str() ) sprintf(defaultParLine,"%s %s ",defaultParLine,sqlRow[i].c_str());
	    }
	  }
	  if (gDebug) cout << " sql defaultParLine = " << defaultParLine << endl;
	  // should get time delay
	}
      }
      
	  if (gDebug) cout << " xx sql defaultParLine = " << defaultParLine << endl;
      if (timeDelay <= 0) {
	cout << "Warning.timeDelay <= 0 =" << timeDelay << " setting it to 10 sec " << endl;
	timeDelay = 10; //consistency
      }

      // now create RRD file using interval time from conf files
      // first check if RRD already exists.
      int CheckIfExists = -1;
      sprintf(aux,"source $AFFAIR_MONITOR/support/isFile.sh $AFFAIR_MONITOR/rrd/%s.rrd",hnameAndMonset);
      
      FILE *fp = popen(aux,"r");
      if(fp !=NULL) {
	fgets(line,256,fp);
      } else {
	cout <<getTime() <<" fp is null. This should not happen " << endl;
      }
      pclose(fp);
      
      CheckIfExists = atoi(line);	
      if (CheckIfExists < 0) {
	// get number of variables monitored for this monitoring set
	// should separate case when have sql and not
	Char_t sql[500];
	sprintf(sql,"SELECT * FROM %s.AFFAIR_%s",gDatabase,monset);
	Int_t variableNumber = fAffairSql->GetNumberOfRowsSql(sql); 
	if (gDebug) cout <<getTime() <<" variableNumber= "<< variableNumber  << " sql command is="<< sql << endl;
	//
	sprintf(createDBCmd, "$AFFAIR_MONITOR/support/createRRD.pl %s.rrd %i %i",hnameAndMonset,timeDelay,variableNumber); 
	system(createDBCmd);

	if (fUseMysql) {
	  Char_t sql[512];	       
	  Char_t rrdName[512];	       
	  Char_t rootFileName[512];	       
	  Char_t baseName[512];	       

	  sprintf(rrdName,"%s_%s.rrd",hname,monset);
	  sprintf(rootFileName,"%s_%s.root",hname,monset);
	  sprintf(baseName,"%s_%s",hname,monset);
  
	  sprintf(sql,"INSERT INTO %s.AFFAIR_RRDLIST VALUES ('%s','%s','%s','%s','%s',CURRENT_TIMESTAMP)",gDatabase,rrdName,rootFileName,baseName,monset,hname);
	  fAffairSql->Exec(sql);
	}
	if (gDebug) cout <<getTime() <<" round robin database create command = "<< createDBCmd << endl;
      } else {
	if (gDebug) cout <<getTime() <<" rrd already exists"<<endl;
      }            
      // send start command to Collector computer, together with parameter line
      sprintf(strCmd,"Affair/%s/StartCommand",hnameAndMonset);
      int statusClient = DimClient::sendCommand(strCmd,defaultParLine);
      if (gDebug == 1) cout <<getTime() <<" statusClient = " << statusClient <<endl;
      // fill hostinfo into log file
      FillLogfile(hname,timeDelay);
    }
  }
  
  else if (strcmp(serviceType,"Data")==0 ) {
    Int_t  arraySize = getSize()/8;
    double buf[arraySize];
    if((int) getDouble() > 0  && arraySize > 1 ) {
      long timeNow = time(0);
      memcpy(buf,getData(), getSize());
      // rrd database case
      if (fOfflineStorageFlag == 0 || fOfflineStorageFlag == 1) {
	if (fbUseRemoteNodeTime == true) {
	  sprintf(storeDataCmd,"%s/rrd/%s.rrd %i",gAffairPath,hnameAndMonset,(int) buf[0]);
	  for(int j = 1; j<= arraySize - 1; j++) sprintf(storeDataCmd,"%s:%f",storeDataCmd,buf[j]);
	} else {
	  sprintf(storeDataCmd,"%s/rrd/%s.rrd %i",gAffairPath,hnameAndMonset,timeNow);
	  for(int j=1; j<= arraySize - 1; j++) sprintf(storeDataCmd,"%s:%f",storeDataCmd,buf[j]);	  
	}
	if (gDebug == 1) cout <<getTime() <<" storeDataCmd=" << storeDataCmd<<endl; 
	char **argv;
	int i, argc;	      
	optind=0;
	if((argv = string_to_argv(storeDataCmd, &argc)) != NULL) {
	  optind = 0;
	  opterr = 0;
	  i = rrd_update(argc, argv); 
	  if ( i < 0) { rrd_clear_error();}
	  //      free up the memory
	  free(argv[0]);
	  free(argv);
	  optind=0;
	  // return i;
	} else {
	  cout <<getTime() <<" argv = null " << endl;
	  //      error
	  // return (-1);
	}	       	  
      }
      // now store in root file	  
      if (fOfflineStorageFlag > 0) cc->SaveNewData(&hnameAndMonset[0],arraySize,&buf[0],timeNow);
    }
  }
  
  else if (strcmp(serviceType,"ParameterLine") == 0 ) {
    if (getSize() == sizeof(hostInfo)) {
      memcpy(&hostInfo, getData(), getSize());
      if (fUseMysql) {
	if (gDebug == 1) printf("AffairMonitor:parameter line. fill computers :sql= %s\n",hostInfo.parameterLine);
	//36=4 +max of 32 pars, defined in TAffairSql.C
	fAffairSql->ReplaceRowWithString("AFFAIR_COMPUTERPARS",hostInfo.parameterLine,36);	      
      } else {
	char strCmd[1028];
	sprintf(strCmd,"$AFFAIR_MONITOR/support/fillLogfileMatchTwo.pl $AFFAIR_MONITOR/config/computerpars.conf %s %s %s",
		hname,monset,hostInfo.parameterLine); 
	system(strCmd);
      }
    }
  }
}


Int_t DimData::UseMysql(TAffairSql * affairSql)
{
  fAffairSql = affairSql;
  fUseMysql    = kTRUE;
  sprintf(gDatabase,"%s",fAffairSql->GetDatabaseName());
  if (gDebug) printf("TAffairDim.h databse name is %s \n",gDatabase);
  return 1;
}


void DimData::SetUseRemoteNodeTime(Int_t useRemoteNodeTime)
{
  fbUseRemoteNodeTime = false;
  if (useRemoteNodeTime > 0 ) {
    fbUseRemoteNodeTime = true;
    printf(" Using time on remote nodes for time stamping of data \n");
  } else {
    printf(" Using local time on monitoring station  for time stamping of data \n");
  }
}


Int_t DimData::OpenFile(char * fileName, Int_t offlineStorageFlag)
{
  Int_t errFlag = 0;
  fOfflineStorageFlag = offlineStorageFlag;
  if (fOfflineStorageFlag > 0) {
   cc = new TOperateFile();
   errFlag = cc->UseMysql(fAffairSql);
   errFlag = cc->OpenFile(fileName);
  }
  return (errFlag);
}


void DimData::Shutdown()
{
  if (fOfflineStorageFlag > 0) {
    cc->CloseFile();
    delete cc;
  }
}


Int_t DimData::GetStatusFlag(void *buf,int size) {
  int statusFlag = -1;
  if (gDebug == 1) cout <<  getTime() <<" bufsize= "<<size<<" hostInfo size= "<< sizeof(hostInfo) << endl;
  if (size == sizeof(hostInfo)) {
      memcpy(&hostInfo, buf, size);
      statusFlag = hostInfo.statusFlag;
  } else {
    if (gDebug == 1) cout <<  getTime() <<" Have something else" << endl;
  }
  return(statusFlag);
}


void DimData::FillLogfile(char* hname, int timeDelay) 
{
  if (gDebug == 1) printf("AffairMonitor:FillLogFile:hname= %s usesql is %d \n",hname,fUseMysql);
  if (fUseMysql) {
    char sql[512];
    //    sprintf(sql," %s %s %s %i %f %f %i %s CURRENT_TIMESTAMP",
    //	    hname,hostInfo.type,hostInfo.kernel,hostInfo.processorNumber,hostInfo.cpuf,hostInfo.maxmem,timeDelay,hostInfo.statusText);
    sprintf(sql," replace into %s.AFFAIR_HOSTINFO VALUES('%s','%s','%s',%i,%f,%f,%i,'%s',CURRENT_TIMESTAMP)",
	    gDatabase,hname,hostInfo.type,hostInfo.kernel,hostInfo.processorNumber,hostInfo.cpuf,hostInfo.maxmem,timeDelay,hostInfo.statusText);
      if (gDebug == 1) printf("AffairMonitor:FillLogFile:sql= %s\n",sql);
      //      fAffairSql->ReplaceRowWithString("AFFAIR_HOSTINFO",sql);
      fAffairSql->Exec(sql);
  } else {      
    char  strCommand[512];
    sprintf(strCommand,"$AFFAIR_MONITOR/support/fillLogfileSed.pl  $AFFAIR_MONITOR/log/hostInfo.log %s %s %s %i %f %f %i %s",
	    hname,hostInfo.type,hostInfo.kernel,hostInfo.processorNumber,hostInfo.cpuf,hostInfo.maxmem,timeDelay,hostInfo.statusText);
    cout << getTime() <<" LOGFILE CMD:= " << strCommand << endl;
    system(strCommand);
  }
  return;
}


char** DimData::string_to_argv(char *argstring, int *argc) {
    char inquotes = 0, *workstring, **argv;
    int i, nchars;

    if((nchars = strlen(argstring)) > 0) {
	workstring = (char*)calloc((nchars + 2), sizeof(char*));
	//	fill the array with space characters
	for(i=0; i < (nchars + 2); i++) workstring[i] = ' '; 	
	//	copy the argstring into the workstring (padded with spaces on both ends)
	for(i=0; i < nchars; i++) workstring[i+1] = argstring[i];
	for((*argc) = 1, i=0; i < nchars + 2; i++) {
	  //	count the number of arguements
	  if(isgraph(workstring[i]) > 0) {
	    if((isgraph(workstring[i-1]) == 0) && (inquotes == 0)) (*argc)++;
	    if(workstring[i] == '"') inquotes = (char)(!inquotes);
	  } else {
	    if(!inquotes) workstring[i] = '\0';
	  }
	}
	//	no arguements in the string
	if((*argc) == 0){
	    free(workstring);
	    return NULL;
	} else {
	  inquotes = 0;
	  argv = (char**)calloc((*argc), sizeof(char**));
	  argv[0] = &workstring[0];
	  
	  for((*argc) = 1, i=1; i < nchars + 2; i++) {
	    // count the number of arguements
	    if(isgraph(workstring[i]) > 0) {
	      // if current character is not whitespace and previous
	      // character is whitespace, increment (*argc)
	      if((isgraph(workstring[i-1]) == 0) && (inquotes == 0)) {
		argv[(*argc)] = &workstring[i];
		(*argc)++;
	      }
	      //	check for the quote character
	      if(workstring[i] == '"') {
		//	toggle inquotes flag
		inquotes = (char)(!inquotes);
	      }
	    }
	  }
	  return argv;
	}
    } else {
	(*argc) = 0;
	return NULL;
    }
}




