#define TAffairSql_cxx
 
#include "TAffairSql.h"

#define DBSQL_OPT_NONE        0x0   /* no option */
#define DBSQL_OPT_WITH_COMA   0x1   /* option : add a coma in the query before the string */
#define DBSQL_OPT_STRIP_QUOTE 0x2   /* option : leading and trailing quotes (") are removed from string */

#define MAX_MONSET_PARAMETERS    33

/// ClassImp(TAffairSql)

Int_t TAffairSql::Exec(const char *query,...)
{
  int result;                            /* error code */

  char buffer_query[MAX_QUERY_SIZE];     /* the buffer for the SQL query */
  va_list ap;                            /* list of optionnal args */
  int query_size;                        /* query size */

  if (!fIsConnected) { 
    printf("Error:Exec: No connections \n");
    return 0;
  }


  /* format query */
  va_start(ap, query);
  query_size=vsnprintf(buffer_query,MAX_QUERY_SIZE,query,ap);
  va_end(ap);

  /* check that query doe not overrun buffer */
  if ((query_size<=0)||(query_size>MAX_QUERY_SIZE)) {
    printf("sql query '%s' ...", query);
    printf("dbsql_query() : bad SQL query size (%d bytes). Maximum is %d bytes.",query_size,MAX_QUERY_SIZE);
    return  0;
  }


  if (gDebug) { printf("Exec: executing : %s\n",buffer_query); }
  
  /* execute query */
  result=fDbSql->Exec(buffer_query);


  if (gDebug) { printf("Exec: result= %i \n",result); }
  
  /* explain error if any */
  if (result == 0) {
    printf("Error:Exec: executing : %s",buffer_query);
    return 0;
  }


  return 1;
}


Int_t TAffairSql::MultiExec()
{
  int result;                            /* error code */

  /* do nothing if not connected */
  if (!fIsConnected) { 
    printf("Error:MutiExec: No connections \n");
    return 0;
  }
  /* is there a query to execute? */
  if (fVChars==0) { 
    printf("Error:MutiExec: No query ro execute \n");
    return 0;
  }
  /* was the query completely stored in buffer? */
  if (fVChars==-1) {
    printf("Error:MultiExec: can not execute truncated query (%s...)",fVBuffer);
    return 0;
  }

  if (gDebug) { printf("MultiExec: executing : %s \n",fVBuffer); }

  /* execute query */
  result=(Int_t) fDbSql->Exec( fVBuffer);
  if (gDebug) printf("MultExec: fIsConnected is %i fVChars is %i  fVBuffer is %s result is %i\n",fIsConnected,fVChars,fVBuffer,result);

  /* explain error if any */
  if (result == 0) {
    printf("Error:MultiExec: %s \n",fVBuffer);
    return 0;
  }

  return 1;
}

/*
Int_t TAffairSql::UpdateRrdList()
{
  Char_t sql[1000];
  Int_t rval = 0;

  sprintf(sql,"TRUNCATE TABLE %s.RRDLIST",fDatabase);
  rval = fDbSql->Exec(sql);


  return rval;
}*/



Int_t TAffairSql::QueryNew()
{
  /* reinit string */
  fVChars     =0;
  fVBuffer[0] =0;
  return 1;
}

Int_t  TAffairSql::QueryAppend(const char *s,...){
  va_list ap;          /* list of optionnal args */
  int query_size;      /* size of string to be inserted to query */

  /* was the query completely stored in buffer up to now? */
  if (fVChars==-1) return  0;
  
  /* format query */
  va_start(ap,s);
  query_size=vsnprintf(&fVBuffer[fVChars],MAX_QUERY_SIZE-fVChars,s,ap);
  va_end(ap);
  
  /* check buffer space */
  if ((query_size<=0)||(query_size>MAX_QUERY_SIZE-fVChars)) {
    /* disable query, it was truncated */
    fVChars=-1;
    return  0;
  }

  /* update query length */  
  fVChars += query_size;
  return 1;    
}



Int_t  TAffairSql::QueryAppendSval(const char *s_in,Int_t options){
  const char *s=NULL;    /* string value to insert */
  char *s_copy=NULL;     /* copy of the string input if needed */
  int rval= 0;           /* return value */
  
  /* if quotes have to be removed, copy string and cut it */
  if ( (s_in!=NULL) && (options & DBSQL_OPT_STRIP_QUOTE) && (s_in[0]=='"') && (s_in[strlen(s_in)-1]=='"') ) {
    s_copy=strdup(&s_in[1]);         /* copy input string and remove starting quote */
    if (s_copy!=NULL) {
      s_copy[strlen(s_copy)-1]=0;   /* remove ending quote */
      s=s_copy;                     /* use the copy as the string to be inserted */
    }
  } else {
    s=s_in;                         /* no need to copy string, insert it directly */
  }

  /* add the string to the query */
  if (options & DBSQL_OPT_WITH_COMA) {    
    /* with leading coma */
    if (s!=NULL) {
      rval=QueryAppend(",'%s'",s);
    } else {
      rval=QueryAppend(",NULL");
    }    
  } else {    
    /* without leading coma */
    if (s!=NULL) {
      rval=QueryAppend("'%s'",s);
    } else {
      rval=QueryAppend("NULL");
    }   
  }
   
  /* free copy if any */
  if (s_copy!=NULL) {
    free(s_copy);
  }
  return rval;
}


Int_t TAffairSql::OpenDatabase(){
  Char_t dbName[1000];

  /* connect database if needed*/
  if (!fIsConnected) {
    // sprintf(dbName,"mysql://%s/%s",fHost,fDatabase);
    sprintf(dbName,"mysql://%s/%s",fHost,fDatabase);
    //sprintf(dbName,"mysql://%s/test",fHost);
    fDbSql = TSQLServer::Connect(dbName,fUserid,fPasswd);

    if (!fDbSql){
      printf("Error Cannot open database for %s host is %s.Maybe mysql not connected, or the affair database does not exist\n.",dbName,fHost);
      printf("Try creating it. This assumes mysql has a root acocunt, with no password, and a mysql database.\n");
      printf("If this is not the case, you will need to create the appropriate database. DatabassName:affair, UserName:%s, Password:%s \n",fUserid,fPasswd);
      sprintf(dbName,"mysql://%s/mysql",fHost);
      //fDbSql = TSQLServer::Connect(dbName,"root","daq");
      fDbSql = TSQLServer::Connect(dbName,"root","");
      if (!fDbSql){      
	printf("Could not log to root account on mysql databse \n");
	return 0;
      } else {
 	printf("Logged in root database with mysql account. Now create affair database \n");
	Char_t sql[500];
	sprintf(sql,"CREATE DATABASE IF NOT EXISTS %s",fDatabase);
	fDbSql->Exec(sql);
	if (!fDbSql){      
	  printf("Could not create %s database  \n",fDatabase);
	  return 0;
	}
	printf(" successfully created %s database \n",fDatabase);

	sprintf(sql,"grant all privileges on *.* to '%s'@'%%' identified by '%s' with grant option",fDatabase,fPasswd);
	//sprintf(sql,"CREATE USER affair IDENTIFIED BY '%s'",fPasswd);
	fDbSql->Exec(sql);

	sprintf(sql,"grant all privileges on *.* to 'affair'@'localhost' identified by '%s' with grant option",fPasswd);
	fDbSql->Exec(sql);

	if (gDebug) printf("created affair user. \n");

	fDbSql->Close();

	if (gDebug) printf("logged out as root \n");
	sprintf(dbName,"mysql://%s/%s",fHost,fDatabase);
	fDbSql = TSQLServer::Connect(dbName,fUserid,fPasswd);

	if (gDebug) printf("connected as affair \n");
	
      }
    }
  }

  /* increase number of 'open' calls */
  fIsConnected++;
  if (gDebug) printf(" number of connections is %d \n",fIsConnected);
  return 1;
}


Int_t TAffairSql::ShowContentsAllTables()
{
   // list databases available on server
  TSQLRow    *row;
  TSQLResult *res;

  TSQLRow    *row2;
  TSQLResult *res2;

  Char_t sql[2000];
  
  printf("\nList all databases on server %s\n", fDbSql->GetHost());
  res = fDbSql->GetDataBases();
  while ((row = res->Next())) {
    printf("%s\n", row->GetField(0));
    delete row;
  }
  delete res;
  
  printf("\nList all tables in database \"affair\" on server %s\n",fDbSql->GetHost());

  // res = fDbSql->GetTables(fDatabase);

  res = fDbSql->GetTables(fDatabase);
  while ((row = res->Next())) {
    printf("table is %s\n", row->GetField(0));
    sprintf(sql,"select * from %s.%s ",fDatabase,row->GetField(0));
    res2 = fDbSql->Query(sql);	
    if (!res2) continue;	
    int nrows = res2->GetRowCount();
    printf("\nGot %d rows in result\n", nrows);
    
    int nfields = res2->GetFieldCount();
    for (int i = 0; i < nfields; i++) printf("%10s", res2->GetFieldName(i));printf("\n");
    for (int i = 0; i < nfields*10; i++) printf("=");printf("\n");	
    for (int i = 0; i < nrows; i++) {
      row2 = res2->Next();
      for (int j = 0; j < nfields; j++) {
	printf("%10s", row2->GetField(j));
      }
      printf("\n");
      delete row2;
    }	
    delete res2;
    delete row;
  }
  
  delete res;
  
  return 1;
}


int TAffairSql::SourceFile(char * file){
  FILE *fp;
  char buffer[MAX_QUERY_SIZE];
  char *ptr;
  int end_query;
  
  fp=fopen(file,"r");
  if (fp==NULL) {
    printf("Error:SourceFile: failed to open %s",file);
    return 0;
  }


  end_query=0;
  QueryNew();

  for(;;){
    /* read line */
    if (fgets(buffer,MAX_QUERY_SIZE,fp)==NULL) break;

    /* skip comments */
    for(ptr=buffer;;ptr++){
      if (*ptr==0) break;
      if (!isspace(*ptr)) break;
    }       
    if (*ptr=='#') continue;

    /* is it the end? */
    for(ptr=&buffer[strlen(buffer)-1];ptr>=0;ptr--){
      if (!isspace(*ptr)) break;
    }       
    if (*ptr==';') {
      end_query=1;
      *ptr=0;
    }
        
    /* add to query */
    QueryAppend("%s",buffer);    
    /* execute query if needed */
    if (end_query) {
      end_query=0;
      MultiExec();
      QueryNew();     
    }      
  }
  fclose(fp);
  return 1;
}


Int_t TAffairSql::CreateDefaultTables()
{
  Int_t retval,retval1,retval2,retval3,retval4;
  retval = 1;
  retval1 = OpenDatabase();
  retval2 = DestroyTables();
  retval3 = CreateTables();
  retval4 = FillDefaultValues();
  if (retval1 == 0 || retval2 == 0 || retval3 == 0 || retval4 == 0 )retval = 0;
  return retval;
}

Int_t TAffairSql::UpgradeTablesForAffairVersion315()
{


  DestroyMonitoringSetTables();
  CreateDefaultMonitoringSetTables();
  FillMonitoringSetDefaultValues();
  CreateParameterTables();
  FillParameterDefaultValues();
  return 1;
}


Int_t TAffairSql::FillRootfilelistDefaultValues()
{
    FILE  * table_fp;
  Char_t    aux[512];
  Char_t    sysCommand[1000];
  Char_t    sql[4096];
  Char_t    line[2000]; 
  char     *ptr;

  Char_t dirName[256];
  Char_t fileName[256];
  Char_t monitorSetName[256];
  Char_t computerName[256];
  Char_t beginTime[256];
  Char_t endTime[256];

  Char_t beginTime2[256];
  Char_t endTime2[256];
  Int_t ncols;
  struct tm *tm_ptr;



  sprintf(sysCommand,"%s/support/listAllRootfiles.pl %s/backup %s/log/rootfilelist.txt ",fAffairPath,fAffairPath,fAffairPath);	
  gSystem->Exec(sysCommand);


  sprintf(aux,"%s/log/rootfilelist.txt",fAffairPath);
  table_fp = fopen(aux,"r");    
  if (table_fp) {
    while (fgets(line,2000,table_fp)) {	      
      if (gDebug)  cout << " cc line is  "<<  line << endl;
      
      for(ptr=line;;ptr++){
	if (*ptr==0) break;
	if (!isspace(*ptr)) break;
      }       
      if (*ptr=='#') continue;
      if (strlen(line) < 2) continue;
      
      ncols = sscanf(line,"%s %s %s %s %s %s",&dirName[0], &fileName[0],&computerName[0],&monitorSetName[0],&beginTime[0],&endTime[0]);
      
      Int_t beginTimeInt = atoi (beginTime);
      Int_t endTimeInt = atoi (endTime);
      tm_ptr = localtime((const time_t *) &beginTimeInt);    
      strftime(beginTime2,64,"%Y-%m-%d %H:%M:%S",tm_ptr);
      
      tm_ptr = localtime((const time_t *)&endTimeInt);    
      strftime(endTime2,64,"%Y-%m-%d %H:%M:%S",tm_ptr);
      
      
      sprintf(sql,"REPLACE INTO %s.AFFAIR_ROOTFILELIST (DIR,FILE,COMPUTER,MONSET,BEGIN,END,TS) VALUES ('%s','%s','%s','%s','%s','%s',CURRENT_TIMESTAMP)",fDatabase,dirName,fileName,computerName,monitorSetName,beginTime2,endTime2);    
      if (gDebug) { printf("FillRootfilelistDefaultValues. query is %s \n",sql); }
       printf("FillRootfilelistDefaultValues. query is %s \n",sql); 
      fDbSql->Exec(sql);          
    }
    fclose(table_fp);
  }
  return 1;

}

Int_t TAffairSql::FillMonitoringSetDefaultValues()
{

  Char_t    aux[512];
  Char_t   sql[4096];
  
  TSQLRow    *row;
  TSQLResult *res;
  
  Char_t line[2000]; 
  Char_t monset[64]; 
  char **argv;
  Int_t argc;
  Int_t exitStatus = 1;

  char *ptr;

  // now loop over monset and create table for each monset entry

  if (!fIsConnected) { 
    printf("Error:FillDefaultValues: No connections \n");
    return 0;
  }

  sprintf(sql,"select * from %s.AFFAIR_MONSET_DEFAULTPARS ",fDatabase);
  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:FillDefaultValues: res==NULL \n");
    return 0;
  }

  Int_t nrows = res->GetRowCount();
  int nfields = res->GetFieldCount();
   
  if (nrows==0 || nfields == 0) {
    printf("Error:FillDefaultValues: nrows==0 || nfields == 0 sql=%s \n",sql);
    return 0;
  }

  for (int i = 0; i < nrows; i++) {
    row = res->Next();
    sprintf(monset,"%s",row->GetField(0));
    sprintf(aux,"%s/config/mysql_default/%s.conf",fAffairPath,monset);
    const char *ins = "INSERT INTO %s.AFFAIR_%s VALUES (%d, '%s', %f, 'GAUGE', %d, %d,  %d, '%s','%s', %d,'%s', %d, %d, %d, %d)";
    if (gDebug) { printf("FillDefaultValues.2 aux is %s \n",aux); }
    
    FILE * table_fp = fopen(aux,"r");
    if (!table_fp) {
      printf("Error: File %s does nor exists\n",aux);
      continue;
    }
    while (fgets(line,2000,table_fp)) {     
      if (gDebug) { printf("FillDefaultValues. line %s \n",line); }
      /* skip comments */
      for(ptr=line;;ptr++){
	if (*ptr==0) break;
	if (!isspace(*ptr)) break;
      }       
      if (*ptr=='#') continue;
      if (strlen(line) < 2) continue;
      
      if((argv = string_to_argv(line, &argc)) != NULL) {
	if (argc < 3) break; //make sure exit if extra line/s at end
	sprintf(sql, ins,fDatabase,monset, atoi(argv[1]),  argv[2],  atof(argv[3]),  atoi(argv[5]), atoi(argv[6]),atoi(argv[7]),argv[8],argv[9],
		atoi(argv[10]),argv[11],atoi(argv[12]),atoi(argv[13]),atoi(argv[14]),atoi(argv[15]) );    
	if (gDebug) { printf("\n FillDefaultValues. query is %s \n\n",sql); }
	fDbSql->Exec(sql);
	
	free(argv[0]);
	free(argv);
      }  
    }
    fclose(table_fp);   
    //
    delete row;
    //
  }
  delete res;

  return exitStatus;

}


Int_t TAffairSql::FillParameterDefaultValues()
{

  Char_t    aux[512];
  Char_t   sql[4096];
  
  
  Char_t line[2000]; 
  Int_t ncols;

  Int_t exitStatus = 1;

  char *ptr;

  Char_t  id[32];
  Char_t  canvasScale[32];
  Char_t  lineWidth[32];
  Char_t  fontSize[32];
  Char_t  timeoutPeriod[32];
  Char_t  highWarningColor[32];
  Char_t  lowWarningColor[32];
  Char_t  maxGraphsPerPlot[32];
  Char_t  plotCycleDelaySec[32];
  Char_t  maxIterationsForStorage[32];
  Char_t  monitoringStationTimeOrRemoteNodeTimeOption[32];

    if (gDebug) printf("FillParameterDefaultValues \n");

  //first open monset,and fill it
  sprintf(aux,"%s/config/mysql_default/pars.conf",fAffairPath);
  FILE * table_fp = fopen(aux,"r");    
  if (table_fp) {
    while (fgets(line,2000,table_fp)) {     
      for(ptr=line;;ptr++){
	if (*ptr==0) break;
	if (!isspace(*ptr)) break;
      }       
      if (*ptr=='#') continue;
      if (strlen(line) < 2) continue;
      
      ncols = sscanf(line,"%s %s %s %s %s %s %s %s %s %s %s",&id, &monitoringStationTimeOrRemoteNodeTimeOption,&canvasScale,&lineWidth,&fontSize,&timeoutPeriod,&highWarningColor,
		     &lowWarningColor,&maxGraphsPerPlot, &plotCycleDelaySec,&maxIterationsForStorage);
      if (gDebug) printf(" pars are %s %s %s %s %s %s %s %s %s %s %s \n",id,monitoringStationTimeOrRemoteNodeTimeOption,canvasScale,lineWidth,fontSize,timeoutPeriod,highWarningColor,
		     lowWarningColor,maxGraphsPerPlot, plotCycleDelaySec,maxIterationsForStorage);
    
      sprintf(sql,"REPLACE INTO %s.AFFAIR_PARS ( ID,USE_TIME_FROM_REMOTE_NODE, CANVAS_SCALE, LINE_WIDTH , FONT_SIZE , TIMEOUT_PERIOD, HIGH_WARNING_COLOR ,LOW_WARNING_COLOR,MAX_GRAPHS_PER_PLOT, PLOT_CYCLE_DELAY_SEC ,MAX_ITER_ROOT_STORE) VALUES (%d,%d,%f,%f,%f,%f,'%s','%s',%d,%f,%d)",fDatabase, atoi(id), atoi(monitoringStationTimeOrRemoteNodeTimeOption) ,atof(canvasScale),atof(lineWidth),atof(fontSize),atof(timeoutPeriod),highWarningColor,lowWarningColor,
	      atoi(maxGraphsPerPlot),atof(plotCycleDelaySec),atoi(maxIterationsForStorage));
      
      Exec(sql);

    }
    fclose(table_fp);   
  }
  return exitStatus;
}


Int_t TAffairSql::FillRrdListDefaultValues()
{


  Char_t baseName[256];
  Char_t rrdName[256];
  Char_t rootName[256];
  Char_t monitorSetName[256];
  Char_t computerName[256];

    FILE * table_fp;
  Char_t    aux[512];
  Char_t   sql[4096];
  Char_t line[2000]; 
  char     *ptr;

  Int_t ncols;

  sprintf(aux,"%s/support/listRRD.pl",fAffairPath);	
  gSystem->Exec(aux);

  sprintf(sql,"TRUNCATE %s.AFFAIR_RRDLIST ",fDatabase);
  if (gDebug) { printf("\n FillDefaultValues. query is %s \n",sql); }
  fDbSql->Exec(sql);
 
  sprintf(aux,"%s/log/rrdlist.txt",fAffairPath);
  table_fp = fopen(aux,"r");   
  if (table_fp) {
    while (fgets(line,2000,table_fp)) {	      
      for(ptr=line;;ptr++){
	if (*ptr==0) break;
	if (!isspace(*ptr)) break;
      }       
      if (*ptr=='#') continue;
      if (strlen(line) < 2) continue;
      
      ncols = sscanf(line,"%s %s %s %s %s",&rrdName[0], &rootName[0],&baseName[0],&monitorSetName[0],&computerName[0]);

      sprintf(sql,"REPLACE INTO %s.AFFAIR_RRDLIST VALUES ('%s','%s','%s','%s','%s',CURRENT_TIMESTAMP)",fDatabase,rrdName,rootName,baseName,monitorSetName,computerName);    
      if (gDebug) { printf("\n FillDefaultValues. query is %s \n",sql); }
      fDbSql->Exec(sql);          
    }
    fclose(table_fp);
  }
  return 1;
}


Char_t * TAffairSql::CreateMonitoringSetDefaultparsRow()
{
  Char_t   sql[4096];
  Int_t exitStatus = 1;
  Int_t i;
  TSQLRow     *row;
  TSQLResult  *res;
  Char_t   newmonset[200];
  Int_t id;


  //should first get name of monitoring set. it should not exist before
  id =0;
  while(id < 100) {
    sprintf(newmonset,"MYNEWMONSET%i",id);
    sprintf(sql,"select * from %s.AFFAIR_MONSET_DEFAULTPARS WHERE MONSET='%s'",fDatabase,newmonset);
    res = fDbSql->Query(sql);
    Int_t nrows = res->GetRowCount();

    if (nrows == 0) {
      printf(" creating a new monitoring set=%s \n ",newmonset);
      break;
    } else {
      id++;
      printf(" already have this monitoring set=%s. try a new one \n ",newmonset);
    }
  }

  QueryNew();
  QueryAppend("INSERT into %s.AFFAIR_MONSET_DEFAULTPARS VALUES('%s','%s',10,1",fDatabase,newmonset,newmonset);
  for (i = 1; i < MAX_MONSET_PARAMETERS; i++)  QueryAppend(",NULL");
  QueryAppend(")");
  if (!MultiExec()) exitStatus = 0;

  //  return exitStatus;
  return   newmonset;

}


Int_t TAffairSql::CreateMonitoringSetRow( Char_t * monset)
{
  Char_t   sql[4096];
  Int_t exitStatus = 1;
  TSQLRow     *row;
  TSQLResult  *res;
  Int_t id;

  // first get last row, to get id, and perhaps values, to make things easier

  sprintf(sql,"select * from %s.AFFAIR_%s ORDER BY ID",fDatabase,monset);
  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:CreateMonitoringSetRow: res==NULL \n");
    return 0;
  }

  Int_t nrows = res->GetRowCount();
  if (nrows == 0) {
    id = 0;
  } else {
    // should go to last row. As I don't know the sql command for it yet, do it the crude way
    for (int i = 0; i < nrows; i++) {
      row = res->Next();
      id = atoi(row->GetField(0));
      delete row;
    }
    id++;
  }
  delete res;

  const char *ins = "INSERT INTO %s.AFFAIR_%s VALUES (%i, 'variable%i', 1.0, 'GAUGE', -1, -1,  -1, 'label','unit', -1,'MYGROUP', -1, -1, -1.0, -1.0)";
  sprintf(sql,ins,fDatabase,monset,id,id);

  if (gDebug) printf("inserting a new row into table AFFAIR_%s. command is %s \n",monset,sql);

  fDbSql->Exec(sql);

  return exitStatus;
}




Int_t TAffairSql::FillDefaultValues()
{
  Char_t    aux[512];
  Char_t   sql[4096];
  
  
  Char_t line[2000]; 
  Char_t node[128]; 
  char **argv;
  Int_t argc;
  Int_t ncols;

  Int_t i;
  Int_t exitStatus = 1;

  char *ptr;

  if (gDebug) { printf("FillDefaultValues.1 \n"); }
 
  //first open monset,and fill it
  sprintf(aux,"%s/config/mysql_default/monsetDefault.conf",fAffairPath);
  FILE * table_fp = fopen(aux,"r");    
  while (fgets(line,2000,table_fp)) {     
    for(ptr=line;;ptr++){
      if (*ptr==0) break;
      if (!isspace(*ptr)) break;
    }       
    if (*ptr=='#') continue;
    if (strlen(line) < 2) continue;

    if((argv = string_to_argv(line, &argc)) != NULL) {

      if (gDebug) { printf("FillDefaultValues.line is %s \n",line); }      
      QueryNew();
      //      QueryAppend("INSERT into affair.AFFAIR_MONSET_DEFAULTPARS VALUES(");
      QueryAppend("INSERT into %s.AFFAIR_MONSET_DEFAULTPARS VALUES(",fDatabase);
      QueryAppendSval(argv[1],DBSQL_OPT_NONE);
      QueryAppendSval(argv[2],DBSQL_OPT_WITH_COMA);
      QueryAppend(",%d",atoi(argv[3]));
      QueryAppend(",1");
      for (i = 1; i < MAX_MONSET_PARAMETERS; i++) {
	if (argc -1 >= i + MONSET_PARAMETERS_OFFSET ) {
	  QueryAppendSval(argv[i+MONSET_PARAMETERS_OFFSET],DBSQL_OPT_WITH_COMA);
	} else {
	  QueryAppend(",NULL");
	}
      }
      QueryAppend(")");
      if (!MultiExec()) exitStatus = 0;

      free(argv[0]);
      free(argv);
    }
  }
  fclose(table_fp);   


  FillMonitoringSetDefaultValues();

  //
  // fill computerlist table from computerlist.config if it exists
  sprintf(aux,"%s/config/computerlist.conf",fAffairPath);

  table_fp = fopen(aux,"r");    
  if (table_fp) {
    while (fgets(line,2000,table_fp)) {
      for(ptr=line;;ptr++){
	if (*ptr==0) break;
	if (!isspace(*ptr)) break;
      }       
      if (*ptr=='#') continue;      
      if (strlen(line) < 2) continue;

      ncols = sscanf(line,"%s",&node);
      sprintf(sql,"INSERT INTO %s.AFFAIR_COMPUTERLIST VALUES ('%s')",fDatabase,node);    
      if (gDebug) { printf("\n FillDefaultValues. query is %s \n",sql); }
      fDbSql->Exec(sql);       
    }    
    fclose(table_fp);
  } else {
    printf("no default values from computerlist.config \n");
  }

  // fill rootfilelist from existing rootfile values, if any, in backup directory
  FillRootfilelistDefaultValues();


  // fill rrdlist from existing rrd values, if any, in rrd directory
  // this is from a method, as this might be called from other places, to ensure
  // have updated rrd files, even if not always taken during times when mysql is up
   
  FillRrdListDefaultValues();

  // fill plotlist values
  // fill plotlist from PlotList.conf
  Char_t plotName[256]; 
  Char_t plotMonset[256]; 
  Char_t plotType[256]; 
  Char_t plotScope[256]; 
  sprintf(aux,"%s/config/mysql_default/PlotList.conf",fAffairPath);
  
  table_fp = fopen(aux,"r");    
  if (table_fp) {
    while (fgets(line,2000,table_fp)) {
      for(ptr=line;;ptr++){
	if (*ptr==0) break;
	if (!isspace(*ptr)) break;
      }       
      if (*ptr=='#') continue;      
      if (strlen(line) < 2) continue;
      
      ncols = sscanf(line,"%s %s %s %s",&plotName,&plotMonset,&plotType,&plotScope);
      sprintf(sql,"REPLACE INTO %s.AFFAIR_PLOTLIST (NAME,USER,MONSET,PLOT_TYPE,PLOT_SCOPE,TS) VALUES('%s','user','%s','%s',%i,CURRENT_TIMESTAMP)",fDatabase,plotName,
	      plotMonset,plotType,atoi(plotScope));
      
      if (gDebug) { printf("\n FillDefaultValues. query is %s \n",sql); }
      fDbSql->Exec(sql);       
    }    
    fclose(table_fp);
  } else {
    printf("no default values from PlotList.conf \n");
  }
 


 // fill AFFAIR_COMPUTERPARS from existing computers.conf values, if any
  sprintf(aux,"%s/config/computers.conf",fAffairPath);
  table_fp = fopen(aux,"r");    
  if (table_fp) {
    while (fgets(line,2000,table_fp)) {	      
      for(ptr=line;;ptr++){
	if (*ptr==0) break;
	if (!isspace(*ptr)) break;
      }       
      if (*ptr=='#') continue;
      if (strlen(line) < 2) continue;
      
      if((argv = string_to_argv(line, &argc)) != NULL) {
	
	if (gDebug) { printf("FillDefaultValues for AFFAIR_COMPUTERPARS.line is %s \n",line); }      
	QueryNew();
	//QueryAppend("INSERT into affair.AFFAIR_COMPUTERPARS VALUES(");
	QueryAppend("INSERT into %s.AFFAIR_COMPUTERPARS VALUES(",fDatabase);
	QueryAppendSval(argv[1],DBSQL_OPT_NONE);
	QueryAppendSval(argv[2],DBSQL_OPT_WITH_COMA);
	// QueryAppend(",%d",atoi(argv[3])); as with version 3.15 this is not a dummy char variable anymore, but USE_HIGH_LOW int, put a 1 here 
	// later have this also from files
	QueryAppend(",1");

	QueryAppendSval(argv[4],DBSQL_OPT_WITH_COMA);
	for (i = 1; i < MAX_MONSET_PARAMETERS; i++) {
	  if (argc -1 >= i + MONSET_PARAMETERS_OFFSET ) {
	    QueryAppendSval(argv[i+MONSET_PARAMETERS_OFFSET],DBSQL_OPT_WITH_COMA);
	  } else {
	    QueryAppend(",NULL");
	  }
	}
	QueryAppend(")");
	if (!MultiExec()) exitStatus = 0;
	
	fDbSql->Exec(sql);
	
	free(argv[0]);
	free(argv);
      }      
    }
    fclose(table_fp);
  }

  FillParameterDefaultValues();
  
  return exitStatus;
}

Int_t TAffairSql::DestroyTables()
{
  TSQLRow    *row;
  TSQLResult *res;

  char sql[MAX_QUERY_SIZE];
  
  if (!fIsConnected) { 
    printf("Error.database not connected\n");    
    return 0;
  }
  res = fDbSql->GetTables(fDatabase);
  while ((row = res->Next())) {
    if (strncmp("AFFAIR",row->GetField(0),6)==0) {
      printf("drop table  if exists %s\n", row->GetField(0));
      sprintf(sql,"drop table  if exists %s ",row->GetField(0));
      fDbSql->Exec(sql);	
    }
    delete row;
  }
  delete res;

  return 1;
}

Char_t * TAffairSql::GetDatabaseName()
{
  char par[128];
  sprintf(par,"%s",fDatabase);
  return par;
}

Char_t * TAffairSql::GetAffairParameter(Int_t col)
{
  TSQLRow    *row;
  TSQLResult *res;
  char par[100];

  char sql[MAX_QUERY_SIZE];

  sprintf(sql,"select * from %s.AFFAIR_PARS  where id=1",fDatabase);
  res = fDbSql->Query(sql);


  if (res==NULL) {
    printf("Error:GetAffairParameter: res == NULL \n");
    return 0;
  }

  Int_t nrows = res->GetRowCount();
  int nfields = res->GetFieldCount();
   
  if (nrows==0 || nfields == 0) {
    printf("Error:GetAffairParameter: nrows==0 || nfields == 0 sql=%s \n",sql);
    delete res;
    return 0;
  }


  if (nrows > 1) {
    printf("Error:GetAffairParameter: nrows != 1. sql=%s \n",sql);
    return 0;
  }
   
  row = res->Next();
  sprintf(par,"%s",row->GetField(col));
  
  delete row;
  delete res;
  
  return par;

}


Int_t TAffairSql::DestroyMonitoringSetTables()
{
  TSQLRow    *row;
  TSQLResult *res;
  Char_t monset[64]; 
  Char_t tableName[200]; 

  char sql[MAX_QUERY_SIZE];
  
  if (!fIsConnected) { 
    printf("Error.database not connected\n");    
    return 0;
  }
  /////

  sprintf(sql,"select * from %s.AFFAIR_MONSET_DEFAULTPARS ",fDatabase);
  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:DestroyMonitoringSetTables: res == NULL \n");
    return 0;
  }

  Int_t nrows = res->GetRowCount();
  int nfields = res->GetFieldCount();
   
  if (nrows==0 || nfields == 0) {
    printf("Error:DestroyMonitoringSetTables: nrows==0 || nfields == 0 sql=%s \n",sql);
    return 0;
  }

  for (int i = 0; i < nrows; i++) {
    row = res->Next();
    sprintf(monset,"%s",row->GetField(0));
    sprintf(tableName,"AFFAIR_%s",monset);
    if (gDebug) printf(" deleting table %s \n",tableName) ;
    sprintf(sql,"drop table  if exists %s ",tableName );
    fDbSql->Exec(sql);	
    delete row;
  }
  delete res;

  return 1;
    
}


Int_t TAffairSql::CreateTables()
{


  if (!fIsConnected) { return -1;}
  Char_t  sql[1024];

  //monset.A bit of a kludge: have up to 32 parameters

  sprintf(sql,"CREATE TABLE IF NOT EXISTS %s.AFFAIR_MONSET_DEFAULTPARS \
    ( \
       MONSET CHAR (64), \
       WEB_TEXT CHAR (128), \
       PERIOD INT, \
       ON_OFF_FLAG INT, \
       PAR1 CHAR (32), \
       PAR2 CHAR (32), \
       PAR3 CHAR (32), \
       PAR4 CHAR (32), \
       PAR5 CHAR (32), \
       PAR6 CHAR (32), \
       PAR7 CHAR (32), \
       PAR8 CHAR (32), \
       PAR9 CHAR (32), \
       PAR10 CHAR (32), \
       PAR11 CHAR (32), \
       PAR12 CHAR (32), \
       PAR13 CHAR (32), \
       PAR14 CHAR (32), \
       PAR15 CHAR (32), \
       PAR16 CHAR (32), \
       PAR17 CHAR (32), \
       PAR18 CHAR (32), \
       PAR19 CHAR (32), \
       PAR20 CHAR (32), \
       PAR21 CHAR (32), \
       PAR22 CHAR (32), \
       PAR23 CHAR (32), \
       PAR24 CHAR (32), \
       PAR25 CHAR (32), \
       PAR26 CHAR (32), \
       PAR27 CHAR (32), \
       PAR28 CHAR (32), \
       PAR29 CHAR (32), \
       PAR30 CHAR (32), \
       PAR31 CHAR (32), \
       PAR32 CHAR (32), \
      PRIMARY KEY (MONSET), \
       INDEX (MONSET) \
     ) ",fDatabase);
    //    )  ENGINE = INNODB");

  printf("fDatabase is %s \n",fDatabase);
  Exec(sql);

  // As AFFAIR_$monitoringset has added pars, for upgrading one should only change this

  CreateDefaultMonitoringSetTables();

  // computerlist
  sprintf(sql,"CREATE TABLE IF NOT EXISTS %s.AFFAIR_COMPUTERLIST \
    ( \
      COMPUTER CHAR (128)  NOT NULL, \
      PRIMARY KEY (COMPUTER) \
    )  ",fDatabase);
  //    )  ENGINE = INNODB");

  Exec(sql);

  //computers
  sprintf(sql,"CREATE TABLE IF NOT EXISTS %s.AFFAIR_COMPUTERPARS \
    ( \
       COMPUTER CHAR (128) NOT NULL, \
       MONSET CHAR (64) NOT NULL, \
       PERIOD INT, \
       USE_HIGH_LOW int, \
       PAR1  CHAR (32), \
       PAR2  CHAR (32), \
       PAR3  CHAR (32), \
       PAR4  CHAR (32), \
       PAR5  CHAR (32), \
       PAR6  CHAR (32), \
       PAR7  CHAR (32), \
       PAR8  CHAR (32), \
       PAR9  CHAR (32), \
       PAR10 CHAR (32), \
       PAR11 CHAR (32), \
       PAR12 CHAR (32), \
       PAR13 CHAR (32), \
       PAR14 CHAR (32), \
       PAR15 CHAR (32), \
       PAR16 CHAR (32), \
       PAR17 CHAR (32), \
       PAR18 CHAR (32), \
       PAR19 CHAR (32), \
       PAR20 CHAR (32), \
       PAR21 CHAR (32), \
       PAR22 CHAR (32), \
       PAR23 CHAR (32), \
       PAR24 CHAR (32), \
       PAR25 CHAR (32), \
       PAR26 CHAR (32), \
       PAR27 CHAR (32), \
       PAR28 CHAR (32), \
       PAR29 CHAR (32), \
       PAR30 CHAR (32), \
       PAR31 CHAR (32), \
       PAR32 CHAR (32), \
      PRIMARY KEY (COMPUTER,MONSET), \
      INDEX (COMPUTER,MONSET) \
    )  ",fDatabase);
  //    )  ENGINE = INNODB");

  Exec(sql);


  //hostinfo
  sprintf(sql,"CREATE TABLE IF NOT EXISTS %s.AFFAIR_HOSTINFO \
    ( \
       COMPUTER CHAR (128) NOT NULL, \
       TYPE CHAR (64), \
       KERNEL CHAR (64), \
       PROC_NUMBER INT, \
       CPU_FREQUENCY FLOAT, \
       MAX_MEMORY FLOAT, \
       PERIOD INT, \
       STATUS CHAR (32), \
       TS TIMESTAMP, \
       PRIMARY KEY (COMPUTER), \
       INDEX (COMPUTER) \
    )  ",fDatabase);
  //    )  ENGINE = INNODB");
  Exec(sql);


  //rrdlist
  sprintf(sql,"CREATE TABLE IF NOT EXISTS %s.AFFAIR_RRDLIST \
    ( \
       RRD_NAME CHAR (128) NOT NULL, \
       ROOT_NAME CHAR (128), \
       BASE_NAME CHAR (128), \
       MONSET  CHAR (128), \
       COMPUTER  CHAR (128), \
       TS TIMESTAMP, \
       PRIMARY KEY (RRD_NAME), \
       INDEX (RRD_NAME) \
    )  ",fDatabase);
  //    )  ENGINE = INNODB");
  Exec(sql);

  sprintf(sql,"CREATE TABLE IF NOT EXISTS %s.AFFAIR_USERNODELIST \
    ( \
       RRD_NAME CHAR (128) NOT NULL, \
       ROOT_NAME CHAR (128), \
       BASE_NAME CHAR (128), \
       MONSET  CHAR (128), \
       COMPUTER  CHAR (128), \
       PRIMARY KEY (RRD_NAME), \
       INDEX (RRD_NAME) \
    )  ",fDatabase);
  //    )  ENGINE = INNODB");
  Exec(sql);

  // root file list
  // ID INT   NOT NULL AUTO_INCREMENT, 

  sprintf(sql,"CREATE TABLE IF NOT EXISTS %s.AFFAIR_ROOTFILELIST \
    ( \
      DIR CHAR (200), \
      FILE CHAR (200) NOT NULL, \
      COMPUTER CHAR (128), \
      MONSET   CHAR (128), \
      BEGIN DATETIME, \
      END DATETIME, \
      TS TIMESTAMP, \
      PRIMARY KEY (FILE) \
    )  ",fDatabase);
  //    )  ENGINE = INNODB");

  Exec(sql);


  // plot list list
  sprintf(sql,"CREATE TABLE IF NOT EXISTS %s.AFFAIR_PLOTLIST \
    ( \
      NAME CHAR (200) NOT NULL, \
      USER CHAR (64) NOT NULL, \
      MONSET   CHAR (64) NOT NULL, \
      PLOT_TYPE     CHAR (64), \
      PLOT_SCOPE INT,   \
      TS TIMESTAMP, \
      PRIMARY KEY (NAME,MONSET,USER) \
    )  ",fDatabase);
  //    )  ENGINE = INNODB");

  Exec(sql);


  CreateParameterTables();

  //computer high low values
  // 
/*
  sprintf(sql,"CREATE TABLE IF NOT EXISTS affair.AFFAIR_HIGH_LOW_WARNINGS \
    ( \
       COMPUTER CHAR (128) NOT NULL, \
       MONSET CHAR (64) NOT NULL, \
       ID INT NOT NULL, \
       LOW FLOAT, \
       HIGH FLOAT, \
      PRIMARY KEY (COMPUTER,MONSET,ID), \
      INDEX (COMPUTER,MONSET,ID) \
    )  ");
*/ 
 //    )  ENGINE = INNODB");

  //  Exec(sql);

  return 1;

}

Int_t TAffairSql::AddEntryToTable( Char_t * tableName, Char_t * tableEntry) {

  Char_t  sql[512];
  Int_t result = 0;

  if (gDebug) cout <<" AddEntryToTable.begin " << endl;

  if (!fIsConnected) { 
    printf("Error:AddEntryToTable: No connections \n");
    return 0;
  }

  //assumption is only one field.
  // sprintf(sql,"insert into %s VALUES ('%s') ",tableName,tableEntry);
  sprintf(sql,"replace into %s.%s VALUES ('%s') ",fDatabase,tableName,tableEntry);
  printf("AddEntryToTable: sql is %s \n",sql);
  result=(Int_t) fDbSql->Exec(sql);
  if (result ==0) {
    printf ("Error:AddEntryToTable: result =0 \n");
  }

  if (gDebug) cout <<" AddEntryToTable.end " << endl;
  
  return 1;
}


Int_t TAffairSql::ReplaceRowWithString( Char_t * tableName, Char_t * rowLine) {
  char **argv;
  Int_t argc;
  Int_t i;

  if (gDebug) { printf("ReplaceRowWithString. 1 tableName is %s \n",tableName); }

  if (!fIsConnected) { 
    printf("Error:ReplaceRowWithString: No connections \n");
    return 0;
  }

  if((argv = string_to_argv(rowLine, &argc)) != NULL) {      
      QueryNew();
      QueryAppend("replace into %s.%s VALUES(",fDatabase,tableName);
      QueryAppendSval(argv[1],DBSQL_OPT_NONE);      
      for (i = 2; i < argc; i++) {
	QueryAppendSval(argv[i],DBSQL_OPT_WITH_COMA);
      }
      QueryAppend(")");
      if (gDebug) printf("ReplaceRowWithString. fvbuffer is %s \n",fVBuffer);
      MultiExec();
  }
  
  return 1;
}


Int_t TAffairSql::ReplaceRowWithString( Char_t * tableName, Char_t * rowLine, Int_t totEntries) {

  char **argv;
  Int_t argc;
  Int_t i;
  //need tot entries.assumption is that if count < tot entries, fill rest with null

  if (gDebug) { printf("ReplaceRowWithStringB. tableName is %s totEntries is %d \n",tableName,totEntries); }

  if (!fIsConnected) { 
    printf("Error:ReplaceRowWithString: No connections \n");
    return 0;
  }
  
  if((argv = string_to_argv(rowLine, &argc)) != NULL) {      
      QueryNew();
      QueryAppend("replace into %s.%s VALUES(",fDatabase,tableName);
      QueryAppendSval(argv[1],DBSQL_OPT_NONE);      
      for (i = 2; i < argc; i++) {
	QueryAppendSval(argv[i],DBSQL_OPT_WITH_COMA);
      }
      if (gDebug) { printf("ReplaceRowWithString. argc is %d \n",argc); }

      if (argc-1 < totEntries) {
	for (i = 1; i <= totEntries - argc + 1; i++) {
	  QueryAppend(",NULL");
	}	
      }
      QueryAppend(")");
      if (gDebug) printf("ReplaceRowWithString. fvbuffer is %s \n",fVBuffer);
      MultiExec();
  }
  
  return 1;
}


Int_t TAffairSql::RemoveEntryFromTable( Char_t * tableName, Char_t * tableEntry) {

  Char_t  sql[512];
  Int_t result = 0;
  //broken. only works for computers table. improve/remove in future
  if (!fIsConnected) { 
    printf("Error:RemoveEntryFromTable: No connections \n");
    return 0;
  }
  //assumption is only one field.
  sprintf(sql,"delete from %s.%s where  computer = '%s'",fDatabase,tableName,tableEntry);
  printf("RemoveEntryFromTable: sql is %s \n",sql);
  result=(Int_t) fDbSql->Exec(sql);
  if (result ==0) {
    printf ("Error:RemoveEntryFromTable: result =0 \n");
  }
  return 1;
}


Int_t TAffairSql::FillListBox( Char_t * tableName, TGListBox * myListBox) {

  Int_t nodeCount;
  nodeCount = 0;


  TSQLRow    *row;
  TSQLResult *res;

  //assumption is that list box filled from only or from first column in sql table
  Char_t  sql[512];

  if (!fIsConnected) { 
    printf("Error:FillListBox: No connections \n");
    return 0;
  }

  sprintf(sql,"select * from %s.%s ",fDatabase,tableName);
  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:FillListBox: res==NULL \n");
    return 0;
  }

  Int_t nrows = res->GetRowCount();
  int nfields = res->GetFieldCount();
   
  if (nrows==0 || nfields == 0) {
    printf("Error:FillListBox: nrows==0 || nfields == 0 sql=%s \n",sql);
    return 0;
  }

  for (int i = 0; i < nrows; i++) {
    row = res->Next();
    nodeCount++;
    myListBox->AddEntry(row->GetField(0),nodeCount);    
    delete row;
  }
  
  delete res;
  return nodeCount;
}




Int_t TAffairSql::FillComputerListTableFromRolesTable( Char_t * domainName) {

  Int_t nodeCount;
  nodeCount = 0;


  TSQLRow    *row;
  TSQLResult *res;

  //assumption is that list box filled from only or from first column in sql table
  Char_t  sql[512];

  if (!fIsConnected) { 
    printf("Error:FillComputerListTableFromRolesTable: No connections \n");
    return 0;
  }

  Char_t dateDatabase[200];

  if (getenv("DATE_DB_MYSQL_DB")) {
    sprintf(dateDatabase,"%s",getenv("DATE_DB_MYSQL_DB"));
  } else {
    cout << " DATE_DB_MYSQL_DB not defined. Exit " << endl;
    return 0;
  }

  sprintf(sql,"select DISTINCT HOSTNAME from %s.ROLES where HOSTNAME IS NOT NULL ",dateDatabase);
  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:FillComputerListTableFromRolesTable: res==NULL \n");
    return 0;
  }

  Int_t nrows = res->GetRowCount();
  int nfields = res->GetFieldCount();
   
  if (nrows==0 || nfields == 0) {
    printf("Error:FillComputerListTableFromRolesTable: nrows==0 || nfields == 0 sql is %s \n",sql);
    return 0;
  }

  for (int i = 0; i < nrows; i++) {
    row = res->Next();
    nodeCount++;
    Exec("REPLACE INTO %s.AFFAIR_COMPUTERLIST VALUES ('%s.%s')",fDatabase,row->GetField(0),domainName);
    delete row;
  }
  
  delete res;
  return nodeCount;
}



 string  * TAffairSql::GetColumn( Char_t * tableName, Int_t *rowNumber, Int_t col, Char_t * whereCondition) {

  TSQLRow    *row;
  TSQLResult *res;

  Char_t  sql[1000];
  string * myList = 0;

  if (!fIsConnected) { 
    printf("Error:GetColumn: No connections \n");
    return 0;
  }

  sprintf(sql,"select * from  %s.%s %s",fDatabase,tableName,whereCondition);
  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:GetColumn: res==NULL \n");
    return 0;
  }

  Int_t nrows = res->GetRowCount();
  int nfields = res->GetFieldCount();

  if (gDebug) printf("GetColumn: tableName is %s nrows = %i nfields =%i \n sql command is %s \n",tableName,nrows,nfields,sql );

  *rowNumber = nrows;
   
  if (nrows==0 || nfields == 0) {
    // printf("GetColumn: nrows==0 || nfields == 0. tableName = %s  sql=%s \n",tableName,sql);
    delete res;
    return 0;
  }

  if ( myList != 0) delete [] myList ;myList = 0;
  myList = new  string[nrows] ;    

  for (int i = 0; i < nrows; i++) {
    row = res->Next();
    myList[i] = row->GetField(col);
    delete row;
  }
  
  delete res;

  return myList;
}




string * TAffairSql::GetRow( Char_t * sql,Int_t myrow, Int_t *rowNumber, Int_t *colNumber) {

  Int_t nodeCount;
  nodeCount = 0;

  TSQLRow    *row;
  TSQLResult *res;
  string  *  myList = 0;

 

  if (!fIsConnected) { 
    printf("Error:FillRowString: No connections \n");
    return 0;
  }

  // sprintf(sql,"select * from %s.%s ",fDatabase,tableName);
  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:FillRowString: res==NULL \n");
    return 0;
  }

  Int_t nrows = res->GetRowCount();
  int nfields = res->GetFieldCount();

  *rowNumber = nrows;
  *colNumber = nfields;

  if (gDebug) printf("FillRowString: sql is %s nrows = %i nfields =%i \n",sql,nrows,nfields );
   
  if (nrows==0 || nfields == 0) {
    printf("Error:FillRowString: nrows==0 || nfields == 0 sql=%s\n",sql);
    return 0;
  }

  if (myrow < 0) {
    printf("Error:FillRowString: myrow <0 = %d \n",myrow);
    return 0;
  }

  if (gDebug) printf("FillRowString:sql is %s  myrow is %d \n ",sql,myrow);

  if ( myList != 0) delete [] myList ;myList = 0;
  myList = new  string[nfields] ;    

  for (int i = 0; i < nrows; i++) {
    row = res->Next();
    if (i== myrow) {
      if (gDebug) printf("FillRowString:selected row =%i values= ",i);
      for (int j = 0; j < nfields; j++) {
	if (row->GetField(j)) myList[j] = row->GetField(j);
	if (gDebug) printf(" %s ",myList[j].c_str());
      }
    }
    delete row;
  }
  if (gDebug) printf(" \n ");

  
  delete res;
  if (nrows) {
    return myList;
  } else {
    return 0;
  }
}


string  * TAffairSql::GetLabels( Char_t * sql, Int_t * fieldNumber, Int_t * rowNumber) {

  Int_t nodeCount;
  nodeCount = 0;

  TSQLResult *res;
  string  *  myList = 0;


  if (!fIsConnected) { 
    printf("Error:FillLabels: No connections \n");
    return 0;
  }

  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:FillLabels: res==NULL \n");
    return 0;
  }

  int nfields          = res->GetFieldCount();
  Int_t nrows          = res->GetRowCount();
   

  if ( nfields == 0) {
    printf("Error:FillLabels:  nfields == 0 \n");
    return 0;
  }

  if (gDebug) printf("FillLabels: nfields s %i \n",nfields);

  if ( myList != 0) delete [] myList ;myList = 0;
  myList = new  string[nfields] ;    


  if (gDebug) printf("FillLabels:nfields =%i ",nfields);
  for (int i = 0; i < nfields; i++) {
    myList[i] = res->GetFieldName(i);
    if (gDebug) printf(" %s ",myList[i].c_str());
  }
  if (gDebug) printf(" \n ");
  
  delete res;
  *fieldNumber = nfields;
  *rowNumber   = nrows;
  return myList;
}
 


string  * TAffairSql::GetMonsetParameterDescription( Char_t * monset) {

  Char_t  sql[500];
  TSQLResult *res;
  TSQLRow    *row;
  string  *  myList = 0;

  if (!fIsConnected) { 
    printf("Error:GetMonsetParameterDescription: No connections \n");
    return 0;
  }

  sprintf(sql,"select * from %s.AFFAIR_MONSET_DEFAULTPARS WHERE MONSET = '%s'",fDatabase,monset);
  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:GetMonsetParameterDescription: res==NULL \n");
    return 0;
  }

  int nfields          = res->GetFieldCount();
  Int_t nrows          = res->GetRowCount();
   
  

  if ( nfields == 0 || nrows == 0) {
    printf("Error:GetMonsetParameterDescription:  nfields == 0 or nrows == 0\n");
    return 0;
  }

  if ( nrows > 1) {
    printf("Error:GetMonsetParameterDescription:  nrows > 1 = %d \n",nrows);
    return 0;
  }
  row = res->Next();

  if (gDebug) printf("GetMonsetParameterDescription: nfields s %i \n",nfields);


  if (nfields <= kMonsetParOffset) {
    if (gDebug) printf("GetMonsetParameterDescription: No paramerers. nfields s %i \n",nfields);
    return 0;
  }


  if ( myList != 0) delete [] myList ;myList = 0;
  myList = new  string[nfields] ;    

  Char_t    fieldName[100];
  Char_t    testIfParField[10];
  Int_t     parId;
  Char_t     parIdChar[100];
  Char_t    field[100];
  Char_t    desc[200];

  Char_t * ptr;

  if (gDebug) printf("GetMonsetParameterDescription:nfields =%i ",nfields);

  for (int i = 0; i < nfields; i++) {
    sprintf(fieldName,res->GetFieldName(i));
    // see if first three letters are PAR
    snprintf(testIfParField,4, "%s",fieldName);      
    myList[i] = "";
    if (strcmp(testIfParField,"PAR")==0) {
      sscanf(fieldName + 3, "%s",parIdChar);
      parId = atoi(parIdChar);
      // now get par desription
      Int_t offsetUnderline = -1;
      // should be juist one row
      if  (row->GetField(i) ) {
	sprintf(field,"%s",row->GetField(i));
	for (ptr=field;;ptr++){
	  if (*ptr==0) break;
	  if (*ptr=='_') offsetUnderline = (ptr - field)/sizeof(Char_t);
	}       	
      }
      if (offsetUnderline>0 && parId > 0 && parId <= nfields) {
	snprintf(desc,offsetUnderline+1, "%s",field);
	myList[MONSET_PARAMETERS_OFFSET+parId-1] = desc;
      }      
    }
  }
  
  delete res;
  return myList;
}



string * TAffairSql::GetVariableInfoForMonitorSet( Char_t * monset, Char_t * varSet, Int_t  *fullVarNumber,  Int_t   *varNumber ) {

  Int_t       globalPlotIndex    =  5-1;
  Int_t       columnPlotIndex    =  6-1;
  Int_t       eachPlotIndex      =  7-1;
  Int_t       multiVarPlotIndex  = 12-1;

  string * myList = 0;

  TSQLRow    *row;
  TSQLResult *res;

  Char_t  sql[512];
  Char_t  line[512];
  Int_t   varNumberTmp =0;


  if (!fIsConnected) { 
    printf("Error:GetVariableInfoForMonitorSet: No connections \n");
    return 0;
  }

  sprintf(sql,"select * from  %s.AFFAIR_%s ",fDatabase,monset);
  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:GetVariableInfoForMonitorSet: res==NULL \n");
    return 0;
  }

  *fullVarNumber        = res->GetRowCount();
  Int_t nfields        = res->GetFieldCount();
   
  if (*fullVarNumber == 0 || nfields == 0) {
    printf("Error:GetVariableInfoForMonitorSet: nrows==0 || nfields == 0 sql=%s\n",sql);
    return 0;
  }

  if ( myList != 0) delete [] myList ;myList = 0;
  myList = new  string[*fullVarNumber] ;    

  varNumberTmp = 0;

  sprintf(line,"");
  for (Int_t i = 0; i < *fullVarNumber; i++) {
    row = res->Next();
    sprintf(line,"");
    for (Int_t j = 0; j < nfields; j++) {
      sprintf(line,"%s %s ",line, row->GetField(j));
    }
    myList[i] = line;
    if (gDebug) printf("GetVariableInfoForMonitorSet: varSet is %s nfields is %d \n myList[i] is %s \n",varSet,nfields,myList[i].c_str());

    if (gDebug) cout << "GetVariableInfoForMonitorSet. have var strcmp(varSet,each)="<<strcmp(varSet,"each") << " atoi(row->GetField(eachPlotIndex))=" << row->GetField(eachPlotIndex)<< " eachPlotIndex="<<eachPlotIndex << endl;

    if (( strcmp(varSet,"global") == 0 && atoi(row->GetField(globalPlotIndex)) > -2 )
	||(strcmp(varSet,"each")  == 0 && atoi(row->GetField(eachPlotIndex)) > -2 )
	||(strcmp(varSet,"all")   == 0 ) 
	||(  atoi(row->GetField(columnPlotIndex)) > -2 ) 
	||(strcmp(varSet,"global_and_each") == 0 && atoi(row->GetField(globalPlotIndex)) > -2 && atoi(row->GetField(eachPlotIndex)) > -2 )
	||( atoi(row->GetField(multiVarPlotIndex)) > -2)) {
      varNumberTmp++;    //counts total number of used variables
      if (gDebug) cout << "GetVariableInfoForMonitorSet. have var " << endl;
    }
    delete row;
  }  
  delete res;

  *varNumber = varNumberTmp;
  if (gDebug) printf("GetVariableInfoForMonitorSet: varNumber is %d  fullVarNumber is %d \n",*varNumber,*fullVarNumber);
  return myList;
}


string * TAffairSql::GetInfoFromComputersTable( string *  rrdList,  Int_t fCompNumber ) {
  
  TSQLRow    *row;
  TSQLResult *res;

  Char_t      sql[512];
  Char_t      line[2000];
  char      **argv;
  Int_t       argc;
  Int_t       compId;
  string  *  myList = 0;

  if (!fIsConnected) { 
    printf("Error:GetInfoFromComputersTable: No connections \n");
    return 0;
  }

  sprintf(sql,"select * from %s.AFFAIR_COMPUTERPARS ORDER BY COMPUTER,MONSET ",fDatabase);
  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:GetInfoFromComputersTable: res==NULL \n");
    return 0;
  }

  if (gDebug) printf("GetInfoFromComputersTable:1 \n");

  Int_t nrows          = res->GetRowCount();
  Int_t nfields        = res->GetFieldCount();
   
  if (nrows  == 0 || nfields == 0) {
    printf("Error:GetInfoFromComputersTable: nrows==0 || nfields == 0 sql=%s \n",sql);
    return 0;
  }

  if ( myList != 0) delete [] myList ;myList = 0;
  myList = new  string[fCompNumber] ;    

  if (gDebug) printf("GetInfoFromComputersTable:nrows =%i nfields = %i \n",nrows,nfields);

  for (Int_t i = 0; i < nrows; i++) {
    row = res->Next();
    // get  compid. In future use maps, when I figure out how to do it in root
    compId = -1;
    for (Int_t ii = 0; ii < fCompNumber; ii++) {
      //    if (gDebug) printf("GetInfoFromComputersTable:rrdList[ii].c_str() = %s \n",rrdList[ii].c_str());

      if((argv = string_to_argv((Char_t *) rrdList[ii].c_str(), &argc)) != NULL) {
	if ( (strcmp(row->GetField(0),argv[5]) == 0) &&  (strcmp(row->GetField(1),argv[4]) == 0) ) {
	  compId = ii;
	  if (gDebug) printf("GetInfoFromComputersTable:compId = %d row->GetField(0)=%s  argv[5]=%s row->GetField(1)=%s argv[4]=%s \n",compId,row->GetField(0),argv[5],row->GetField(1),argv[4]);
	  break;
	}  
      }
    }
    if ( compId > -1) {
      sprintf(line,"");
      for (Int_t j = 0; j < nfields; j++) sprintf(line,"%s %s ",line,row->GetField(j));
      myList[compId] = line;
      if (gDebug) printf("GetInfoFromComputersTable:myList[compId] = %s \n",myList[compId].c_str());
    }
    delete row;
  }
  
  delete res;
  return myList;
}


string  *  TAffairSql::GetRrdRows( Char_t * monitorSet, Int_t *compNumber) 
{
  TSQLRow    *row;
  TSQLResult *res;
  string * myList = 0;

  Char_t  sql[512];
  Char_t  line[1000];

  if (!fIsConnected) { 
    printf("Error:GetRrdRows: No connections \n");
    return 0;
  }

  sprintf(sql,"select * from %s.AFFAIR_RRDLIST WHERE MONSET = '%s' ORDER BY COMPUTER",fDatabase,monitorSet);

  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:GetRrdRows: res==NULL \n");
    return 0;
  }

  Int_t nrows          = res->GetRowCount();
  Int_t nfields        = res->GetFieldCount();
   
  if (nrows == 0 || nfields == 0) {
    *compNumber =  nrows;
    if (gDebug) printf("Error:GetRrdRows: nrows==0 || nfields == 0 sql=%s \n",sql);
    return 0;
  }

  if ( myList != 0) delete [] myList ;myList = 0;
  myList = new  string[nrows] ;    

  Int_t count = 0;

  for (Int_t i = 0; i < nrows; i++) {
    row = res->Next();
    sprintf(line,"%s"," ");
    for (Int_t j = 0; j < nfields; j++) {
      sprintf(line,"%s %s",line, row->GetField(j));
    }
    myList[i] = line;
    if (gDebug) printf("GetRrdRows: mylist[i]= %s \n",myList[i].c_str());
    count++;  
    delete row;
  }
  
  delete res;
  *compNumber =  nrows;
  return myList;
}


Int_t TAffairSql::GetNumberOfRows( Char_t * tableName) {

  Int_t nodeCount;
  nodeCount = 0;

  TSQLResult *res;

  Char_t  sql[512];

  if (!fIsConnected) { 
    printf("Error:GetTableRowCount: No connections \n");
    return 0;
  }

  sprintf(sql,"select * from %s.%s ",fDatabase,tableName);
  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:GetTableRowCount: res==NULL \n");
    return 0;
  }

  Int_t nrows = res->GetRowCount();
  
  delete res;
  return nrows;
}



Int_t TAffairSql::GetNumberOfRowsSql( Char_t * sql) {

  Int_t nodeCount;
  nodeCount = 0;

  TSQLResult *res;


  if (!fIsConnected) { 
    printf("Error:GetTableRowCount: No connections \n");
    return 0;
  }

  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:GetTableRowCount: res==NULL \n");
    return 0;
  }

  Int_t nrows = res->GetRowCount();
  
  delete res;
  return nrows;
}


string * TAffairSql::GetRootFileList(Char_t * beginTimeSqlFormat,Char_t *endTimeSqlFormat,Char_t *compname,Char_t *gMonitorSet, Int_t *entries) {

  //  Int_t nodeCount;
  //  nodeCount = 0;

  TSQLRow    *row;
  TSQLResult *res;

  string * myList = 0;

  //assumption is that list box filld from only or from first column in sql table
  Char_t  sql[MAX_QUERY_SIZE];

  if (!fIsConnected) { 
    printf("Error:SetRootfilelist: No connections \n");
    return 0;
  }

  sprintf(sql,"SELECT DIR,FILE FROM %s.AFFAIR_ROOTFILELIST WHERE BEGIN <='%s' AND  END >='%s' AND MONSET = '%s'",fDatabase,endTimeSqlFormat,beginTimeSqlFormat,gMonitorSet);

  if (strlen(compname) > 2 ) sprintf(sql,"%s AND COMPUTER = '%s'",sql,compname);

  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:SetRootfilelist: res==NULL \n");
    return 0;
  }

  Int_t nrows = res->GetRowCount();
  int nfields = res->GetFieldCount();
   
  if (nrows==0 || nfields == 0) {
    printf("Error:SetRootfilelist: nrows==0 || nfields == 0 sql is %s \n",sql);
    return 0;
  }

  if ( myList != 0) delete [] myList ;myList = 0;
  myList = new  string[nrows] ;    

  Char_t fullfilename[512];

  for (int i = 0; i < nrows; i++) {
    row = res->Next();
    sprintf(fullfilename,"%s/%s",row->GetField(0),row->GetField(1));
    myList[i] = fullfilename;
    if (gDebug) printf("selected root file name is %s \n",fullfilename);
    delete row;
  }
  
  delete res;
  *entries = nrows;
  return myList;
}


/*
string * TAffairSql::GetRootFileHistory(Char_t * beginTimeSqlFormat,Char_t *endTimeSqlFormat,Char_t *compname,Char_t *gMonitorSet, Int_t *entries) {

  //  Int_t nodeCount;
  //  nodeCount = 0;

  TSQLRow    *row;
  TSQLResult *res;

  string * myList = 0;

  //assumption is that list box filled from only or from first column in sql table
  Char_t  sql[MAX_QUERY_SIZE];

  if (!fIsConnected) { 
    printf("Error:SetRootfilelist: No connections \n");
    return 0;
  }

  sprintf(sql,"SELECT DIR,FILE,BEGIN,END FROM %s.AFFAIR_ROOTFILELIST WHERE BEGIN <='%s' AND  END >='%s' AND MONSET = '%s'",fDatabase,endTimeSqlFormat,beginTimeSqlFormat,gMonitorSet);

  if (strlen(compname) > 2 ) sprintf(sql,"%s AND COMPUTER = '%s'",sql,compname);

  res = fDbSql->Query(sql);

  if (res==NULL) {
    printf("Error:SetRootfilelist: res==NULL \n");
    return 0;
  }

  Int_t nrows = res->GetRowCount();
  int nfields = res->GetFieldCount();
   
  if (nrows==0 || nfields == 0) {
    printf("Error:SetRootfilelist: nrows==0 || nfields == 0 sql is %s \n",sql);
    return 0;
  }

  if ( myList != 0) delete [] myList ;myList = 0;
  myList = new  string[nrows] ;    

  Char_t fullfilename[512];

  timeRange=endTimeSqlFormat-beginTimeSqlFormat;
  binSize=timeRange/binNumber; endbin && endtimerow >=


  for (int i = 0; i < nrows; i++) {
    row = res->Next();
    sprintf(fullfilename,"%s/%s",row->GetField(0),row->GetField(1));
    myList[i] = fullfilename;

    if begintimerow <= endbin && endtimerow >= beginbin 
    yValues[(i*varNumber)*binNumber+varUsed*binNumber+bin] = ; 
    if (gDebug) printf("selected root file name is %s \n",fullfilename);
    delete row;
  }
  
  delete res;
  *entries = nrows;
  return myList;
}

*/


/*................................................................
    check if the DATE database is configured
    returns: 1 if configured, 0 if not
................................................................*/
Int_t TAffairSql::IsConfiguredMysql(){
  char *v;
  char *affairSqlStatus;
   
  Int_t status = 1;
  /* SQL DB activated? */
  v               =  getenv("DATE_DB_MYSQL");
  //affairSqlStatus =  getenv("AFFAIR_DB_MYSQL");

  if (gDebug) cout << "IsConfiguredMysql:DATE_DB_MYSQL= "<< getenv("DATE_DB_MYSQL")<< " affairSqlStatus= "<<affairSqlStatus <<endl; 

  if (v != NULL) {
    if (strcmp(v,"TRUE")) status = 0;
  }

  // AFFAIR_DB_MYSQL has been added to have an option to explicitly not use mysql. 
  //If set to anything execpt false, 
    if ( v==NULL  && affairSqlStatus != NULL ) {
      if (strcmp(affairSqlStatus,"FALSE")==0)  {
	status = 0;
      } else {
	status = 1;
      }
    }
    if ( v==NULL  && affairSqlStatus == NULL ) status = 0;

  return status;
}


Int_t TAffairSql::CreateDefaultMonitoringSetTables() 
{
  //  Char_t  sql[1024];
  Char_t  aux[512];
  Char_t  line[512]; 

  Char_t *ptr;
  Char_t **argv;
  Int_t  argc;

  sprintf(aux,"%s/config/mysql_default/monsetDefault.conf",fAffairPath);
  FILE * table_fp = fopen(aux,"r");    
  while (fgets(line,512,table_fp)) {     
    for(ptr=line;;ptr++){
      if (*ptr==0) break;
      if (!isspace(*ptr)) break;
    }       
    if (*ptr=='#') continue;
    if (strlen(line) < 2) continue;
    if((argv = string_to_argv(line, &argc)) != NULL) {
      //      if (gDebug) printf("about to execute sql=%s with arg %s \n",sql,argv[1]);
      CreateMonitoringSetTable(argv[1],0) ;
      //      Exec(sql,argv[1]);
    }
  }

  return 1;

}




Int_t TAffairSql::CreateMonitoringSetTable(Char_t * monset,Int_t newSetFlag) 
{
  Char_t  sql[1024];

  //DATEMON GDC LDC
  sprintf(sql,"CREATE TABLE IF NOT EXISTS %s.AFFAIR_%s \
    ( \
       ID INT UNIQUE NOT NULL, \
       VARIABLE CHAR (32), \
       SCALING FLOAT, \
       TYPE CHAR(32), \
       GLOBAL_PLOT_OPT INT NOT NULL, \
       SNAPSHOT_PLOT_OPT INT NOT NULL, \
       INDIVIDUAL_PLOT_OPT INT NOT NULL, \
       SHARED_TITLE CHAR(64), \
       Y_AXIS_TITLE CHAR(64), \
       PAR_INDEX INT NOT NULL, \
       PLOT_TYPE CHAR(64), \
       GLOBAL_PLOT_MULTIVAR_OPT INT NOT NULL, \
       REF_VARIABLE_ID INT NOT NULL, \
       LOW_LEVEL_WARNING FLOAT, \
       HIGH_LEVEL_WARNING FLOAT, \
       PRIMARY KEY (ID), \
       INDEX (ID), \
       INDEX (VARIABLE) \
    )  ",fDatabase,"%s");
  //    )  ENGINE = INNODB","%s");


  if (gDebug) { printf("query is %s monset is %s \n",sql,monset); }
  Exec(sql,monset);
  //should add one row
  // WHY????????
  // perhaps only if there is no default values????
  // anyway, this is a bug so, uncomment the next line and get back to it later

  if(newSetFlag == 1)  CreateMonitoringSetRow(monset);


  return 1;

}



Int_t TAffairSql::CreateParameterTables() 
{
  Char_t  sql[1024];

  sprintf(sql,"drop table  if exists AFFAIR_PARS " );
  fDbSql->Exec(sql);	

  sprintf(sql,"drop table  if exists AFFAIR_STATUS " );
  fDbSql->Exec(sql);	


  // parameters
  sprintf(sql,"CREATE TABLE IF NOT EXISTS %s.AFFAIR_PARS \
    ( \
      ID  INT NOT NULL, \
      USE_TIME_FROM_REMOTE_NODE INT NOT NULL, \
      CANVAS_SCALE FLOAT, \
      LINE_WIDTH FLOAT, \
      FONT_SIZE FLOAT, \
      TIMEOUT_PERIOD FLOAT, \
      HIGH_WARNING_COLOR CHAR (32), \
      LOW_WARNING_COLOR CHAR (32), \
      MAX_GRAPHS_PER_PLOT INT, \
      PLOT_CYCLE_DELAY_SEC FLOAT, \ 
      MAX_ITER_ROOT_STORE INT, \
      PRIMARY KEY (ID) \
    )  ",fDatabase);
  //    )  ENGINE = INNODB");

  Exec(sql);



  // parameters
  sprintf(sql,"CREATE TABLE IF NOT EXISTS %s.AFFAIR_STATUS \
    ( \
      MONSET   CHAR (64) NOT NULL, \
      HIGH_WARNING_COUNT INT, \
      LOW_WARNING_COUNT INT, \
      DEAD_NODE_COUNT INT, \
      ACTIVE_NODE_COUNT INT, \
      TS TIMESTAMP, \
      TS_PLOT_SCOPE_1  TIMESTAMP, \ 
      TS_PLOT_SCOPE_2  TIMESTAMP, \ 
      TS_PLOT_SCOPE_3  TIMESTAMP, \ 
      PRIMARY KEY (MONSET) \
    )  ",fDatabase);
  //    )  ENGINE = INNODB");

  Exec(sql);


  return 1;

}


char** TAffairSql::string_to_argv(char *argstring, int *argc)
{
  char inquotes = 0, *workstring, **argv;
  int i, nchars;
  
  if((nchars = strlen(argstring)) > 0) {
    workstring = (char*)calloc((nchars + 2), sizeof(char*));
    //	fill the array with space characters
    for(i=0; i < (nchars + 2); i++) workstring[i] = ' '; 	  
    //	copy the argstring into the workstring (padded with spaces on both ends)
    for(i=0; i < nchars; i++) workstring[i+1] = argstring[i];
    for((*argc) = 1, i=0; i < nchars + 2; i++) {
      //	count the number of arguements
      if(isgraph(workstring[i]) > 0) {
	//	if current character is not whitespace and previous 
	//      character is whitespace, increment (*argc)
	if((isgraph(workstring[i-1]) == 0) && (inquotes == 0)) (*argc)++;
	//	check for the quote character
	if(workstring[i] == '"') {
	  //	toggle inquotes flag
	  inquotes = (char)(!inquotes);
	}		
      } else {
	//	convert whitespace to null characters
	if(!inquotes) workstring[i] = '\0';
      }
    }
    
    //	no arguements in the string
    if((*argc) == 0){
      free(workstring);
      return NULL;
    } else {
      inquotes = 0;		  
      argv = (char**)calloc((*argc), sizeof(char**));	    
      argv[0] = &workstring[0];
      
      for((*argc) = 1, i=1; i < nchars + 2; i++) {
	// count the number of arguements
	if(isgraph(workstring[i]) > 0) {
	  // if current character is not whitespace and previous 
	  // character is whitespace, increment (*argc)
	  if((isgraph(workstring[i-1]) == 0) && (inquotes == 0)) {
	    argv[(*argc)] = &workstring[i];
	    (*argc)++;
	  }
	  //	check for the quote character
	  if(workstring[i] == '"') {
	    //	toggle inquotes flag
	    inquotes = (char)(!inquotes);
	  }
	}
      }	  
      return argv;
    }
  }
  else {
    (*argc) = 0;
    return NULL;
  }
}
