#ifndef TAffairSql_h
#define TAffairSql_h

#include <TROOT.h>
#include <stdio.h>
#include <TSystem.h>
#include <TString.h>
#include <TError.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <string>

#include <sys/stat.h>

#include <TSQLServer.h>
#include <TSQLResult.h>
#include <TSQLRow.h>
#include <TGListBox.h>
#include <TList.h>
#include <stdarg.h>
#include "TObject.h"
#include "MonitorConstants.h"



namespace std {}
using namespace std;


/* DB access - default definitions */
#define MYSQL_HOST   "localhost"          /* database server name */
#define MYSQL_DB     "affair"          /* database name */
#define MYSQL_USERID "affair"          /* database user id */
#define MYSQL_PASSWD "affair"          /* userid password */
/* constants */
#define MAX_QUERY_SIZE 2000      /* maximum size of a SQL query */
#define MONSET_PARAMETERS_OFFSET 4




class TAffairSql:public TObject {
 private:

  Char_t  fUserid[128];
  Char_t  fPasswd[128];
  Char_t  fDatabase[128];
  Char_t  fHost[128];
  Char_t  fAffairPath[256];

  TSQLServer *fDbSql;

  Int_t   fIsConnected;  /* set to 1 (or more) when mysql connected */
  
  char    fVBuffer[MAX_QUERY_SIZE];     /* a buffer for SQL queries */
  Int_t   fVChars;                      /* number of chars in the current query */  

  Int_t  MultiExec();
  Int_t  QueryNew();
  Int_t  QueryAppend(const char *s,...);
  Int_t  QueryAppendSval(const char *s_in,Int_t options);
  char** string_to_argv(char *argstring, int *argc);
  Int_t  CreateDefaultMonitoringSetTables();
  Int_t  DestroyMonitoringSetTables();
  Int_t  FillMonitoringSetDefaultValues();
  Int_t  CreateParameterTables();
  Int_t  FillParameterDefaultValues();

 public:

  Int_t  OpenDatabase();
  Int_t  SourceFile(char * file);
  Int_t  CreateDefaultTables();
  Int_t  DestroyTables();
  Int_t  CreateTables();
  Int_t  FillDefaultValues();
  Int_t  FillListBox( Char_t * tableName, TGListBox * myListBox);
  string  * GetColumn( Char_t * tableName, Int_t *rowNumber, Int_t col, Char_t * whereCondition);
  string *  GetRow( Char_t * sql,Int_t myrow, Int_t *rowNumber, Int_t *colNumber);
  Int_t  ShowContentsAllTables();
  Int_t  AddEntryToTable( Char_t * tableName, Char_t * tableEntry);
  Int_t  RemoveEntryFromTable( Char_t * tableName, Char_t * tableEntry);
  Int_t  ReplaceRowWithString( Char_t * tableName, Char_t * rowLine);
  Int_t  ReplaceRowWithString( Char_t * tableName, Char_t * rowLine, Int_t totEntries);
  Int_t  IsConfiguredMysql();
  Int_t  Exec(const char *query,...);
  Int_t  GetNumberOfRows( Char_t * tableName);
  Int_t  UpgradeTablesForAffairVersion315();
  Int_t  GetNumberOfRowsSql( Char_t * sql) ;
  string * GetRootFileList(Char_t * beginTimeSqlFormat,Char_t *endTimeSqlFormat,Char_t *compname,Char_t *monitorSet,Int_t * entries);
  string * GetVariableInfoForMonitorSet( Char_t * tableName, Char_t * varSet, Int_t  *fullVarNumber,  Int_t   *varNumber );
  string * GetRrdRows( Char_t * monitorSet, Int_t *compNumber);
  string * GetInfoFromComputersTable( string *  rrdList,  Int_t fCompNumber );
  string * GetLabels( Char_t * sql, Int_t * fieldNumber, Int_t * rowNumber);
  string * GetMonsetParameterDescription( Char_t * monset);
  Int_t  FillRrdListDefaultValues();
  Int_t  FillComputerListTableFromRolesTable( Char_t * domainName);
  Int_t  FillRootfilelistDefaultValues();
  // Int_t  CreateMonitoringSetDefaultparsRow();
  Char_t * CreateMonitoringSetDefaultparsRow();
  Int_t  CreateMonitoringSetRow( Char_t * monset);
  Int_t  CreateMonitoringSetTable(Char_t * monset,Int_t newSetFlag);
  Char_t *  GetAffairParameter(Int_t col);
  Char_t *  GetDatabaseName();

  TAffairSql();
  ~TAffairSql();

   ClassDef (TAffairSql,0) // affair class
};

#endif

#ifdef TAffairSql_cxx
TAffairSql::~TAffairSql()
{
  if(gDebug) cout << "TAffairSql.destructor " <<endl;
}

TAffairSql::TAffairSql()
{

  if (getenv("AFFAIR_MYSQL_USER")) {
    sprintf(fUserid,"%s",getenv("AFFAIR_MYSQL_USER"));
  } else {
    sprintf(fUserid,"%s",MYSQL_USERID);
  }


  if (getenv("AFFAIR_MYSQL_PWD")) {
    sprintf(fPasswd,"%s",getenv("AFFAIR_MYSQL_PWD"));
  } else {
    sprintf(fPasswd,"%s",MYSQL_PASSWD);

  }


  if (getenv("AFFAIR_MYSQL_DB")) {
    sprintf(fDatabase,"%s",getenv("AFFAIR_MYSQL_DB"));
  } else {
    sprintf(fDatabase,"%s",MYSQL_DB);
  }
  
  if (strlen(fDatabase) < 2 ) {
    printf("ERROR! NO DATABASE NAME SPECIFIED. =%s \n",fDatabase);
  }


  if (getenv("AFFAIR_MYSQL_HOST")) {
    sprintf(fHost,"%s",getenv("AFFAIR_MYSQL_HOST"));
  } else {
    sprintf(fHost,"%s",MYSQL_HOST);
  }

  if (gDebug) cout << " fHost is " << fHost << " fDatabase is " << fDatabase << " fUserid is " << fUserid << " fPasswd is " << fPasswd << endl;

  fIsConnected = 0;  /* set to 1 (or more) when mysql connected */

  sprintf(fAffairPath,"%s",getenv("AFFAIR_MONITOR"));

  fDbSql       = 0;

  //  if (fDbSql != 0) delete fDbSql;
  //fDbSql = new TSqlServer;

};

#endif // #ifdef TAffairSql_cxx

