/* AffairRoot
 * reads info from rrd files, stores in root files and creates gif plots
 *
 *
 * Start Date    : 1-8-2001
 * Written by    : tome.anticic@cern.ch
 */

#define TDataProcess_cxx

#include "affairGlobals.h"
#include "TDataProcess.h"

/// ClassImp(TDataProcess)

void TDataProcess::GlobalPlotsOffline(Char_t *plotLabel,Double_t timeUnit,Int_t gridNumber,Int_t timeNow, Int_t globalTypeOpt,
			       Int_t selectionVariable, Double_t selectionLowCut,Double_t selectionHighCut)      
{
    Float_t X[fBinNumber];   

    //create directory plotLable(username really)
    //fPlotMaker->SetOutputMode(k_YES_PNG);
    fPlotMaker->SetUser(plotLabel);

    // default case, when have a simple last time period
    fTimeMax = timeNow;
    fBinSize = timeUnit;    
    fTimeMin = (Int_t)(fTimeMax-fBinNumber*fBinSize);

    if (selectionVariable > -1) {
	// case when told to take a particular run period 
	FindRange(selectionVariable,selectionLowCut,selectionHighCut);
	fBinSize = ((Double_t)(fTimeMax-fTimeMin))/( (Double_t)fBinNumber);    
    }

    if( fCompNumber == 0 ) {cout   << getTime() << "GlobalPlotsOffline. Loop plotsfCompNumber=0. Don't fill at all " << endl; return;}
    

    fPlotMaker->SetRange(fTimeMin,fTimeMax);
    fPlotMaker->SetCut(selectionVariable,selectionLowCut,selectionHighCut);
    fPlotMaker->SetGraphOptions(gridNumber);

    if (fChain == 0) return;   

    InitXArray(X);   
    InitArrays();    // initialize to 0     
    FillEachOffline();
    if( fAnalysisLevel > 0 ) DoAnalysis();
    FillSumArrays();

    //prepare stuff for global plots. take care of compup status,as relevant only for online, using rrd
    // so bypass
    fCompActiveNumber= fCompNumber;
    for (Int_t i = 0; i < fCompNumber; i++){ 
     fReduced2FullCompActiveIndex[i]  = i;
    }

    DoGlobalGraphs(plotLabel, X, globalTypeOpt);    // plot summed plots
    if (gDebug > 0) cout  << getTime() << " about to do global histo " << endl;
    DoGlobalHisto(plotLabel);                       // plot histograms
    if (gDebug > 0) cout  << getTime() << " end global histo " << endl;

    //make dirs in root file
    if (gDebug > 0) cout  << getTime() << " about to print each plots " << " fEachPlotsOption=" << fEachPlotsOption <<endl;
    if (fEachPlotsOption > 0) {
      if (fFillRootFileFlag == true) for (Int_t lComp = 0; lComp < fCompNumber; lComp++) fPlotMaker->MkDir(fCompNumber,lComp,fCompPars[lComp].compName);
      if (gDebug > 0) cout  << getTime() << " about to print each pgraphs " << endl;
      DoEachGraphs(plotLabel,X); // plot individual plots
      if (gDebug > 0) cout  << getTime() << " about to print each histos " << endl;
      DoEachHisto(plotLabel);  // plot histograms
    }
    if (fFillRootFileFlag == true) fPlotMaker->CloseRootFile();
}


void TDataProcess::GlobalPlotsRrd(Char_t *plotLabel,Double_t timeUnit,Int_t gridNumber, Int_t timeNow, Int_t globalTypeOpt)
{

    Float_t X[fBinNumber];   

    fTimeMax = timeNow;
    fBinSize = timeUnit;    
    fTimeMin = (Int_t)(fTimeMax-fBinNumber*fBinSize);

    if( fCompNumber == 0 ) {
	cout  << getTime() << " Loop plotsfCompNumber=0. Don't fill at all " << endl;
	return;
    }

    fPlotMaker->SetRange(fTimeMin,fTimeMax);
    fPlotMaker->SetGraphOptions(gridNumber);

    InitXArray(X);    
    InitArrays(); 
        
    //Int_t returnFlag = FillEachRrd(plotLabel, globalTypeOpt);
    Int_t returnFlag = FillEachRrd();
    if (gDebug > 0) cout  << getTime() << " global return flags = " << returnFlag << endl; 
    if (returnFlag > 0) {
	FillSumArrays();     
	DoGlobalGraphs(plotLabel, X, globalTypeOpt);     
    }
    else {
	if (gDebug > 0) cout  << getTime() << kBeep << " ot succesfully filleachrrd return = " << returnFlag <<endl;
    }

    fInitFlag = false;
    fPlotMaker->SetLogFlag(fInitFlag);
}


void TDataProcess::EachPlotsRrd(Char_t *plotLabel,Double_t timeUnit, Int_t gridNumber, Int_t timeNow)
{

    Float_t X[fBinNumber];   
    //    Int_t globalTypeOpt = k_SUM_AND_SUPERIMPOSE; //dummy variable for this method
    if(fCompNumber==0){
	cout  << getTime()  << " Loop plotsfCompNumber=0. Don't fill at all " << endl;
	return;
    }

    fTimeMax = timeNow;
    fBinSize = timeUnit;    
    fTimeMin = (Int_t)(fTimeMax-fBinNumber*fBinSize);

    fPlotMaker->SetRange(fTimeMin,fTimeMax);
    fPlotMaker->SetGraphOptions(gridNumber);

    InitXArray(X);   
    InitArrays();
    // Int_t returnFlag = FillEachRrd(plotLabel,globalTypeOpt);   
    Int_t returnFlag = FillEachRrd();   
    
    if (returnFlag > 0) {         
	DoEachGraphs(plotLabel, X);
    }
    else {
	if (gDebug > 0) cout  << getTime() << kBeep << " not succesfully filleachrrd return = " << returnFlag <<endl;
    }
    fInitFlag = false;
    fPlotMaker->SetLogFlag(fInitFlag);
}

void TDataProcess::EachPlotsOffline(Char_t *plotLabel,Double_t timeUnit, Int_t gridNumber,
			     Int_t timeNow, Int_t selectionVariable, Double_t selectionLowCut,Double_t selectionHighCut)
{

    Float_t X[fBinNumber];   

    //create directory plotLable(username really)
    //fPlotMaker->SetOutputMode(k_YES_PNG);
    fPlotMaker->SetUser(plotLabel);

    // default case, when have a simple last time period
    fTimeMax = timeNow;
    fBinSize = timeUnit;    
    fTimeMin = (Int_t) (fTimeMax-fBinNumber*fBinSize);

    if (selectionVariable > -1) {
	// case when told to take a particular run period 
	FindRange(selectionVariable,selectionLowCut,selectionHighCut);
	fBinSize = ((Double_t)(fTimeMax-fTimeMin))/( (Double_t)fBinNumber);    
    }

    if(fCompNumber == 0){ cout  << getTime()  << " Loop plotsfCompNumber=0. Don't fill at all " << endl; return;}
    //make dirs in root file
    if (fFillRootFileFlag == true)  for (Int_t lComp = 0; lComp < fCompNumber; lComp++) fPlotMaker->MkDir(fCompNumber,lComp,fCompPars[lComp].compName);
    

    fPlotMaker->SetRange(fTimeMin,fTimeMax);
    fPlotMaker->SetCut(selectionVariable,selectionLowCut,selectionHighCut);
    fPlotMaker->SetGraphOptions(gridNumber);

    if (fChain == 0) return;       

    InitXArray(X);   
    // initialize to 0
    InitArrays();         
    FillEachOffline();
    DoEachGraphs(plotLabel, X); // plot individual plots
    DoEachHisto(plotLabel);  // plot histograms
    if (fFillRootFileFlag == true) fPlotMaker->CloseRootFile();
}



void TDataProcess::MakeGlobalBoxGraphs( Float_t* X,Int_t globalTypeOpt)
{
    Char_t        plotName[256];
    Int_t         l;
    Char_t        labels[512];
    Char_t        title[512];
    unsigned int  pos;
    
    if (gDebug > 0) cout << "begin MakeGlobalBoxGraphs " <<endl;

    //////////////////////////////////////////////////////////////////////
    //  do box graphs
    //////////////////////////////////////////////////////////////////////
    if (globalTypeOpt == k_BOX || globalTypeOpt == k_ALL_PLOT_TYPES ) {
      //printf(" about to create box canvas\n");
      //fPlotMaker->CreateCanvas(1.0,0.5);     
      fPlotMaker->ResizeCanvas(1.0,0.5);     

      for (Int_t iVar = 0; iVar <fVarNumber; iVar++) { 
	l = fReduced2FullVarIndex[iVar];	
	if (fVarPars[l].GlobalMultiBoxType > -2) {	  
	  sprintf(plotName,"%s_%s_%s_Box",fMonitorSet,fVarPars[l].PlotType,fVarPars[l].Name);
	  if ((gbSetPlotNameOption == kTRUE && strcmp(gSetPlotName,plotName) == 0) ||  gbSetPlotNameOption == kFALSE) {	    
	    	    fPlotMaker->GraphBox(fVarPars[l].Name,fVarPars[l].YUnit,0.0,&fEachCurrentValue[(iVar*fCompNumber)],fCompNumber);
	          fPlotMaker->DoGif(plotName,kCurrentTimeScope,fVarPars[l].PlotType);
	  }
	  // kludged to plot cpu. unlike for each plots, here get actual number of plots. In future
	  // do like it is done in for each plots
	  if (gDebug > 0) printf("fVarPars[l].GlobalMultiBoxType =%d l=%d \n",fVarPars[l].GlobalMultiBoxType,l);

	  if (fVarPars[l].GlobalMultiBoxType ==  l ) {
	    
	    Int_t multiBoxCount = fVarPars[l].GlobalMultiBoxCount;
	    sprintf(labels,"");
	    // printf("a multiBoxCount=%d \n",multiBoxCount);
	    for (Int_t ll = 0; ll  < multiBoxCount; ll++) {
	      pos = strcspn(fVarPars[l+ll].MultiName, "_");
	      if (pos <  strlen(fVarPars[l+ll].MultiName)) {		  
		sprintf(labels,"%s %s ",labels,fVarPars[l+ll].MultiName+pos+1);
		if ( ll == 0) snprintf(title,pos+1,"%s ",fVarPars[l+ll].MultiName);
		if (gDebug > 0) printf( "labels= %s multiname=%s title=%s \n",labels,fVarPars[l+ll].MultiName,title);
	      } else {
		sprintf(labels,"%s %s ",labels,fVarPars[l+ll].Name);
		if (ll == 0) sprintf(title,"%s ",fVarPars[l+ll].MultiName);
	      }
	    }
	    sprintf(plotName,"%s_%s_%s_MultiBox",fMonitorSet,fVarPars[l].PlotType,title);

	    if (gDebug > 0) printf("plot name is %s \n",plotName);

	    if ((gbSetPlotNameOption == kTRUE && strcmp(gSetPlotName,plotName) == 0) ||  gbSetPlotNameOption == kFALSE) {	     	      
	      Int_t plotStatus = fPlotMaker->GraphMultiBox(title,labels,fVarPars[l].YUnit,0.0,&fEachCurrentValue[(iVar*fCompNumber)],fCompNumber, multiBoxCount,
	      			&fEachVarRef[(iVar*fCompNumber)]);
	      	      if (plotStatus) fPlotMaker->DoGif(plotName,kCurrentTimeScope,fVarPars[l].PlotType);
	    }	    
	  }
	}

      }

      //fPlotMaker->CreateCanvas(1.0,1.0);
      fPlotMaker->ResizeCanvas(1.0,1.0);
    }
    if (gDebug > 0) cout << "end MakeGlobalBoxGraphs " <<endl;
}



void TDataProcess::MakeGlobalSuperimposedGraphs(Char_t *plotLabel, Float_t* X,Int_t globalTypeOpt)
{

    Char_t  plotName[256];
    Char_t  plotName2[256];
    TString label;
    Int_t   l;
    Int_t   trueCompIndex;

 
    Int_t   graphsPerPlot ;
    Int_t   remainingGraphs ;
    Int_t   plottedGraphs ;
    Int_t   plotCount ;
    Bool_t  noDummyLabelFlag ;
    Bool_t  bSkipDummyLables ;

    bSkipDummyLables = false;

    if (gDebug > 0) cout << "begin MakeGlobalSuperimposedGraphs " <<endl;

    //////////////////////////////////////////////////////////////////////
    // do superimposed individual plots 
    //////////////////////////////////////////////////////////////////////
    if (globalTypeOpt == k_SUM_AND_SUPERIMPOSE || globalTypeOpt == k_SUPERIMPOSE || globalTypeOpt == k_ALL_PLOT_TYPES) {

      for (Int_t iVar = 0; iVar <fVarNumber; iVar++) { 
	l = fReduced2FullVarIndex[iVar];
	// take care of case when don't want to plot superimpose plots
	if (fVarPars[l].GlobalSumType > -2 ) {	      
	  //  do only for computers which are alive during this time period.
	  //  so use fCompUpNumber instead of fCompNumber

	  TGraph ** individualPlot    = 0; individualPlot    = new TGraph*[fCompActiveNumber];
	  TGraph ** individualPlotMax = 0; individualPlotMax = new TGraph*[fCompActiveNumber];
	  TText **tText    = 0; tText    = new TText*[fCompActiveNumber];
	  TText **tTextMax = 0; tTextMax = new TText*[fCompActiveNumber];
	  
	  for (Int_t i = 0; i < fCompActiveNumber; i++) { 		   
	    trueCompIndex = fReduced2FullCompActiveIndex[i];
	    if (trueCompIndex < 0  ) {
	      cout  << getTime() << " trueCompIndex < 0 . This should not happen.debug " << endl;
	    } 
	    individualPlot[i]    = new TGraph(fBinNumber,X,&fEachValue[(trueCompIndex*fVarNumber)*fBinNumber+iVar*fBinNumber]);
	    individualPlotMax[i] = new TGraph(fBinNumber,X,&fEachMaxValue[(trueCompIndex*fVarNumber)*fBinNumber+iVar*fBinNumber]);
	    tText[i]= new TText();
	    tTextMax[i]= new TText();
	  }
	  
	  ////////////////////////////////////////////////////////////////////////////////
	  // take care to not have too many graphs per plot. If too many, create new plot
	  ////////////////////////////////////////////////////////////////////////////////

	  graphsPerPlot = 0;
	  remainingGraphs = fCompActiveNumber;
	  plottedGraphs = 0;
	  plotCount = 0;
	  noDummyLabelFlag = false;
	  
	  while (remainingGraphs > 0) {		  
	    if (remainingGraphs >= fMaxGraphsPerPlot ) {
	      graphsPerPlot = fMaxGraphsPerPlot;
	    } else {
	      graphsPerPlot = remainingGraphs;
	    }
	    remainingGraphs = remainingGraphs - graphsPerPlot;	       
	    label = GetLabel(fCompPars[fReduced2FullCompActiveIndex[plottedGraphs]].compName,iVar,fReduced2FullCompActiveIndex[plottedGraphs]);		    		  

	    sprintf(plotName,"%s_%s_%s%i_Each_%s",fMonitorSet,fVarPars[l].PlotType,fVarPars[l].Name,plotCount,plotLabel);		  
	    if (gDebug > 0) cout << getTime() << " plotName="<<plotName<<" iVar="<<iVar<< " label="<<label
				      <<" comp="<< fCompPars[fReduced2FullCompActiveIndex[plottedGraphs]].compName<<endl;
	    
	    if ((gbSetPlotNameOption == kTRUE && strcmp(gSetPlotName,plotName) == 0) ||  gbSetPlotNameOption == kFALSE) {		    
	      //do first plot
	      fPlotMaker->Graph(fVarPars[l].Name,fVarPars[l].YUnit,kColorCode[0],fGlobalMin[iVar]*0.9,fGlobalMax[iVar]*1.1,
				(Char_t *)label.Data(),"AL",individualPlot[plottedGraphs]);		    
	      
	      //superimpose rest
	      noDummyLabelFlag = false;
	      if (fVarHasLabelButIsDummyFlag[0+iVar] != true)   noDummyLabelFlag = true;

	      if (gDebug > 0) cout << getTime() << " 2 plotName="<<plotName<<" graphsPerPlot="<<graphsPerPlot <<endl;
	      
	      
	      Int_t  tmpCount = 1;

	      for (Int_t lComp = 1; lComp < graphsPerPlot; lComp++) {
		// ignore if variable not used. Will not look too good on plot. Improve in future
		if (fVarHasLabelButIsDummyFlag[lComp*fVarNumber+iVar] != true || bSkipDummyLables == false) {			    
		  //label = GetLabel(fCompPars[fReduced2FullCompActiveIndex[plottedGraphs+lComp]].compName,l,fReduced2FullCompActiveIndex[plottedGraphs+lComp]);
		  label = GetLabel(fCompPars[fReduced2FullCompActiveIndex[plottedGraphs+lComp]].compName,iVar,fReduced2FullCompActiveIndex[plottedGraphs+lComp]);
		  //		  fPlotMaker->SuperimposeGraph(kColorCode[lComp],(Char_t *) label.Data(),lComp,kFullLine,"L",tText[plottedGraphs+lComp],
		  //			       individualPlot[plottedGraphs+lComp]);
		  fPlotMaker->SuperimposeGraph(kColorCode[tmpCount],(Char_t *) label.Data(),tmpCount,kFullLine,"L",tText[plottedGraphs+lComp],
					       individualPlot[plottedGraphs+lComp]);
		  noDummyLabelFlag = true;
		  tmpCount++;		  		  
		}   		   
	      }
	      // superimpose plt of max values
	      tmpCount = 0;
	      for (Int_t lComp =0; lComp < graphsPerPlot; lComp++) {
		if (fVarHasLabelButIsDummyFlag[lComp*fVarNumber+iVar] != true  || bSkipDummyLables == false) {
		  // check if lComp should be -1
		  //		  fPlotMaker->SuperimposeGraph(kColorCode[lComp]," ", lComp,kDashedLine,"L",tTextMax[plottedGraphs+lComp],
		  //			     individualPlotMax[plottedGraphs+lComp]);
		  fPlotMaker->SuperimposeGraph(kColorCode[tmpCount]," ", tmpCount,kDashedLine,"L",tTextMax[plottedGraphs+lComp],
					     individualPlotMax[plottedGraphs+lComp]);
		  tmpCount++;		  		  
		}
	      }		    
	      if (fFillRootFileFlag == true && (noDummyLabelFlag == true  || bSkipDummyLables == false )) {
	      //if (fFillRootFileFlag == true) {
		sprintf(plotName2,"%s_%s_%sVsTimeSuperimposedSet%i_%s",fMonitorSet,fVarPars[l].PlotType,fVarPars[l].Name,plotCount,plotLabel);		  
		fPlotMaker->WriteCanvas(plotName2);
	      }
	      //only plot if at least one label OK.
	      if (noDummyLabelFlag == true  || bSkipDummyLables == false)  fPlotMaker->DoGif(plotName,kGlobalScope,fVarPars[l].PlotType);
	      //fPlotMaker->DoGif(plotName,kGlobalLog);
	    }		  
	    plottedGraphs += graphsPerPlot;
	    plotCount++;
	  }
	  // Free memory
	  FreeMemory(individualPlot, fCompActiveNumber);
	  FreeMemory(individualPlotMax, fCompActiveNumber);
	  FreeMemory(tText, fCompActiveNumber);
	  FreeMemory(tTextMax, fCompActiveNumber);
	}
      }
    }
    if (gDebug > 0) cout << "end MakeGlobalSuperimposedGraphs " <<endl;
    
}


void TDataProcess::MakeGlobalSumGraphs(Char_t *plotLabel, Float_t* X,Int_t globalTypeOpt)
{

    Char_t plotName[256];
    Char_t plotName2[256];
    TString label;
    Char_t plotTitle[256];
    Int_t l;


    if (gDebug > 0) cout << "MakeGlobalSumGraphs " <<endl;

    // du summed plots
    // 1 = summed, 2 = superimposed, 0 = both
    if (globalTypeOpt == k_SUM_AND_SUPERIMPOSE || globalTypeOpt == k_SUM || globalTypeOpt == k_ALL_PLOT_TYPES) {
      // do status of nodes
      // use just the first variable (i.e. var = 0) to see if any entry. Colud be any variable of course.
      // kludge to only to this for datemon or sysmon. Make more elegant in future
      // this is not really summed, but need to place in one category, so that only one process writes it.            

      if ((   (gbSetPlotNameOption == kTRUE && strcmp(gSetPlotName,plotName) == 0) ||  gbSetPlotNameOption == kFALSE ) 
	  && (strcmp(fMonitorSet,"DATEMON") == 0 ||  strcmp(fMonitorSet,"SYSMON") == 0 )  ) {
	//	fPlotMaker->CreateCanvas(1.0,0.5);

	sprintf(plotName,"%s_NodeUpStatus_Box_%s",fMonitorSet,plotLabel);
	sprintf(plotTitle, "Node monitored status: previous %s",plotLabel);
	fPlotMaker->GraphCompActiveStatusBox(plotTitle,plotLabel,&fEachValueCount[0],fCompNumber,fBinNumber,fVarNumber);
	fPlotMaker->DoGif(plotName,kGlobalScope,"UPTIME");
	//	fPlotMaker->CreateCanvas(1.0,1.0);
      }

      for (Int_t iVar = 0; iVar < fVarNumber; iVar++) { 
	l = fReduced2FullVarIndex[iVar];
	if (fVarPars[l].GlobalSumType == l) {   
	  sprintf(plotName,"%s_%s_%s_%s",fMonitorSet, fVarPars[l].PlotType, fVarPars[l].GlobalSumName,plotLabel);
	  
	  
	  if ((gbSetPlotNameOption == kTRUE && strcmp(gSetPlotName,plotName) == 0) ||  gbSetPlotNameOption == kFALSE) {
	    Int_t totGraphs = fVarPars[l].GlobalSumCount;
	    TGraph **additivePlot = 0; additivePlot = new TGraph*[totGraphs];
	    TText **tText = 0; tText = new TText*[totGraphs];
	    Float_t sumMin; 
	    Float_t sumMax; 
	    for (Int_t i = 0; i < totGraphs; i++) {
	      additivePlot[i] = new TGraph(fBinNumber,X,&fGlobalSumValue[(iVar+i)*fBinNumber]);
	      tText[i]= new TText();
	      // determine min and max for all of the summed plots
	      if ( i == 0 ) {
		sumMin = fGlobalSumMin[iVar+i];
		sumMax = fGlobalSumMax[iVar+i];
	      }
	      if ( fGlobalSumMin[iVar+i] < sumMin ) sumMin = fGlobalSumMin[iVar+i]; 
	      if ( fGlobalSumMax[iVar+i] > sumMax ) sumMax = fGlobalSumMax[iVar+i]; 
	    }		 

	    fPlotMaker->Graph(fVarPars[l].GlobalSumTitle, fVarPars[l].YUnit,kColorCode[0],sumMin - 0.1*(sumMax - sumMin), sumMax + 0.1*(sumMax - sumMin),
			      fVarPars[l].Name,"AL",additivePlot[0]);
	    
	    if (gDebug > 0) cout  << getTime()  << " var == "<<iVar<< "fGlobalSumMin[iVar]=" <<fGlobalSumMin[iVar] << " fGlobalSumMax[iVar] = " <<fGlobalSumMax[iVar]
				  << " sumMin = " << sumMax << " sumMax= " << sumMax<< endl;
	    
	    for (Int_t graphNum = 1; graphNum < totGraphs; graphNum++) {	    
	      fPlotMaker->SuperimposeGraph(kColorCode[graphNum],fVarPars[l+graphNum].Name, graphNum+1,kFullLine,"L",tText[graphNum],additivePlot[graphNum]); 
	    }
	    
	    if (fFillRootFileFlag == true) {
	      sprintf(plotName2,"%s_%s_%sVsTime_%s",fMonitorSet,fVarPars[l].PlotType,fVarPars[l].GlobalSumName,plotLabel);
	      fPlotMaker->WriteCanvas(plotName2);
	    }
	    fPlotMaker->DoGif(plotName,kGlobalScope,fVarPars[l].PlotType);
	    // free memory	   
	    FreeMemory(additivePlot,totGraphs);
	    FreeMemory(tText,totGraphs);	    
	    
	  }  
	} 
      }
    }
    if (gDebug > 0) cout << "end MakeGlobalSumGraphs " <<endl;
}


void TDataProcess::MakeGlobalSuperimposedMultiVarGraphs(Char_t *plotLabel, Float_t* X,Int_t globalTypeOpt)
{
    Char_t  title[256];
    Char_t  plotName[256];
    Char_t  plotName2[256];
    TString label;
    Int_t   l;
    Int_t   trueCompIndex;
    Bool_t  labelDrawnStatus;
    Char_t  labelName[256];
    Int_t   labelOrder;
    Float_t ymin;
    Float_t ymax;

    Int_t   compsPerPlot;
    Int_t   remaininfComps;
    Int_t   plottedComps;
    Int_t   plotCount;
    Int_t   previousSuperimposeMultiVarType;

    Bool_t  firstGraphDrawnStatus; 
    unsigned int   pos;

    //////////////////////////////////////////////////////////////////////
    // do superimposed individual plots, with different variables
    //////////////////////////////////////////////////////////////////////
    

    if (gDebug > 0) cout  << getTime()  << " MakeGlobalSuperimposedMultiVarGraphs" << endl;

    if (globalTypeOpt == k_SUM_AND_SUPERIMPOSE || globalTypeOpt == k_SUPERIMPOSE || globalTypeOpt == k_ALL_PLOT_TYPES) {
      previousSuperimposeMultiVarType = -1;
      for (Int_t iVar = 0; iVar <fVarNumber; iVar++) { 
	l = fReduced2FullVarIndex[iVar];
	if (fVarPars[l].SuperimposeMultiVarType >= 0  &&  fVarPars[l].SuperimposeMultiVarType !=  previousSuperimposeMultiVarType) {	      
	  previousSuperimposeMultiVarType = fVarPars[l].SuperimposeMultiVarType;
	  TGraph ** individualPlot = 0;    individualPlot = new TGraph*[fCompActiveNumber*fVarPars[l].SuperimposeMultiVarCount];
	  TText **tText    = 0; tText     = new TText*[fCompActiveNumber];
	  
	  for (Int_t i = 0; i < fCompActiveNumber; i++) { 		    
	    trueCompIndex = fReduced2FullCompActiveIndex[i];
	    for (Int_t j = 0; j < fVarPars[l].SuperimposeMultiVarCount; j++) { 	
	      individualPlot[ i*fVarPars[l].SuperimposeMultiVarCount+j] = new TGraph(fBinNumber,X,&fEachValue[(trueCompIndex*fVarNumber)*fBinNumber+(iVar+j)*fBinNumber]);
	    }
	    tText[i]   = new TText();
	  }
	  
	  ////////////////////////////////////////////////////////////////////////////////
	  // take care to not have too many graphs per plot. If too many, create new plot	  
	  compsPerPlot = 0;
	  remaininfComps = fCompActiveNumber;
	  

	  plottedComps = 0;
	  plotCount = 0;
	  
	  while (remaininfComps > 0) {		  
	    if (remaininfComps >= fMaxGraphsPerPlot ) {
	      compsPerPlot = fMaxGraphsPerPlot;
	    } else {
	      compsPerPlot = remaininfComps;
	    }
	
	    remaininfComps = remaininfComps - compsPerPlot;	       
	    sprintf(plotName,"%s_%s_All%s%i_Each_%s",fMonitorSet,fVarPars[l].PlotType,fVarPars[l].Name,plotCount,plotLabel);		  
	    labelDrawnStatus = false;
	    if ((gbSetPlotNameOption == kTRUE && strcmp(gSetPlotName,plotName) == 0) ||  gbSetPlotNameOption == kFALSE) { 

	      // calculate global min and max for all variables not just first
	      ymin = 99999999.;
	      ymax = 0.0;
	      for (Int_t ii = 0 ; ii < compsPerPlot; ii++) {
		for (Int_t jj = 0; jj < fVarPars[l].SuperimposeMultiVarCount; jj++) {			
		  if ( fEachMin[fVarNumber*fReduced2FullCompActiveIndex[plottedComps+ii]+iVar+jj] < ymin )  
		    ymin =  fEachMin[fVarNumber*fReduced2FullCompActiveIndex[plottedComps+ii]+iVar+jj];
		  if ( fEachMax[fVarNumber*fReduced2FullCompActiveIndex[plottedComps+ii]+iVar+jj] > ymax )  
		    ymax =  fEachMax[fVarNumber*fReduced2FullCompActiveIndex[plottedComps+ii]+iVar+jj];
		}
	      }
	      
	      //
	      firstGraphDrawnStatus = false; 
	      for (Int_t lComp = 0 ; lComp < compsPerPlot; lComp++) {		
		labelDrawnStatus = false;
		// plot first graph only if ddl is used. labaled by fEachVarRef[totvar*comp +var]
		// can one assume that each comp has at least one active ddl?
		for (Int_t iMultiVar = 0; iMultiVar < fVarPars[l].SuperimposeMultiVarCount; iMultiVar++) {
		  // draw label only once per computer
		  if (labelDrawnStatus == false) {
		    label = GetLabel(fCompPars[fReduced2FullCompActiveIndex[plottedComps+lComp]].compName,iVar+0,fReduced2FullCompActiveIndex[plottedComps+lComp]);
		    sprintf(labelName,"%s",(Char_t *)label.Data());
		    labelOrder = lComp;
		  } else {
		    sprintf(labelName," ");
		    labelOrder = -1;
		  }		  
		  if (firstGraphDrawnStatus == false && fEachVarRef[(iVar+iMultiVar)*fCompNumber+ fReduced2FullCompActiveIndex[plottedComps+lComp]] > 0 ) {		    
		    //do first plot		      
		    pos = strcspn(fVarPars[l+0].MultiName, "_");
		    if (pos < strlen(fVarPars[l+0].MultiName)) {		  
		      snprintf(title,pos+1,"%s ",fVarPars[l+0].MultiName);
		    } else {
		      sprintf(title,"%s ",fVarPars[l+0].MultiName);
		    }
		    
//		    fPlotMaker->Graph(fVarPars[l+0].Name,fVarPars[l+0].YUnit,kColorCode[0],ymin*0.9,ymax*1.1,
		    fPlotMaker->Graph(title,fVarPars[l+0].YUnit,kColorCode[0],ymin*0.9,ymax*1.1,
				      labelName,"AL",individualPlot[(plottedComps + 0)*fVarPars[l].SuperimposeMultiVarCount+0]); 
		    firstGraphDrawnStatus = true;
		    labelDrawnStatus      = true;
		  } else if (firstGraphDrawnStatus == true && fEachVarRef[(iVar+iMultiVar)*fCompNumber+ fReduced2FullCompActiveIndex[plottedComps+lComp]] > 0 ) {		    
		    fPlotMaker->SuperimposeGraph(kColorCode[lComp],labelName,labelOrder,kFullLine,"L",tText[plottedComps + lComp],
						 individualPlot[(plottedComps + lComp)*fVarPars[l].SuperimposeMultiVarCount+iMultiVar]);
		    labelDrawnStatus = true;
		  }
		}
	      }	      
	      if (fFillRootFileFlag == true && (labelDrawnStatus == true)) {
		sprintf(plotName2,"%s_%s_%sVsTimeMultiVarSuperimposedSet%i_%s",fMonitorSet,fVarPars[l].PlotType,fVarPars[l].Name,plotCount,plotLabel);		  
		fPlotMaker->WriteCanvas(plotName2);
	      }
	      if (labelDrawnStatus == true) fPlotMaker->DoGif(plotName,kGlobalScope,fVarPars[l].PlotType);
	    }		  
	    plottedComps += compsPerPlot;
	    plotCount++;
	  }
	  // Free memory
	  FreeMemory(individualPlot,fCompActiveNumber*fVarPars[l].SuperimposeMultiVarCount);
	  FreeMemory(tText, fCompActiveNumber);
	}
      }
    }
    if (gDebug > 0) cout  << getTime()  << " end MakeGlobalSuperimposedMultiVarGraphs" << endl;
}


void TDataProcess::FillEachArrays(Int_t deltaTime)
{	    
    Int_t j;

    Int_t bin = (Int_t) (( (Double_t) deltaTime)/fBinSize);
    Int_t lComp = GetCompId(NameAndMonitorSet);

    if ( bin >= 0 && bin < fBinNumber && lComp > -1) {	
      //      for (Int_t lComp = 0;lComp < fCompNumber; lComp++) {	
      //	if (strcmp(fCompPars[lComp].baseName,NameAndMonitorSet)==0) { 	    
      for (Int_t i = 0; i <nVariables; i++) {
	j = fVarPars[i].ReducedVarIndex;
	if ( j < 0)                            continue; // ignore variables not used for plots
	if (isinf( MonitorValue[i]) != 0 )     continue; 
	if (  MonitorValue[i] < -0.000000001 ) continue;
	
	fEachValue[(lComp*fVarNumber)*fBinNumber+j*fBinNumber+bin] += MonitorValue[i]*fVarPars[i].MultFactor;
	fEachValueCount[(lComp*fVarNumber)*fBinNumber+j*fBinNumber+bin] = fEachValueCount[(lComp*fVarNumber)*fBinNumber+j*fBinNumber+bin] + 1.0;
	
	//fill array of max values at each bin. These will be superimposed on graphs
	if (MonitorValue[i]*fVarPars[i].MultFactor > fEachMaxValue[(lComp*fVarNumber)*fBinNumber+j*fBinNumber+bin]) 
	  fEachMaxValue[(lComp*fVarNumber)*fBinNumber+j*fBinNumber+bin] = MonitorValue[i]*fVarPars[i].MultFactor; 
	
	//find max for graph for each computer and variable separately 
	if (MonitorValue[i]*fVarPars[i].MultFactor > fEachMax[lComp*fVarNumber+j])
	  fEachMax[lComp*fVarNumber+j] = MonitorValue[i]*fVarPars[i].MultFactor;   
	
	//find min value for each comp separately
	if (MonitorValue[j]*fVarPars[i].MultFactor < fEachMin[lComp*fVarNumber+j])
	  fEachMin[lComp*fVarNumber+j] = MonitorValue[i]*fVarPars[i].MultFactor;		    
	
	//find max for graph for superimposed plots
	if (MonitorValue[i]*fVarPars[i].MultFactor > fGlobalMax[j]) fGlobalMax[j] = MonitorValue[i]*fVarPars[i].MultFactor; 
	
	//find min for graph for superimposed plots
	if (MonitorValue[i]*fVarPars[i].MultFactor < fGlobalMin[j]) fGlobalMin[j] = MonitorValue[i]*fVarPars[i].MultFactor; 	
	
	if (fVarPars[i].SuperimposeMultiVarType >= 0 ) {	      
	  if (fVarPars[i].SuperimposeMultiVarRef >= 0 && MonitorValue[fVarPars[i].SuperimposeMultiVarRef]  > 0.0) { 
	    fEachVarRef[(j*fCompNumber)+lComp] = 1;		
	  }
	  if (fVarPars[i].SuperimposeMultiVarRef < 0) { 
	    fEachVarRef[(j*fCompNumber)+lComp] = 1;		
	  }
	} else {
	  fEachVarRef[(j*fCompNumber)+lComp] = 1;	    
	}
      }
      //	}
      //}
    } 
}


void TDataProcess::FillEachArraysMaxRrd(Int_t deltaTime)
{	    
  Int_t j;
  Int_t bin = (Int_t) (( (Double_t) deltaTime)/fBinSize);

  Int_t lComp = GetCompId(NameAndMonitorSet);
  if ( bin >= 0 && bin < fBinNumber && lComp > -1) {	
    //  if ( bin >= 0 && bin < fBinNumber) {	
    //for (Int_t lComp = 0; lComp < fCompNumber; lComp++) {
    //  if (strcmp(fCompPars[lComp].baseName,NameAndMonitorSet)==0) { 	    
    // load values 
    for (Int_t i = 0; i < nVariables; i++) {
      j = fVarPars[i].ReducedVarIndex;
      if ( j < 0) continue; // ignore variables not used for plots
      //fill array of max values at each bin. These will be superimposed on graphs
      if ( MonitorValue[i]*fVarPars[i].MultFactor >  fEachMaxValue[(lComp*fVarNumber)*fBinNumber+j*fBinNumber+bin]) 
	fEachMaxValue[(lComp*fVarNumber)*fBinNumber+j*fBinNumber+bin] = MonitorValue[i]*fVarPars[i].MultFactor; 		  
      //find max for graph for each computer and variable separately 
      if (MonitorValue[i]*fVarPars[i].MultFactor > fEachMax[lComp*fVarNumber+j])
	fEachMax[lComp*fVarNumber+j] = MonitorValue[i]*fVarPars[i].MultFactor;   		  
      //find min value for each comp separately
      if (MonitorValue[i]*fVarPars[i].MultFactor < fEachMin[lComp*fVarNumber+j])
	fEachMin[lComp*fVarNumber+j] = MonitorValue[i]*fVarPars[i].MultFactor;		    
      //find max for graph for superimposed plots
      if (MonitorValue[i]*fVarPars[i].MultFactor > fGlobalMax[j]) fGlobalMax[j] = MonitorValue[i]*fVarPars[i].MultFactor; 		
      //find min for graph for superimposed plots
      //		    if (MonitorValue[i]*fVarPars[i].MultFactor < fGlobalMin[j]) fGlobalMin[j] = 
      //    MonitorValue[i]*fVarPars[i].MultFactor; 		
    }
  }
  //}
  //} 
}


void TDataProcess::FillEachHisto(TH1F **  histoEach)
{	    
  Int_t iVar;
  
  Int_t lComp = GetCompId(NameAndMonitorSet);
  if (lComp > -1) {	
    //for (Int_t lComp = 0; lComp < fCompNumber; lComp++) {
    //if (strcmp(fCompPars[lComp].baseName, NameAndMonitorSet) ==0 ) { 	    
    // load values 
    // for (Int_t iVar = 0; iVar < fVarNumber; iVar++) {
    for (Int_t i = 0; i < nVariables; i++) {
      iVar = fVarPars[i].ReducedVarIndex;
      if ( iVar < 0)                         continue; // ignore variables not used for plots
      // if ( gPickedVariable >= 0 && iVar != gPickedVariable) continue;
      // fill individual histo
      
      if (isinf( MonitorValue[i]) != 0 )     continue; 
      if (  MonitorValue[i] < -0.000000001 ) continue;
      
      if (MonitorValue[i] > 0.0) histoEach[lComp*fVarNumber+iVar]->Fill(MonitorValue[i]*fVarPars[i].MultFactor);
    }
  }
}



void TDataProcess::FillEachOffline()
{	   
    fRunCount = 0;
    for (Int_t i = 0; i < kMaxRunCount; i++) fRunNumArray[i] = -999;
    Bool_t runExistsFlag;

    // loop over all tree entries
    Int_t nentries = Int_t(fChain->GetEntries());   
    
    for (Int_t jentry = 0; jentry < nentries; jentry++) {
      //Int_t treeRet = LoadTree(jentry); //in case of a TChain, ientry is the entry number in the current file
      (Int_t) LoadTree(jentry); //in case of a TChain, ientry is the entry number in the current file
      fChain->GetEntry(jentry);     
      if(difftime(fTimeMax,MonitorTime) < fBinNumber*fBinSize) FillEachArrays(MonitorTime - fTimeMin);      
      if ( fAnalysisLevel > 0) {
	if(difftime(fTimeMax,MonitorTime) < fBinNumber*fBinSize) {
	  runExistsFlag = false;
	  for (Int_t i = 0; i < kMaxRunCount; i++) if (((Int_t) MonitorValue[kRunNumIndex]) == fRunNumArray[i] ) runExistsFlag = true;  
	  if ( runExistsFlag == false || fRunCount == 0) {
	    fRunNumArray[fRunCount] = (Int_t) (MonitorValue[kRunNumIndex]*fVarPars[kRunNumIndex].MultFactor); 
	    fRunCount++;	
	  }
	}
      }
    }
    NormalizeEachArrays();
}


void TDataProcess::DoAnalysis()
{	   
    
    Char_t resultFile[256];
    Char_t timeLabel[64];


    typedef struct myRunResults_t{
	Int_t   runNumber;
	Float_t varSum[kMaxVariables];
	Int_t   timeFirst;
	Int_t   timeLast;
	Int_t   entryCount;
    }RunResults_t;
    
    RunResults_t * runResults = 0 ;

    if (runResults != 0) delete [] runResults ;
    runResults = new  RunResults_t[fRunCount];

    for ( Int_t i = 0; i < fRunCount; i++ ) {
	runResults[i].entryCount   = 0;
	runResults[i].runNumber    = fRunNumArray[i];
	for ( Int_t j = 0; j < kMaxVariables; j++ ) runResults[i].varSum[j]  = 0;
    }

    // loop over all tree entries
    Int_t nentries = Int_t(fChain->GetEntries());   
    for (Int_t jentry = 0; jentry < nentries; jentry++) {
	(Int_t) LoadTree(jentry); //in case of a TChain, ientry is the entry number in the current file
	fChain->GetEntry(jentry);     
	if(difftime(fTimeMax,MonitorTime) < fBinNumber*fBinSize) {  
	    for ( Int_t i = 0 ; i < fRunCount ; i++ ) {
		if ( MonitorValue[kRunNumIndex]*fVarPars[kRunNumIndex].MultFactor == runResults[i].runNumber ) {
		    //found entry in structure with correct run number
		    if ( runResults[i].entryCount == 0 ) {
			// init begin and end time
		        runResults[i].timeFirst = MonitorTime;
			runResults[i].timeLast  = MonitorTime;
		    }
		    if ( MonitorTime < runResults[i].timeFirst )   runResults[i].timeFirst = MonitorTime;
		    if ( MonitorTime > runResults[i].timeLast )    runResults[i].timeLast = MonitorTime;
		    for (Int_t iVar = 0 ; iVar < nVariables ; iVar++) {
			runResults[i].varSum[iVar] +=  MonitorValue[iVar]*fVarPars[iVar].MultFactor;
		    }
		    runResults[i].entryCount++;
		}
	    }   
	}
    }

    // determine 1st snd last run. probably good enough to use fRunNumArray[0] and at max, but...
    Int_t runFirst = fRunNumArray[0];
    Int_t runLast = fRunNumArray[0];
    for (Int_t i = 0; i < fRunCount; i++) {
	if ( fRunNumArray[i] < runFirst ) runFirst = fRunNumArray[i];
	if ( fRunNumArray[i] > runLast ) runLast = fRunNumArray[i];
    }


    sprintf(resultFile,"%s/log/%s_run_%i_to_%i.info",gAffairPath,fMonitorSet,runFirst,runLast);
    std::ofstream fp;
    fp.open(resultFile, std::ios::out);

    struct tm *tm_ptr;

    for (Int_t i = 0 ; i < fRunCount ; i++ ) {
	Int_t   runLength     = runResults[i].timeLast - runResults[i].timeFirst; 
	Float_t timeEntries  = (Float_t) runLength/kMaxRes;

	fp << endl << endl << "********************************************************************";
	fp << endl<< " run Number = " << runResults[i].runNumber  << " from ";

	if (gDebug > 0) cout  << getTime()  << " runResults[i].timeFirst = " <<runResults[i].timeFirst<< endl;
 	tm_ptr = localtime((const time_t *) &runResults[i].timeFirst);    
	strftime(timeLabel,64,"%H:%M %d/%b/%y",tm_ptr);
	if (gDebug > 0)cout  << getTime()  << "timeLabel = " <<timeLabel<<endl;

	fp << timeLabel << " (" << runResults[i].timeFirst << ") to ";

	tm_ptr = localtime((const time_t *)&runResults[i].timeLast);    
	strftime(timeLabel,64,"%H:%M %d/%b/%y",tm_ptr);

	fp << timeLabel << " (" << runResults[i].timeLast << ")" << endl << endl;

	for (Int_t iVar = 0; iVar < fFullVarNumber; iVar++) {
	    fp << fVarPars[iVar].Name << " : Sum = ";
	    if ( timeEntries > 0 )  fp  << runResults[i].varSum[iVar]/timeEntries; 
	    fp << fVarPars[iVar].YUnit << " ,Aver = ";
	    if ( runResults[i].entryCount > 0) fp << runResults[i].varSum[iVar]/((Float_t) runResults[i].entryCount);
	    fp << fVarPars[iVar].YUnit << " ,tot entries = " << runResults[i].entryCount << " ,separate times=" << timeEntries << endl;
	}
    }
    fp.close();
    if (gDebug > 0) cout  << getTime()  << " do analysis done " <<endl;
}


void TDataProcess::FindRange(Int_t selectionVariable, Double_t selectionLowCut,Double_t selectionHighCut)
{	       
  // loop over all tree entries
  Int_t nentries = Int_t(fChain->GetEntries());   
  Bool_t firstFlag = true;
  
  
  for (Int_t jentry =0;  jentry < nentries; jentry++) {
    (Int_t) LoadTree(jentry); //in case of a TChain, ientry is the entry number in the current file
    fChain->GetEntry(jentry);     	
    
    if (MonitorValue[selectionVariable] >= selectionLowCut && MonitorValue[selectionVariable] <= selectionHighCut) {
      if (firstFlag == true) {
	fTimeMin = MonitorTime;
	fTimeMax = MonitorTime;
	firstFlag = false;
      }
      if (MonitorTime < fTimeMin) fTimeMin = MonitorTime;	    
      if (MonitorTime > fTimeMax) fTimeMax = MonitorTime;
    }   
  }
}


//Int_t TDataProcess::FillEachRrd(Char_t *timePeriod, Int_t globalTypeOpt)
Int_t TDataProcess::FillEachRrd()
{	   
    Char_t RRDCommandX[512];
    Char_t RRDCommand[512];
    Int_t  CurrentTime;
    Int_t  returnStatus = 1;
    CurrentTime = time(0);
    time_t timeLastUpdate;

    char **argv;
    int    argc;
    
    time_t        start,end;
    unsigned long step, ds_cnt,i,ii;
    rrd_value_t   *data,*datai;
    char          **ds_namv;
    
    fCompActiveNumber = 0;

    for (Int_t rrdFileNumber = 0; rrdFileNumber < fCompNumber; rrdFileNumber++) {
        // first find if last time rrd updated is > start time      
        sprintf(RRDCommand,"last %s/rrd/%s ",gAffairPath,fCompPars[rrdFileNumber].rrdName);
	if((argv = string_to_argv(RRDCommand, &argc)) != NULL) {
	  timeLastUpdate = rrd_last(argc-1, &argv[1]);
	  free(argv[0]);
	  free(argv);

	  // add here code to mark node as not updated, i.e. dead. Put this in an arrrray 
	  // of size fCompNumber
	  if (timeLastUpdate > fTimeMin) {
	      fReduced2FullCompActiveIndex[fCompActiveNumber] = rrdFileNumber; 
	      fCompActiveNumber++;
	  } else {
	    fReduced2FullCompActiveIndex[fCompActiveNumber] = -1; //this not really necessary 
	  }

	  if (timeLastUpdate < fTimeMin) continue;	  
	}
	
	// get data 
	sprintf(RRDCommandX,"fetch %s/rrd/%s AVERAGE -r %i -s %i -e %i ", gAffairPath,fCompPars[rrdFileNumber].rrdName,(int) (fBinSize/3),fTimeMin,CurrentTime);
		
	if((argv = string_to_argv(RRDCommandX, &argc)) != NULL) {
	  optind=0; /* reset gnu getopt */
	  opterr=0; /* no error messages */

	  if (rrd_fetch(argc-1, &argv[1],&start,&end,&step,&ds_cnt,&ds_namv,&data) != -1) {
	    datai=data;
	    nVariables = ds_cnt;

	    Char_t xtitleFinal[256];
	    time_t the_time;
	    struct tm *tm_ptr;
	    (void) time(&the_time);
	    tm_ptr = localtime(&the_time);    
	    strftime(xtitleFinal,256,"%H:%M %d/%b/%y",tm_ptr);
	    
	    for (i =  (unsigned long) start; i <=  (unsigned long)  end; i += step){ 
	      MonitorTime = i;	      
	      for (ii = 0; ii < ds_cnt; ii++)  {
		MonitorValue[ii] = *(datai++);
		if(isinf(MonitorValue[ii]) != 0) MonitorValue[ii] = 0;
	      }
	      if (MonitorTime > 0){
		sprintf(NameAndMonitorSet,"%s",fCompPars[rrdFileNumber].baseName);
		if (isnan(MonitorValue[0]) ) {
		} else {			
		  if(difftime(fTimeMax,MonitorTime) <= fBinNumber*fBinSize) FillEachArrays(MonitorTime - fTimeMin);  
		  if (gDebug > 1000) {
		    for (ii = 0; ii < ds_cnt; ii++)  cout  << MonitorValue[ii] << " ";
		    cout << endl;
		  }
		}
	      }
	    }
	    for (i = 0; i<ds_cnt ; i++) free(ds_namv[i]);    
	    free(ds_namv);
	    free (data);
	  }
	  free(argv[0]);
	  free(argv);
	}
	
	///////////////////    
	//
	// now do the same to get correct max values
	//
 	sprintf(RRDCommandX,"fetch %s/rrd/%s MAX -r %i -s %i -e %i ",
	 	gAffairPath,fCompPars[rrdFileNumber].rrdName,(int) (fBinSize/3),fTimeMin,CurrentTime);  
	
	if((argv = string_to_argv(RRDCommandX, &argc)) != NULL) {
	  optind = 0; /* reset gnu getopt */
	  opterr = 0; /* no error messages */
		
	  if (rrd_fetch(argc-1, &argv[1],&start,&end,&step,&ds_cnt,&ds_namv,&data) != -1) {
	    datai=data;
	    for (i =  (unsigned long) start; i <=  (unsigned long) end; i += step){ 
	      MonitorTime = i;
	      for (ii = 0; ii < ds_cnt; ii++)  {
		MonitorValue[ii] = *(datai++);
	       	if(isinf(MonitorValue[ii]) != 0) MonitorValue[ii] = 0;
	      }
	      if (MonitorTime > 0){
		sprintf(NameAndMonitorSet,"%s",fCompPars[rrdFileNumber].baseName);
		if (isnan(MonitorValue[0])) {
		} else {			
		  if(difftime(fTimeMax,MonitorTime) <= fBinNumber*fBinSize) FillEachArraysMaxRrd(MonitorTime - fTimeMin);  
		}
	      }
	    }
	    for (i=0;i<ds_cnt;i++) free(ds_namv[i]);    
	    free(ds_namv);
	    free (data);
	  }
	  free(argv[0]);
	  free(argv);
	}
    }
    NormalizeEachArrays();
    //////
    return (returnStatus);
} 

    

void TDataProcess::NormalizeEachArrays()
{	 
  Int_t j;
  for (Int_t lComp = 0; lComp < fCompNumber; lComp++) { 
    for (Int_t i = 0; i <nVariables; i++) {
      j = fVarPars[i].ReducedVarIndex;
      if ( j < 0) continue; // ignore variables not used for plots	 
      for (Int_t  bin = 0; bin < fBinNumber; bin++) {	

	  if (fEachValueCount[(lComp*fVarNumber)*fBinNumber+j*fBinNumber+bin] > 0.01 ) {
	      fEachValue[(lComp*fVarNumber)*fBinNumber+j*fBinNumber+bin] = fEachValue[(lComp*fVarNumber)*fBinNumber+j*fBinNumber+bin]*fEachVarRef[(j*fCompNumber)+lComp]/
		  fEachValueCount[(lComp*fVarNumber)*fBinNumber+j*fBinNumber+bin];	  
	  }

	  if ( bin == fBinNumber - 1) {
	  fEachCurrentValue[(j*fCompNumber)+lComp] = fEachValue[(lComp*fVarNumber)*fBinNumber+j*fBinNumber+bin];	  
	}	
      }
      fEachMax[lComp*fVarNumber+j] =fEachMax[lComp*fVarNumber+j]*fEachVarRef[(j*fCompNumber)+lComp] ;   

      // shoud add here code to have mult factor here. Avoids repetitive multiplication
    }
  }
}


void TDataProcess::FillSumArrays()
{	 
    Int_t j,k;

    for (Int_t lComp = 0;lComp < fCompNumber; lComp++) {
	for (Int_t varNumber = 0; varNumber < fFullVarNumber; varNumber++) {
	    j = fVarPars[varNumber].ReducedVarIndex;
	    if ( j < 0) continue; // ignore variables not used for plots	 	    
	    // this is for plots involving sums over all machines
	    if (fVarPars[varNumber].GlobalSumType >= 0 ){
		for (Int_t bin = 0; bin < fBinNumber; bin++)   		   
		    fGlobalSumValue[j*fBinNumber+bin] += fEachValue[(lComp*fVarNumber)*fBinNumber+j*fBinNumber+bin];
	    }
	}
    }

    // determine max/min 
    for (Int_t varNumber = 0; varNumber < fFullVarNumber; varNumber++) {
	j = fVarPars[varNumber].ReducedVarIndex;
	if ( j < 0) continue; // ignore variables not used for plots	    
	// this is for plots involving sums over all machines
	if (fVarPars[varNumber].GlobalSumType >= 0 ){
	    //k = fVarPars[fVarPars[varNumber].GlobalSumType].ReducedVarIndex; // to get summed index in reduced var space
	  k = j;

	  for (Int_t bin = 0; bin <fBinNumber; bin++) {  
	    // initialise global additive min
	    if (bin == 0 ) fGlobalSumMin[k] = fGlobalSumValue[j*fBinNumber+bin];		
	    if (isinf(fGlobalSumValue[j*fBinNumber+bin]) == 0 && fGlobalSumValue[j*fBinNumber+bin] > fGlobalSumMax[k]) 
	      fGlobalSumMax[k] = fGlobalSumValue[j*fBinNumber+bin];
	    if (fGlobalSumValue[j*fBinNumber+bin] < fGlobalSumMin[k]) fGlobalSumMin[k] = fGlobalSumValue[j*fBinNumber+bin];
	  }
	}
    }
}

void TDataProcess::DoEachHisto(Char_t *plotLabel)
{
   Char_t plotName[256];
   TString label;
   Int_t l;

   TH1F **  histo = 0; histo = new  TH1F*[fCompNumber*fVarNumber] ;
   InitEachHisto(histo);
   //
   // loop over all tree entries for histo fillig purposes only
   //
   Int_t nentries = Int_t(fChain->GetEntries());   
   Int_t nbytes = 0, nb = 0;
   for (Int_t jentry=0; jentry<nentries;jentry++) {
       (Int_t) LoadTree(jentry); 
       nb = fChain->GetEntry(jentry);   nbytes += nb;
       if(difftime(fTimeMax,MonitorTime) < fBinNumber*fBinSize) FillEachHisto(histo);       
   }
   // check if this shoiuld be here, Make sure it will not clash
   // with f->Close in end of run
   //draw individual histos
   for (Int_t lComp = 0; lComp < fCompNumber; lComp++ ) {	   
     for (Int_t iVar = 0; iVar < fVarNumber; iVar++ ) { 
       l = fReduced2FullVarIndex[iVar];
       //label = GetLabel(fVarPars[l].Name,l,lComp);
       label = GetLabel(fVarPars[l].Name,iVar,lComp);
       if (fVarHasLabelButIsDummyFlag[lComp*fVarNumber+iVar] != true && fVarPars[l].EachSumType > -2 ) {
	 fPlotMaker->Histo((Char_t *) label.Data(),fVarPars[l].YUnit,kColorCode[0]," ", histo[lComp*fVarNumber+iVar],-1.);
	 if (fFillRootFileFlag == true) {
	   sprintf(plotName,"%s_%s_%s_HistoCanvas_%s",fCompPars[lComp].monsetName,fVarPars[l].PlotType,fVarPars[l].Name,plotLabel);
	   fPlotMaker->CdDir(lComp,"comp");
	   fPlotMaker->WriteCanvas(plotName);
	   fPlotMaker->WriteHisto(histo[lComp*fVarNumber+iVar]);
	 }
	 sprintf(plotName,"%s/_%s_%s_Histo%s_Each_%s",fCompPars[lComp].compName,fCompPars[lComp].monsetName,fVarPars[l].PlotType, 
		 fVarPars[l].Name,plotLabel);
	 fPlotMaker->DoGif(plotName,kOneNodeScope,fVarPars[l].PlotType);  // do gif
       }
     }       
   }
   FreeMemory(histo,fCompNumber*fVarNumber);
}


void TDataProcess::InitEachHisto(TH1F **  histo)
{
    TString plotTitle;
    Int_t l;
    // do histogram initialization
    // create histograms which are not additive   
    for (Int_t i = 0; i < fCompNumber; i++) {      
	for (Int_t j = 0; j < fVarNumber; j++) {
	  l = fReduced2FullVarIndex[j];
	  //plotTitle = GetLabel(fVarPars[l].Name,l,i);
	  plotTitle = GetLabel(fVarPars[l].Name,j,i);
	  if(fEachMax[i*fVarNumber+j]<=fEachMin[i*fVarNumber+j]) fEachMax[i*fVarNumber+j]=fEachMin[i*fVarNumber+j]+1.0;
	  histo[i*fVarNumber+j] = new TH1F(plotTitle.Data(),plotTitle.Data(),100,fEachMin[i*fVarNumber+j]*0.9, fEachMax[i*fVarNumber+j]*1.1);
	}
    }   
}



void TDataProcess::InitGlobalSuperimposeHisto(TH1F** histo,TH1F** histo2,TText ** text1,TText ** text2)
{
    Char_t plotTitle[256];
    Int_t l;
//
// nonsummed, i.e. superimposed for each comp
//    
    for (Int_t iVar = 0; iVar < fVarNumber; iVar++) { 
	histo2[iVar] = 0;
	text2[iVar] = 0;
	// take care of case when don't want to plot superimpose plots
	l = fReduced2FullVarIndex[iVar];
	sprintf(plotTitle,"%s_eachNodeSuperimposed",fVarPars[l].Name);
	for (Int_t lComp = 0; lComp < fCompNumber; lComp++) { 	     
	    histo[lComp*fVarNumber+iVar] = 0;
	    histo[lComp*fVarNumber+iVar] = new TH1F(plotTitle,plotTitle,100,fGlobalMin[iVar]*0.9,fGlobalMax[iVar]*1.1);
	    text1[lComp*fVarNumber+iVar] = 0;
	    text1[lComp*fVarNumber+iVar] = new TText();
	}
	sprintf(plotTitle,"%s_AllNodesTogether",fVarPars[l].Name);
	histo2[iVar] = new TH1F(plotTitle,plotTitle,100,fGlobalMin[iVar]*0.9,fGlobalMax[iVar]*1.1);
	text2[iVar] = new TText();
    }   
}


void TDataProcess::InitGlobalSumHisto(TH1F** histoSumAllTimePoints,TText **tTextSumAllTimePoints, Int_t binAllTimePoints,Float_t timeRange)
{
    Char_t plotTitle[256];    
    Char_t plotTitle2[256];    
    Int_t l;
    
    for (Int_t iVar = 0; iVar <fVarNumber; iVar++) { 
	histoSumAllTimePoints[iVar] = 0;
	tTextSumAllTimePoints[iVar] = 0;

	l = fReduced2FullVarIndex[iVar];
	//	if (fVarPars[l].GlobalSumType == l) {   
	if (fVarPars[l].GlobalSumType > -2) {   
	  //	    Int_t totGraphs = fVarPars[l].GlobalSumCount;
	    sprintf(plotTitle,"%s: %s",fMonitorSet,fVarPars[l].Name);	    
	    sprintf(plotTitle2,"%s 2: %s", fMonitorSet, fVarPars[l].Name);	    	    

	    //	    for (Int_t i=0; i < totGraphs; i++) { 
		histoSumAllTimePoints[iVar+0] = new TH1F(plotTitle2,plotTitle2,binAllTimePoints,0.5,timeRange+0.5);
		tTextSumAllTimePoints[iVar+0] = new TText();
		//}
	}
    }    
}


void TDataProcess::FillGlobalSumHisto(TH1F ** histoSumAllTimePoints, TH1F ** histoSumAllTimePointsInv, 
				      TText ** tTextSumAllTimePointsInv, Int_t binAllTimePoints)
{
  // extract min and max from histoSumAllTimePoints
  Double_t histMin = 0.0 ;
  Double_t histMax = 0.0 ;
  Double_t histTmpValue;
  Int_t l;
  
  Char_t plotTitle[256];    
  
  for (Int_t iVar = 0; iVar < fVarNumber; iVar++) { 
    histoSumAllTimePointsInv[iVar] = 0;
    tTextSumAllTimePointsInv[iVar] = 0;
    l = fReduced2FullVarIndex[iVar];
    
    if (fVarPars[l].GlobalSumType > -1) {   
      //	    Int_t totGraphs = fVarPars[l].GlobalSumCount;
      sprintf(plotTitle,"Total %s",fVarPars[l].Name);	    
      //      for (Int_t i = 0; i < totGraphs; i++) { 
      // get min and max for this 
      for (Int_t j = 0; j < binAllTimePoints; j++) {
	histTmpValue = histoSumAllTimePoints[iVar+0]->GetBinContent(j);
	//		if (j == 1) {
	if (j == 0) {
	  histMin =  histTmpValue;
	  histMax =  histTmpValue;
	}
	if ( histTmpValue > histMax ) histMax = histTmpValue;
	if ( histTmpValue < histMin ) histMin = histTmpValue;
      }
      // create histo
      histoSumAllTimePointsInv[iVar+0]  = new TH1F(plotTitle,plotTitle,100, histMin - 0.1*(histMax - histMin),histMax + 0.1*(histMax - histMin) );
      tTextSumAllTimePointsInv[iVar+0] = new TText();
      // fill histo
      for (Int_t j = 0; j < binAllTimePoints ; j++) {
	histTmpValue = histoSumAllTimePoints[iVar+0]->GetBinContent(j);
	histoSumAllTimePointsInv[iVar+0]->Fill(histTmpValue);
      }
    }    
  }
}
  
    
void TDataProcess::DoGlobalHisto(Char_t *plotLabel)
{

    Int_t j,l;
    Char_t plotTitle[256];
    TString label;
    Char_t plotName[256];


    if (gDebug > 0) cout  << getTime()  << " DoGlobalHisto begin " << endl;

    if (fFillRootFileFlag == true) fPlotMaker->CdDir(-1,"histo");

   //create max number of histos. some may not be filled...

   //histos for each variable and comp separately
   TH1F **  histoSuperimpose = 0; histoSuperimpose = new  TH1F*[fCompNumber*fVarNumber] ;
   TText **tTextSuperimpose = 0; tTextSuperimpose = new TText*[fCompNumber*fVarNumber];

   //histos for each variable  separately
   TH1F **  histoSimple = 0; histoSimple = new  TH1F*[fVarNumber];
   TText **tTextSimple = 0; tTextSimple = new TText*[fVarNumber];

   //histos for each summed variable  as a function of time.only used to
   // get histoSumAllTimePointsInv
   TH1F **  histoSumAllTimePoints = 0; histoSumAllTimePoints = new  TH1F*[fVarNumber];
   TText **tTextSumAllTimePoints = 0; tTextSumAllTimePoints = new TText*[fVarNumber];
   //histos for each summed variable  separately
   TH1F **  histoSumAllTimePointsInv = 0; histoSumAllTimePointsInv = new  TH1F*[fVarNumber];
   TText ** tTextSumAllTimePointsInv = 0; tTextSumAllTimePointsInv = new TText*[fVarNumber];

   Int_t binAllTimePoints = (Int_t) ((1.0/kHistoSmoothFactor)*difftime(fTimeMax,fTimeMin)/((Float_t) kMaxRes));
   InitGlobalSumHisto(histoSumAllTimePoints, tTextSumAllTimePoints,binAllTimePoints,difftime(fTimeMax,fTimeMin));

   // create gloabal superimposed histos,and fill, including tmp one global summed histo
   //create max number of histos. some may not be filled,(if GlobalSumType> ==-2)
   InitGlobalSuperimposeHisto(histoSuperimpose,histoSimple,tTextSuperimpose,tTextSimple);
   //
   // loop over all tree entries for histo filling purposes only
   //  
   Int_t nentries = Int_t(fChain->GetEntries());   
   Int_t nbytes = 0, nb = 0;
   for (Int_t jentry = 0; jentry < nentries; jentry++) {
     (Int_t) LoadTree(jentry); 
     nb = fChain->GetEntry(jentry);   nbytes += nb;
     if(difftime(fTimeMax,MonitorTime) < fBinNumber*fBinSize ) {
       for (Int_t lComp = 0;lComp < fCompNumber; lComp++) {
	 if ( strcmp(fCompPars[lComp].baseName,NameAndMonitorSet) == 0 ) { 	    
	   // load values 
	   for ( Int_t i = 0; i < nVariables; i++ ) {
	     j = fVarPars[i].ReducedVarIndex;
	     if ( j < 0)                            continue;
	     if (isinf( MonitorValue[i]) != 0 )     continue; 
	     if (  MonitorValue[i] < -0.000000001 ) continue;
	     // fill appropriate histo
	     if (MonitorValue[i] > 0.0 && (fVarPars[i].GlobalSumType > -2) ) {
	       histoSuperimpose[lComp*fVarNumber+j]->Fill(MonitorValue[i]*fVarPars[i].MultFactor);
	       histoSimple[j]->Fill(MonitorValue[i]*fVarPars[i].MultFactor);			   
	     }
	     // histoSumAllTimePoints
	     //		       if (fVarPars[i].GlobalSumType == i ) {
	     if (fVarPars[i].GlobalSumType > -1 ) {
	       histoSumAllTimePoints[j]->Fill( (Float_t) difftime(MonitorTime,fTimeMin), MonitorValue[i]*fVarPars[i].MultFactor/kHistoSmoothFactor);
	     }
	   }
	 }
       }
     }
   }

   if (gDebug > 0) cout  << getTime()  << " DoGlobalHisto 2 " << endl;
   FillGlobalSumHisto(histoSumAllTimePoints, histoSumAllTimePointsInv,tTextSumAllTimePointsInv, binAllTimePoints);
   FreeMemory(histoSumAllTimePoints,fVarNumber);
   FreeMemory(tTextSumAllTimePoints,fVarNumber);
   //
   //now that have the histoSumAllTimePointsInv, plot them
   //this is the histogram of summed values like total mbytes
   //
   for (Int_t iVar = 0; iVar <fVarNumber; iVar++) { 
       l = fReduced2FullVarIndex[iVar];
       if (fVarPars[l].GlobalSumType > -1) {   
	 sprintf(plotTitle, "%s: %s",fMonitorSet,fVarPars[l].Name);
	 fPlotMaker->Histo(plotTitle,fVarPars[l].YUnit,kColorCode[0],fVarPars[l].Name,histoSumAllTimePointsInv[iVar],-1.);	 
	 if (fFillRootFileFlag == true) {
	   sprintf(plotName,"%s_%s_Aggregate%s_HistoCanvas_%s",fMonitorSet,fVarPars[l].PlotType,fVarPars[l].Name,plotLabel);
	   fPlotMaker->WriteCanvas(plotName);
	   fPlotMaker->WriteHisto(histoSumAllTimePointsInv[iVar]);
	 } 
	 //	 sprintf(plotName,"%s_Aggregate%s_Histo_%s",fMonitorSet,fVarPars[l].Name,plotLabel);
	 sprintf(plotName,"%s_%s_HistoAggregate%s_%s",fMonitorSet,fVarPars[l].PlotType,fVarPars[l].Name,plotLabel);
	 fPlotMaker->DoGif(plotName,kGlobalScope,fVarPars[l].PlotType);
       }
   }

   FreeMemory(histoSumAllTimePointsInv,fVarNumber);
   FreeMemory(tTextSumAllTimePointsInv,fVarNumber);

    if (gDebug > 0) cout  << getTime()  << " DoGlobalHisto 3 " << endl;

   //now that have the histoSimple, plot them
   // this is the histogram of say cpu system for all nodes, together, no superimposing
   for (Int_t iVar =0; iVar <fVarNumber; iVar++) { 
       l = fReduced2FullVarIndex[iVar];
       //        if (fVarPars[l].GlobalSumType == l) {   
        if (fVarPars[l].GlobalSumType > -2) {   
	  //	   sprintf(plotTitle, "%s AllNodesTogether: %s",fMonitorSet,fVarPars[l].Name);
	   sprintf(plotTitle, "AllNodesTogether: %s",fVarPars[l].Name);
	   fPlotMaker->Histo(plotTitle,fVarPars[l].YUnit,kColorCode[0],fVarPars[l].Name,histoSimple[iVar],-1.);
	   if (fFillRootFileFlag == true) {
	     sprintf(plotName,"%s_%s_%s_AllNodesTogether_HistoCanvas_%s",fMonitorSet,fVarPars[l].PlotType,fVarPars[l].Name,plotLabel);
	     fPlotMaker->WriteCanvas(plotName);	     
	     fPlotMaker->WriteHisto(histoSimple[iVar]);	     
	   }
 	   sprintf(plotName,"%s_%s_Histo%sAllNodesTogether_%s",fMonitorSet,fVarPars[l].PlotType,fVarPars[l].Name,plotLabel);
	   fPlotMaker->DoGif(plotName,kGlobalScope,fVarPars[l].PlotType);
	}
   }
   FreeMemory(histoSimple,fVarNumber);
   FreeMemory(tTextSimple,fVarNumber);
	   
   //////////////////////////////////////////////////////////////////////
   // do superimposed individual histos
   /////////////////////////////////////////////////////////////////////
   // maybe do not use statistics for this, as it is meaningless?
   //

   fPlotMaker->SetStatOption(kFALSE);

   Bool_t bSkipDummyLables ;
   
   if (gDebug > 0) cout  << getTime()  << " DoGlobalHisto 4 " << endl;
   
   bSkipDummyLables = false;
   
   for (Int_t iVar = 0; iVar <fVarNumber; iVar++) { 
       l = fReduced2FullVarIndex[iVar];
       // take care of case when don't want to plot superimposed plots
       if (fVarPars[l].GlobalSumType > - 2) {	   
	   ////////////////////////////////////////////////////////////////////////////////
	   // take care to not have too many graphs per plot. If too many, create new plot
	   Int_t graphsPerPlot   = 0;
	   Int_t remainingGraphs = fCompNumber;
	   Int_t  plottedGraphs  = 0;
	   Int_t plotCount       = 0;
	   Double_t histMax; 

	   while (remainingGraphs > 0) {
	       if (remainingGraphs >= fMaxGraphsPerPlot ) {
		   graphsPerPlot = fMaxGraphsPerPlot;
	       }
	       else {
		   graphsPerPlot = remainingGraphs;
	       }
	       remainingGraphs = remainingGraphs - graphsPerPlot;	       

	       histMax      = 0; 

	       // 1st get max of all histos and set max
	       for (Int_t lComp = 0; lComp < graphsPerPlot; lComp++) {
		 if (fVarHasLabelButIsDummyFlag[lComp*fVarNumber+iVar] != true || bSkipDummyLables == false) {
		   if (histoSuperimpose[(plottedGraphs+lComp)*fVarNumber+iVar]->GetMaximum() > histMax)
		     histMax = histoSuperimpose[(plottedGraphs+lComp)*fVarNumber+iVar]->GetMaximum();		   
		 }
	       }   		   	       

	       // now plot. first histo
	       //label = GetLabel(fCompPars[plottedGraphs].compName,l,plottedGraphs);
	       label = GetLabel(fCompPars[plottedGraphs].compName,iVar,plottedGraphs);
	       fPlotMaker->Histo(fVarPars[l].Name,fVarPars[l].YUnit,kColorCode[0],
				 (Char_t *)label.Data(),histoSuperimpose[plottedGraphs*fVarNumber+iVar],histMax*1.1);

	       //superimpose rest
	       for (Int_t lComp = 1; lComp < graphsPerPlot; lComp++) {
		   // ignore if variable not used. Will not look too good on plot. Improve in future
		   if (fVarHasLabelButIsDummyFlag[lComp*fVarNumber+iVar] != true || bSkipDummyLables == false) { 
		       label = GetLabel(fCompPars[plottedGraphs+lComp].compName,iVar,plottedGraphs+lComp);
		       fPlotMaker->SuperimposeHisto(kColorCode[lComp],lComp,(Char_t *)label.Data(),
			    tTextSuperimpose[(plottedGraphs+lComp)*fVarNumber+iVar],histoSuperimpose[(plottedGraphs+lComp)*fVarNumber+iVar]);		      
		       //		       if (histoSuperimpose[(plottedGraphs+lComp)*fVarNumber+iVar]->GetMaximum() > histMax)
		       //	   histMax = histoSuperimpose[(plottedGraphs+lComp)*fVarNumber+iVar]->GetMaximum();
		   }
	       }   		   	       
	       //	       sprintf(plotName,"%s_%s%i_SuperimposedNodesEach_Histo_%s",fMonitorSet,fVarPars[l].Name,plotCount,plotLabel);
	       if (fFillRootFileFlag == true) {
		 sprintf(plotName,"%s_%s_%s_SuperimposedSet%i_HistoCanvas_%s",fMonitorSet,fVarPars[l].PlotType,fVarPars[l].Name,plotCount,plotLabel);
		 fPlotMaker->WriteCanvas(plotName);
	       }
	       sprintf(plotName,"%s_%s_Histo%s%i_Each_%s",fMonitorSet,fVarPars[l].PlotType,fVarPars[l].Name,plotCount,plotLabel);
	       fPlotMaker->DoGif(plotName,kGlobalScope,fVarPars[l].PlotType);
	       plottedGraphs += graphsPerPlot;
	       plotCount++;
	   }
       }
   }
   fPlotMaker->SetStatOption(kTRUE);

   FreeMemory(histoSuperimpose,fCompNumber*fVarNumber);
   FreeMemory(tTextSuperimpose,fCompNumber*fVarNumber);

    if (gDebug > 0) cout  << getTime()  << " DoGlobalHisto end " << endl;

   //   fPlotMaker->WriteFile();
}


void TDataProcess::DoGlobalGraphs(Char_t *plotLabel, Float_t* X,Int_t globalTypeOpt)
{

    if (gDebug > 0) cout  << getTime()  << " DoGlobalGraphs begin " << endl;

    if (fFillRootFileFlag == true) fPlotMaker->CdDir(-1,"graph");

    
    MakeGlobalSumGraphs(plotLabel, X,globalTypeOpt);
    MakeGlobalSuperimposedGraphs(plotLabel, X,globalTypeOpt);
    MakeGlobalBoxGraphs(X,globalTypeOpt);
    MakeGlobalSuperimposedMultiVarGraphs(plotLabel, X,globalTypeOpt);
     
    if (gDebug > 0) cout  << getTime()  << " DoGlobalGraphs end " << endl;
}



////////////////////////////////////////////////////////////////////////////////////////////////////
// appears on pages for each individual computer
////////////////////////////////////////////////////////////////////////////////////////////////////
void TDataProcess::DoEachGraphs(Char_t *plotLabel,Float_t* X)
{
   Char_t basePlotName[256];
   Char_t plotName[256];
   Char_t plotName2[256];
   TString label;
   Int_t l;

   if (gDebug > 0) cout  << getTime()  << " Do each Graphs begin " <<endl;

   ////////////////////////////////////////////////////////////////////////////////////////////////////
   // do individual plots, no superimposing
   ////////////////////////////////////////////////////////////////////////////////////////////////////

   for (Int_t lComp = 0; lComp < fCompNumber; lComp++) {	   
     for (Int_t iVar =0; iVar  < fVarNumber; iVar++) { 
       l = fReduced2FullVarIndex[iVar];
       //if ( gPickedVariable >= 0 && iVar != gPickedVariable) continue;
       if (fVarHasLabelButIsDummyFlag[lComp*fVarNumber+iVar] == true || fVarPars[l].EachSumType != -1 ) continue;
       //if (fVarPars[l].EachSumType != -1 ) continue;
       sprintf(plotName,"%s/_%s_%s_%s_Each_%s",fCompPars[lComp].compName,fCompPars[lComp].monsetName,fVarPars[l].PlotType,&fVarPars[l].Name[0],plotLabel);
       // sprintf(basePlotName,"%s_%s_Each_%s",fCompPars[lComp].monsetName,&fVarPars[l].Name[0],plotLabel);
       sprintf(basePlotName,"_%s_%s_%s_Each_%s",fCompPars[lComp].monsetName,fVarPars[l].PlotType,&fVarPars[l].Name[0],plotLabel);
       if (gDebug > 0)cout  << getTime()  << " individual plots, no sup. plotName= "<<plotName<<" gSetPlotName="<<gSetPlotName <<endl;

       if ((gbSetPlotNameOption == kTRUE && strcmp(gSetPlotName,basePlotName) == 0) ||  gbSetPlotNameOption == kFALSE) {
	 //label = GetLabel(fVarPars[l].Name,l,lComp);
	 label = GetLabel(fVarPars[l].Name,iVar,lComp);
	 // initialze
	 TGraph* grh    = new TGraph(fBinNumber,X,&fEachValue[(lComp*fVarNumber)*fBinNumber+iVar*fBinNumber]);
	 TGraph* grh2   = new TGraph(fBinNumber,X,&fEachMaxValue[(lComp*fVarNumber)*fBinNumber+iVar*fBinNumber]);
	 TText* tText2  = new TText();
	 //1st graph
	 fPlotMaker->Graph((Char_t *) label.Data(),fVarPars[l].YUnit, kColorCode[0], fEachMin[lComp*fVarNumber+iVar]*0.9,fEachMax[lComp*fVarNumber+iVar]*1.1," ","AL",grh);
	 // superimpose plt of max values
	 fPlotMaker->SuperimposeGraph(2," ", -1, kDashedLine,"L",tText2,grh2);
	 //	 if (fVarHasLabelButIsDummyFlag[lComp*fVarNumber+iVar] != true && fVarPars[l].EachSumType == -1 ) {
	 if (fVarPars[l].EachSumType == -1 ) {
	   if (fFillRootFileFlag == true) {
	     sprintf(plotName2,"%s_%s_%sVsTime_%s",fCompPars[lComp].monsetName,fVarPars[l].PlotType,&fVarPars[l].Name[0],plotLabel);
	     fPlotMaker->CdDir(lComp,"comp");
	     fPlotMaker->WriteCanvas(plotName2);
	   }
	   fPlotMaker->DoGif(plotName,kOneNodeScope,fVarPars[l].PlotType);  // do gif
	 }
	 // delete from memory
	 delete grh;
	 delete grh2;	       
	 delete tText2;
       }	       
     }       
   }
 
   //////////////////////////////////////////////////////////////////////////////////////
   // do superimposed individual plots for each comp
   //
   // do for each comp
   
    Char_t   labels[512];
    Char_t   title[512];
    unsigned int  pos;
    char    **argv;
    int      argc;
    Int_t    varUsedNumber      ;
    Int_t    plottedGraphsNumber;


   for (Int_t lComp = 0; lComp < fCompNumber; lComp++) {	        
     for (Int_t iVar =0; iVar <fVarNumber; iVar++) {
       l = fReduced2FullVarIndex[iVar];
       //if (gPickedVariable >= 0 && iVar != gPickedVariable) continue;
       if (fVarPars[l].EachSumType == l) {

	 varUsedNumber           = 0;
	 plottedGraphsNumber     = 0;

	 Int_t totGraphs = fVarPars[l].EachSumCount;
	 sprintf(labels,"");
	 for (Int_t ll = 0; ll  < totGraphs; ll++) {
	   pos = strcspn(fVarPars[l+ll].MultiName, "_");
	   if (pos < strlen(fVarPars[l+ll].MultiName)) {		  
	     if (strcmp(fVarPars[l+ll].MultiName+pos+1,"AUTO") == 0) {
	       sprintf(labels,"%s %s ",labels,fLabel[lComp*fVarNumber+iVar+ll].c_str());
	     } else {
	       sprintf(labels,"%s %s ",labels,fVarPars[l+ll].MultiName+pos+1);
	     }
	     if ( ll == 0) snprintf(title,pos+1,"%s ",fVarPars[l+ll].MultiName);
	   } else {
	     sprintf(labels,"%s %s ",labels,fVarPars[l+ll].Name);
	     if (ll == 0) sprintf(title,"%s ",fVarPars[l+ll].MultiName);
	   }
	 }

	 sprintf(plotName,"%s/_%s_%s_%s_Each_%s",fCompPars[lComp].compName,fCompPars[lComp].monsetName,fVarPars[l].PlotType, title,plotLabel);
	 // sprintf(basePlotName,"%s_%s_Each_%s",fCompPars[lComp].monsetName, title,plotLabel);	     
	 sprintf(basePlotName,"_%s_%s_%s_Each_%s",fCompPars[lComp].monsetName,fVarPars[l].PlotType,title,plotLabel);	     
	 
	 if ((gbSetPlotNameOption == kTRUE && strcmp(gSetPlotName,basePlotName) == 0) ||  gbSetPlotNameOption == kFALSE) {
	   argv = string_to_argv(labels, &argc);
	   // aha! A  cummulative plot. and aha! First variable to be plotted
	   // first add up the graphs so can plot superimposed, barred graphs
	   // graphSumBar will contain all the summed plots to be bar superimposed, starting from last
	   Float_t *graphSumBar = 0; graphSumBar = new Float_t[fBinNumber*totGraphs];
	   Float_t sumGraphMax  = 0.0;
	   // initialise first fBinNumber to first array
	   for (Int_t i = 0; i <  fBinNumber*totGraphs; i++) graphSumBar[i] = fEachValue[(lComp*fVarNumber)*fBinNumber+iVar*fBinNumber+i];       
	   // fill array, summed over previous
	   for (Int_t graphNum = 1; graphNum < totGraphs; graphNum++){
	     for (Int_t i=0; i < fBinNumber; i++) {
	       graphSumBar[graphNum*fBinNumber+i] = graphSumBar[(graphNum-1)*fBinNumber+i] + 
		 fEachValue[(lComp*fVarNumber)*fBinNumber+(iVar+graphNum)*fBinNumber+i];
	       if (graphSumBar[graphNum*fBinNumber+i] > sumGraphMax) sumGraphMax = graphSumBar[graphNum*fBinNumber+i];
	     }   
	   }

	   // reserve memory for appropriate graph number
	   TGraph ** sumGraph = 0; sumGraph = new TGraph*[totGraphs];
	   TText **tText = 0;         tText = new TText*[totGraphs];
	   for (Int_t  i = 0; i < totGraphs; i++) { 
	     sumGraph[i] = new TGraph(fBinNumber,X,&graphSumBar[i*fBinNumber]); 
	     tText[i]    = new TText();
	     if (fEachVarRef[ (iVar+i)*fCompNumber+lComp] > 0) varUsedNumber++; // get actual number of used graphs
	   }
	   
	   // plot graphs in reverse order
	   // do first graph
	   // first get last graph whose varref says it is active
	   //Int_t lastUsedMultiVar  = totGraphs-1;
	   Int_t lastUsedMultiVar  = 0;
	   for (Int_t testVar = totGraphs-1; testVar >= 0; testVar--) {
	     if(fEachVarRef[ (iVar+testVar)*fCompNumber+lComp] > 0 &&  testVar > lastUsedMultiVar) lastUsedMultiVar = testVar;
	   }

	   fPlotMaker->Graph(title,fVarPars[l].YUnit,kColorCodeEach[varUsedNumber-1],0.0,sumGraphMax,argv[lastUsedMultiVar+1],"ALB", sumGraph[lastUsedMultiVar]);
	   if (gDebug > 0) cout  << getTime()  << "  varUsedNumber="<<varUsedNumber<< " lastUsedMultiVar="<<lastUsedMultiVar<< " argv[lastUsedMultiVar+1]="<<argv[lastUsedMultiVar+1] 
		<< " argv[lastUsedMultiVar+1]="<<argv[lastUsedMultiVar+1] <<endl;
	   plottedGraphsNumber++;
	   // do the rest
	   for (Int_t graphNum = lastUsedMultiVar-1; graphNum >= 0; graphNum--) {
	       if (fEachVarRef[ (iVar+graphNum)*fCompNumber+lComp] > 0) {
	       fPlotMaker->SuperimposeGraph(kColorCodeEach[varUsedNumber-1-plottedGraphsNumber],argv[graphNum+1],plottedGraphsNumber,kFullLine,"LB", tText[graphNum],sumGraph[graphNum]); 
	       plottedGraphsNumber++;
	       if (gDebug > 0) cout  << getTime() << "  graphNum="<<graphNum << " fEachVarRef[ (iVar+graphNum)*fCompNumber+lComp] ="<<fEachVarRef[ (iVar+graphNum)*fCompNumber+lComp]<< " argv[graphNum+1]="<<argv[graphNum+1] << endl;
	     }
	   }

	   if (fFillRootFileFlag == true) {
	     sprintf(plotName2,"%s_%s_%sVsTime_%s",fCompPars[lComp].monsetName,fVarPars[l].PlotType,fVarPars[l].GlobalSumName,plotLabel);
	     fPlotMaker->CdDir(lComp,"comp");
	     fPlotMaker->WriteCanvas(plotName2);
	   }
	  
	   fPlotMaker->DoGif(plotName,kOneNodeScope,fVarPars[l].PlotType); //plot postscript and gif
	   // free memory
	   FreeMemory(sumGraph, totGraphs);
	   FreeMemory(tText, totGraphs);	       
	   delete [] graphSumBar; graphSumBar = 0;	       
	   free(argv[0]);
	   free(argv);
	 }
       }
     }
     // draw here line of node up status
   }
   if (gDebug > 0) cout  << getTime()  << " Do each Graphs end " <<endl;
}
   

void TDataProcess::FreeMemory(TGraph ** grh, Int_t totGraphs)
{
    for (Int_t i=0;i < totGraphs; i++) {
	if ( grh != 0 ) delete  grh[i];
	grh[i] = 0;
    }
    delete [] grh;		       
    grh = 0;    
}

void TDataProcess::FreeMemory(TH1F ** hst, Int_t totGraphs)
{
    for (Int_t i=0;i < totGraphs; i++) {
	if (hst[i] != 0)  delete  hst[i];
	hst[i] = 0;
    }
    delete [] hst;		       
    hst = 0;	   
}


void TDataProcess::FreeMemory(TText ** tText, Int_t totGraphs)
{
    for (Int_t i = 0;i < totGraphs; i++) {
	if (tText[i] != 0) delete  tText[i];
	tText[i]=0;
    }
    delete [] tText;    
    tText = 0;	   
}



TString TDataProcess::GetLabel(Char_t * label,Int_t iVar,Int_t lComp) 
{
    Char_t plotTitleTemp[256]="";
    TString xxx;
   
    Int_t l = fReduced2FullVarIndex[iVar];

    sprintf(plotTitleTemp,"%s",label);
    if (fVarPars[l].labelIndex >= 0){ 
      sprintf(plotTitleTemp,"%s_%s",plotTitleTemp,fLabel[lComp*fVarNumber+iVar].c_str());
    }
    xxx = plotTitleTemp;
    return(xxx);
}



Int_t TDataProcess::SetLabel() {

    Char_t  line[2000]="";
    Int_t   l;
    char    **argv;
    Int_t   argc;
    Char_t  *ptr;
    Char_t  aux[512];
    string  * computersFileSelectedEntries = 0;
    Int_t   lComp;
    
    if (gDebug > 0) cout  << getTime()  << " SetLabel. begin. fUseMysql="<<fUseMysql  << endl;  


    if (!fUseMysql) {

      if ( computersFileSelectedEntries != 0) delete [] computersFileSelectedEntries ;
      computersFileSelectedEntries = new string[fCompNumber];

      sprintf(aux,"%s/config/computerpars.conf",gAffairPath);
      FILE *table_fp = fopen(aux,"r");          
      if (gDebug > 0) cout  << getTime()  << " SetLabel. begin "  << endl;        
      
      while (fgets(line,2000,table_fp)) { 
	for(ptr=line;;ptr++){
	  if (*ptr==0) break;
	  if (!isspace(*ptr)) break;
	}       
	if (*ptr=='#')        continue;
	if (strlen(line) < 2) continue;
	if((argv = string_to_argv(line, &argc)) != NULL) {
	  lComp = GetCompId(argv[1],argv[2]); 
	  if (lComp > -1) { 
	    computersFileSelectedEntries[lComp] = line;
	  }
	}
	free(argv[0]);
	free(argv);
	
      }
      fclose(table_fp);   
    } else {
      // BUG! have to specify monitor set??? is it fixed??
      computersFileSelectedEntries = fAffairSql->GetInfoFromComputersTable(fRrdString,fCompNumber);
      if (!computersFileSelectedEntries) return 0;
    }
    
    for (Int_t lComp = 0; lComp < fCompNumber; lComp++) {
      if((argv = string_to_argv((Char_t *) computersFileSelectedEntries[lComp].c_str(), &argc)) != NULL) {
	for (Int_t iVar = 0; iVar < fVarNumber; iVar++) { 	   	    
	  l = fReduced2FullVarIndex[iVar];
	  if (fVarPars[l].labelIndex + kComputersConfZeroOffset >= argc) {
	    cout  << getTime()  << " SetLabel method.fVarPars[varCount].labelIndex +5="<<
	      fVarPars[l].labelIndex<<" >= argc. Should not happen. Set it to -1 " << 
	      " name=" << fVarPars[l].Name << " argc=" << argc << " varCount=" << l << endl;
	    // fVarPars[l].labelIndex = argc -kComputersConfZeroOffset-1;
	    fVarPars[l].labelIndex = -1;
	  }
	  
	  if (fVarPars[l].labelIndex >= 0) {
	    fLabel[lComp*fVarNumber+iVar] = argv[kComputersConfZeroOffset+fVarPars[l].labelIndex];
	    if (strcmp(argv[kComputersConfZeroOffset+fVarPars[l].labelIndex],"DUMMY") == 0) fVarHasLabelButIsDummyFlag[lComp*fVarNumber+iVar] = true;
	  }
	}      
	free(argv[0]);
	free(argv);
      }
    }
    if (gDebug > 0) cout  << getTime()  << " SetLabel. end "  << endl;  
    return 1;
}

char* TDataProcess::getTime(void){
      time_t the_time;
      struct tm *tm_ptr;
      char timeLabel[256];
      (void) time(&the_time);
      tm_ptr = localtime(&the_time);    
      strftime(timeLabel,74,"%H:%M %d/%b/%y",tm_ptr);
      sprintf(gDebugOutput,"DataProcess: %s",timeLabel);
      return gDebugOutput;
  }







