#ifndef TDataProcess_h
#define TDataProcess_h

//#define __cplusplus


#include <TROOT.h>
#include <TChain.h>
#include <TFile.h>
#include <TCanvas.h>
#include <TText.h>
#include <TH1.h>
#include <TGraph.h>
#include <TText.h>
#include <TH2.h>
#include <stdio.h>
#include <TSystem.h>
#include <TString.h>

#include <time.h>
#include <iostream>
#include <fstream>
#include <string>
#include "TStyle.h"
#include <sys/stat.h>
#include "rrd_tool.h"
#include "TObject.h"
#include "TPlotMaker.h"
#include "TAffairSql.h"

#include "TMap.h"

#include <math.h>

namespace std {}
using namespace std;




class TDataProcess:public TObject {
 private:
    Float_t   gCanvasYFactor;
    Int_t     fAnalysisLevel;
    Int_t     fEachPlotsOption;
    Char_t    fRootFile[256];

    Float_t  *fEachMaxValue; //max value
    Float_t  *fEachCurrentValue;
    Float_t  *fEachValue;
    Float_t  *fEachValueCount;
    
    Float_t  *fGlobalMax;
    Float_t  *fGlobalMin;
   
    Float_t *fGlobalSumMax;
    Float_t *fGlobalSumMin;
    Float_t *fGlobalSumValue;
    Float_t *fGlobalSumMaxValue;

    Char_t   fPlotRootFile[256];
    Float_t *fEachMax;
    Float_t *fEachMin;
    Int_t   *fEachVarRef;

    TPlotMaker *fPlotMaker;
    TAffairSql *fAffairSql;

    FILE*    gLogFile;  
    Char_t   gAffairPath[256];
    Char_t   gAffairPlotPath[256];
    Double_t fBinSize; 

    Int_t  fTimeMax;
    Int_t  fTimeMin;

    Int_t  fVarNumber;
    Int_t  fFullVarNumber;

    typedef struct myComputerPars_t{
	Char_t  rrdName[128];
	Char_t  rootName[128];
	Char_t  baseName[128];
	Char_t  compName[128];
	Char_t  monsetName[128]; 
    }ComputerPars_t; 

    typedef struct myVariablePars_t{
      Float_t MultFactor;
      Int_t   GlobalSumType;
      Int_t   SuperimposeMultiVarType;
      Int_t   SuperimposeMultiVarCount;
      Int_t   SuperimposeMultiVarRef;
      Int_t   GlobalMultiBoxType;
      Int_t   GlobalMultiBoxCount;
      Int_t   EachSumType;
      Int_t   GlobalSumCount;
      Int_t   EachSumCount;
      Int_t   labelIndex;
      Int_t   ReducedVarIndex;
      Char_t  YUnit[32];
      Char_t  GlobalSumName[128];    
      Char_t  GlobalSumTitle[128];  
      Char_t  MultiName[128];   
      Char_t  PlotType[128];
      Char_t  Name[128];
    }VariablePars_t; 
    

    // the two declarations below may cause problems.
    // they probably have to be redifen somewhere else, or something...
    // they have a problem to be streamed. The ! excludes them from the streamer

    VariablePars_t  * fVarPars; //! temporary state value
    ComputerPars_t  * fCompPars;              //! temporary state value
    string *fRrdString;
    Int_t   fVarLabelNumber;

    Int_t  fCompNumber;
    Int_t  gPickedVariable;

    Char_t fMonitorSet[256];
    Char_t fComputerList[256];
    Char_t fRunOption[128];

    Int_t  fBinNumber;
    Int_t  kColorCode[20];    
    Int_t  kColorCodeEach[10];    
    TFile *f;

    char     kBeep;
    Bool_t   fInitFlag;
    Bool_t  *fVarHasLabelButIsDummyFlag;
    string  *fLabel;
    Int_t    fRunCount;
    Int_t    fRunNumArray[kMaxRunCount];
    Bool_t   fFillRootFileFlag;
    Int_t   *fReduced2FullVarIndex;

    Char_t   gSetPlotName[256]; //used by affair control to pick graph to display
    Bool_t   gbSetPlotNameOption;
    Char_t   gDebugOutput[256];

    Int_t    fCompActiveNumber;
    Int_t *  fReduced2FullCompActiveIndex;

    Bool_t   gStoreOldPlotsFlag;
    Int_t    fUseMysql;

    Int_t   fMaxGraphsPerPlot;

   FILE*  getLogfile(int logOptionX);
   void   FillEachArrays(Int_t deltaTime);
   void   NormalizeEachArrays();
   void   FillEachArraysMaxRrd(Int_t deltaTime);
   void   FillSumArrays();
   void   InitArrays();
   void   SetVariables(Char_t * varSet);
   void   SetComputers();
   void   SetOneComputer(Char_t * computerName);
   void   SetColorCodes();
   void   CreateArrays();
   void   InitEachHisto(TH1F **  histo);
   void   InitGlobalSuperimposeHisto(TH1F **  histo,TH1F **  histo2,TText **  text1,TText **  text2);
   void   InitGlobalSumHisto(TH1F** histo2,TText **  text2, Int_t binAllTimePoints,Float_t timeRange);
   void   FillGlobalSumHisto(TH1F **  histo,TH1F** histo2,TText ** tTextSumAllTimePointsInv,Int_t binAllTimePoints);
   void   DoEachHisto(Char_t *plotLabel);
   void   DoGlobalHisto(Char_t *plotLabel);
   void   FillEachOffline();
   //Int_t  FillEachRrd(Char_t *timePeriodX, Int_t globalTypeOpt);
   Int_t  FillEachRrd();
   void   InitXArray(Float_t* XArray);
   void   DoGlobalGraphs(Char_t *plotLabel,Float_t* X, Int_t globalTypeOpt);
   void   DoEachGraphs(Char_t *plotLabel,Float_t* X);
   void   MakeGlobalSuperimposedMultiVarGraphs(Char_t *plotLabel, Float_t* X,Int_t globalTypeOpt);
   void   MakeGlobalSuperimposedGraphs(Char_t *plotLabel, Float_t* X,Int_t globalTypeOpt);
   void   MakeGlobalBoxGraphs(Float_t* X,Int_t globalTypeOpt);
   void   MakeGlobalSumGraphs(Char_t *plotLabel, Float_t* X,Int_t globalTypeOpt);
   void   FreeMemory(TGraph ** graph,Int_t totGraphs);
   void   FreeMemory(TH1F ** histo,Int_t totGraphs);
   void   FreeMemory(TText ** tText,Int_t totGraphs);
   Bool_t IsDummyLabel(Int_t iVar,Int_t lComp);
   TString GetLabel(Char_t * label,Int_t iVar,Int_t lComp);
   Int_t  SetLabel();
   void   FindRange( Int_t selectionVariable, Double_t selectionLowCut,Double_t selectionHighCut);
   Bool_t Notify();
   void   DoAnalysis();
   char** string_to_argv(char *argstring, int *argc);
   char*  getTime(void);
   Int_t  CheckIfFileExists(Char_t * rootfile);
   Int_t  GetCompId(Char_t* compName,Char_t* monitorSet);
   Int_t  GetCompId(Char_t* nameAndMonitorSet); 

   public :
   TTree          *fTree;   //pointer to the analyzed TTree or TChain
   TChain          *fChain;   //pointer to the analyzed TTree or TChain
   Int_t           fCurrent; //current Tree number in a TChain
   //Declaration of leaves types
   Char_t          NameAndMonitorSet[256];
   Int_t           MonitorTime;
   Int_t           OrigMonitorTime;
   Int_t           nVariables; 
   Float_t         MonitorValue[kMaxVariables];
   
//List of branches
   TBranch        *b_NameAndMonitorSet;
   TBranch        *b_MonitorTime;  
   TBranch        *b_OrigMonitorTime;  
   TBranch        *b_nVariables;  
   TBranch        *b_MonitorValue;  
   
   TDataProcess();
   ~TDataProcess();
   Int_t  Cut(Int_t entry);
   Int_t  GetEntry(Int_t entry);
   Int_t  GetEntryTree(Int_t entry);
   Int_t  LoadTree(Int_t entry);
   Int_t  LoadFillTree(Int_t entry);

   Int_t  Init(Char_t monSet[256],Int_t xUnitsInit,Char_t runOption[256],Char_t computerList[256],
	       Char_t plotRootFile[256],Char_t varSet[64]);
   Int_t  InitQuick(Char_t computerList[256],Char_t varSet[64]);
   Int_t  SetRootFileList(string * rootfilelist, Int_t entries);
   void   PickVariable(Int_t pickedVariable);
   void   CloseFile();
   void   GlobalPlotsOffline(Char_t *plotLabel,Double_t timeUnit, Int_t gridNumber,Int_t TimeNow, 
	   Int_t sumOrSuperimposedOption,Int_t selectionVariable, Double_t selectionLowCut, Double_t selectionHighCut);
   void   GlobalPlotsRrd(Char_t *plotLabel,Double_t timeUnit, Int_t gridNumber,Int_t TimeNow, Int_t sumOrSuperimposedOption);
   void   EachPlotsOffline(Char_t *plotLabel,Double_t timeUnit, Int_t gridNumber,
			    Int_t TimeNow,Int_t selectionVariable, Double_t selectionLowCut, Double_t selectionHighCut);
   void   EachPlotsRrd(Char_t *plotLabel, Double_t timeUnit, Int_t gridNumber, Int_t TimeNow);
   void   Show(Int_t entry = -1);
   void   FillEachHisto(TH1F** histo);
   void   SetPlotDir(Char_t *affairPlotPath) { sprintf(gAffairPlotPath,"%s",affairPlotPath);}
   void   SetCanvasYFactor(Float_t canvasYFactor) { gCanvasYFactor = canvasYFactor;}
   void   SetEachPlotsOption(Int_t eachPlotsOption) { fEachPlotsOption = eachPlotsOption;}
   void   SetCanvas(TCanvas *c1);
   TCanvas *  GetCanvas();
   void   SetOutputMode(Int_t outMode);
   void   SetPlotName(Char_t * plotName);
   void   UnsetPlotName();
   void   SetStoreOldPlotsFlag(Bool_t storeOldPlotsFlag){ gStoreOldPlotsFlag = storeOldPlotsFlag;}
   void   SetMysql(TAffairSql * affairSql);

//0 option creates a dummy streamer
// with real stremer have problems with fVarPars and fCompPars
   ClassDef (TDataProcess,0) // affair class
};

#endif

#ifdef TDataProcess_cxx
//TDataProcess::TDataProcess(TTree *tree)

TDataProcess::TDataProcess()
{   
    fEachCurrentValue    = 0;
    fEachMaxValue        = 0;
    fEachValue           = 0;
    fEachValueCount      = 0;
    fGlobalMax           = 0;
    fGlobalMin           = 0;
    fGlobalSumMax        = 0;
    fGlobalSumMin        = 0;
    fGlobalSumValue      = 0;
    fGlobalSumMaxValue   = 0;
    fEachMax             = 0;
    fEachVarRef          = 0;
    fEachMin             = 0;
    fCompPars            = 0;
    fRrdString           = 0;
    fVarPars             = 0;
    fVarHasLabelButIsDummyFlag      = 0;
    fLabel               = 0;
    fPlotMaker           = 0;
    f                    = 0;
    fAffairSql           = 0;
    fUseMysql            = 0;

    fFillRootFileFlag     = false;
    fReduced2FullVarIndex = 0;

    fCompActiveNumber = 0;
    fReduced2FullCompActiveIndex  = 0;


    // initialize plotting class
    if (fPlotMaker != 0) delete fPlotMaker;
    fPlotMaker = new TPlotMaker;


    gbSetPlotNameOption = kFALSE;
    sprintf(gSetPlotName,"ALL"); ///dummy name
    SetColorCodes();
    gCanvasYFactor = 1.0;

    kBeep = '\a';
    gPickedVariable  = -1;   
    SetStoreOldPlotsFlag(false);
    fEachPlotsOption = 0;

    fAnalysisLevel   = 0;

    // setup logfile
    // it can be stdout if logOption ==0, /dev/null if logOption ==1, and in log dir if logoption = 2
    // gLogFile = getLogfile(0);

    sprintf(gAffairPath,"%s",getenv("AFFAIR_MONITOR"));
    sprintf(gAffairPlotPath,"%s/plots",gAffairPath);
    fMaxGraphsPerPlot = kMaxGraphsPerPlot;
}



TDataProcess::~TDataProcess()
{
   if (!fChain) return;
   delete fChain->GetCurrentFile();
}

Int_t TDataProcess::GetEntry(Int_t entry)
{
// Read contents of entry.
   if (!fChain) return 0;
   return fChain->GetTree()->GetEntry(entry);
}

Int_t TDataProcess::GetEntryTree(Int_t entry)
{
// Read contents of entry.
   if (!fTree) return 0;
   return fTree->GetEntry(entry);
}

Int_t TDataProcess::LoadFillTree(Int_t entry)
{
    // Set the environment to read one entry
   if (!fTree) return -5;
   Int_t centry = fTree->LoadTree(entry);
   if (centry < 0) return centry;
   if (fTree->IsA() != TChain::Class()) {
       return centry;
   }
   return centry;
}


Int_t TDataProcess::LoadTree(Int_t entry)
{
  // Set the environment to read one entry
   if (!fChain) return -5;
   Int_t centry = fChain->LoadTree(entry);
   if (centry < 0) return centry;
   if (fChain->IsA() != TChain::Class()) {
       return centry;
   }
   TChain *chain = (TChain*)fChain;
   if (chain->GetTreeNumber() != fCurrent) {
       fCurrent = chain->GetTreeNumber();
       Notify();
   }
   return centry;
}


void   TDataProcess::SetPlotName(Char_t * plotName) 
{ 

  sprintf(gSetPlotName,"%s",plotName);
  gbSetPlotNameOption = kTRUE;
}

void   TDataProcess::UnsetPlotName() 
{ 
  gbSetPlotNameOption = kFALSE;
}


void TDataProcess::SetCanvas(TCanvas * c1)
{
        fPlotMaker->SetCanvas(c1); 
}

void   TDataProcess::SetOutputMode(Int_t outMode)
{
        fPlotMaker->SetOutputMode(outMode); 
}


TCanvas * TDataProcess::GetCanvas()
{
  return(fPlotMaker->GetCanvas()); 
}

void TDataProcess::PickVariable(Int_t pickedVariable)
{
  gPickedVariable = pickedVariable;
}

void   TDataProcess::SetMysql(TAffairSql * affairSql) 
{
  fAffairSql = affairSql;
  fUseMysql = 1;
}

Int_t TDataProcess::InitQuick(Char_t computerList[256],Char_t varSet[64])
{

  Int_t ncols;
  Char_t line[256];
  Char_t baseName[256];
  Char_t rrdName[256];
  Char_t rootName[256];
  Char_t monitorSetName[256];
  Char_t computerName[256];

  Char_t aux[256];
  Int_t nameAndMonitorSetTotalTmp = 0;

  if (gDebug > 0) cout  << getTime()  << " init quick. computerList="<<computerList <<" varSet"<<varSet<<endl; 

  if (!fUseMysql) {
    if (strcmp(computerList,"ONLINE_RRD_LIST") == 0) {
      // case when rrdlist from computer online.
      sprintf(aux,"%s/log/rrdlist.txt",gAffairPath);
    }
    else {
      sprintf(aux,"%s",computerList);
    }    
    
    /// first get fCompNumber
    FILE *table_fp = fopen(aux,"r");    
    while (fgets(line,256,table_fp)) {	      
      if (strcmp(computerList,"ONLINE_RRD_LIST") == 0) {
	ncols = sscanf(line,"%s %s %s %s %s",&rrdName[0], &rootName[0],&baseName[0],&monitorSetName[0],&computerName[0]);
      }
      else {
	ncols = sscanf(line,"%s",&computerName[0]);
      }
      if (strcmp(monitorSetName,fMonitorSet) == 0 ) { 	  
	nameAndMonitorSetTotalTmp++;
      }
    }
    fclose(table_fp);   
  } else {
    //mysql case
    // case when use a file is not coverd here. Do this, if needed, in future
    Char_t sql[500];
    sprintf(sql,"SELECT * FROM AFFAIR_RRDLIST WHERE MONSET='%s' ",fMonitorSet);
    nameAndMonitorSetTotalTmp = fAffairSql->GetNumberOfRowsSql(sql);
  }

  if (gDebug > 0) 
    cout  << getTime()  << " init quick. nameAndMonitorSetTotalTmp=" << nameAndMonitorSetTotalTmp <<"fCompNumber"<<fCompNumber<<" computerList"<<computerList<<endl; 

  if (nameAndMonitorSetTotalTmp != fCompNumber) {
    fInitFlag = true;
    printf("TDataProccess. number of nodes monitored has changed since last cycle. call init \n");
    Init(fMonitorSet, fBinNumber, fRunOption, computerList,fPlotRootFile,varSet);
  } else {
    fInitFlag = false;
    fPlotMaker->SetLogFlag(fInitFlag);
  }
  return(1);
}


//
Int_t  TDataProcess::SetRootFileList(string *rootfileList, Int_t entries)
{

    TChain *chainTmp = 0;
    chainTmp = new TChain("ntuple");

    for (Int_t ii = 0; ii < entries; ii++) chainTmp->Add(rootfileList[ii].c_str());

    if (chainTmp == 0) return(0);
    fChain = chainTmp;
    fCurrent = -1;	
    fChain->SetBranchAddress("NameAndMonitorSet",&NameAndMonitorSet);
    fChain->SetBranchAddress("MonitorTime",&MonitorTime);
    fChain->SetBranchAddress("OrigMonitorTime",&OrigMonitorTime);
    fChain->SetBranchAddress("nVariables",&nVariables);  
    fChain->SetBranchAddress("MonitorValue",&MonitorValue);
    
    Notify();
    if (gDebug > 0) cout  << getTime()  << "SetRootFileList: chain loaded " << endl;
    fAnalysisLevel = 1;

    return 1;
}

//
Int_t TDataProcess::Init(Char_t monSet[256],Int_t xUnitsInit, Char_t runOption[256],
			 Char_t computerList[256],Char_t plotRootFile[256],Char_t varSet[64])
{
    Char_t aux[256];    
    Int_t  ncols;
    Char_t line[256];
    Char_t baseName[256];
    Char_t rrdName[256];
    Char_t rootName[256];
    Char_t monitorSetName[256];
    Char_t computerName[256];
    Int_t  checkIfExists;
    Char_t tableName[256];
   
    gROOT->GetListOfFunctions()->Delete();
    gROOT->GetList()->Delete();


    fBinNumber = xUnitsInit;

    sprintf(fRunOption,"%s",runOption); 
    sprintf(fComputerList,"%s",computerList);
    sprintf(fPlotRootFile,"%s",plotRootFile);
    sprintf(fMonitorSet,"%s",monSet);
    fInitFlag = true;
        
    std::ifstream fin;
    fCompNumber = 0;    

    //    cout = std::stdout;
    printf("starting data process. \n");
    //find number of computers monitored
    if (strcmp(fRunOption,RRD_ONE_COMPUTER_OPTION) != 0 && strcmp(fRunOption,ANALYSIS_ONE_COMPUTER_OPTION) ) {
      if (!fUseMysql) {
	if (strcmp(fComputerList,"ONLINE_RRD_LIST") == 0) {
	  // case when rrdlist from computer online.
	  sprintf(aux,"%s/log/rrdlist.txt",gAffairPath);
	  checkIfExists = CheckIfFileExists(aux);
	  if (checkIfExists <= 0) {
	    cout  << getTime()  << " you chose online rrdlist, but it does not exist. Exiting " << endl;
	    exit(2);
	  }
	} else  {
	  // case when rrdlist from file
	  sprintf(aux,"%s",fComputerList);
	}    
	FILE *table_fp = fopen(aux,"r");    
	while (fgets(line,256,table_fp)) {	
	  if (strcmp(fComputerList,"ONLINE_RRD_LIST") == 0) {
	    ncols = sscanf(line,"%s %s %s %s %s",&rrdName[0], &rootName[0],&baseName[0],&monitorSetName[0],&computerName[0]); //dummy vars here
	    if (strcmp(monitorSetName,fMonitorSet) == 0 ) fCompNumber++;
	  } else {
	    ncols = sscanf(line,"%s",&computerName[0]);
	    fCompNumber++;
	  }
	}
	fclose(table_fp);   
      } else {
	// mysql case
	// case when use a file is not coverd here. Do this, if needed, in future
	// fCompNumber = fAffairSql->ReplaceRrdRowWithString(fMonitorSet,fRrdString);
	if ( fRrdString!= 0) {
	  delete [] fRrdString; 
	  fRrdString = 0;
	}
	fRrdString = fAffairSql->GetRrdRows(fMonitorSet,&fCompNumber);
	if (fRrdString == 0) {
	  printf(" fRrdString = 0. No computers. exit \n");
	  fCompNumber = 0;
	  return 0;
	} else {
	  if (gDebug) cout  << getTime() << " fRrdString0="<< fRrdString[0].c_str() <<endl;
	}
      }
     } else {
	// case when run  from web click for individual computer
	fCompNumber = 1;
    }
	
    if ( fCompNumber <= 0 ) {
	cout  << getTime()  << " fCompNumber= " <<fCompNumber << " runOption= " << runOption << " monitorSet= " <<fMonitorSet <<" fUseMysql=" << fUseMysql<<" This should not happen!. Exiting" << endl;
	return(0);
    }
    
    if (gDebug > 0) cout  << getTime()  << " Init: fCompNumber= "<< fCompNumber <<" fUseMysql=" << fUseMysql <<endl;

    if ( strcmp(fPlotRootFile,NO_ROOTFILE_STORAGE) != 0 ) fFillRootFileFlag = true; // see if want a final root file
    if (fUseMysql) fPlotMaker->SetMysql(fAffairSql);
    fPlotMaker->SetCanvasYFactor(gCanvasYFactor);
    fPlotMaker->SetPlotDir(gAffairPlotPath);
    fPlotMaker->Init(fMonitorSet,fFillRootFileFlag,fPlotRootFile,fMonitorSet);
    fPlotMaker->SetLogFlag(fInitFlag);


    SetVariables(varSet);
    CreateArrays();    
    if (strcmp(fRunOption,RRD_ONE_COMPUTER_OPTION) != 0 && strcmp(fRunOption,ANALYSIS_ONE_COMPUTER_OPTION) ) {
      SetComputers();
    } else {
	SetOneComputer(fComputerList);
    }
    SetLabel();
    if (gDebug > 0) cout  << getTime()  << "Init. labels filled " <<endl;
    // set now max graphs per plot, if use mysql
    if (fUseMysql) { fMaxGraphsPerPlot = atoi(fAffairSql->GetAffairParameter(kMaxGraphsPerPlotIndex)); 
    if (gDebug) printf(" default max graphs per plot =%d. New max graphs per plot =%d \n",kMaxGraphsPerPlot,fMaxGraphsPerPlot );
    }
    return(1);
}


//
//
//
Bool_t TDataProcess::Notify()
{
    //   called when loading a new file
    //   get branch pointers
    b_NameAndMonitorSet = fChain->GetBranch("NameAndMonitorSet");
    b_MonitorTime       = fChain->GetBranch("MonitorTime");
    b_OrigMonitorTime   = fChain->GetBranch("OrigMonitorTime");
    b_nVariables        = fChain->GetBranch("nVariables");
    b_MonitorValue      = fChain->GetBranch("MonitorValue");
    return kTRUE;
}


void TDataProcess::CloseFile()
{
    if (gDebug > 0) cout  << getTime()  << " close file begin " << endl;
    if (fEachMaxValue != 0) delete [] fEachMaxValue ; fEachMaxValue = 0;
    if (fEachValue != 0) delete [] fEachValue ; fEachValue = 0;
    if (fEachValueCount != 0) delete [] fEachValueCount ; fEachValueCount = 0;
    if (fEachCurrentValue != 0) delete [] fEachCurrentValue ; fEachCurrentValue = 0;
    if ( fGlobalMax!= 0) delete [] fGlobalMax ; fGlobalMax = 0;
    if ( fGlobalMin!= 0) delete [] fGlobalMin ; fGlobalMin=0;
    if ( fGlobalSumMax!= 0) delete [] fGlobalSumMax ; fGlobalSumMax=0;
    if ( fGlobalSumMin!= 0) delete [] fGlobalSumMin ; fGlobalSumMin=0;
    if ( fGlobalSumValue!= 0) delete [] fGlobalSumValue ; fGlobalSumValue = 0;
    if ( fGlobalSumMaxValue!= 0) delete [] fGlobalSumMaxValue; fGlobalSumMaxValue = 0;
    if ( fEachMax!= 0) delete [] fEachMax ; fEachMax = 0;
    if ( fEachMin!= 0) delete [] fEachMin ; fEachMin = 0;
    if ( fEachVarRef!= 0) delete [] fEachVarRef ; fEachVarRef = 0;
    if ( fReduced2FullCompActiveIndex!= 0) delete [] fReduced2FullCompActiveIndex ; fReduced2FullCompActiveIndex = 0;
    if ( fVarHasLabelButIsDummyFlag != 0) delete [] fVarHasLabelButIsDummyFlag ; fVarHasLabelButIsDummyFlag = 0;
    if ( fLabel != 0) delete [] fLabel ; fLabel = 0;

    if ( fCompPars!= 0) delete [] fCompPars ; fCompPars = 0;
    if ( fRrdString != 0) delete [] fRrdString ; fRrdString = 0;
    if (fVarPars!= 0) delete [] fVarPars ; fVarPars = 0;

    if ( fReduced2FullVarIndex!= 0) delete [] fReduced2FullVarIndex ; fReduced2FullVarIndex = 0;

    //    if ( strcmp(fRootfileList,"DUMMY") == 0 )  {
    //if (!fRootfileList )  {
    //	if (f != 0 ) f->Close(); // only close file if not using a chain
    //} else {
      //	fChain->Close();
	//	delete fChain;
    //}
    if (gDebug > 0) cout  << getTime()  << " close file fPlotMaker="<<fPlotMaker << endl;

    if (fPlotMaker != 0 ) {
      //      fPlotMaker->~TPlotMaker();
      //delete fPlotMaker ; fPlotMaker = 0;
    }
    gROOT->GetListOfFunctions()->Delete();
    gROOT->GetList()->Delete();
    if (f != 0 ) delete f;
    if (gDebug > 0) cout  << getTime()  << " close file end " << endl;
}


FILE* TDataProcess::getLogfile(int logOptionX) {
  Char_t aux[256];
    FILE* logFileX = 0;  
    if (logOptionX == 0) {
	logFileX = stdout;
    }
    else if (logOptionX == 1) { 
	logFileX = fopen("/dev/null","a");
    }
    else if (logOptionX == 2) {
       sprintf(aux,"%s/log/affairroot.log",gAffairPath);
      //sprintf(aux,"/tmp/rrd.log",gAffairPath);
	logFileX = fopen(aux,"a");
	cout  << getTime()  << " opened file = " << aux << " return = " <<  logFileX << endl;
    }
    return(logFileX);
}

void TDataProcess::Show(Int_t entry)
{
    // Print contents of entry.
    // If entry is not specified, print current entry
   if (!fChain) return;
   fChain->Show(entry);
}

Int_t TDataProcess::Cut(Int_t entry)
{
    // This function may be called from Loop.
    // returns  1 if entry is accepted.
    // returns -1 otherwise.
    return 1;
}

void TDataProcess::CreateArrays()
{
  // now that fVarNumber and fCompNumber determined,
  // initialsize arrays
  //    
  if (fEachCurrentValue != 0)      delete [] fEachCurrentValue ;    
  fEachCurrentValue = new  Float_t[fCompNumber*fVarNumber] ;
  
  if (fEachMaxValue != 0)      delete [] fEachMaxValue ;    
  fEachMaxValue = new  Float_t[fCompNumber*fVarNumber*fBinNumber] ;
  
  if (fEachValue != 0)      delete [] fEachValue ;    
  fEachValue = new  Float_t[fCompNumber*fVarNumber*fBinNumber] ;
  
  if (fEachValueCount != 0)      delete [] fEachValueCount ;    
  fEachValueCount = new  Float_t[fCompNumber*fVarNumber*fBinNumber] ;
  
  if ( fGlobalMax!= 0) delete [] fGlobalMax ;
  fGlobalMax = new  Float_t[fVarNumber] ;
  
  if ( fGlobalMin!= 0) delete [] fGlobalMin ;
  fGlobalMin = new  Float_t[fVarNumber] ;
  
  if ( fGlobalSumMax!= 0) delete [] fGlobalSumMax ;
  fGlobalSumMax = new  Float_t[fVarNumber] ;
  
  if ( fGlobalSumMin!= 0) delete [] fGlobalSumMin ;
  fGlobalSumMin = new  Float_t[fVarNumber] ;
  
  if ( fGlobalSumValue!= 0) delete [] fGlobalSumValue ;
  fGlobalSumValue = new  Float_t[fVarNumber*fBinNumber] ;
  
  if ( fGlobalSumMaxValue!= 0) delete [] fGlobalSumMaxValue ;
  fGlobalSumMaxValue = new  Float_t[fVarNumber*fBinNumber] ;
  
  if ( fEachMax!= 0) delete [] fEachMax ;
  fEachMax = new  Float_t[fVarNumber*fCompNumber] ;
  
  if ( fEachMin!= 0) delete [] fEachMin ;
  fEachMin = new  Float_t[fVarNumber*fCompNumber] ;
  
  if ( fEachVarRef!= 0) delete [] fEachVarRef ;
  fEachVarRef = new  Int_t[fVarNumber*fCompNumber] ;
  
  if ( fReduced2FullCompActiveIndex!= 0) delete [] fReduced2FullCompActiveIndex ;
  fReduced2FullCompActiveIndex = new  Int_t[fCompNumber] ;
  
  if ( fVarHasLabelButIsDummyFlag != 0) delete [] fVarHasLabelButIsDummyFlag ;
  fVarHasLabelButIsDummyFlag = new  Bool_t[fVarNumber*fCompNumber] ;
  
  if ( fLabel != 0) delete [] fLabel ;
  fLabel = new  string[fVarNumber*fCompNumber] ;
  
  if ( fCompPars!= 0) delete [] fCompPars ;
  fCompPars = new  ComputerPars_t[fCompNumber] ;
  
  //fRrdString set in taffairsql before call to createarray, so only do this step if
  //mysql not used
  if ( !fUseMysql ) {
    if ( fRrdString!= 0) delete [] fRrdString;
    fRrdString = new  string[fCompNumber] ;
  }
  
  for (Int_t i = 0; i < fVarNumber*fBinNumber;i++){
    fGlobalSumValue[i]    = 0.0;
    fGlobalSumMaxValue[i] = 0.0;
  }
  
  for (Int_t i = 0; i < fVarNumber*fCompNumber; i++){
    fEachMax[i]           = 0.0;
    fEachMin[i]           =  9999999999.; // kludge
    fEachCurrentValue[i]  = 0.0;
    //fVarHasLabelButIsDummyFlag[i]    = true;
    fVarHasLabelButIsDummyFlag[i]    = false;
    fEachVarRef[i]        = 0;
  }
  
  for (Int_t i = 0; i<  fCompNumber*fVarNumber*fBinNumber;i++){
    fEachValue[i]         = 0.0;
    fEachValueCount[i]    = 0.0;
    fEachMaxValue[i]      = 0.0;
  }
  
  
  for (Int_t i = 0; i < fVarNumber; i++){
    fGlobalMax[i]    =          0.0;
    fGlobalMin[i]    = 9999999999.;    // kludge
    fGlobalSumMax[i] =           0.0;
    fGlobalSumMin[i] =  9999999999.;   // kludge
  }
  
  
  for (Int_t i = 0; i < fCompNumber; i++) { 
    fReduced2FullCompActiveIndex[i]  = -1;
  }
  
}



void  TDataProcess::SetComputers()
{
    Char_t aux[256];
    Int_t  countTemp = 0;
    Int_t  ncols;
    Char_t line[512];
    Char_t baseName[256];
    Char_t rrdName[256];
    Char_t rootName[256];
    Char_t monitorSetName[256];
    Char_t computerName[256];
    Char_t newDir[256];
    Char_t chownCmd[512];

    Char_t   *ptr;
    char    **argv;
    Int_t     argc;

    countTemp = 0;

    if (gDebug > 0) cout  << getTime()  << "SetComputers:begin "  << endl;  

    if ( !fUseMysql ) {
      if (strcmp(fComputerList,"ONLINE_RRD_LIST") == 0) {
	// case when rrdlist from computer online.
	sprintf(aux,"%s/log/rrdlist.txt",gAffairPath);      
	FILE *table_fp = fopen(aux,"r");        
	while (fgets(line,512,table_fp)) {		
	  for(ptr=line;;ptr++){
	    if (*ptr==0) break;
	    if (!isspace(*ptr)) break;
	  }       
	  if (*ptr=='#')        continue;
	  if (strlen(line) < 2) continue;
	  if (strcmp(fComputerList,"ONLINE_RRD_LIST") == 0) {
	    ncols = sscanf(line,"%s %s %s %s %s",&rrdName[0], &rootName[0],&baseName[0],&monitorSetName[0],&computerName[0]);
	    if (strcmp(monitorSetName,fMonitorSet) == 0 ) {
	      fRrdString[countTemp] = line;
	      countTemp++;
	    }
	  }
	}
	fclose(table_fp);
      } else {
	// case when input from list, not online
      }
    }

    if (gDebug > 0) cout  << getTime()  << "SetComputers:fCompNumber "<<fCompNumber  << endl;  

    for (Int_t ii = 0; ii < fCompNumber; ii++) {
      if (gDebug > 0) cout  << getTime()  << " SetComputers: fRrdString[ii].c_str()= "<< fRrdString[ii].c_str() <<endl;
      if((argv = string_to_argv((Char_t *) fRrdString[ii].c_str(), &argc)) != NULL) {
	sprintf(fCompPars[ii].rrdName,   "%s",argv[1]); 
	sprintf(fCompPars[ii].rootName,  "%s",argv[2]); 
	sprintf(fCompPars[ii].baseName,  "%s",argv[3]); 
	sprintf(fCompPars[ii].monsetName,"%s",argv[4]); 
	sprintf(fCompPars[ii].compName,  "%s",argv[5]); 

	if (gDebug > 0) cout  << getTime()  << " SetComputers: argv1 to 5 = "<< argv[1]<< argv[2]<< argv[3]<< argv[4]<< argv[5] <<endl;
	
	// create directory AFFAIR/ComputerName if it does not exist
	// do this only for non offline
	if (strcmp(fRunOption,ANALYSIS_ONE_COMPUTER_OPTION) != 0 && strcmp(fRunOption,ANALYSIS_GLOBAL_OPTION) != 0 ) {
	  sprintf(newDir,"%s/%s",gAffairPlotPath,fCompPars[ii].compName);
	  sprintf(chownCmd,"if [ ! -d %s ]; then mkdir %s; chmod 777 %s;chmod 777 %s/.; chmod a+w %s/.; chmod a+r %s/.; fi ",newDir,newDir,newDir,newDir,newDir,newDir);
	  if (gDebug) cout << " newDir = " << newDir << endl << " chownCmd=" << chownCmd <<endl;  
	  gSystem->Exec(chownCmd);
	}
	free(argv[0]);
	free(argv);
      } else {
	cout  << getTime()  << "WARNING!! SetComputers:argv = string_to_argv(fRrdString[ii].c_str(), &argc)) == NULL.fRrdString[ii]= "<< fRrdString[ii].c_str() <<endl;
      }
    }

    if (gDebug > 0) cout  << getTime()  << "SetComputers:end "  << endl;  
}


void  TDataProcess::SetOneComputer(Char_t * computerName)
{
    Char_t baseName[256];
    Char_t rrdName[256];
    Char_t rootName[256];
    Char_t monitorSetName[256];
    Char_t newDir[256];
    Char_t chownCmd[256];
    Char_t line[1000];

    if (gDebug > 0) cout  << getTime()  << "SetOneComputers:begin fCompNumber="<<fCompNumber  << endl;  

    sprintf(rrdName,"%s_%s.rrd",  computerName,fMonitorSet);
    sprintf(rootName,"%s_%s.root",computerName,fMonitorSet);
    sprintf(baseName,"%s_%s",     computerName,fMonitorSet);
    sprintf(monitorSetName,"%s",  fMonitorSet);

    if (gDebug > 0) cout  << getTime()  << "SetOneComputers:begin2 rrdName="<<rrdName  << endl;  
    sprintf(fCompPars[0].rrdName   ,"%s",rrdName); 
    sprintf(fCompPars[0].rootName  ,"%s",rootName); 
    sprintf(fCompPars[0].baseName  ,"%s",baseName); 
    sprintf(fCompPars[0].compName  ,"%s",computerName); 
    sprintf(fCompPars[0].monsetName,"%s",monitorSetName); 

    if (gDebug > 0) cout  << getTime()  << "SetOneComputers:begin3 fCompNumber="<<fCompNumber  << endl;  
    // for comptability with general case with multiple computers. Needed in SetLabel
    sprintf(line,"%s %s %s %s %s",rrdName, rootName,baseName,monitorSetName,computerName);

    if (gDebug > 0) cout  << getTime()  << "SetOneComputers:begin4 rrdName="<<rrdName  << " line=" << line <<endl;  
    // when one computer,fRrdString not initialized till this point if sql used
    if ( fUseMysql ) {
      if ( fRrdString!= 0) delete [] fRrdString;
      fRrdString = new  string[fCompNumber] ;
    }

    if (gDebug > 0) cout  << getTime()  << "SetOneComputers:begin5 rrdName="<<rrdName  << " fRrdString=" << fRrdString <<endl;  
    fRrdString[0] = line;


    if (gDebug > 0) cout  << getTime()  << " basename=" << fCompPars[0].baseName << " comp name= " << fCompPars[0].compName  << endl; 

    // create directory AFFAIR/ComputerName if it does not exist
    sprintf(newDir,"%s/%s",gAffairPlotPath,computerName);
    sprintf(chownCmd,"if [ ! -d %s ]; then mkdir %s; chmod 777 %s/.; chmod a+w %s/.; chmod a+r %s/.; fi ",newDir,newDir,newDir,newDir,newDir);
    gSystem->Exec(chownCmd);

    if (gDebug > 0) cout  << getTime()  << "SetOneComputers:end "  << endl;  

}

void TDataProcess::SetVariables(Char_t * varSet)
{    
    Int_t     countTemp = 0;
    Char_t    aux[512];
    Int_t     varCount = 0;
    Int_t     l;

    ifstream fin;
    ifstream fin2;
    
    
    FILE *table_fp;    
    Char_t line[512]; 
    char **argv;
    Int_t argc;
    Int_t arfCount = 0;

    Int_t globalPlotIndex    =  -1;
    Int_t columnPlotIndex    =  -1;
    Int_t eachPlotIndex      =  -1;
    Int_t multiVarPlotIndex  =  -1;
    Int_t multiPlotNameIndex =  -1;
    Int_t yUnitIndex         =  -1;
    Int_t varLabelIndex      =  -1;
    Int_t plotTypeIndex      =  -1;
    Int_t multiVarRefIndex   =  -1;

    globalPlotIndex    =  5;
    columnPlotIndex    =  6;
    eachPlotIndex      =  7;
    multiPlotNameIndex =  8;
    yUnitIndex         =  9;
    varLabelIndex      = 10;
    plotTypeIndex      = 11;
    multiVarPlotIndex  = 12;
    multiVarRefIndex  =  13;
    
    Char_t   *ptr;
    string   *varLines = 0;


    fFullVarNumber  = 0;
    fVarNumber      = 0;
    fVarLabelNumber = 0;

    if (gDebug > 0) cout  << getTime()  << " SetVariables: begin " <<endl;

    if ( !fUseMysql ) {
      sprintf(aux,"%s/config/%s.conf",gAffairPath,fMonitorSet);
      table_fp = fopen(aux,"r");      
      // first determine the full number of variables            
      while (fgets(line,512,table_fp)) {      
	for(ptr=line;;ptr++){
	  if (*ptr==0) break;
	  if (!isspace(*ptr)) break;
	}       
	if (*ptr=='#') continue;
	if (strlen(line) < 2) continue;

	if((argv = string_to_argv(line, &argc)) != NULL) {
	  if (argc < 3) break; //make sure exit if extra line/s at end
	  if (fFullVarNumber == 0)  {
	    arfCount =  argc -1;
	    // for compatibility
	    if (arfCount == 11) {
	      globalPlotIndex    =  5;
	      columnPlotIndex    =  6;
	      eachPlotIndex      =  7;
	      multiPlotNameIndex =  8;
	      yUnitIndex         =  9;
	      varLabelIndex      = 10;
	      plotTypeIndex      = 11;
	    }
	  }
	  
	  if (gDebug) cout <<line << endl << " argv5,6,7,8=" << argv[5] << argv[5] << argv[6] << argv[7] << argv[8] <<endl;

	  if (( strcmp(varSet,"global") == 0 && atoi(argv[globalPlotIndex ]) > -2 )
	      ||(strcmp(varSet,"each")  == 0 &&  atoi(argv[eachPlotIndex ]) > -2 )
	      ||(strcmp(varSet,"all")   == 0 ) 
	      ||(atoi(argv[columnPlotIndex]) > -2 ) 
	      ||(strcmp(varSet,"global_and_each") == 0 && atoi(argv[globalPlotIndex ]) > -2 && atoi(argv[eachPlotIndex ]) > -2 )
	      ||(multiVarPlotIndex >= 0 && atoi(argv[multiVarPlotIndex ]) > -2)) {
	    fVarNumber++;    //counts total number of used variables
	  }
	  free(argv[0]);
	  free(argv);
	}  
	fFullVarNumber++;         //counts total number of variables 
      }
      fclose(table_fp);   
            
      if (gDebug) cout  << getTime()   << " fVariableNumber=" <<fVarNumber  << "varCount= " << fFullVarNumber<< " par number="<< arfCount << endl;     

      if ( varLines != 0) delete [] varLines ;varLines = 0;
      varLines = new  string[fFullVarNumber] ;    
      if (gDebug > 0) cout  << getTime()  << " SetVariables: fFullVarNumber= "<< fFullVarNumber<< " fVarNumber="<<fVarNumber <<endl;
      
      countTemp       = 0;
      table_fp = fopen(aux,"r");    
      while (fgets(line,512,table_fp)) {
	for(ptr=line;;ptr++){
	  if (*ptr==0)        break;
	  if (!isspace(*ptr)) break;
	}       
	if (*ptr=='#')        continue;
	if (strlen(line) < 2) continue;
	//	if((argv = string_to_argv(line, &argc)) != NULL) {
	  varLines[countTemp]  =  line;
	  countTemp++;
	//}
      }
      fclose(table_fp);   
    } else {
      //mysql
      if ( varLines != 0) delete [] varLines ;varLines = 0;
      varLines = fAffairSql->GetVariableInfoForMonitorSet(fMonitorSet,varSet,&fFullVarNumber,&fVarNumber);
    }

    // now create the appropriate arrays
    if (fVarPars!= 0) delete [] fVarPars;
    fVarPars = new  VariablePars_t[fFullVarNumber] ;
    if (fReduced2FullVarIndex != 0) delete [] fReduced2FullVarIndex ;
    fReduced2FullVarIndex = new  Int_t[fVarNumber] ;
    

    countTemp       = 0;
    varCount        = 0;
    for (Int_t ii = 0; ii < fFullVarNumber; ii++) {
      if (gDebug > 0) cout  << getTime()  << " SetVariables: varLines[ii].c_str()= "<< varLines[ii].c_str() <<endl;
      if((argv = string_to_argv((Char_t *) varLines[ii].c_str(), &argc)) != NULL) {
	fVarPars[varCount].ReducedVarIndex            = -1;      // initialize
	sprintf(fVarPars[varCount].Name,"%s",           argv[2]);		
	fVarPars[varCount].MultFactor                 = atof(argv[3]);		
	fVarPars[varCount].GlobalSumType              = atoi(argv[globalPlotIndex]);
	fVarPars[varCount].GlobalMultiBoxType         = atoi(argv[columnPlotIndex]);
	fVarPars[varCount].GlobalMultiBoxCount        = 0;
	fVarPars[varCount].EachSumType                = atoi(argv[eachPlotIndex]); 
	sprintf(fVarPars[varCount].YUnit,"%s",          argv[yUnitIndex]);
	sprintf(fVarPars[varCount].PlotType,"%s",       argv[plotTypeIndex]);
	sprintf(fVarPars[varCount].GlobalSumName,"%s",  argv[multiPlotNameIndex]);
	sprintf(fVarPars[varCount].MultiName,"%s",      argv[multiPlotNameIndex]);
	fVarPars[varCount].GlobalSumCount             =  0;
	fVarPars[varCount].labelIndex                  = atoi(argv[varLabelIndex]);	


	fVarPars[varCount].SuperimposeMultiVarType    = -2;
	fVarPars[varCount].SuperimposeMultiVarRef     = -2;
	if (multiVarPlotIndex >= 0) {
	  fVarPars[varCount].SuperimposeMultiVarType  = atoi(argv[multiVarPlotIndex]);
	  fVarPars[varCount].SuperimposeMultiVarRef   = atoi(argv[multiVarRefIndex]);
	}
	fVarPars[varCount].SuperimposeMultiVarCount   = 0;


	if (( strcmp(varSet,"global") == 0 && atoi(argv[globalPlotIndex ]) > -2 )
	    ||(strcmp(varSet,"each") == 0 &&  atoi(argv[eachPlotIndex ]) > -2 )
	    ||(strcmp(varSet,"all") == 0 ) 
	    ||(atoi(argv[columnPlotIndex]) > -2 ) 
	    ||(strcmp(varSet,"global_and_each") == 0 && atoi(argv[globalPlotIndex ]) > -2 && atoi(argv[eachPlotIndex ]) > -2 )
	    ||(multiVarPlotIndex >= 0 && atoi(argv[multiVarPlotIndex ]) > -2)) {


	  fVarPars[varCount].ReducedVarIndex = countTemp;
	  fReduced2FullVarIndex[countTemp]   = varCount;

	  if (gDebug) printf(" fVarPars[varCount].ReducedVarIndex= %d fReduced2FullVarIndex[countTemp]=%d varCount=%d \n",fVarPars[varCount].ReducedVarIndex,fReduced2FullVarIndex[countTemp],varCount);

	  if ( fVarPars[varCount].labelIndex >= 0) fVarLabelNumber++;  

	  countTemp++;    //counts total number of used variables
	}
	free(argv[0]);
	free(argv);
      }  
      varCount++;         //counts total number of variables 
    }

    if ( varLines != 0) delete [] varLines ;varLines = 0;

    //
    // give graph title names, and find number of plots involved in additive 
    // 
    Int_t monitorValueFirstPlotted = 0;
    for (Int_t i = 0; i < fVarNumber; i++) {
      l = fReduced2FullVarIndex[i];
      if (fVarPars[l].GlobalSumType >= 0){	  
	fVarPars[l].GlobalSumCount = 0;
	if ( fVarPars[l].GlobalSumType == l ) {
	  // this conditions defines first graph.graph title will
	  // be inserted at this i
	  monitorValueFirstPlotted = l;			 
	  sprintf(fVarPars[l].GlobalSumTitle,"Total_%s",fVarPars[l].Name);		
	  fVarPars[l].GlobalSumCount++;     
	} else { 
	  // other variables to be superimposed
	  sprintf(fVarPars[monitorValueFirstPlotted].GlobalSumTitle,"%s, %s",
		  &fVarPars[monitorValueFirstPlotted].GlobalSumTitle[0],&fVarPars[l].Name[0]);
	  ++fVarPars[monitorValueFirstPlotted].GlobalSumCount;
	}
      }
    }



    if (multiVarPlotIndex >= 0) {      
      monitorValueFirstPlotted = 0;
      for (Int_t i = 0; i < fVarNumber; i++) {
	l = fReduced2FullVarIndex[i];
	if (fVarPars[l].SuperimposeMultiVarType >= 0){	  
	  fVarPars[l].SuperimposeMultiVarCount = 0;
	  if ( fVarPars[l].SuperimposeMultiVarType == l ) {
	    // this conditions defines first graph.graph title will
	    // be inserted at this i
	    monitorValueFirstPlotted = l;			 
	    //	  sprintf(fVarPars[l].SuperimposeMultiVarTitle,"Total_%s",fVarPars[l].Name);		
	    fVarPars[l].SuperimposeMultiVarCount++;     
	  } else { 
	    // other variables to be superimposed
	    //		sprintf(fVarPars[monitorValueFirstPlotted].GlobalSumTitle,"%s, %s",&fVarPars[monitorValueFirstPlotted].GlobalSumTitle[0],
	    //		&fVarPars[l].Name[0]);
	    ++fVarPars[monitorValueFirstPlotted].SuperimposeMultiVarCount;
	  }
	}      
      }  
    }



    monitorValueFirstPlotted = 0;
    for (Int_t i = 0; i < fVarNumber; i++) {
      l = fReduced2FullVarIndex[i];
      if (fVarPars[l].GlobalMultiBoxType >= 0){	  
	fVarPars[l].GlobalMultiBoxCount = 0;
	if ( fVarPars[l].GlobalMultiBoxType == l ) {
	  // this conditions defines first graph.graph title will
	  // be inserted at this i
	  monitorValueFirstPlotted = l;			 
	  //	  sprintf(fVarPars[l].SuperimposeMultiVarTitle,"Total_%s",fVarPars[l].Name);		
	  fVarPars[l].GlobalMultiBoxCount++;     
	} 
	else { 
	  // other variables to be superimposed
	  //		sprintf(fVarPars[monitorValueFirstPlotted].GlobalSumTitle,"%s, %s",&fVarPars[monitorValueFirstPlotted].GlobalSumTitle[0],
	  //		&fVarPars[l].Name[0]);
	  ++fVarPars[monitorValueFirstPlotted].GlobalMultiBoxCount;
	}
      }      
    }  

    //
    // for each compouter, for superimposed variables
    // give graph title names, and find number of plots involved in additive 
    // 
    monitorValueFirstPlotted=0;  
    for (Int_t i = 0; i < fVarNumber; i++) {
	l = fReduced2FullVarIndex[i];
	fVarPars[l].EachSumCount = 0;
	if ( fVarPars[l].EachSumType >= 0 ){	  
	if ( fVarPars[l].EachSumType == l ) {
	  // this conditions defines first graph.graph title will
	  // be inserted at this i
	  monitorValueFirstPlotted = l;			 
	  fVarPars[l].EachSumCount++;
	} 
	else { 
	  ++fVarPars[monitorValueFirstPlotted].EachSumCount;
	}
      }
    }

    if (gDebug > 0) cout  << getTime()  << " SetVariables: end " <<endl;

}

 

void TDataProcess::SetColorCodes()
{    
    // initialize color codes

    Int_t colorOffset = 0;
    kColorCode[0]  = colorOffset + 2;
    kColorCode[1]  = colorOffset + 1;
    kColorCode[2]  = colorOffset + 4;
    kColorCode[3]  = colorOffset + 6;
    kColorCode[4]  = colorOffset + 16;
    //    kColorCode[5]  = colorOffset + 7;
    kColorCode[5]  = colorOffset + 7;
    kColorCode[6]  = colorOffset + 50;
    kColorCode[7]  = colorOffset + 3;
    kColorCode[8]  = colorOffset + 28;
    kColorCode[9]  = colorOffset + 39;
    kColorCode[10] = colorOffset +  2;
    kColorCode[11] = colorOffset +  1;
    kColorCode[12] = colorOffset +  4;
    kColorCode[13] = colorOffset +  6;
    kColorCode[14] = colorOffset +  16;
    kColorCode[15] = colorOffset +  7;
    kColorCode[16] = colorOffset +  50;
    kColorCode[17] = colorOffset +  3;
    kColorCode[18] = colorOffset +  28;
    kColorCode[19] = colorOffset +  39;

    // initialize sum each color codes
    kColorCodeEach[0] = 3;
    kColorCodeEach[1] = 16;
    kColorCodeEach[2] = 2;
    kColorCodeEach[3] = 4;
    kColorCodeEach[4] = 6;
    //    kColorCodeEach[5] = 7;
    kColorCodeEach[5] = 7;
    kColorCodeEach[6] = 1;
    kColorCodeEach[7] = 46;
    kColorCodeEach[8] = 38;
    kColorCodeEach[9] = 41;
}


void TDataProcess::InitArrays()
{    
    for (Int_t l = 0; l < fCompNumber*fVarNumber*fBinNumber; l++) {
	fEachValue[l]      = 0.0;
	fEachValueCount[l] = 0.0;
	fEachMaxValue[l]   = 0.0;
    }


    for (Int_t l = 0; l < fCompNumber*fVarNumber; l++ ) {
	fEachMax[l]          = 0.0;
	fEachMin[l]          = 9999999999.; // kludge
	fEachCurrentValue[l] = 0.0;
	fEachVarRef[l]       = 0;
    }

    for (Int_t l = 0; l < fVarNumber*fBinNumber; l++) {
	fGlobalSumValue[l]    = 0;
	fGlobalSumMaxValue[l] = 0;
    }

    for (Int_t i = 0; i < fVarNumber; i++) { 
	fGlobalMax[i]    = 0.0;
	fGlobalMin[i]    = 9999999999.; // kludge
	fGlobalSumMax[i] = 0.0;
	fGlobalSumMin[i] = 9999999999.; // kludge
    }

    for (Int_t i = 0; i < fCompNumber; i++){ 
      fReduced2FullCompActiveIndex[i]  = -1;
    }

}


void TDataProcess::InitXArray(Float_t* XArray)
{
  for (Int_t i = 0; i < fBinNumber; i++) XArray[i] = (Float_t) (fBinSize*i);  
}


//
Int_t  TDataProcess::GetCompId(Char_t* compName,Char_t* monitorSet) {  
  Int_t selectedCompId = -1;
  
  for (Int_t lComp = 0; lComp < fCompNumber; lComp++) {	   
    if (( strcmp(fCompPars[lComp].compName,compName) == 0 ) && ( strcmp(fCompPars[lComp].monsetName,monitorSet ) == 0 ) ) {
      selectedCompId = lComp;
      break;
    }
  }
  //  if (selectedCompId < 0) {
  //    cout <<"WARNING!! GetCompId. id is="<<selectedCompId << " should never happen" <<endl; 
  //}
  return selectedCompId;
}



Int_t  TDataProcess::GetCompId(Char_t* nameAndMonitorSet) {  
  Int_t selectedCompId = -1;
  
  for (Int_t lComp = 0; lComp < fCompNumber; lComp++) {	   
    if ( strcmp(fCompPars[lComp].baseName,nameAndMonitorSet) == 0 ) {
      selectedCompId = lComp;
      break;
    }
  }
  //if (selectedCompId < 0) {
  //    cout <<"WARNING!! GetCompId. id is="<<selectedCompId << " should never happen" <<endl; 
  //}
  return selectedCompId;
}


char** TDataProcess::string_to_argv(char *argstring, int *argc)
{
  char inquotes = 0, *workstring, **argv;
  int i, nchars;
  
  if((nchars = strlen(argstring)) > 0) {
    workstring = (char*)calloc((nchars + 2), sizeof(char*));
    //	fill the array with space characters
    for(i=0; i < (nchars + 2); i++) workstring[i] = ' '; 	  
    //	copy the argstring into the workstring (padded with spaces on both ends)
    for(i=0; i < nchars; i++) workstring[i+1] = argstring[i];
    for((*argc) = 1, i=0; i < nchars + 2; i++) {
      //	count the number of arguements
      if(isgraph(workstring[i]) > 0) {
	//	if current character is not whitespace and previous 
	//      character is whitespace, increment (*argc)
	if((isgraph(workstring[i-1]) == 0) && (inquotes == 0)) (*argc)++;
	//	check for the quote character
	if(workstring[i] == '"') {
	  //	toggle inquotes flag
	  inquotes = (char)(!inquotes);
	}		
      } else {
	//	convert whitespace to null characters
	if(!inquotes) workstring[i] = '\0';
      }
    }
    
    //	no arguements in the string
    if((*argc) == 0){
      free(workstring);
      return NULL;
    } else {
      inquotes = 0;		  
      argv = (char**)calloc((*argc), sizeof(char**));	    
      argv[0] = &workstring[0];
      
      for((*argc) = 1, i=1; i < nchars + 2; i++) {
	// count the number of arguements
	if(isgraph(workstring[i]) > 0) {
	  // if current character is not whitespace and previous 
	  // character is whitespace, increment (*argc)
	  if((isgraph(workstring[i-1]) == 0) && (inquotes == 0)) {
	    argv[(*argc)] = &workstring[i];
	    (*argc)++;
	  }
	  //	check for the quote character
	  if(workstring[i] == '"') {
	    //	toggle inquotes flag
	    inquotes = (char)(!inquotes);
	  }
	}
      }	  
      return argv;
    }
  }
  else {
    (*argc) = 0;
    return NULL;
  }
}
 
Int_t TDataProcess::CheckIfFileExists(Char_t * rootfile)
{
  Char_t line[256];
  Char_t aux[256];    
  Int_t fileExistsStatus = -1;
  sprintf(aux,"source %s/support/isFile.sh %s",gAffairPath,rootfile);	
  FILE *fp = gSystem->OpenPipe(aux,"r");
  if(fp != NULL) {
    // consistency check. Should get -1 if not exists, and 1 if exists
    fgets(line,256,fp);
  }
  else {		
    cout  << getTime()  << " fp is null. This should not happen " << endl;
  }
  gSystem->ClosePipe(fp);
  fileExistsStatus = atoi(line);	    	
  return(fileExistsStatus);
}


/////////////////////////////////////


#endif // #ifdef TDataProcess_cxx












