#ifndef TPlotMaker_h
#define TPlotMaker_h

#include <TROOT.h>
#include <TChain.h>
#include <TFile.h>
#include <TCanvas.h>
#include <TFrame.h>
#include <TText.h>
#include <TH1.h>
#include <TGraph.h>
#include <TH2.h>
#include <stdio.h>
#include <TSystem.h>
#include <TString.h>
#include <TGaxis.h>
#include <TColor.h>
#include <TError.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <string>
#include "TStyle.h"
#include <sys/stat.h>
#include "TObject.h"
#include "TAffairSql.h"
//

#include "affairGlobals.h"
#include "MonitorConstants.h"
namespace std {}
using namespace std;

class TPlotMaker:public TObject {
 private:
    TCanvas *fC1;
    TPad    *fP1;

    TWbox   **gBox;
    TWbox   **gMultiBox;
    TLine   **fLine;

    TLine   *fHorizontalAxisLine;

    Int_t    gBoxNumber;


    Char_t   fAffairPath[256];
    Char_t   fAffairPlotPath[256];
    Char_t   fUser[256];
    Char_t   fMonitorSet[256];
    // Double_t fBinSize; 

    Int_t    fTimeMax;
    Int_t    fTimeMin;

    Bool_t   gLogFlag; 

    Int_t    fCutVar;
    Double_t fCutLow;
    Double_t fCutHigh;

    Char_t   fXTitle[64];
    Char_t   fTimeFormat[64];
    Int_t    fXDiv;
    Int_t    fCompNumber;
    Float_t  gYStretchFactor;

    TText *fText1;
    TText *fText2;
    TText *fText3;
    TText *fText4;
    TText *fText5;
    TText *fText6;
    TText *fText7;

    TGaxis *fAxis2;


    TFile       *gRootFile;
    TDirectory *fDirTop;
    TDirectory *fDirTopGraph;
    TDirectory *fDirTopHisto;
    TDirectory *fDirTopComp;
    TDirectory **fDirComp;
    Float_t gGraphMinimum;
    Bool_t  gExtCanvas;
    Int_t   gOutputMode;
    Int_t   gLoopCount;
    Int_t  kColorCode[20];    
    Int_t  kColorCodeEach[10];    
    Int_t  kGreen;
    Int_t  kRed;
    Int_t  kBlack;
    Float_t kLabelFontSize;
    Bool_t gStatOption;

    TAffairSql *fAffairSql;
    Int_t     fUseMysql;
    Float_t   fRootVersion;
    Int_t     fUseRootToPlotGif;
    Char_t    fDatabase[128];

    void   CreateRootFile(Char_t * fileName, Char_t  headerName[128]); 
   char** string_to_argv(char *argstring, int *argc);

   public :
   void Init(Char_t *monset, Bool_t fillFlag,Char_t * fileName, Char_t *headerName);
   void SetLogFlag(Bool_t logFlag);
   void SetCut(Int_t cutVariable, Double_t cutLow,Double_t cutHigh);
   
   void SetRange(Int_t timeMax,Int_t timeMin);
   void SetGraphOptions(Int_t xDiv);

   void Graph(Char_t *PlotName,Char_t *YTitle,Int_t LineColor,Float_t graphMinimum,Float_t graphMaximum, 
		    Char_t *computerName,Char_t *drawStyle,TGraph * grh);
   
   void GraphBox(Char_t *plotTitle,Char_t *yTitle,Float_t graphMinimum,  Float_t * yValues, Int_t pointNumber);
   Int_t GraphMultiBox(Char_t *plotTitle,Char_t *labels,Char_t *yTitle,Float_t graphMaximum, Float_t * yValues, Int_t pointNumber,Int_t boxNumber,Int_t * yVarRef);
   void SuperimposeGraph(Int_t LineColor,Char_t *computerName,Int_t plotCount,Int_t lineStyle,Char_t *drawStyle,TText *t,TGraph * grh);
   void   Histo(Char_t *plotTitle,Char_t *xUnit, Int_t LineColor, Char_t *compName,TH1F * hst,Float_t histMaximum);
   void   SuperimposeHisto(Int_t LineColor, Int_t plotCount, Char_t* labelName, TText *t, TH1F * hst);
   // void   DoGif(Char_t *PlotName, Int_t eachOrGlobalLog);
   void   DoGif(Char_t *plotName, Int_t eachOrGlobalLog, Char_t * plotType);
   void   WriteHisto(TH1F *h){h->Write();} 
   void   WriteGraph(TGraph *g){g->Write();} 
   void   WriteFile(); 
   void   WriteCanvas(Char_t *cName); 
   void   CloseRootFile(); 
   void   SetPlotDir(Char_t *affairPlotPath) {sprintf(fAffairPlotPath,"%s",affairPlotPath);}
   void   SetUser(Char_t *user);
   void   MkDir(Int_t fCompNumber, Int_t lComp, Char_t * compName); 
   void   CdDir(Int_t lComp, Char_t * dirName); 
   TCanvas *  GetCanvas(){return fC1;} 
   void   SetCanvas(TCanvas *c1); 
   void   SetOutputMode(Int_t mode); 
   void   CreateCanvas(Float_t resizeFactor, Float_t yStretchFactor);
   void   ResizeCanvas(Float_t resizeFactor,Float_t yStretchFactor);
   void   SetCanvasYFactor(Float_t yStretchFactor);
   void   SetStatOption(Bool_t statOption){gStatOption = statOption;}
   void GraphCompActiveStatusBox(Char_t *plotTitle,Char_t *yTitle,Float_t * yValues, Int_t compNumber,Int_t binNumber,Int_t varNumber);
   void   SetMysql(TAffairSql * affairSql);


   TPlotMaker();
   ~TPlotMaker();

   ///   ClassDef (TPlotMaker,1) // affair class
   // 0 option creates a dummy streamer
   // with real stremer have problems with gVarPars and fCompPars
   ClassDef (TPlotMaker,0) // affair class
};

#endif

#ifdef TPlotMaker_cxx
//TPlotMaker::TPlotMaker(TTree *tree)
TPlotMaker::TPlotMaker()
{   
    fC1       = 0;
    fP1       = 0;
    gBox      = 0;
    gMultiBox = 0;
    fLine  = 0;
    fAxis2 = 0;
    fText1 = 0;
    fText2 = 0;
    fText3 = 0;
    fText4 = 0;
    fText5 = 0;
    fText6 = 0;
    fText7 = 0;
    fHorizontalAxisLine = 0;
    fDirComp     = 0;
    fDirTop      = 0;
    fDirTopGraph = 0;
    fDirTopHisto = 0;
    fDirTopComp  = 0;
    gExtCanvas   = kFALSE;
    gOutputMode  = k_YES_EPS;
    gLoopCount   = 0;
    gRootFile    = 0;

    gStyle->SetPalette(1);

    // initialize color codes
    //Int_t colorOffset = 100;
    Int_t colorOffset = 0;
    kColorCode[0] = colorOffset + 2;
    kColorCode[1] = colorOffset + 1;
    kColorCode[2] = colorOffset + 4;
    kColorCode[3] = colorOffset + 6;
    kColorCode[4] = colorOffset + 16;
    //    kColorCode[5] = colorOffset + 7;
    kColorCode[5] = colorOffset + 7;
    kColorCode[6] = colorOffset + 50;
    kColorCode[7] = colorOffset + 3;
    kColorCode[8] = colorOffset + 28;
    kColorCode[9] = colorOffset + 41;
    kColorCode[10] = 2;
    kColorCode[11] = 1;
    kColorCode[12] = 4;
    kColorCode[13] = 6;
    kColorCode[14] = 16;
    kColorCode[15] = 7;
    kColorCode[16] = 50;
    kColorCode[17] = 3;
    kColorCode[18] = 28;
    kColorCode[19] = 41;

    kGreen   = 3;
    kRed     = 2;
    kBlack   = 1;
    // initialize sum each color codes
    kColorCodeEach[0] = 3;
    kColorCodeEach[1] = 16;
    kColorCodeEach[2] = 2;
    kColorCodeEach[3] = 4;
    kColorCodeEach[4] = 6;
    //    kColorCodeEach[5] = 7;
    kColorCodeEach[5] = 7;
    kColorCodeEach[6] = 1;
    kColorCodeEach[7] = 46;
    kColorCodeEach[8] = 38;
    kColorCodeEach[9] = 41;

    gBoxNumber        = 0;

    gErrorIgnoreLevel = 1;
    gYStretchFactor   = -1.0;

    kLabelFontSize    = 0.04;
    gStatOption       = kTRUE;
    gStyle->SetOptStat("emr"); 
    gStyle->SetStatW(0.25);
    gStyle->SetStatX(0.9);
    gStyle->SetStatY(0.9);
    gStyle->SetStatBorderSize(0);
    gStyle->SetStatColor(38);
    
    fAffairSql           = 0;
    fUseMysql            = 0;

    Char_t *  testx;
    testx = (Char_t * ) gROOT->GetVersion();
    fRootVersion = atof(testx);
    fUseRootToPlotGif = 0; // don't use this feature until figure out how to make more detailed gif/png plots
    //cout << "TPlotMaker. fRootVersionascii = " << testx << "  fRootVersion= " << fRootVersion << endl;
    
    //    gDebug = 1; 
}

TPlotMaker::~TPlotMaker()
{
  if (fP1 != 0 )    delete fP1;fP1 = 0;
  if (fC1 != 0 )    delete fC1; fC1 = 0;
  if (fText1 != 0 ) delete fText1; fText1 = 0;
  if (fHorizontalAxisLine != 0 ) delete fHorizontalAxisLine; fHorizontalAxisLine = 0;
  if (gBox != 0 ) delete  [] gBox; gBox = 0; // not fully deleted. Correct in future
  if (gMultiBox != 0 ) delete [] gMultiBox; gMultiBox = 0; // not fully deleted. Correct in future
  if (fLine != 0 ) delete [] fLine; fLine = 0; // not fully deleted. Correct in future
  if (fAxis2 != 0 ) delete fAxis2; fAxis2  = 0;
  if (fText2 != 0 ) delete   fText2; fText2 = 0;
  if (fText3 != 0 ) delete   fText3; fText3 = 0;
  if (fText4 != 0 ) delete   fText4; fText4 = 0;
  if (fText5 != 0 ) delete   fText5; fText5 = 0;
  if (fText6 != 0 ) delete   fText6; fText6 = 0;
  if (fText7 != 0 ) delete   fText7; fText7 = 0;
  if (fDirComp != 0 ) delete   fDirComp; fDirComp = 0;
  if (fDirTop != 0 )  delete   fDirTop;  fDirTop  = 0;
  if (fDirTopGraph != 0 ) delete   fDirTopGraph;   fDirTopGraph = 0;
  if (fDirTopHisto != 0 ) delete   fDirTopHisto;   fDirTopHisto = 0;
  if (fDirTopComp != 0 ) delete    fDirTopComp;    fDirTopComp  = 0;
}

void TPlotMaker::SetRange(Int_t timeMin,Int_t timeMax)
{
    fTimeMax = timeMax;
    fTimeMin = timeMin;

    if ( (fTimeMax-fTimeMin) > 0 && (fTimeMax-fTimeMin) <= kSecondsHour ) {
	sprintf(fXTitle,"%s","min");
	sprintf(fTimeFormat,"%s","%M");
    } 
    else if ( (fTimeMax-fTimeMin) > kSecondsHour && (fTimeMax-fTimeMin) <= 2*kSecondsDay ) {
	sprintf(fXTitle,"%s","hour");
	sprintf(fTimeFormat,"%s","%H");
    } 
    else {
	sprintf(fXTitle,"%s","day");
	sprintf(fTimeFormat,"%s","%d");
    }
}


void  TPlotMaker::SetMysql(TAffairSql * affairSql) 
{
  fAffairSql  = affairSql;
  fUseMysql   = 1;

  sprintf(fDatabase,"%s",fAffairSql->GetDatabaseName());

  // set root version here.
  Char_t sql[500];
  //  sprintf(sql,"REPLACE INTO AFFAIR_PARS (ID,ROOT_VERSION) VALUES (1,%f) ",fRootVersion);
  //  fAffairSql->Exec(sql);
}


void TPlotMaker::SetGraphOptions(Int_t xDiv)
{
    //sprintf(fXTitle,"%s",xTitle);
    //sprintf(fTimeFormat,"%s",timeFormat);
    fXDiv = xDiv;
}

void TPlotMaker::SetCut(Int_t cutVariable, Double_t cutLow,Double_t cutHigh)
{
    fCutVar  = cutVariable;
    fCutLow  = cutLow;
    fCutHigh = cutHigh;
}


void TPlotMaker::SetLogFlag(Bool_t logFlag)
{    
    gLogFlag = logFlag;
}

void TPlotMaker::SetCanvas(TCanvas * c1)
{    
  //  cout << " TPlotMaker::SetCanvas "<<endl;
    fC1 = c1;
    gExtCanvas = kTRUE;
}

void  TPlotMaker::SetOutputMode(Int_t mode)
{
  gOutputMode = mode;
}


void TPlotMaker::Init(Char_t *monset, Bool_t fillFlag,Char_t * fileName, Char_t *headerName)
{    
  Float_t yStretchFactor;
  
  sprintf(fAffairPath,"%s",getenv("AFFAIR_MONITOR"));
  if (strlen(fAffairPlotPath) < 2) sprintf(fAffairPlotPath,"%s/plots",fAffairPath);

  sprintf(fMonitorSet,"%s",monset);
  sprintf(fUser,"");
  
  // set up stuff to plot
  if (gYStretchFactor > 0.) {
    yStretchFactor = gYStretchFactor;
  } else {
    yStretchFactor = 1.0;
  }
  
  if (fillFlag == true ) CreateRootFile(fileName, headerName);
  gLoopCount++;
  
  if (gLoopCount > 1) return;
  CreateCanvas(1.0,yStretchFactor);
  
  gStyle->SetTitleX(0.2);
  gStyle->SetTitleW(0.5);
  gStyle->SetTitleH(0.08);
  gStyle->SetLabelSize(0.05);
  gStyle->SetLabelSize(0.07,"Y");
  
  gStyle->SetHistLineWidth(6);
  
  gStyle->SetTextFont(62);
  gStyle->SetTextSize(0.063);
  
  
  if ( fText1 != 0 ) delete fText1; fText1 = new TText();
  if ( fHorizontalAxisLine != 0 ) delete fHorizontalAxisLine; fHorizontalAxisLine = new TLine();
  if ( fText2 != 0 ) delete fText2; fText2 = new TText();
  if ( fText3 != 0 ) delete fText3; fText3 = new TText();
  if ( fText4 != 0 ) delete fText4; fText4 = new TText();
  if ( fText5 != 0 ) delete fText5; fText5 = new TText();
  if ( fText6 != 0 ) delete fText6; fText6 = new TText();
  if ( fText7 != 0 ) delete fText7; fText7 = new TText();
  
  fCutVar = -1;  

}

void TPlotMaker::WriteCanvas(Char_t *cName){
    fC1->Write(cName);
} 


void   TPlotMaker::SetUser(Char_t *user) {
  sprintf(fUser,"%s_",user);
}


void TPlotMaker::ResizeCanvas(Float_t resizeFactor,Float_t yStretchFactor){
  //  fC1->SetWindowSize ((Int_t) (resizeFactor*300.0),(Int_t) (yStretchFactor*resizeFactor*110.0));
  fC1->SetCanvasSize ((Int_t) (resizeFactor*30000.0),(Int_t) (yStretchFactor*resizeFactor*11000.0));
} 

void TPlotMaker::SetCanvasYFactor(Float_t yStretchFactor) {
  gYStretchFactor = yStretchFactor;
}

void TPlotMaker::CreateCanvas(Float_t resizeFactor,Float_t yStretchFactor) {
  if (fP1 != 0) delete fP1; fP1 = 0;
  // printf(" aa yStretchFactor=%f \n",yStretchFactor);
  if (gExtCanvas == kFALSE ) {
    if (fC1 != 0) delete  fC1; fC1 = 0;
    // fC1 = new TCanvas("c1","plot canvas",200,10,(Int_t) (resizeFactor*300.0),(Int_t) (yStretchFactor*resizeFactor*110.0));
    fC1 = new TCanvas("c1","plot canvas",30000,(Int_t) (yStretchFactor*11000));
    //  printf(" bbaa \n");
  }
  
  fC1->SetBorderSize(0);

  fC1->Range(0,0,1,1);
  fC1->SetFillColor(0);
  fC1->SetGrid();
  fC1->Update();
  
  fP1 = new TPad("pad","pad",.0001,0.001,.92,.99);

  fP1->SetFillColor(0);  

  fP1->SetFrameBorderSize(0);  
  fP1->SetFrameBorderMode(0);  
  fP1->SetFrameLineWidth(0);  
  fP1->SetFrameLineColor(1);  
  fP1->SetFrameLineStyle(0);  
  fP1->SetFrameFillColor(0);  
  fP1->SetFrameBorderMode(0);  


  fP1->SetBorderMode(0);  
  fP1->SetBorderSize(0);  

  fC1->Update();

  fC1->GetFrame()->SetFillColor(0);  
  fC1->GetFrame()->SetBorderMode(0);  
  fC1->GetFrame()->SetBorderSize(0);  
  fC1->SetFrameBorderMode(0);


  fP1->SetBottomMargin(0.13); 

  fP1->Draw();
  fP1->cd();
  fP1->SetGrid();
  gStyle->SetGridStyle(3);
  gStyle->SetGridColor(17);

  
}



char ** TPlotMaker::string_to_argv(char *argstring, int *argc)
{
  char inquotes = 0, *workstring, **argv;
  int i, nchars;
  
  if((nchars = strlen(argstring)) > 0) {
    workstring = (char*)calloc((nchars + 2), sizeof(char*));
    //	fill the array with space characters
    for(i=0; i < (nchars + 2); i++) workstring[i] = ' '; 	  
    //	copy the argstring into the workstring (padded with spaces on both ends)
    for(i=0; i < nchars; i++) workstring[i+1] = argstring[i];
    for((*argc) = 1, i=0; i < nchars + 2; i++) {
      //	count the number of arguements
      if(isgraph(workstring[i]) > 0) {
	//	if current character is not whitespace and previous 
	//      character is whitespace, increment (*argc)
	if((isgraph(workstring[i-1]) == 0) && (inquotes == 0)) (*argc)++;
	//	check for the quote character
	if(workstring[i] == '"') {
	  //	toggle inquotes flag
	  inquotes = (char)(!inquotes);
	}		
      } else {
	//	convert whitespace to null characters
	if(!inquotes) workstring[i] = '\0';
      }
    }
    
    //	no arguements in the string
    if((*argc) == 0){
      free(workstring);
      return NULL;
    } else {
      inquotes = 0;		  
      argv = (char**)calloc((*argc), sizeof(char**));	    
      argv[0] = &workstring[0];
      
      for((*argc) = 1, i=1; i < nchars + 2; i++) {
	// count the number of arguements
	if(isgraph(workstring[i]) > 0) {
	  // if current character is not whitespace and previous 
	  // character is whitespace, increment (*argc)
	  if((isgraph(workstring[i-1]) == 0) && (inquotes == 0)) {
	    argv[(*argc)] = &workstring[i];
	    (*argc)++;
	  }
	  //	check for the quote character
	  if(workstring[i] == '"') {
	    //	toggle inquotes flag
	    inquotes = (char)(!inquotes);
	  }
	}
      }	  
      return argv;
    }
  }
  else {
    (*argc) = 0;
    return NULL;
  }
}

#endif // #ifdef TPlotMaker_cxx












