#! /usr/bin/perl
# batch.pl, simple program to help add datasources to an existing RRD
#   goes with add_ds.pl
#
#    Copyright (C) 2000 Selena M. Brewington
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# 
# for use with add_ds.pl script to add datasources to an RRD
#
# usage: ls -1 | ./batch.pl [-o] <# ds to add>
#
# -o will let you overwrite your rrds.  i don't recommend using this 
# switch the first time you run this program.  If something
# gets messed up, the original xml files will be in the xml
# directory and you can run 'rrdtool restore' on all of them
# uncomment the commented out lines below to make this work.
#
# also, you can change the name of the directory the XML is
# getting dumped to, where your rrdtool binary is located, 
# and where add_ds.pl is located.  the variables are listed 
# below.
#
# This script could theoretically take fully-qualified pathnames
# as input from STDIN rather than the output from ls -1. 
#

use strict;

my $AffairDir     = "$ENV{'AFFAIR_MONITOR'}";
########### USER CONFIGURABLE SECTION #######################
my $rrddir    = "$AffairDir/rrd";
my $newdir    = "$AffairDir/rrd/xml";
my $backupdir = "$AffairDir/rrd/backup";
my $rrdtool   = "$AffairDir/rrdtool/bin/rrdtool";
my $add_ds    = "$AffairDir/support/add_ds.pl";  # path to add_ds.pl script

########### END CONFIGURE SECTION ###########################

my $i;
my $argCount = @ARGV;

if ( $argCount < 2) {
    print "not enought arguments \n";
    exit 0;
}

my $overwrite = 1;

my $monset = $ARGV[0];
my $dsNumber = $ARGV[1];

if ( $dsNumber < 1) {
    print "ds number  should be greater than 0 \n";
    exit 0;
}

if (! (-x $newdir)) {
  system(" mkdir $newdir ");
}

if (! (-x $backupdir)) {
  system(" mkdir $backupdir ");
}
system(" cp $AffairDir/rrd/*$monset*.rrd  $backupdir ");


opendir(DIRHANDLE,"$rrddir");
my @tmpfiles = grep(/.*rrd$/,readdir(DIRHANDLE));
my @allfiles = sort @tmpfiles;
my $number_of_elements = @allfiles; 



for( $i=0; $i<$number_of_elements; $i++) {
  next unless ( $allfiles[$i] =~ /rrd/ && $allfiles[$i] =~ /$monset/ );

  chop;
  my $file = $allfiles[$i];
  my $rrdfile = $file;
  $file =~ s/rrd$/xml/;
  my $xmlfile = $file;
  
  open(FILE, ">$newdir/$xmlfile");
  my @output = `$rrdtool dump $rrddir/$rrdfile`;
  print FILE @output;

  close(FILE);

  open (FILE, ">$newdir/$xmlfile.2");
  my @new = `cat $newdir/$xmlfile | $add_ds $dsNumber`; 

  print FILE @new;
  close (FILE);

  system("$rrdtool restore $newdir/$xmlfile.2 $newdir/$rrdfile") == 0 
    or die "rrdtool restore failed for $file";
  if ($overwrite == 1) {
      print "expanded  rrd file with $dsNumber additional data sources created for monitoring set $monset and file $rrdfile. Old files are in $AffairDir/backup \n";
      system("mv $newdir/$rrdfile $rrddir/$rrdfile") == 0
	  or die "can't overwrite $file";
  }
}




